// Copyright, Finjan Software Ltd., 2006.
// Greasemonkey software- Copyright 2004-2005 Aaron Boodman
// Turnabout software- Copyright 2005, Reify Software, Inc.


// this file is the javascript backing for the UI wrangling which happens in
// browser.xul. It also initializes the Greasemonkey singleton which contains
// all the main injection logic, though that should probably be a proper XPCOM
// service and wouldn't need to be initialized in that case.
var TA_BrowserUI = new Object();

/**
 * nsISupports.QueryInterface
 */
TA_BrowserUI.QueryInterface = function(aIID) {
  if (!aIID.equals(Components.interfaces.nsISupports) &&
      !aIID.equals(Components.interfaces.taIBrowserWindow) &&
      !aIID.equals(Components.interfaces.nsISupportsWeakReference))
    throw Components.results.NS_ERROR_NO_INTERFACE;

  return this;
}


/**
 * Called when this file is parsed, by the last line. Set up initial objects,
 * do version checking, and set up listeners for browser xul load and location
 * changes.
 */
TA_BrowserUI.init = function() {
  TA_log("TA_BrowserUI.init");

  this.menuCommanders = [];
  this.currentMenuCommander = null;

  TA_listen(window, "load", TA_hitch(this, "chromeLoad"));
  TA_listen(window, "unload", TA_hitch(this, "chromeUnload"));

}

/**
 * The browser XUL has loaded. Find the elements we need and set up our
 * listeners and wrapper objects.
 */
TA_BrowserUI.chromeLoad = function(e) {
  // get all required DOM elements
  this.tabBrowser = document.getElementById("content");
  this.appContent = document.getElementById("appcontent");

  this.statusImage = document.getElementById("ta-status-image");
  this.statusLabel = document.getElementById("ta-status-label");
  this.statusPopup = document.getElementById("ta-status-popup");
  this.statusEnabledItem = document.getElementById("ta-status-enabled-item");

  this.enabledWatcher = TA_hitch(this, "refreshStatus");
  TA_prefRoot.watch("enabled", this.enabledWatcher);

  TA_listen(this.appContent, "DOMContentLoaded", TA_hitch(this, "contentLoad"));
  // listen for clicks on the install bar
  Components.classes["@mozilla.org/observer-service;1"]
            .getService(Components.interfaces.nsIObserverService)
            .addObserver(this, "install-userscript", true);
  // this gives us onLocationChange
  document.getElementById("content").addProgressListener(this,
    Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);

  // register for notifications from greasemonkey-service about ui type things
  this.gmSvc = Components.classes["@securebrowsing.mozdev.org/securebrowsing-service;1"]
                         .getService(Components.interfaces.taIsecurebrowsingService);


  this.gmSvc.registerBrowser(this);

  TA_initta(false);

  this.refreshStatus();
}

TA_initta = function(force) {
	// Reset version showing bits if a new version was installed
	var CURRENT_VERSION = '3.007';
	var SCRIPTS_VERSION = '3.000'
	if (TA_prefRoot.getValue("forceInit", false) || (TA_prefRoot.getValue("engineVersion", "0.0") != CURRENT_VERSION) || force) {
		TA_log("This is a new version of the XPI");
        TA_prefRoot.setValue("engineVersion", CURRENT_VERSION);
		TA_prefRoot.setValue("ta.scriptsVersion", SCRIPTS_VERSION);
		var fileObj = TA_getScriptDir();
		fileObj.append("ta.dynamic.user.js");
		var content = TA_readFile(fileObj.path);
		if ((content != null) && (content.length > 1)) {
			   TA_log("Installing dynamic script");
			   TA_prefRoot.setValue("ta.modules.app", content);
		}

		var customFileObj = TA_getScriptDir();
		customFileObj.append("ta.custom.user.js");
		var customContent = TA_readFile(customFileObj.path);
		if ((customContent != null) && (customContent.length > 1)) {
			   TA_log("Installing custom script");
			   TA_prefRoot.setValue("ta.modules.custom", customContent);
		}
	}
}

TA_readFile = function(fileName) {
	var file = Components.classes["@mozilla.org/file/local;1"]
		.createInstance(Components.interfaces.nsILocalFile);
	file.initWithPath(fileName);
	if (!file.exists()) {
    TA_log("File does not exist: " + fileName);
	}

	var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
            		.createInstance( Components.interfaces.nsIFileInputStream );
	is.init(file, 0x01, 00004, null);
	var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
    		.createInstance( Components.interfaces.nsIScriptableInputStream );
	sis.init(is);
	var content = sis.read(sis.available());
	return content;
}

/**
 * gmIBrowserWindow.registerMenuCommand
 */
TA_BrowserUI.registerMenuCommand = function(menuCommand) {
  if (this.isMyWindow(menuCommand.window)) {
    var commander = this.getCommander(menuCommand.window);

    commander.registerMenuCommand(menuCommand.name,
                                  menuCommand.doCommand,
                                  menuCommand.accelKey,
                                  menuCommand.accelModifiers,
                                  menuCommand.accessKey);
  }
}

/**
 * gmIBrowserWindow.openInTab
 */
TA_BrowserUI.openInTab = function(domWindow, url) {
  if (this.isMyWindow(domWindow)) {
    document.getElementById("content").addTab(url);
  }
}

/**
 * Gets called when a DOMContentLoaded event occurs somewhere in the browser.
 * If that document is in in the top-level window of the focused tab, find
 * it's menu items and activate them.
 */
TA_BrowserUI.contentLoad = function(e) {
  var unsafeWin;
  var href;
  var commander;

  if (TA_deepWrappersEnabled(window)) {
    // when deep wrappers are enabled, e.target is already a deep xpcnw
    unsafeWin = e.target.defaultView;

    // in DPa2, there was a bug that made this *not* a deep wrapper.
    if (unsafeWin.wrappedJSObject) {
      unsafeWin = unsafeWin.wrappedJSObject;
    }

    href = e.target.location.href;
  } else {
    // otherwise we need to wrap it manually
    unsafeWin = new XPCNativeWrapper(
                  new XPCNativeWrapper(e, "target").target,
                  "defaultView").defaultView;
    href = new XPCNativeWrapper(
              new XPCNativeWrapper(unsafeWin, "location").location,
              "href").href;
  }

  if (TA_getEnabled() && TA_isGreasemonkeyable(href)) {
    commander = this.getCommander(unsafeWin);

    // if this content load is in the focused tab, attach the menuCommaander
    if (unsafeWin == this.tabBrowser.selectedBrowser.contentWindow) {
      this.currentMenuCommander = commander;
      this.currentMenuCommander.attach();
    }

    if (TA_prefRoot.getValue("devMode", false)) {
        TA_initta(true);
    }


    this.gmSvc.domContentLoaded({ wrappedJSObject: unsafeWin });

    TA_listen(unsafeWin, "pagehide", TA_hitch(this, "contentUnload"));
  }

}

/**
 * Implements nsIObserve.observe. Right now we're only observing our own
 * install-userscript, which happens when the install bar is clicked.
 */
TA_BrowserUI.observe = function(subject, topic, data) {
}

/**
 * The browser's location has changed. Usually, we don't care. But in the case
 * of tab switching we need to change the list of commands displayed in the
 * User Script Commands submenu.
 */
TA_BrowserUI.onLocationChange = function(a,b,c) {
  if (this.currentMenuCommander != null) {
    this.currentMenuCommander.detach();
    this.currentMenuCommander = null;
  }

  var menuCommander = this.getCommander(this.tabBrowser.selectedBrowser.
                                        contentWindow);

  if (menuCommander) {
    this.currentMenuCommander = menuCommander;
    this.currentMenuCommander.attach();
  }
}

/**
 * A content document has unloaded. We need to remove it's menuCommander to
 * avoid leaking it's memory.
 */
TA_BrowserUI.contentUnload = function(e) {
  if (e.persisted) {
    return;
  }

  var unsafeWin = e.target.defaultView;

  // remove the commander for this document
  var commander = null;

  // looping over commanders rather than using getCommander because we need
  // the index into commanders.splice.
  for (var i = 0; item = this.menuCommanders[i]; i++) {
    if (item.win == unsafeWin) {

      TA_logInt("* Found corresponding commander. Is currentMenuCommander: " +
          (item.commander == this.currentMenuCommander));

      if (item.commander == this.currentMenuCommander) {
        this.currentMenuCommander.detach();
        this.currentMenuCommander = null;
      }

      this.menuCommanders.splice(i, 1);

      TA_logInt("* Found and removed corresponding commander")
      break;
    }
  }
}

/**
 * The browser XUL has unloaded. We need to let go of the pref watcher so
 * that a non-existant window is not informed when greasemonkey enabled state
 * changes. And we need to let go of the progress listener so that we don't
 * leak it's memory.
 */
TA_BrowserUI.chromeUnload = function() {
  TA_prefRoot.unwatch("enabled", this.enabledWatcher);
  this.tabBrowser.removeProgressListener(this);
  this.gmSvc.unregisterBrowser(this);
  delete this.menuCommanders;
}


/**
 * Helper method which gets the menuCommander corresponding to a given
 * document
 */
TA_BrowserUI.getCommander = function(unsafeWin) {
  try {
  for (var i = 0; i < this.menuCommanders.length; i++) {
    if (this.menuCommanders[i].win == unsafeWin) {
      return this.menuCommanders[i].commander;
    }
  }

  // no commander found. create one and add it.
  var commander = new TA_MenuCommander(document);
  this.menuCommanders.push({win:unsafeWin, commander:commander});

  return commander;
  } catch (e) {
	TA_log("GOT: " + e);
  }
}

/**
 * Helper to determine if a given dom window is in this tabbrowser
 */
TA_BrowserUI.isMyWindow = function(domWindow) {
  var tabbrowser = getBrowser();
  var browser;

  for (var i = 0; browser = tabbrowser.browsers[i]; i++) {
    if (browser.contentWindow == domWindow) {
      return true;
    }
  }

  return false;
}

TA_BrowserUI.refreshStatus = function() {
  if (TA_getEnabled()) {
    this.statusImage.src = "chrome://securebrowsing/content/status_on.gif";
    this.statusImage.tooltipText = "M86Security Secure Browsing is enabled";
    this.statusEnabledItem.label = "Disable M86Security Secure Browsing"
  } else {
    this.statusImage.src = "chrome://securebrowsing/content/status_off.gif";
    this.statusImage.tooltipText = "M86Security Secure Browsing is disabled";
    this.statusEnabledItem.label = "Enable M86Security Secure Browsing"
  }

  document.getElementById("ta-options-unsafe-urls-only-item").setAttribute("checked", TA_getOptionsUnsafeURLsOnly());
  document.getElementById("ta-options-allow-unchecked-access-item").setAttribute("checked", TA_getOptionsAllowUncheckedAccess());
}


// necessary for webProgressListener implementation
TA_BrowserUI.onProgressChange = function(webProgress,b,c,d,e,f){}
TA_BrowserUI.onStateChange = function(a,b,c,d){}
TA_BrowserUI.onStatusChange = function(a,b,c,d){}
TA_BrowserUI.onSecurityChange = function(a,b,c){}
TA_BrowserUI.onLinkIconAvailable = function(a){}

TA_loggify(TA_BrowserUI, "TA_BrowserUI");

TA_logInt("calling init...")
TA_BrowserUI.init();

// the following functions were copied wholesale from old code without
// refactoring. need to be reorganized a little.

// this is a debug function to make sure that menuCommanders are being
// created and destroyed at the correct time.

// window.setInterval(TA_checkState, 3000);

function TA_checkState() {
  var commander;
  var urls = [];
  for (var i = 0; commander = TA_BrowserUI.menuCommanders[i]; i++) {
    urls.push(commander.win.location.href);
  }

  TA_logInt(urls.length + " active commanders: " + urls.join(", "));
}


function TA_getOptionsUnsafeURLsOnly() {
  return TA_prefRoot.getValue("ta.options.showUnsafeOnly", false);
}

function TA_setOptionsUnsafeURLsOnly(flag) {
  TA_prefRoot.setValue("ta.options.showUnsafeOnly", flag);
}

function TA_getOptionsAllowUncheckedAccess() {
  return TA_prefRoot.getValue("ta.options.allowUncheckedAccess", true);
}

function TA_setOptionsAllowUncheckedAccess(flag) {
  TA_prefRoot.setValue("ta.options.allowUncheckedAccess", flag);
}

function TA_getEngineVersion() {

    return TA_prefRoot.getValue("engineVersion", "0.0");
}

function TA_getScriptsVersion() {

    return TA_prefRoot.getValue("ta.scriptsVersion", "0.0");
}

function TA_getBrowserVersion() {
    var index = navigator.userAgent.indexOf('Firefox/');
    if (index == -1) {
        return "0.0";
    }
    return navigator.userAgent.substring(index + 8);
}
