// Copyright, Finjan Software Ltd., 2006.
// Greasemonkey software- Copyright 2004-2005 Aaron Boodman
// Turnabout software- Copyright 2005, Reify Software, Inc.


var TA_consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                        .getService(Components.interfaces.nsIConsoleService);

function TA_isDef(thing) {
  return typeof(thing) != "undefined";
}

function TA_hitch(obj, meth) {
  if (!obj[meth]) {
    throw "method '" + meth + "' does not exist on object '" + obj + "'";
  }

  var staticArgs = Array.prototype.splice.call(arguments, 2, arguments.length);

  return function() {
    // make a copy of staticArgs (don't modify it because it gets reused for
    // every invocation).
    var args = staticArgs.concat();

    // add all the new arguments
    for (var i = 0; i < arguments.length; i++) {
      args.push(arguments[i]);
    }

    // invoke the original function with the correct this obj and the combined
    // list of static and dynamic arguments.
    return obj[meth].apply(obj, args);
  };
}

function TA_listen(source, event, listener, opt_capture) {
  Components.lookupMethod(source, "addEventListener").apply(
    source, [event, listener, opt_capture]);
}

function TA_unlisten(source, event, listener, opt_capture) {
  Components.lookupMethod(source, "removeEventListener").apply(
    source, [event, listener, opt_capture]);
}

/**
 * Utility to create an error message in the log without throwing an error.
 */
function TA_logError(e, opt_warn) {
  var consoleService = Components.classes['@mozilla.org/consoleservice;1']
    .getService(Components.interfaces.nsIConsoleService);

  var consoleError = Components.classes['@mozilla.org/scripterror;1']
    .createInstance(Components.interfaces.nsIScriptError);

  var flags = opt_warn ? 1 : 0;

  consoleError.init(e.message, e.fileName, e.lineNumber, e.lineNumber,
                    e.columnNumber, flags, null);

  consoleService.logMessage(consoleError);
}

function TA_log(message, force) {
  if (force || TA_prefRoot.getValue("logChrome", false)) {
    TA_consoleService.logStringMessage(message);
  }
}

function TA_getContents(aURL, charset){
  if( !charset ) {
    charset = "UTF-8"
  }
  var ioService=Components.classes["@mozilla.org/network/io-service;1"]
    .getService(Components.interfaces.nsIIOService);
  var scriptableStream=Components
    .classes["@mozilla.org/scriptableinputstream;1"]
    .getService(Components.interfaces.nsIScriptableInputStream);
  // http://lxr.mozilla.org/mozilla/source/intl/uconv/idl/nsIScriptableUConv.idl
  var unicodeConverter = Components
    .classes["@mozilla.org/intl/scriptableunicodeconverter"]
    .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
  unicodeConverter.charset = charset;

  var channel=ioService.newChannelFromURI(aURL);
  var input=channel.open();
  scriptableStream.init(input);
  var str=scriptableStream.read(input.available());
  scriptableStream.close();
  input.close();

  try {
    return unicodeConverter.ConvertToUnicode(str);
  } catch( e ) {
    return str;
  }
}

function TA_getWriteStream(file) {
  var stream = Components.classes["@mozilla.org/network/file-output-stream;1"]
    .createInstance(Components.interfaces.nsIFileOutputStream);

  stream.init(file, 0x02 | 0x08 | 0x20, 420, -1);

  return stream;
}

function TA_getScriptFileURI(fileName) {
  return Components.classes["@mozilla.org/network/io-service;1"]
                   .getService(Components.interfaces.nsIIOService)
                   .newFileURI(TA_getScriptFile(fileName));
}

function TA_getScriptFile(fileName) {
  var file = TA_getScriptDir();
  file.append(fileName);
  return file;
}

function TA_getScriptDir() {
  return TA_getNewScriptDir();
}

function TA_getNewScriptDir() {
  var file = TA_getContentDir();
  file.append("scripts");
  return file;
}

function TA_getContentDir() {
  var reg = Components.classes["@mozilla.org/chrome/chrome-registry;1"]
                      .getService(Components.interfaces.nsIChromeRegistry);

  var ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
                        .getService(Components.interfaces.nsIIOService);

  var proto = Components.classes["@mozilla.org/network/protocol;1?name=file"]
                        .getService(Components.interfaces.nsIFileProtocolHandler);

  var chromeURL = ioSvc.newURI("chrome://securebrowsing/content", null, null);
  var fileURL = reg.convertChromeURL(chromeURL);
  var file = proto.getFileFromURLSpec(fileURL.spec).parent;

  return file
}

function TA_deepWrappersEnabled(someXPCObject) {
  // the old school javacript wrappers had this property containing their
  // untrusted variable. the new ones don't.
  return !(new XPCNativeWrapper(someXPCObject).mUntrustedObject);
}

function TA_isGreasemonkeyable(url) {
  var scheme = Components.classes["@mozilla.org/network/io-service;1"]
               .getService(Components.interfaces.nsIIOService)
               .extractScheme(url);

  return (scheme == "http" || scheme == "https" || scheme == "file") &&
         !/hiddenWindow\.html$/.test(url);
}

function TA_isFileScheme(url) {
  var scheme = Components.classes["@mozilla.org/network/io-service;1"]
               .getService(Components.interfaces.nsIIOService)
               .extractScheme(url);

  return scheme == "file";
}

function TA_getEnabled() {
  return TA_prefRoot.getValue("enabled", true);
}

function TA_setEnabled(enabled) {
  TA_prefRoot.setValue("enabled", enabled);
}


/**
 * Logs a message to the console. The message can have python style %s
 * thingers which will be interpolated with additional parameters passed.
 */
function TA_logInt(message) {
  if (TA_prefRoot.getValue("logChrome", false)) {
    TA_logf.apply(null, arguments);
  }
}

function TA_logf(message) {
  for (var i = 1; i < arguments.length; i++) {
    message = message.replace(/\%s/, arguments[i]);
  }

  dump(message + "\n");
}

/**
 * Loggifies an object. Every method of the object will have it's entrance,
 * any parameters, any errors, and it's exit logged automatically.
 */
function TA_loggify(obj, name) {
  for (var p in obj) {
    if (typeof obj[p] == "function") {
      obj[p] = TA_gen_loggify_wrapper(obj[p], name, p);
    }
  }
}

function TA_gen_loggify_wrapper(meth, objName, methName) {
return function() {
     var retVal;
    //var args = new Array(arguments.length);
    var argString = "";
    for (var i = 0; i < arguments.length; i++) {
      //args[i] = arguments[i];
      argString += arguments[i] + (((i+1)<arguments.length)? ", " : "");
    }

    TA_logInt("> %s.%s(%s)", objName, methName, argString);//args.join(", "));

    try {
      return retVal = meth.apply(this, arguments);
    } finally {
      TA_logInt("< %s.%s: %s",
          objName,
          methName,
          (typeof retVal == "undefined" ? "void" : retVal));
    }
  }
}
