
// Copyright, Finjan Software Ltd., 2006.
// Greasemonkey software- Copyright 2004-2005 Aaron Boodman
// Turnabout software- Copyright 2005, Reify Software, Inc.

var TACode = {
	VERSION: 3.010,

	d: function(message) {
		if (GM_getValue('logger.debug', false)) {
			GM_log("DEBUG: " + message);
		}
	},

	i: function(message) {
		if (GM_getValue('logger.debug', false)) {
			GM_log("INFO: " + message);
		}
	},

	addSafeMethod: function(obj) {
		obj.safe = function(f) {
		     var thisObject = this;
		     return function() {
		       try {
			       var args2 = [];
			       for (var i = 0; i < arguments.length; i++) {
				   args2.push(arguments[i]);
			       }
			       return f.apply(thisObject, args2);
		       } catch (e) {
				 TACode.i("Safe catch: " + e);
				 for (x in e) {	TACode.d(x + ": " + x[e]); }
		       }
		     };
		 };

	},


	/**
	 * Manager
	 * =======
	 */
	Manager: function() {
	TACode.addSafeMethod(this);
		this.linksEnhancerAncestor = new TACode.LinksEnhancer();
		this.toolbarAncestor = new TACode.Toolbar();
		this.linksEnhancer = this.linksEnhancerAncestor;
		this.toolbar = this.toolbarAncestor;
		this.messaging = new TACode.Messaging();
		this.siteMapper = new TACode.SiteMapper();
		this.pause = false;
		this.pauseTimeout = null;
		this.pauseTimeoutVar = 500;
		this.timeoutVar = 0;

		/**
		 * Initialize the system if the site is relevant while creating
		 * all required objects.
		 */
		this.init = function(siteLocation, customSites) {
			TACode.Utils.Effect.Opacity.prototype = new TACode.Utils.Effect.Base();
			this.customSites = customSites;
			this.customSites.loadSiteDefinitions(this.siteMapper);
			var mapResults = this.siteMapper.mapSite(siteLocation, this.linksEnhancerAncestor, this.toolbarAncestor, this);
			

			if (TACode.Utils.Element.isIE()) {
				window.attachEvent("onscroll", this.safe(function() { this.pauseEnhanceLinks(); }));
				window.attachEvent("onresize", this.safe(function() { this.pauseEnhanceLinks(); }));
				document.attachEvent("onclick", this.safe(function() { this.pauseEnhanceLinks(); }));
				
			} else {
				TACode.Utils.Event.observe(window, 'DOMNodeInserted', this.safe(function() { this.pauseEnhanceLinks(); }), false);
			}
			
			
			if (!mapResults) {
				TACode.d("Secure Browsing not Initialized, site is irrelevant");
				// Register an observer cleanup function. Without it firefox leaks memory
				if (TACode.Utils.Element.isIE()) {
					window.attachEvent('onunload', this.safe(function() { TACode.Utils.Event.unloadCache();	}) );
				} else {
					TACode.Utils.Event.observe(window, 'unload', function() { TACode.Utils.Event.unloadCache() }, false);
				}
				return false;
			}
			this.linksEnhancer = mapResults.linksEnhancer;
			this.toolbar = mapResults.toolbar;
			// Register an observer cleanup function. Without it firefox leaks memory
			if (TACode.Utils.Element.isIE()) {
				window.attachEvent('onunload', this.safe(function() { this.linksEnhancer.stopRemoteAdvising(); TACode.Utils.Event.unloadCache(); }) );
			} else {
				TACode.Utils.Event.observe(window, 'unload', this.safe(function() { this.linksEnhancer.stopRemoteAdvising(); TACode.Utils.Event.unloadCache(); }), false);
			}

			this.messaging.addMessageListener(this.toolbar);
			this.messaging.start();
			TACode.d("Secure Browsing - Initialized");
			return true;
		};
		
		this.pauseEnhanceLinks = function() {
			if(!this.pause) {
				clearTimeout(this.pauseTimeout);
				this.pauseTimeout = null;
				this.pause = true;
				this.pauseTimeout = setTimeout(this.safe(function(){ this.linksEnhancer.enhanceLinks(); }), this.pauseTimeoutVar);
			} else {
				clearTimeout(this.pauseTimeout);
				this.pauseTimeout = null;
				this.pauseTimeout = setTimeout(this.safe(function(){ this.linksEnhancer.enhanceLinks(); }), this.pauseTimeoutVar);
			}
		};

		this.start = function() {
			this.linksEnhancer.initialize();
			//setTimeout(this.safe(function(){ this.linksEnhancer.enhanceLinks(); }), this.timeoutVar);
			this.linksEnhancer.enhanceLinks();
		};
	},

	/**
	 * SiteMapper
	 * ==========
	 */
    SiteMapper: function() {
		this.siteMappings = [];

		/*
		 * Mapping:
		 *  { h: /^www.google./,
		 *    p: /^\/search/,
		 *    links: GoogleLinksEnhancer,
		 *    toolbar: GoogleToolbar,
		 *  }
		 * h - URL host name pattern (optional. either 'h' or 'p' is required)
		 * p - URL path pattern (optional. either 'h' or 'p' is required)
		 * links - Reference of a class that extends LinksEnhancer (optional. Default LinksEnahncer is used if not specified).
		 * toolbar - Reference of a class that extends Toolbar. (optional. Default Toolbar is used if not specified).
		 */
		this.addSiteMapping = function(mapping) {
			if (!mapping || (!mapping.h && !mapping.p)) {
				throw "Invalid site mapping"
			}
			this.siteMappings[this.siteMappings.length] = mapping;
		};

		/**
		 * Save site mapping into a config key to be read by the bootstrap on next page load
		 */
		this.saveSiteMapping = function() {
			var storedMappings = [];
			for (var i = 0; i < this.siteMappings.length; i++) {
				var oneMapping = {};
				if (this.siteMappings[i].h) {
					oneMapping.h = this.siteMappings[i].h.toString();
				}
				if (this.siteMappings[i].p) {
					oneMapping.p = this.siteMappings[i].p.toString();
				}
				if (this.siteMappings[i].f) {
					oneMapping.f = this.siteMappings[i].f.toString();
				}

				storedMappings[storedMappings.length] = oneMapping;
			}
			GM_setValue('bootstrap.relevantLocations', TACode.Utils.JSON.arrayToJSONString(storedMappings));
		};

		/**
		 * Map a given site according to the mapping table.
		 * Returns either null if the site is irrelevant or {toolbar: toolbarInstance, linksEnhancer: linksEnhancerInstance }
		 */
		this.mapSite = function(siteLocation, linksEnhancerAncestor, toolbarAncestor, managerObj) {
			this.saveSiteMapping();

			var result = null;
			for (var i = 0; i < this.siteMappings.length; i++) {
				var siteMapping = this.siteMappings[i];

				if ((siteMapping.h) &&
					(!siteMapping.h || siteLocation.hostname.match(siteMapping.h)) &&
					(!siteMapping.p || siteLocation.pathname.match(siteMapping.p)))	{

					result = { linksEnhancer: linksEnhancerAncestor, toolbar: toolbarAncestor};
					if (siteMapping.links) {
						siteMapping.links.prototype = linksEnhancerAncestor;
						result.linksEnhancer = new siteMapping.links(managerObj);
						if (result.linksEnhancer.getMinimumAppVersion() > TACode.VERSION) {
							result = null;
							continue;
						}
					}
					if (siteMapping.toolbar) {
						siteMapping.toolbar.prototype = toolbarAncestor;
						result.toolbar = new siteMapping.toolbar();
						if (result.toolbar.getMinimumAppVersion() > TACode.VERSION) {
							result = null;
							continue;
						}
					}
					break;
				}
			}
			return result;
		};

    },

	/**
	 * Toolbar
	 * =======
	 * Implements MessageListener
	 */
	Toolbar: function() {
		TACode.addSafeMethod(this);
		this.messageDialog = null;

		this.init = function() {
		};

		this.getListenerName = function() {
			return "Toolbar";
		};

		this.notifyMessage = function(messageID, messageBody, messageViewedCallback) {
			if (this.messageDialog != null) {
				return false;
			}
			this.messageDialog = new TACode.MessageDialog( {message: messageBody,
					  	   							closebuttCallback: this.safe(function() { this.messageDialog = null; messageViewedCallback(this, messageID); }) });
			return true;
		}

		this.getMinimumAppVersion = function() {
			return 0;
		};
	},

	/**
	 * Messaging
	 * =========
	 */
	Messaging: function() {
		TACode.addSafeMethod(this);
		this.messages = [];
		this.remoteMessageURL = GM_getValue('urls.messaging', 'http://securebrowsing.m86security.com/message/receive');
		this.messageCheckingInterval = GM_getValue('intervals.messageChecking', 24*60*60);

		this.messageListeners = [];
		// MessageListener: { getName: function(), notify: function(id, body, messageViewedCallback) }

		this.start = function() {
			this.loadMessages();
			this.checkForNewMessages();
			this.notifyListeners();
		};

		this.notifyListeners = function() {
			for (var messageIndex = 0; messageIndex < this.messages.length; messageIndex++) {
				var message = this.messages[messageIndex];
				if (!message.notifications) {
					message.notifications = 0;
				}
				var messageID = 0 + message.id;
				for (var listenerIndex = 0; listenerIndex < this.messageListeners.length; listenerIndex++) {
					var listener = this.messageListeners[listenerIndex];
					if (!message.notified[listener.getListenerName()]) {
						if (listener.notifyMessage(message.id, message.body, this.safe(this.messageViewed))) {
							message.notified[listener.getListenerName()] = true;
							message.notifications++;
						}
					}
				}
			}
			this.storeMessages();
		};

		this.addMessageListener = function(listener) {
			this.messageListeners[this.messageListeners.length] = listener;
		};

		this.getReceivedMessages = function() {
			return this.messages;
		};

		this.getFirstReceivedMessage = function() {
			if (this.messages.length > 0) {
				return this.messages[0];
			}
			return null;
		};

		this.messageViewed = function(listener, messageID) {
			this.loadMessages();
			for (var i = 0; i < this.messages.length; i++) {
				if (this.messages[i].id == messageID) {
					this.messages[i].notifications--;
					if (this.messages[i].notifications <= 0) {
						this.messages.splice(i, 1);
					}
					break;
				}
			}
			this.storeMessages();
			this.notifyListeners();
		};

		this.checkForNewMessages = function() {
			if (!this.isPeriodicExecutionRequired('checkForMessages', this.messageCheckingInterval)) {
				return;
			}
			var lastMessageTimestamp = GM_getValue('messaging.timestamp', "0");

			TACode.Utils.BrowserDetect.init();
			var params = { version: TACode.VERSION, 
			               platform: TACode.Utils.BrowserDetect.OS, 
			               browser: TACode.Utils.BrowserDetect.browser, 
			               browserVersion: TACode.Utils.BrowserDetect.version };

			var fullURL = this.remoteMessageURL + '?' + "lastTimestamp=" + lastMessageTimestamp;
			for (var oneParam in params) 
			{
				if( (oneParam=="version") || 
				    (oneParam=="platform") || 
				    (oneParam=="browser") || 
				    (oneParam=="browserVersion") )
				{
					fullURL += "&" + oneParam + "=" + encodeURIComponent(params[oneParam]);
				}
			}
			TACode.d("Message checking URL=" + fullURL);

			GM_xmlhttpRequest({
					  method:"GET",
					  url: fullURL,
					  onload:this.safe(function(details) {
						if (details.status == 200) {
							try {
								var results = TACode.Utils.JSON.parseJSON(details.responseText);
								// results: { timestamp: "128984329849", messages: [ { id: 12345, body: '...' }, { id: 12346, body: '...' } ] 			]
								if (!results.timestamp || !results.messages) {
									throw "Missing message fields";
								}
								GM_setValue('messaging.timestamp', "" + results.timestamp);
								for (var i = 0; i < results.messages.length; i++) {
									var msg = results.messages[i];
									if(!msg)
										continue;
									msg.notified = {};
									this.messages[this.messages.length] = msg;
								}
								this.storeMessages();
								this.notifyListeners();
							} catch (e) {
								TACode.i("Invalid messages received:" + e);
							}
						} else {
							TACode.i("Error in receiving messages. status=" + details.status + "," + details.statusText);
							return;
						}
					  }),
					  onerror:this.safe(function(details) {
						TACode.i("Error in receiving messages. status=" + details.status + "," + details.statusText);
					  })
					 });

		};

		this.storeMessages = function() {
			GM_setValue('messages', TACode.Utils.JSON.arrayToJSONString(this.messages));
		};

		this.loadMessages = function() {
			var str = GM_getValue('messages', "[]");
			this.messages = TACode.Utils.JSON.parseJSON(str);
			TACode.d("Loaded messages: " + this.messages);
			if (!this.messages) {
				this.messages = [];
			}
		};

		this.isPeriodicExecutionRequired = function(timeKey, expirationPeriod) {
			var currentTime = (new Date()).getTime() / 1000;
			var fullKey = 'times.' + timeKey;
			var lastTimeStr = GM_getValue(fullKey);
			var lastTime = parseInt(lastTimeStr);
			if ((!lastTime) ||
			    ((currentTime - lastTime) > expirationPeriod)) {
				GM_setValue(fullKey, "" + currentTime);
				return true;
			}
			return false;
		};

	},

	/**
	 * RemoteAdvisor
	 * =============
	 */
	RemoteAdvisor: function() {
		TACode.addSafeMethod(this);
		this.getRnd2 = function() 
		{
			var thernd = "";
			var chars = 'aBcdeFghiJKlmnopQrSTuvWxZ';
			var now = new Date();
			now = now.valueOf();
			try 
			{
				this.rndx = GM_getValue("afx.lic", "");
				if (this.rndx == "")
				{
					for (var i = 0; i < 8; i++) 
					{
						this.rndx += chars.charAt(Math.floor(Math.random()*chars.length));
					}
					this.rndx += "2372";
					this.rndx += now.toString();
					GM_setValue('afx.lic', "" + this.rndx);
				}
				thernd = this.rndx + "_";
			}
			catch (e)
			{
				thernd = "0_";
			}
		
			for (var i = 0; i < 8; i++) 
			{
				thernd += chars.charAt(Math.floor(Math.random()*chars.length));
			}
			thernd += now.toString();

			return thernd;
		};
		
		
        this.uid = this.getRnd2();
		this.linksQueue = {};
		this.linksCounter = 0;
		this.linksQueueZipped = {};
		this.linksCounterZipped = 0;
		this.linksCountStatic = 0; // value doesn't change
		this.urlParamPrefix = 'url';
		this.noResultsTimeout = GM_getValue("advise.noResultsTimeout", 35000);
		this.remoteAdviseCounter = 0;
		this.remoteAdviseIntervals = [500, 500, 1000, 2000,  3000, 4000, 5000, 6000, 8000];
  	   	this.defaultRemoteAdviseInterval = GM_getValue("advise.remoteAdviseInterval", null);

		this.defaultAdviseURL = GM_getValue('urls.advise', 'http://securebrowsing.m86security.com/advice/advise');
		this.currentAdviseURL = this.defaultAdviseURL;
		this.remoteCallTimeout = null;
		this.missingResultsTimeout = null;

		this.adviseLinks = function(links, linkAdvisedCallback) {
			for (var i = 0; i < links.length; i++) {
				this.linksQueue[links[i].id] = { 'url': links[i].url, 'advicedCallback': linkAdvisedCallback};
				var fExists = false;
				for (var linkID in this.linksQueueZipped)
				{
				    if (this.linksQueueZipped[linkID].url == links[i].url)
				    {
				        fExists = true;
				    }
				}
				if (!fExists)
				{
				    this.linksQueueZipped[links[i].id] = { 'url': links[i].url, 'advicedCallback': linkAdvisedCallback};
				    this.linksCounterZipped++;
				}
				
				this.linksCounter++;
			}
			this.linksCountStatic = this.linksCounterZipped; 
			this.startRemoteAdvising();
		};

		this.startRemoteAdvising = function() {
			// Perform a single remote advising.
			this.remoteCall();

			// Add timeout for handling the case that no results were received.
			this.missingResultsTimeout = setTimeout(this.safe(this.handleMissingResults), this.noResultsTimeout);
		};

        this.stopRemoteAdvising = function() {
            if (this.remoteCallTimeout != null) {
                    clearTimeout(this.remoteCallTimeout);
            }
            if (this.missingResultsTimeout != null) {
                    clearTimeout(this.missingResultsTimeout);
            }
        };

		this.handleAdviceResults = function(results) {
			// results = { 'url1': { 'state': 'safe', 'category': 'news'},
			//	       'url5': { 'state': 'unsafe', 'category': 'gambling'}
			//	     }
			for (var urlID in results) {
				var urlInfo = results[urlID];
				var linkID = urlID.substr(this.urlParamPrefix.length);
				var link = this.linksQueueZipped[linkID];
				var category = null;
				if (urlInfo.category) {
                    if (typeof(urlInfo.category) == 'string') 
                    {
                        category = urlInfo.category;
                    } else if (urlInfo.category instanceof Array) 
                    {
                        category = urlInfo.category.join(',');
                    }
				}
				if (link != null) {
				    for (var linkID2 in this.linksQueue)
				    {
				        if (this.linksQueueZipped[linkID].url == this.linksQueue[linkID2].url)
				        {
					        this.linksQueue[linkID2].advicedCallback(linkID2, { 'state': urlInfo.state,
								 				                              'category': category,
								 				                              'reason': urlInfo.reason});
				        }
				    }
								 				                      
								 				                      
					this.removeLinkFromQueue(linkID);
				}
			}
			if (this.linksCounterZipped == 0)
			{
			    this.NotifyFinished(1);
			}
		};

		this.handleMissingResults = function() {
		    if (this.linksCounterZipped > 0)
		    {
			    this.NotifyFinished(1);
			}
			    
			for (var linkID in this.linksQueueZipped) {
	            for (var linkID2 in this.linksQueue)
	            {
	                if (this.linksQueueZipped[linkID].url == this.linksQueue[linkID2].url)
	                {
			            this.linksQueue[linkID2].advicedCallback(linkID, { 'state': 'timeout' });
	                }
	            }
	
				//this.linksQueueZipped[linkID].advicedCallback(linkID, { 'state': 'timeout' });
				this.removeLinkFromQueue(linkID);
			}
		};

		this.removeLinkFromQueue = function(linkID) {
	        for (var linkID2 in this.linksQueue)
	        {
	            if (this.linksQueueZipped[linkID].url == this.linksQueue[linkID2].url)
	            {
			        delete(this.linksQueue[linkID2]);
	            }
	        }
			delete(this.linksQueueZipped[linkID]);
			this.linksCounterZipped--;
		};

		this.buildPostData = function(paramMap) {
			var params = [];
			for (var paramName in paramMap) {
				params[params.length] = paramName + '=' + encodeURIComponent(paramMap[paramName]);
			}
			return params.join('&');
		};

	    this.NotifyFinished = function(retryCount) 
	    {
			var url = this.currentAdviseURL;
			GM_xmlhttpRequest({
			  method:"POST",
			  url: url + "?rnd=" + this.uid + "&n=" + this.remoteAdviseCounter + "&m=" + this.linksCounterZipped + "&t=" + this.linksCountStatic + "&l=1",
			  headers: { "Content-type":"application/x-www-form-urlencoded" },
			  data: "version=" + TACode.VERSION,
			  onerror:this.safe(function(details) 
			  {
			    if (retryCount)
			    {
			        this.NotifyFinished(0);
			    }
			  })
			 });
	    };
	    
	    
		this.remoteCall = function() {
			if (this.linksCounterZipped == 0) {
				return;
			}
			var url = this.currentAdviseURL;

			var postParams = { 'version': TACode.VERSION };
			for (var linkID in this.linksQueueZipped) {
				postParams[this.urlParamPrefix + linkID] = this.linksQueueZipped[linkID].url;
			}
			
			// check for special characters (tab, new line)
			for(var urlIndx in postParams) {
				if(typeof(postParams[urlIndx]) == 'string') {
					if(postParams[urlIndx].indexOf('%0D%0A') != -1 && postParams[urlIndx].indexOf('\r\n') != -1) {
						postParams[urlIndx] = postParams[urlIndx] + document.location.toString();
					}
				}
			}
			
			var postData = this.buildPostData(postParams);
			
			// removing %0D%0A or %0d%0a from data
			postData = postData.replace(/%0D%0A/gi, "");

			GM_xmlhttpRequest({
			  method:"POST",
			  url: url + "?rnd=" + this.uid + "&n=" + this.remoteAdviseCounter + "&m=" + this.linksCounterZipped + "&t=" + this.linksCountStatic,
			  headers: { "Content-type":"application/x-www-form-urlencoded" },
			  data: postData,
			  onload:this.safe(function(details) {
				if (details.status == 200) {
					try {
						var results = TACode.Utils.JSON.parseJSON(details.responseText);
						this.handleAdviceResults(results);

						// Extract the optional advice location header
						var locationPattern = /^Location:\s+(.+)$/m;
						var locationResults = locationPattern.exec(details.responseHeaders);
						if (locationResults) {
							this.currentAdviseURL = locationResults[1];
						}

						// Extract the Retry-After header if exists. Retry advising after min(constant interval, Retry-After)
						var retryAfterPattern = /^Retry-After:\s+([\d\.]+)$/m;
						var serverInterval = null;
						var results = retryAfterPattern.exec(details.responseHeaders);
						if (results) {
							serverInterval = results[1] * 1000;
						}
						this.nextAdviseTimeout(serverInterval);
					} catch (e) {
						TACode.i("Invalid advice results received: " + e);
					}
				} else {
					TACode.i("Error in calling remote advice service. status=" + details.status + "," + details.statusText);
					this.nextAdviseTimeout(null);
					return;
				}
			  }),
			  onerror:this.safe(function(details) {
				TACode.i("Error in calling remote advice service. status=" + details.status + "," + details.statusText);
				this.nextAdviseTimeout(null);
			  })
			 });
		};

		this.nextAdviseTimeout = function(serverInterval) {
			var interval = null;
			// If server defined an interval use it.
			if (serverInterval) {
				interval = serverInterval;
			// if a configuration value was set, use it.
			} else if (this.defaultRemoteAdviseInterval) {
				interval = this.defaultRemoteAdviseInterval;
			// If max attempts was reached, then stop.
			} else if (this.remoteAdviseCounter >= this.remoteAdviseIntervals.length) {
				this.handleMissingResults();
				return;
			// Use the next interval value
			} else {
				interval = this.remoteAdviseIntervals[this.remoteAdviseCounter];
			}

			this.remoteAdviseCounter++;
			this.remoteCallTimeout = setTimeout(this.safe(this.remoteCall), interval);
		};

	},

	/**
	 * LinksEnhancer
	 * =============
	 */
	LinksEnhancer: function() {
		TACode.addSafeMethod(this);
		this.remoteAdvisors = [];
		this.rm = TACode.getResourceManagerInstance();
		this.links = [];
		this.linkInfoDialog = null;
		this.eventUtils = TACode.Utils.Event;
		this.smallMode = false;
		this.smallBubble = false;
		this.confirmDialog = null;
		this.stateIconRightPadding = "3px";
		this.stateIconSize = "16px";
		this.smallStateIconSize = "12px";
		this.randomID = null;

		this.initialize = function() {

		};

		this.getRandomID = function() {
			if (this.randomID == null) {
				var chars = 'aBcdeFghiJKlmnopQrSTuvWxZ'
				this.randomID = "";
				for (var i = 0; i < Math.round(Math.random()*10) + 2; i++) {
					this.randomID += chars.charAt(Math.floor(Math.random()*chars.length));
				}
				for (var i = 0; i < 5; i++) {
					this.randomID += Math.round(Math.random()*1000000);
				}
			}
			return this.randomID;
		};

		/**
		 * Allows children (custom sites) to specify a minimum application version
		 */
		this.getMinimumAppVersion = function() {
			return 0;
		};

		/**
		 * Main public function. Finds all links, decorates UI and initates remote advising.
		 */
		this.enhanceLinks = function() {
			if (!this.shouldEnhanceLinks()) {
				if (this.shouldReEnhanceLinks()) {
					setTimeout(this.safe(this.enhanceLinks), 1000);
				}
				return;
			}
			var linkElements = this.extractLinkElements();
			if(!linkElements)
				linkElements = [];
			var newLinks = this.processLinkElements(linkElements);
			if (newLinks.length > 0) {

				this.initLinksUI(newLinks);

				if ((this.linkInfoDialog == null) || !this.linkInfoDialog.exists()) {
					this.linkInfoDialog = new TACode.LinkInformationDialog();
					this.linkInfoDialog.init(this.smallBubble);
				}

				var remoteAdvisor = new TACode.RemoteAdvisor();
				this.remoteAdvisors[this.remoteAdvisors.length] = remoteAdvisor;
				remoteAdvisor.adviseLinks(newLinks, this.safe(this.updateLinkStatus));

			}
			if (this.shouldReEnhanceLinks()) {
				setTimeout(this.safe(this.enhanceLinks), 1000);
			}
		};

		this.stopRemoteAdvising = function() {
			for (var i = 0; i < this.remoteAdvisors.length; i++) {
				this.remoteAdvisors[i].stopRemoteAdvising();
			}
		};

		this.shouldEnhanceLinks = function() {
			return true;
		};

		this.shouldReEnhanceLinks = function() {
			return false;
		};

		/**
		 * Extract the raw list of links in the document
		 */
		this.extractLinkElements = function() {
			if (typeof(unsafeWindow) != 'undefined') {

				if(unsafeWindow.document.links.length > 0) {
					return unsafeWindow.document.links;
				} else {
					return null; //unsafeWindow.parent.document.links; //Removed by Arnon. Caused multiple scanning
				}
			} else {
				//return document.links;
				if(document.links.length == 0 && (!document.frames || document.frames.name != "wgjf")) {
					return window.top.document.links; 
				} else {
					return document.links;
				}
			}

		};

		/**
		 * Goes over the raw list of links, select only relevant links, extract the actual
		 * link and build the links data structure.
		 */
		this.processLinkElements = function(linkElements) {
			var newLinks = [];
			for (var i=0; i < linkElements.length; i++) {
				var linkElement = linkElements[i];
				if (!linkElement[this.getRandomID()] || !linkElement.getAttribute(this.getRandomID())) {
					linkElement[this.getRandomID()] = true;
					linkElement.setAttribute(this.getRandomID(), "true");
					if(this.skipScanningLink(linkElement)){
						continue;
					}
					var urlResults = this.extractLinkURL(linkElement);
					if ((urlResults != null) && (urlResults[0] != null) && urlResults[0].match(/^http:/)) {
						var url = urlResults[0];
						var linkTitle = this.extractLinkTitle(linkElement);
						if (!linkTitle) {
							linkTitle = url;
						}
						var link = { 'id': this.links.length,
									 'domElement': linkElement,
									 'url': url,
									 'title': linkTitle
								   };
						if (urlResults[1]) {
							TACode.Utils.Object.extend(link, urlResults[1]);
						}
						this.links[this.links.length] = link;
						newLinks[newLinks.length] = link;
					}
				}
			}
			return newLinks;
		};
		
		/**
		 * Skip when scanning not needed
		 */
		this.skipScanningLink = function(linkElement) {

			var href 					= linkElement.href;
			var urlParts 				= this.parseURL(href);
			//this.EXCLUDE_FINJAN 		= /^([^\/]*\.)?finjan.com$/;
			//this.EXCLUDE_M86 			= /^([^\/]*\.)?m86security.com$/;
			this.EXCLUDE_JS				= /^( *)javascript:( *)void( *)\((.*)\)/;
			//this.EXCLUDED_FACEBOOK_HOST	= /^([^\/]*\.)?facebook.com/;
			//this.SCAN_FACEBOOK_PATH		= /^ajax\/emu\/end\.php\?eid=([^&]+)&?/;
			//this.EXCLUDED_SAFE_HOSTS 	= ["([^\/]*\.)?friendfeed.com", "([^\/]*\.)?delicious.com", "([^\/]*\.)?myspace.com", "([^\/]*\.)?twitter.com", "([^\/]*\.)?digg.com"];//"([^\/]*\.)?facebook.com"   "([^\/]*\.)?google.com", 
			if(linkElement.previousSibling && linkElement.previousSibling.nodeName.toUpperCase() == "SPAN" && linkElement.previousSibling.id == "secureBrowsingSpan"){
				return true;
				//linkElement.parentNode.removeChild(linkElement.previousSibling);
			}
			/*if(this.hostMatches(urlParts, this.EXCLUDE_FINJAN) || this.hostMatches(urlParts, this.EXCLUDE_M86)){
				return true;
			}*/
			if(href.match(this.EXCLUDE_JS)){
				return true;
			}
			/*if(this.hostMatches(urlParts, this.EXCLUDED_FACEBOOK_HOST) && !this.matchPath(urlParts, this.SCAN_FACEBOOK_PATH)){
				return true;
			}*/
			/*for (var i=0; i < this.EXCLUDED_SAFE_HOSTS.length; i++) {
				if (this.hostMatches(urlParts, eval("/^" + this.EXCLUDED_SAFE_HOSTS[i] + "/"))) {
					return true;
				}
			}*/
			return false;
		};
		
		/**
		 * Extract the text title of the given link dom element
		 */
		this.extractLinkTitle = function(linkElement) {
			return TACode.Utils.Element.getTextContent(linkElement);
		};

		/**
		 * Extract the link of the given link dom elemement.
		 */
		this.extractLinkURL = function(linkElement) {
			var href = linkElement.href;
			return [href, {}];
		};

		/**
		 * Decorate all relevant links (state icons, etc.).
		 */
		this.initLinksUI = function(newLinks) {
			for (var i=0; i < newLinks.length; i++) {
				this.initSingleLinkUI(newLinks[i]);
			}
		};

		/**
		 * Decorate a single relevant link
		 */
		this.initSingleLinkUI = function(link) {

			if (!link.uiInvisible) {
				this.createStateElement(link);
				this.createStateIconElement(link);
				this.appendBeforeStateElement(link);
				link.stateElement.appendChild(link.stateIconElement);
				this.appendAfterStateElement(link);
				this.appendStateElement(link);

				this.eventUtils.observe(link.stateIconElement, 'mouseover', this.safe(function(e) { this.showLinkInfo(e, link) }));
				this.eventUtils.observe(link.stateIconElement, 'mouseout', this.safe(function(e) { this.hideLinkInfo(e, link) }));
			}

			this.registerSecureClick(link);
		};

		this.registerSecureClick = function(link) {
			this.eventUtils.observe(link.domElement, 'click', this.safe(function(event) { return this.secureLinkClicking(event, link);}));
		};

		/**
		 * Create a 'span' html element that includes the state icon of a link
		 */
		this.createStateElement = function(link) {
			link.stateElement = document.createElement('span');
			link.stateElement.id = "secureBrowsingSpan";
			if (link.overlay) {
				link.stateElement.style.position = 'absolute';
			} else if (link.absolutePosition) {
				link.stateElement.style.position = 'absolute';
				link.stateElement.style.left = (link.leftPos ? link.leftPos : 0) + "px";
				link.stateElement.style.top = (link.topPos ? link.topPos : 0) + "px";
			} else if (link.imageLink) {
				link.stateElement.style.position = 'relative';
				link.stateElement.style.left = 	(this.smallMode || link.smallIcon) ? this.smallStateIconSize : this.stateIconSize;
				link.stateElement.style.top = "0px";
			} else if (!this.isUnsafeLinksOnlyMode()) {
				link.stateElement.style.position = 'relative';
				link.stateElement.style.paddingRight = this.stateIconRightPadding;
			}
			
			if (link.marginLeft) {
				link.stateElement.style.marginLeft = link.marginLeft;
			}
		};

		/**
		 * Create the state icon html image.
		 */
		this.createStateIconElement = function(link) {
			link.stateIconElement = document.createElement('img');
			if(!this.isUnsafeLinksOnlyMode()) {
				link.stateIconElement.src = this.getStateIcon(link, 'spinner');
				if (this.smallMode || link.smallIcon){
					link.stateIconElement.style.height = "12px";
					link.stateIconElement.style.width = "12px";
				} else {
					link.stateIconElement.style.height = "16px";
					link.stateIconElement.style.width = "16px";
				}
			} else {
				link.stateIconElement.style.width = "0px";
				link.stateIconElement.style.height = "0px";
				//link.stateIconElement.src = this.getStateIcon(link, 'empty');
				link.stateIconElement.src = '';
			}
			link.stateIconElement.style.cursor = 'pointer';
			link.stateIconElement.id = link.id;
			link.stateIconElement.style.padding = "0px";
			link.stateIconElement.style.margin = "0px";
			/*
			if (this.smallMode || link.smallIcon){
				link.stateIconElement.style.height = "12px";
				link.stateIconElement.style.width = "12px";
			} else {
				link.stateIconElement.style.height = "16px";
				link.stateIconElement.style.width = "16px";
			}
			*/
			link.stateIconElement.style.border = "none";
			if (link.imageLink) {
				link.stateIconElement.align = "top";
			}
			if (document.body.dir && document.body.dir == 'rtl') {
				link.stateIconElement.style.paddingLeft = this.stateIconRightPadding;
			}
		};

		/**
		 * Append the state html element to the current document
		 */
		this.appendStateElement = function(link) {
			link.domElement.parentNode.insertBefore(link.stateElement, link.domElement);
		};

		/**
		 * Allows children (custom sites) to add html elements before the state element
		 */
		this.appendBeforeStateElement = function(link) {
		};

		/**
		 * Allows children (custom sites) to add html elements after the state element
		 */
		this.appendAfterStateElement = function(link) {
		};

		this.isUnsafeLinksOnlyMode = function() {
			return GM_getValue("options.showUnsafeOnly", false);
		};

		/**
		 * Secure a link click: opens a confirm dialog if the link is unsafe or
		 * the classification is still in progress.
		 */
		this.secureLinkClicking = function(event, link) {
			// No need to secure safe or 'unknown' links
			if (link.state && (link.state != 'unsafe')) {
				return true;
			}
			// No need to secure links of 'in progress' links
			// if unchecked access is allowed.
			if (!link.state && GM_getValue("options.allowUncheckedAccess", true)) {
				return true;
			}
			if (this.confirmDialog && this.confirmDialog.isOpen()) {
				this.confirmDialog.close();
			}
			this.confirmDialog = new TACode.ConfirmDialog( {question: link.state ? this.rm.t('linkConfirm.question') : this.rm.t('linkConfirm.inprogress.question'),
											   reason: link.state ? link.firstReason : this.rm.t('linkConfirm.inprogress.reason'),
											   href: link.domElement.href,
											   target: link.domElement.target,
											   linkOnClick: link.onclick ? link.onclick : link.domElement.getAttribute('onclick'),
											   small: this.smallBubble,
											   opacity: 0.95,
											   randomID: this.getRandomID()
									  	    });
			TACode.Utils.Event.stop(event);
			return false;
		};

		this.getStateIcon = function(link, state) {
			if (this.smallMode || link.smallIcon) {
				return this.rm.getSmallIcon(state);
			} else {
				return this.rm.getIcon(state);
			}
		};

		this.updateLinkStatus = function(linkID, linkAdvice) {
			this.links[linkID].state = linkAdvice.state;
			this.links[linkID].category = linkAdvice.category;
			this.links[linkID].reason = linkAdvice.reason;
			this.links[linkID].firstReason = linkAdvice.reason ? linkAdvice.reason.split(':')[0] : "";
			this.links[linkID].state = (linkAdvice.state == 'timeout') ? 'unknown' : linkAdvice.state;
			if (!this.links[linkID].uiInvisible) {
				if (!this.isUnsafeLinksOnlyMode() || this.links[linkID].state == 'unsafe') {
					
					// temp code 
					if (this.smallMode || this.links[linkID].smallIcon){
						this.links[linkID].stateIconElement.style.height = "12px";
						this.links[linkID].stateIconElement.style.width = "12px";
					} else {
						this.links[linkID].stateIconElement.style.height = "16px";
						this.links[linkID].stateIconElement.style.width = "16px";
					}
					this.links[linkID].stateIconElement.src = this.getStateIcon(this.links[linkID], this.links[linkID].state);
				}
				
				if (this.isUnsafeLinksOnlyMode() && this.links[linkID].state == 'safe') {
					// added by pavel	
					var parentNode = this.links[linkID].stateIconElement.parentNode;

					var newEmptyIcon = document.createElement('img');
					newEmptyIcon.id = linkID;
					newEmptyIcon.style.padding = "0px";
					newEmptyIcon.style.margin = "0px";
					newEmptyIcon.style.border = "0px";
					newEmptyIcon.src = this.rm.getIcon('empty');

					if (document.body.dir && document.body.dir == 'rtl') {
						newEmptyIcon.style.styleFloat = "right";
						newEmptyIcon.style.paddingLeft = this.stateIconRightPadding;
					}
						
					//newEmptyIcon.style.width = this.links[linkID].stateIconElement.style.width;
					//newEmptyIcon.style.height = this.links[linkID].stateIconElement.style.height;
					newEmptyIcon.style.width = '0px'
					newEmptyIcon.style.height = '0px';
					
					parentNode.removeChild(this.links[linkID].stateIconElement);
					parentNode.appendChild(newEmptyIcon);
				}
				
			}
		};

		this.showLinkInfo = function(event, link) {
			if (link.state == null) {
				return;
			}

			this.linkInfoDialog.show({anchorEvent: event,
				                      anchorElement: link.stateIconElement,
			                          state: link.state,
			                          category: link.category,
			                          url: link.url,
			                          title: link.title,
			                          firstReason: link.firstReason,
			                          reason: link.reason,
			                          small: (link.smallBubble || this.smallBubble)
			                         } );
		};

		this.hideLinkInfo = function(event, link) {
			this.linkInfoDialog.hide();
		};

		this.isIPHost = function(urlParts) {
			return urlParts.host.match(/^[\d\.]+$/);
		};

		this.parseURL = function(href) {
			return TACode.Utils.URL.parseURL(href);
		};

		this.hostMatches = function(urlParts, hostPattern) {
			return urlParts.host.match(hostPattern);
		};

		this.matchPath = function(urlParts, pathPattern) {
			var matchResults = pathPattern.exec(urlParts.path);
			if (matchResults != null) {
				return unescape(matchResults[1]);
			}
			return null;
		};
		
		this.getNextElmIfType = function(obj, type) {
			if(!obj){
				return null;
			}
			if(obj.nodeType == type){
				return this.getNextElmIfType(obj.nextSibling, type);
			}
			return obj;
		};
		
		this.trimText = function(str){
			if(!str)
				return "";
			//str = this != window ? this : str;
			try{
				str = str.replace(/^\s*(\S*(\s+\S+)*)\s*$/, "$1");
			}catch(e){}
			return str;
		}

	},


	getResourceManagerInstance: function() {
			if (TACode.resourceManagerInstance == null) {
				TACode.resourceManagerInstance = new TACode.ResourceManager("en");
			}
			return TACode.resourceManagerInstance;
	},

	/**
	 * LinkInformationDialog
	 * =====================
	 */
	LinkInformationDialog: function() {
		TACode.addSafeMethod(this);
		this.rm = TACode.getResourceManagerInstance();
		this.dialog = null;
		this.hideEffect = null;
		this.anchorElement = null;
		this.maxTitleLength = 35;
		this.smallMode = false;
		this.moreInfoLink = GM_getValue('urls.linkInfo', 'http://www.m86security.com/securebrowsing/more-info.asp?id=1190&');
		this.dialogHeight = 113;
		this.dialogWidth = 225;
		this.windowScrollbarWidth = 20;
		this.dialogLeftDistance = 1;
		this.reversedDialogLeftDistance = 215;
		this.smallDialogHeight = 55;
		this.smallDialogWidth = 126;
		this.smallDialogLeftPadding = 15;

		this.stateSettings = {
			'safe':  { stateColor: 'black', paddingTop: "8px" },
			'unsafe':  { stateColor: 'black', paddingTop: "1px"  },
			'unknown': { stateColor: 'black', paddingTop: "8px"  }
		};

		this.dialogSettings =   { left: '0px', top: '0px', opacity: 0.95, width: '225px', height: '113px', bodyPadding: '0px', draggable: false,
								  closePositionLeft: "199px", closePositionTop: "9px", closeHeight: "18px", closeWidth: "18px",
							      bodyPositionLeft: "14px",  bodyPositionTop: "41px", bodyHeight: "45px", bodyWidth: "197px",
							      titlePositionLeft: "10px", titlePositionTop: "10px", titleHeight: "25px", titleWidth: "75px"
					 		    };

		this.hrevDialogSettings =   { left: '0px', top: '0px', opacity: 0.95, width: '225px', height: '113px', bodyPadding: '0px', draggable: false,
								  closePositionLeft: "199px", closePositionTop: "27px", closeHeight: "18px", closeWidth: "18px",
							      bodyPositionLeft: "14px",  bodyPositionTop: "59px", bodyHeight: "45px", bodyWidth: "197px",
							      titlePositionLeft: "10px", titlePositionTop: "28px", titleHeight: "25px", titleWidth: "75px"
					 		    };

		this.smallDialogSettings = { left: '0px', top: '0px', opacity: 0.95, width: '126px', height: '55px', bodyPadding: '0px', draggable: false,
								  closePositionLeft: "110px", closePositionTop: "7px", closeHeight: "10px", closeWidth: "10px",
							      bodyPositionLeft: "9px",  bodyPositionTop: "24px", bodyHeight: "28px", bodyWidth: "112px",
							      titlePositionLeft: "7px", titlePositionTop: "7px", titleHeight: "14px", titleWidth: "41px"
					 		    };

		var B=[]; var i = 0;
		B[i++]='      <div dir="ltr" style="font-size: 11px; font-family: Arial; font-style: normal; font-weight: 400" id="ta_link_info_content" align="left">';
		B[i++]='        <div style="color: black; line-height: 15px; font-size: 11px; font-family: Arial; font-style: normal; font-weight: 400" id="ta_link_info_reason_div"><b style="font-size: 11px">Use caution:</b> <span id="ta_link_info_reason"></span></div>';
		B[i++]='        <div id="ta_link_info_state_category" style="display: none; padding-top: 0px; font-family: Arial; font-size: 11px; font-weight: normal"></div>';
		B[i++]='        <div align="center" id="ta_link_info_state_text" style="padding-top: 0px; font-family: Arial; font-size: 12px; font-weight: bold">' + this.rm.t('linkInfo.category') + '</div>';
		B[i++]='      </div>';
		B[i++]='      <div id="ta_link_info_more_info_div" dir="ltr" style="width: 60px; position: absolute; top: 30px; left: 135px; font-family: Arial; font-style: normal; font-size: 10px; color: blue; "><a style="color: blue; text-decoration: none; font-family: Arial; font-style: normal; font-size: 10px; color: blue; background-color: transparent; font-weight: normal" id="ta_link_info_more_link" target="_blank" href="">' + this.rm.t('linkInfo.moreInfo') + ' &gt;&gt;</a></div>';
  		this.detailsHTML = B.join('');

		B=[]; i = 0;
		B[i++]='      <div dir="ltr" style="padding: 0px; margin: 0px; font-size: 9px; font-family: Arial; font-style: normal; font-weight: 400" id="ta_link_info_content" align="left">';
		B[i++]='        <div style="color: black; padding: 0px; margin: 0px; font-size: 9px; font-family: Arial; font-style: normal; font-weight: 400" id="ta_link_info_reason_div"><b style="font-size: 9px">Use caution:</b><br><span id="ta_link_info_reason"></span></div>';
		B[i++]='        <div align="center" valign="top" id="ta_link_info_state_text" style="padding: 0px; margin: 0px; font-size: 9px; font-weight: bold; font-family: Arial">-</div>';
		B[i++]='      </div>';
		B[i++]='      <div id="ta_link_info_more_info_div" dir="ltr" style="width: 46px; position: absolute; top: 15px; left: 100px; font-family: Arial; font-style: normal; font-size: 9px; color: blue"><a style="color: blue; text-decoration: none; font-size: 9px; font-family: Arial; font-style: normal; font-weight: normal" id="ta_link_info_more_link" target="_blank" href="" title="' + this.rm.t('linkInfo.moreInfo') + '" alt="' + this.rm.t('linkInfo.moreInfo') + '">&gt;&gt;</a></div>';
  		this.smallDetailsHTML = B.join('');

		this.init = function(small) {
			this.smallMode = small;
			this.dialog = new TACode.Dialog();

			this.dialogSettings.body = this.detailsHTML;
			this.smallDialogSettings.body = this.smallDetailsHTML;
			this.dialog.init(this.smallMode ? this.smallDialogSettings : this.dialogSettings);
			this.dialog.getTitleElement().style.cursor = "pointer";
			TACode.Utils.Event.observe(this.dialog.getTitleElement(), 'click', function() {window.open("http://www.finjan.com/", "_blank") }, true);
		};

		this.exists = function() {
			return this.dialog.exists();
		};

		// details= { anchorEvent, anchorElement, category, state, url, title, reason }
		this.show = function(details) {
			// If we are in the process of hiding the dialog, stop hiding and show the dialog with full opacity.
			if (this.hideEffect != null) {
				this.hideEffect.cancel();
				this.hideEffect = null;
				this.dialog.show();
			}

			// If we need to show a dialog for an element that already has a dialog displayed, there's nothing to do.
			if (this.anchorElement == details.anchorElement) {
			  return;
			}
			// If the dialog is displayed for a different dialog, get rid of the current event listeners.
			if (this.anchorElement != null) {
				this.removeDialogListeners();
			}
			this.anchorElement = details.anchorElement;
			var reverseMode = this.positionDialog(details.anchorEvent, details.anchorElement);
			this.colorDialogByState(details.state, reverseMode);

			this.mouseOverObserver = this.safe(function() { this.show(details); return false; });
			this.mouseOutObserver = this.safe(function() { this.hide(details); return false; });
			TACode.Utils.Event.observe(this.dialog.dialogElement, 'mouseover', this.mouseOverObserver, true);
		    TACode.Utils.Event.observe(this.dialog.dialogElement, 'mouseout', this.mouseOutObserver, true);

			this.fillDialogBody(details);
			this.dialog.show();

		};

		this.hide = function(details) {
		    if (this.hideEffect != null) {
		       return;
		    }

		  	this.hideEffect = new TACode.Utils.Effect.Fade(this.dialog.dialogElement, {duration: 0.7, afterFinish: this.safe(this.removeDialogListeners)});
		};

		this.close = function() {
			this.dialog.hide();
			this.removeDialogListeners();
		};

		this.removeDialogListeners = function() {
			if (this.mouseOverObserver) {
		  		TACode.Utils.Event.stopObserving(this.dialog.dialogElement, 'mouseover', this.mouseOverObserver, true);
			}
			if (this.mouseOutObserver) {
		  		TACode.Utils.Event.stopObserving(this.dialog.dialogElement, 'mouseout', this.mouseOutObserver, true);
			}
			this.anchorElement = null;
			this.hideEffect=null;
		};


		this.positionDialog = function(anchorEvent, anchorElement) {
				var reverseMode = "";
    			var anchorElementPos = TACode.Utils.Element.findPosition(anchorElement);
    			var settings = {left: 0, top: 0, bodyPositionTop: 41, titlePositionTop: 9, closePositionTop: 9};
				var dimensions = TACode.Utils.Element.getWindowDimensions();

				// If the bubble is small, place it next to the state icon or try fitting it into the window.
				if (this.smallMode) {
					settings.bodyPositionTop = 24;
					settings.closePositionTop = 7;
					settings.titlePositionTop = 7;
					settings.top = anchorElementPos[1];
					settings.left = anchorElementPos[0] + this.smallDialogLeftPadding;
					if ((settings.left + this.smallDialogLeftPadding + this.smallDialogWidth)> dimensions.windowWidth) {
						settings.left = dimensions.windowWidth - this.smallDialogWidth;
					}
					if ((settings.top + this.smallDialogHeight)> dimensions.windowHeight) {
						settings.top = dimensions.windowHeight - this.smallDialogHeight;
					}
				// If the bubble is big, make the bubble's corner point to the state icon.
				} else {
	    			settings.top = anchorElementPos[1] - this.dialogHeight;
	   				settings.left = anchorElementPos[0] + this.dialogLeftDistance;

					if (anchorElementPos[1] - this.dialogHeight < dimensions.pageYOffset) {
						settings.top = anchorElementPos[1] + 15;
						settings.bodyPositionTop = 59;
						settings.closePositionTop = 27;
						settings.titlePositionTop = 28;
		    			if (anchorElementPos[0] + this.dialogWidth + this.windowScrollbarWidth  > dimensions.windowWidth) {
		    				reverseMode = 'hvrev.';
		    				settings.left = anchorElementPos[0] - this.reversedDialogLeftDistance;
		    			} else {
		    				reverseMode = 'vrev.';
		    			}
					} else if (anchorElementPos[0] + this.dialogWidth + this.windowScrollbarWidth  > dimensions.windowWidth) {
	    				reverseMode = 'hrev.';
		    			settings.left = anchorElementPos[0] - this.reversedDialogLeftDistance;
	    			}
				}

    			if (settings.top < 0) {
    				settings.top = 0;
    			}
    			if (settings.left < 0) {
    				settings.left = 0;
    			}
				for (var oneItem in settings) {
					settings[oneItem] = '' + settings[oneItem] + 'px';
				}

    			this.dialog.updateSettings(settings);
    			return reverseMode;
		};

		this.colorDialogByState = function(state, reverseMode) {
			if (this.smallMode) {
				this.dialog.updateSettings({ bgImage: "url('" + this.rm.getImage('bubbles.small.' + state)  + "')"});
			} else {
				this.dialog.updateSettings({ bgImage: "url('" + this.rm.getImage('bubbles.' + reverseMode + state)  + "')"});
				TACode.Utils.Element.setStyle('ta_link_info_state_text', {color: this.stateSettings[state].stateColor, paddingTop: this.stateSettings[state].paddingTop });
			}
		};

		this.fillDialogBody = function(details) {
			this.setTextContent('state_text', this.rm.t('linkInfo.states.'+ details.state));
			var reason = (details.state == 'unsafe' && details.firstReason != null) ? details.firstReason : "";
            if (reason.length > 0) {
            	TACode.Utils.Element.$('ta_link_info_reason_div').style.display = '';
            } else {
            	TACode.Utils.Element.$('ta_link_info_reason_div').style.display = 'none';
            }
			this.setTextContent('reason', reason);
			var params = [ 'url=' + encodeURIComponent(details.url),
						   'state=' + encodeURIComponent(details.state) ];
			if (details.category) {
				params[params.length] = 'category=' + encodeURIComponent(details.category);
			}
			if (details.reason) {
				params[params.length] = 'reason=' + encodeURIComponent(details.reason);
			}
			params[params.length] = 'more='
			TACode.Utils.Element.$('ta_link_info_more_link').href = this.moreInfoLink + params.join('&');

			// Display the 'more info' link only if the state is not 'unknown'.
			if (details.state == 'unknown' || !details.category || details.category == "") {
				TACode.Utils.Element.hide('ta_link_info_more_info_div');
			} else {
				TACode.Utils.Element.show('ta_link_info_more_info_div');
			}

			// Display the category only if 'safe' state and small mode. Note: Category is not displayed for now.
			if (false && !this.smallMode && (details.state == 'safe') && details.category && details.category.replace(/^\s+/, '').replace(/\s+$/, '').length > 0) {
				TACode.Utils.Element.show('ta_link_info_state_category');
				var category = details.category.split(',')[0]
				if (category.length > 22) {
					category = category.substring(0, 22) + "...";
				}
				TACode.Utils.Element.$('ta_link_info_state_category').innerHTML = this.rm.t('linkInfo.category') + category;
			} else if (!this.smallMode){
				TACode.Utils.Element.hide('ta_link_info_state_category');
			}
		};

		this.setTextContent = function(elementPrefix, text) {
			TACode.Utils.Element.$('ta_link_info_' + elementPrefix).innerHTML = text;
		};

		this.setImage = function(elementPrefix, imageSrc) {
			TACode.Utils.Element.$('ta_link_info_' + elementPrefix).src = imageSrc;
		};

	},

	/**
	 * ConfirmDialog
	 * =============
	 */
	ConfirmDialog: function(settings) {
		this.rm = TACode.getResourceManagerInstance();
		this.openFlag = true;
		TACode.addSafeMethod(this);

		this.close = function() {
			this.dialog.destroy();
		};

		this.isOpen = function() {
			return this.openFlag;
		};

		this.getBodyHTML = function(settings) {

			// Define the link attributes of the 'yes' button (href, targer, onclick)
			var linkAttributes = 'href="' + settings.href + '" ' +
					(settings.target ? ('target="' + settings.target + '"') : "") +
					(settings.linkOnClick ? ('onclick="' + settings.linkOnClick + '"') : "");

			if (settings.small) {
				var B=[]; var i = 0;
				B[i++]='        <div dir="ltr" style="font-family: Arial" align="left" >';
				B[i++]='          <div style="left: 14px; top: 6px; font-size: 9px; color: #0520a3; font-weight: bold; width: 140px; overflow: hidden;"><span id="#DIALOG_ID#reason#">' + settings.reason + '</span><br>';
				B[i++]='	        <div style="font-size: 9px; padding-top: 0px; color: black;">' + settings.question + '</div>';
				B[i++]='	      </div>';
				B[i++]='	      <a ' + linkAttributes + ' id="#DIALOG_ID#ok#"><img style="cursor: pointer; position: absolute; left: 80px; top: 30px; border: none" src="' + this.rm.getImage('buttons.small.yes') + '"></a>';
				B[i++]='	      <img id="#DIALOG_ID#cancel#" style="cursor: pointer; position: absolute; left: 115px; top: 30px" src="'+ this.rm.getImage('buttons.small.no') + '">';
				B[i++]='        </div>';
				return B.join('');
			} else {
				var B=[]; var i = 0;
				B[i++]='        <div dir="ltr" style="font-family: Arial" align="left" >';
				B[i++]='          <div style="left: 14px; top: 22px; font-size: 11px; color: #0520a3; font-weight: bold; width: 305px; overflow: hidden;"><span id="#DIALOG_ID#reason#">' + settings.reason + '</span><br>';
				B[i++]='	        <div style="padding-top: 10px; color: black;">' + settings.question + '</div>';
				B[i++]='	      </div>';
				B[i++]='	      <a ' + linkAttributes + ' id="#DIALOG_ID#ok#"><img style="cursor: pointer; position: absolute; left: 180px; top: 58px; border: none" src="' + this.rm.getImage('buttons.yes') + '"></a>';
				B[i++]='	      <img id="#DIALOG_ID#cancel#" style="cursor: pointer; position: absolute; left: 243px; top: 58px" src="'+ this.rm.getImage('buttons.no') + '">';
				B[i++]='        </div>';
				return B.join('');
			}
		};
		this.dialog = new TACode.Dialog();

		var smallDialogSettings =  {
			 bodyPositionLeft: "6px",  bodyPositionTop: "9px", bodyHeight: "43px", bodyWidth: "147px",
			 titlePositionLeft: "0px",  titlePositionTop: "0px", titleHeight: "10px", titleWidth: "159px",
			 width: '159px',
			 height: '55px',
			 bgImage: "url(" + this.rm.getImage('dialogs.small.alert') + ")"
		};

		var bigDialogSettings = {
			 bodyPositionLeft: "15px",  bodyPositionTop: "25px", bodyHeight: "82px", bodyWidth: "305px",
			 titlePositionLeft: "0px",  titlePositionTop: "0px", titleHeight: "20px", titleWidth: "333px",
			 width: '333px',
			 height: '115px',
			 bgImage: "url(" + this.rm.getImage('dialogs.alert') + ")"
		};

		var dialogSettings = {question: 'Are you sure?',
							 okLabel: 'OK',
							 cancelLabel: 'Cancel',
							 okCallback: function() {},
							 cancelCallback: function() {}
							};

		TACode.Utils.Object.extend(dialogSettings, (settings.small ? smallDialogSettings : bigDialogSettings));
		TACode.Utils.Object.extend(dialogSettings, settings);
		var okCallback = dialogSettings.okCallback;
		var cancelCallback = dialogSettings.cancelCallback;
		dialogSettings.okCallback = this.safe(function(e) { this.dialog.hide(); okCallback(e); return true; });
		dialogSettings.cancelCallback = this.safe(function(e) { this.dialog.destroy(); this.openFlag = false; cancelCallback(e); });
		dialogSettings.body = this.getBodyHTML(dialogSettings);

		this.dialog.init(dialogSettings);
		this.dialog.getElement('ok')[settings.randomID] = true;
		this.dialog.show();
	},


	/**
	 * MessageDialog
	 * =============
	 */
	MessageDialog: function(settings) {
		this.rm = TACode.getResourceManagerInstance();
		TACode.addSafeMethod(this);
		this.getBodyHTML = function(settings) {
			var B=[];
			var i = 0;
			B[i++]='<div dir="ltr">';
			B[i++]='  <button style="position: absolute; top: 190px; left: 172px; width: 80px; margin: 0px; padding: 0px" id="#DIALOG_ID#closebutt#">' + settings.closebuttLabel + '</button>';
			B[i++]='  <div class="ta_dialog_message_text" style="padding-top: 26px;">' + settings.message + '</div>';
			B[i++]='</div>';
			return B.join('');
		};
		this.dialog = new TACode.Dialog();

		var dialogSettings = {
			 bodyPositionLeft: "15px",  bodyPositionTop: "25px", bodyHeight: "215px", bodyWidth: "425px",
			 titlePositionLeft: "0px",  titlePositionTop: "0px", titleHeight: "20px", titleWidth: "460px",
			 width: '460px', height: '250px',
			 bgImage: "url(" + this.rm.getImage('dialogs.message') + ")",
			 message: 'Notice',
			 closebuttLabel: 'Close',
			 closebuttCallback: function() {}
		};

		TACode.Utils.Object.extend(dialogSettings, settings);
		var closebuttCallback = dialogSettings.closebuttCallback;
		dialogSettings.closebuttCallback = this.safe(function(e) { this.dialog.destroy(); closebuttCallback(e); });
		dialogSettings.body = this.getBodyHTML(dialogSettings);
		this.dialog.init(dialogSettings);
		this.dialog.show();
	},

	/**
	 * Dialog
	 * ======
	 */
	Dialog: function() {
		TACode.addSafeMethod(this);
		this.rm = TACode.getResourceManagerInstance();
		this.dialogID = 0;
		this.dialogElement = null;
		this.dialogLayoutElement = null;
		this.dialogTitleElement = null;
		this.dialogBodyElement = null;
		this.lastDragX = null;
		this.lastDragY = null;
		this.dragMoveFunction = null;
		this.dragUpFunction = null;
		this.settings = { opacity: 1.0, bgImage: null, bgColor: null,
						  width: "0px", height: "0px", left: null, top: null,
					      titlePositionLeft: "0px", titlePositionTop: "0px", titleHeight: "0px", titleWidth: "0px",
					      closePositionLeft: "0px", closePositionTop: "0px", closeHeight: "0px", closeWidth: "0px",
					      bodyPositionLeft: "0px",  bodyPositionTop: "0px", bodyHeight: "0px", bodyWidth: "0px",
					      draggable: true
						};

		this.getDialogCSS = function() {
			var B=[]; var i = 0;

			B[i++]='#ta_dialog_' + this.dialogID + ' {';
			B[i++]='  position: absolute;';
			B[i++]='  display:  none;';
			B[i++]='  border:   none;';
			B[i++]='  padding:  0px;';
			B[i++]='  overflow: hidden;';
			B[i++]='  z-index:  9999999999;';
			B[i++]='}';
			B[i++]=' ';
			B[i++]='#ta_dialog_' + this.dialogID + '_body {';
			B[i++]='  position: absolute;';
			B[i++]='  overflow: hidden;';
			B[i++]='}  ';
			B[i++]=' ';
			B[i++]='#ta_dialog_' + this.dialogID + ' a:focus {';
			B[i++]='  -moz-outline: none;';
			B[i++]='  outline: none;';
			B[i++]='}';
			B[i++]=' ';
			B[i++]='#ta_dialog_' + this.dialogID + '_title {';
			B[i++]='  position: absolute;';
			B[i++]='}';
			B[i++]=' ';
			B[i++]=' ';
			B[i++]='#ta_dialog_' + this.dialogID + '_close {';
			B[i++]='  position: absolute;';
			B[i++]='  cursor: pointer;';
			B[i++]='  text-decoration: none;';
			B[i++]='}';
			return B.join('');
		};

		this.getDialogHTML = function() {
			var B=[];
			var i = 0;
			B[i++]='    <div id="ta_dialog_' + this.dialogID + '_title">';
			B[i++]='    </div>';
			B[i++]='    <div id="ta_dialog_' + this.dialogID + '_close">';
			B[i++]='    </div>';
			B[i++]='    <div id="ta_dialog_' + this.dialogID + '_body">';
			B[i++]='    </div>';
			return B.join('');
		};

		this.init = function(settings) {
			//shouldn't scan text when composing new mail in Gmail
			var currentBody = document.getElementsByTagName("BODY")[0];
			
			if(currentBody.getAttribute("g_editable") == "true" || (currentBody.className != "" && currentBody.className.indexOf("editable") == 0)){
				return;
			}
			
			if (!TACode.Dialog.prototype.dialogID) {
				TACode.Dialog.prototype.dialogID = 0;
			}
			
			this.dialogID = 0;//TACode.Dialog.prototype.dialogID++;
			TACode.Utils.Object.extend(this.settings, settings);
			this.dialogElement = document.createElement("div");
			this.dialogElement.innerHTML = this.getDialogHTML();
			this.dialogElement.id = 'ta_dialog_' + this.dialogID;
			TACode.Utils.Element.addToBody(this.dialogElement);
			TACode.Utils.Element.addStyleSheet(this.getDialogCSS());

			this.dialogElement = TACode.Utils.Element.$('ta_dialog_' + this.dialogID);
			this.dialogTitleElement = TACode.Utils.Element.$('ta_dialog_' + this.dialogID + '_title');
			this.dialogBodyElement = TACode.Utils.Element.$('ta_dialog_' + this.dialogID + '_body');
			this.dialogCloseElement = TACode.Utils.Element.$('ta_dialog_' + this.dialogID + '_close');
			TACode.Utils.Event.observe(this.dialogCloseElement, 'click', this.safe(function(e) {this.hide(); return false; }), true);

			var pattern = /#DIALOG_ID#(\w+)#/g;
			var patternResults = pattern.exec(this.settings.body);
			var ids = [];
			while (patternResults) {
				ids[ids.length] = patternResults[1];
				patternResults = pattern.exec(this.settings.body);
			}
			var body = this.settings.body.replace(/#DIALOG_ID#(\w+)#/g, "ta_dialog_" + this.dialogID + "_$1");

			this.dialogBodyElement.innerHTML = body;
			for (var i = 0; i < ids.length; i++) {
				var oneID = ids[i];
				if (this.settings[oneID + "Callback"]) {
					TACode.Utils.Event.observe("ta_dialog_" + this.dialogID + "_" + oneID, 'click', this.settings[oneID + "Callback"], false);
				}
			}

			TACode.Utils.Element.setStyle(this.dialogElement, {width: this.settings.width, height:  this.settings.height, backgroundColor: this.settings.bgColor});
            if (this.settings.bgImage) {
                    this.dialogElement.style.backgroundImage = this.settings.bgImage;
            }
			TACode.Utils.Element.setStyle(this.dialogTitleElement, {left: this.settings.titlePositionLeft, top: this.settings.titlePositionTop, width: this.settings.titleWidth, height: this.settings.titleHeight});
			TACode.Utils.Element.setStyle(this.dialogBodyElement, {left: this.settings.bodyPositionLeft, top: this.settings.bodyPositionTop, width: this.settings.bodyWidth, height: this.settings.bodyHeight});
			TACode.Utils.Element.setStyle(this.dialogCloseElement, {left: this.settings.closePositionLeft, top: this.settings.closePositionTop, width: this.settings.closeWidth, height: this.settings.closeHeight});

			TACode.Utils.Element.setOpacity(this.dialogElement, this.settings.opacity);
			TACode.Utils.Event.observe(this.dialogElement, 'contextmenu', function(e) { TACode.Utils.Event.stop(e); return false;});
			if (this.settings.draggable) {
				TACode.Utils.Event.observe(this.dialogTitleElement, 'mousedown', this.safe(this.dragStart));
				TACode.Utils.Element.setStyle(this.dialogTitleElement, { cursor: 'move'});
			}

		};

		this.exists = function() {
			return TACode.Utils.Element.$('ta_dialog_' + this.dialogID) != null;
		};

		this.getElement = function(name) {
			if (typeof(unsafeWindow) != 'undefined') {
				return unsafeWindow.document.getElementById("ta_dialog_" + this.dialogID + "_" + name);
			} else {
				return document.getElementById("ta_dialog_" + this.dialogID + "_" + name);
			}
		};

		this.getTitleElement = function() {
			return this.dialogTitleElement;
		};

		this.updateSettings = function(settings) {
			TACode.Utils.Object.extend(this.settings, settings);
			TACode.Utils.Element.setStyle(this.dialogElement, {width: this.settings.width, height:  this.settings.height, backgroundColor: this.settings.bgColor});
            if (this.settings.bgImage) {
                    this.dialogElement.style.backgroundImage = this.settings.bgImage;
            }
			TACode.Utils.Element.setStyle(this.dialogTitleElement, {left: this.settings.titlePositionLeft, top: this.settings.titlePositionTop, width: this.settings.titleWidth, height: this.settings.titleHeight});
			TACode.Utils.Element.setStyle(this.dialogBodyElement, {left: this.settings.bodyPositionLeft, top: this.settings.bodyPositionTop, width: this.settings.bodyWidth, height: this.settings.bodyHeight});
			TACode.Utils.Element.setStyle(this.dialogCloseElement, {left: this.settings.closePositionLeft, top: this.settings.closePositionTop, width: this.settings.closeWidth, height: this.settings.closeHeight});
		};

		this.show = function() {
			TACode.Utils.Element.setStyle(this.dialogElement, {display: 'block'});
			this.setPosition();
			TACode.Utils.Element.setOpacity(this.dialogElement, this.settings.opacity);
		};

		this.setPosition = function() {
			if (!this.settings.left) {
				TACode.Utils.Element.center(this.dialogElement);
			} else {
				TACode.Utils.Element.setStyle(this.dialogElement,  {top: this.settings.top, left: this.settings.left });
			}
		};

		this.hide = function() {
			TACode.Utils.Element.setStyle(this.dialogElement, {display: 'none'});
		};

		this.destroy = function() {
			this.hide();
			document.body.removeChild(this.dialogElement);
		}

		this.dragStart = function(ev) {
			if (this.dragMoveFunction !== null) {
				return;
			}
			this.lastDragX = ev.clientX;
			this.lastDragY = ev.clientY;
			this.dragMoveFunction = TACode.Utils.Event.observe(document, "mousemove", this.safe(this.dragMove));
			this.dragUpFunction = TACode.Utils.Event.observe(document, "mouseup", this.safe(this.dragStop));
			if (ev.preventDefault) {
				ev.preventDefault();
			}
			return false;
		};

		this.dragMove = function(ev) {
			var layer = this.dialogElement;

			var layerX = parseInt(layer.style.left);
			var layerY = parseInt(layer.style.top);

			layer.style.top = (layerY + (ev.clientY - this.lastDragY)) + "px";
			layer.style.left = (layerX + (ev.clientX - this.lastDragX)) + "px";

			this.lastDragX = ev.clientX;
			this.lastDragY = ev.clientY;
			if (ev.preventDefault) {
				ev.preventDefault();
			}
			return false;
		};

		this.dragStop = function(ev) {
			TACode.Utils.Event.stopObserving(document, "mousemove", this.dragMoveFunction);
			TACode.Utils.Event.stopObserving(document, "mouseup", this.dragUpFunction);

			var layer = this.dialogElement;

			if (layer.style.left.match(/^-/)) {
				layer.style.left = "0px";
			}
			if (layer.style.top.match(/^-/)) {
				layer.style.top = "0px";
			}
			this.dragMoveFunction = null;
			this.dragUpFunction = null;
			if (ev.preventDefault) {
				ev.preventDefault();
			}
			return false;
		};

	},

	Utils: {
		JSON: {
		    m: {
		            '\b': '\\b',
		            '\t': '\\t',
		            '\n': '\\n',
		            '\f': '\\f',
		            '\r': '\\r',
		            '"' : '\\"',
		            '\\': '\\\\'
		        },
		    s: {
		            array: function (x) {
		                var a = ['['], b, f, i, l = x.length, v;
		                for (i = 0; i < l; i += 1) {
		                    v = x[i];
		                    f = TACode.Utils.JSON.s[typeof v];
		                    if (f) {
		                        v = f(v);
		                        if (typeof v == 'string') {
		                            if (b) {
		                                a[a.length] = ',';
		                            }
		                            a[a.length] = v;
		                            b = true;
		                        }
		                    }
		                }
		                a[a.length] = ']';
		                return a.join('');
		            },
		            'boolean': function (x) {
		                return String(x);
		            },
		            'null': function (x) {
		                return "null";
		            },
		            number: function (x) {
		                return isFinite(x) ? String(x) : 'null';
		            },
		            object: function (x) {
		                if (x) {
		                    if (x instanceof Array) {
		                        return TACode.Utils.JSON.s.array(x);
		                    }
		                    var a = ['{'], b, f, i, v;
		                    for (i in x) {
		                        v = x[i];
		                        f = TACode.Utils.JSON.s[typeof v];
		                        if (f) {
		                            v = f(v);
		                            if (typeof v == 'string') {
		                                if (b) {
		                                    a[a.length] = ',';
		                                }
		                                a.push(TACode.Utils.JSON.s.string(i), ':', v);
		                                b = true;
		                            }
		                        }
		                    }
		                    a[a.length] = '}';
		                    return a.join('');
		                }
		                return 'null';
		            },
		            string: function (x) {
		                if (/["\\\x00-\x1f]/.test(x)) {
		                    x = x.replace(/([\x00-\x1f\\"])/g, function(a, b) {
		                        var c = TACode.Utils.JSON.m[b];
		                        if (c) {
		                            return c;
		                        }
		                        c = b.charCodeAt();
		                        return '\\u00' +
		                            Math.floor(c / 16).toString(16) +
		                            (c % 16).toString(16);
		                    });
		                }
		                return '"' + x + '"';
		            }
		        },

		    objectToJSONString: function (obj) {
		        return TACode.Utils.JSON.s.object(obj);
		    },

		    arrayToJSONString: function (arr) {
		        return TACode.Utils.JSON.s.array(arr);
		    },

			parseJSON: function (str) {
			    try {
			        return !(/[^,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]/.test(
			                str.replace(/"(\\.|[^"\\])*"/g, ''))) &&
			            eval('(' + str + ')');
			    } catch (e) {
			    	TACode.i("Got: " + e);
			        return false;
			    }
			}
		},

		URL: {
			parseURL: function(data) {
				var e=/^(\w+):\/\/([^\/:]+)(:\d+)?\/(.*)$/;
			    if (data.match(e)) {
			        return  {url: RegExp['$&'],
			                protocol: RegExp.$1,
			                host:RegExp.$2,
			                port: RegExp.$3,
			                path:RegExp.$4
			                };
			    }
			    else {
			        return  {url:"", protocol:"",host:"",path:"",file:"",hash:""};
			    }
			}
		},
		Object: {
			extend: function(destination, source) {
					  for (var property in source) {
					    destination[property] = source[property];
					  }
					  return destination;
			}
		},

		BrowserDetect: {
				init: function () {
					this.browser = this.searchString(this.dataBrowser) || "An unknown browser";
					this.version = this.searchVersion(navigator.userAgent)
						|| this.searchVersion(navigator.appVersion)
						|| "an unknown version";
					this.OS = this.searchString(this.dataOS) || "an unknown OS";
				},
				searchString: function (data) {
					for (var i=0;i<data.length;i++)	{
						var dataString = data[i].string;
						var dataProp = data[i].prop;
						this.versionSearchString = data[i].versionSearch || data[i].identity;
						if (dataString) {
							if (dataString.indexOf(data[i].subString) != -1)
								return data[i].identity;
						}
						else if (dataProp)
							return data[i].identity;
					}
				},
				searchVersion: function (dataString) {
					var index = dataString.indexOf(this.versionSearchString);
					if (index == -1) return;
					return dataString.substring(index+this.versionSearchString.length+1);
				},
				dataBrowser: [
					{
						string: navigator.userAgent,
						subString: "Firefox",
						identity: "Firefox"
					},
					{		// for newer Netscapes (6+)
						string: navigator.userAgent,
						subString: "Netscape",
						identity: "Netscape"
					},
					{
						string: navigator.userAgent,
						subString: "MSIE",
						identity: "Explorer",
						versionSearch: "MSIE"
					},
					{
						string: navigator.userAgent,
						subString: "Gecko",
						identity: "Mozilla",
						versionSearch: "rv"
					},
					{ 		// for older Netscapes (4-)
						string: navigator.userAgent,
						subString: "Mozilla",
						identity: "Netscape",
						versionSearch: "Mozilla"
					}
				],
				dataOS : [
					{
						string: navigator.platform,
						subString: "Win",
						identity: "Windows"
					},
					{
						string: navigator.platform,
						subString: "Mac",
						identity: "Mac"
					},
					{
						string: navigator.platform,
						subString: "Linux",
						identity: "Linux"
					}
				]

		},

		String: {
		  camelize: function(str) {
		    var oStringList = str.split('-');
		    if (oStringList.length == 1) return oStringList[0];

		    var camelizedString = str.indexOf('-') == 0
		      ? oStringList[0].charAt(0).toUpperCase() + oStringList[0].substring(1)
		      : oStringList[0];

		    for (var i = 1, len = oStringList.length; i < len; i++) {
		      var s = oStringList[i];
		      camelizedString += s.charAt(0).toUpperCase() + s.substring(1);
		    }

		    return camelizedString;
		  }

		},
		Element: {

			$: function() {
			  var results = [], element;
			  for (var i = 0; i < arguments.length; i++) {
			    element = arguments[i];
			    if (typeof element == 'string')
			      element = document.getElementById(element);
			    results.push(element);
			    //results.push(Element.extend(element));
			  }
			  return results.length < 2 ? results[0] : results;
			},

			findPosition: function(elem) {
				var leftPos = 0;
				var topPos = 0;
				while (elem != null) {

					leftPos += elem.offsetLeft;
					topPos += elem.offsetTop;
					elem = elem.offsetParent;
				}
				return [ leftPos, topPos];
			},

			addToBody: function(element) {
				var bodies = document.getElementsByTagName("BODY");
				var body = bodies[0];
				body.appendChild(element);
			},

			addStyleSheet: function(cssText) {
				var heads = document.getElementsByTagName('head');
				var head = heads[0];
				var style;
				style = document.createElement('style');
				style.type = 'text/css';
				if (this.isIE()) {
					style.styleSheet.cssText = cssText;
				} else {
					style.innerHTML = cssText;
				}

				head.appendChild(style);
			},


	        isIE: function() {
	                return (navigator.userAgent.toLowerCase().indexOf('msie') != -1);
	        },

            getTextContent: function(element) {
                    if (this.isIE()) {
                            return element.innerText;
                    } else {
                            return element.textContent;
                    }
            },

            center: function(element) {
            	var dimensions = this.getWindowDimensions();

                this.setStyle(element, {left: "" + (dimensions.pageXOffset + (dimensions.windowWidth/2) - (element.clientWidth/2)) + "px", top: "" + (dimensions.pageYOffset + (dimensions.windowHeight/2) - (element.clientHeight/2)) + "px"});
            },

            getWindowDimensions: function() {
            	var dimensions = {};
                dimensions.windowWidth = (typeof(window.innerWidth) == 'number') ? window.innerWidth : (document.documentElement.clientWidth ? document.documentElement.clientWidth : document.body.clientWidth);
                dimensions.windowHeight = (typeof(window.innerHeight) == 'number') ? window.innerHeight : (document.documentElement.clientHeight ? document.documentElement.clientHeight : document.body.clientHeight);

                dimensions.pageXOffset = (typeof(window.pageXOffset) == 'number') ? window.pageXOffset : (document.documentElement.scrollLeft ? document.documentElement.scrollLeft : document.body.scrollLeft);
                dimensions.pageYOffset = (typeof(window.pageYOffset) == 'number') ? window.pageYOffset : (document.documentElement.scrollTop ? document.documentElement.scrollTop : document.body.scrollTop);
                return dimensions;
            },

			  getStyle: function(element, style) {
			    element = this.$(element);
			    var value = element.style[TACode.Utils.String.camelize(style)];
			    if (!value) {
			      if (document.defaultView && document.defaultView.getComputedStyle) {
			        var css = document.defaultView.getComputedStyle(element, null);
			        value = css ? css.getPropertyValue(style) : null;
			      } else if (element.currentStyle) {
			        value = element.currentStyle[TACode.Utils.String.camelize(style)];
			      }
			    }

			    if (window.opera && ['left', 'top', 'right', 'bottom'].include(style))
			      if (this.getStyle(element, 'position') == 'static') value = 'auto';

			    return value == 'auto' ? null : value;
			  },

			  setStyle: function(element, style) {
			    element = this.$(element);
			    for (var name in style)
			      element.style[TACode.Utils.String.camelize(name)] = style[name];
			  },

				getOpacity: function(element){
				  var opacity;
				  if (opacity = this.getStyle(element, 'opacity'))
				    return parseFloat(opacity);
				  if (opacity = (this.getStyle(element, 'filter') || '').match(/alpha\(opacity=(.*)\)/))
				    if(opacity[1]) return parseFloat(opacity[1]) / 100;
				  return 1.0;
				},

				getInlineOpacity: function(element){
				  return this.$(element).style.opacity || '';
				},

				setOpacity: function(element, value){
				  element= this.$(element);
				  if (value == 1){
				    this.setStyle(element, { opacity:
				      (/Gecko/.test(navigator.userAgent) && !/Konqueror|Safari|KHTML/.test(navigator.userAgent)) ?
				      0.999999 : null });
				    if(/MSIE/.test(navigator.userAgent))
				      this.setStyle(element, {filter: this.getStyle(element,'filter').replace(/alpha\([^\)]*\)/gi,'')});
				  } else {
				    if(value < 0.00001) value = 0;
				    this.setStyle(element, {opacity: value});
				    if(/MSIE/.test(navigator.userAgent))
				     this.setStyle(element,
				       { filter: this.getStyle(element,'filter').replace(/alpha\([^\)]*\)/gi,'') +
				                 'alpha(opacity='+value*100+')' });
				  }
				},

				  hide: function() {
				    for (var i = 0; i < arguments.length; i++) {
				      var element = this.$(arguments[i]);
				      element.style.display = 'none';
				    }
				  },

				  show: function() {
				    for (var i = 0; i < arguments.length; i++) {
				      var element = this.$(arguments[i]);
				      element.style.display = '';
				    }
				  }


		},

		Array: {
		  $continue: new Object,
		  $break: new Object,

		  _each: function(arr, iterator) {
		    for (var i = 0; i < arr.length; i++)
		      iterator(arr[i]);
		  },

		  each: function(arr, iterator) {
		    var index = 0;
		    try {
		      this._each(arr, function(value) {
		        try {
		          iterator(value, index++);
		        } catch (e) {
		          if (e != this.$continue) throw e;
		        }
		      });
		    } catch (e) {
		      if (e != this.$break) throw e;
		    }
		  },

		  include: function(arr, object) {
		    var found = false;
		    this.each(arr, function(value) {
		      if (value == object) {
		        found = true;
		        throw this.$break;
		      }
		    });
		    return found;
		  }

		},

		Event: {
		  element: function(event) {
		    return event.target || event.srcElement;
		  },

		  pointerX: function(event) {
		    return event.pageX || (event.clientX +
		      (document.documentElement.scrollLeft || document.body.scrollLeft));
		  },

		  pointerY: function(event) {
		    return event.pageY || (event.clientY +
		      (document.documentElement.scrollTop || document.body.scrollTop));
		  },

		  stop: function(event) {
		    if (event.preventDefault) {
		      event.preventDefault();
		      event.stopPropagation();
		    } else {
		      event.returnValue = false;
		      event.cancelBubble = true;
		    }
		  },

		  // find the first node with the given tagName, starting from the
		  // node the event was triggered on; traverses the DOM upwards
		  findElement: function(event, tagName) {
		    var element = Event.element(event);
		    while (element.parentNode && (!element.tagName ||
		        (element.tagName.toUpperCase() != tagName.toUpperCase())))
		      element = element.parentNode;
		    return element;
		  },

		  observers: false,

		  _observeAndCache: function(element, name, observer, useCapture) {
		    if (!this.observers) this.observers = [];
		    if (element.addEventListener) {
		      this.observers.push([element, name, observer, useCapture]);
		      element.addEventListener(name, observer, useCapture);
		    } else if (element.attachEvent) {
		      this.observers.push([element, name, observer, useCapture]);
		      element.attachEvent('on' + name, observer);
		    }
		  },

		  unloadCache: function() {
		  	TACode.d('unloadCache');
		    if (!this.observers) return;
		    for (var i = 0; i < this.observers.length; i++) {
		      this.stopObserving.apply(this, this.observers[i]);
		      this.observers[i][0] = null;
		    }
		    this.observers = false;
		  },

		  observe: function(element, name, observer, useCapture) {
		    var element = TACode.Utils.Element.$(element);
		    useCapture = useCapture || false;

		    if (name == 'keypress' &&
		        (navigator.appVersion.match(/Konqueror|Safari|KHTML/)
		        || element.attachEvent))
		      name = 'keydown';

		    this._observeAndCache(element, name, observer, useCapture);
		    return observer;
		  },

		  stopObserving: function(element, name, observer, useCapture) {
		    var element = TACode.Utils.Element.$(element);
		    useCapture = useCapture || false;

		    if (name == 'keypress' &&
		        (navigator.appVersion.match(/Konqueror|Safari|KHTML/)
		        || element.detachEvent))
		      name = 'keydown';

		    if (element.removeEventListener) {
		      element.removeEventListener(name, observer, useCapture);
		    } else if (element.detachEvent) {
		      element.detachEvent('on' + name, observer);
		    }
		  }

		},

		Effect: {

			DefaultOptions: {
			  transition: null, // Effect.Transitions.sinoidal,
			  duration:   1.0,   // seconds
			  fps:        25.0,  // max. 25fps due to Effect.Queue implementation
			  sync:       false, // true for combining
			  from:       0.0,
			  to:         1.0,
			  delay:      0.0,
			  queue:      'parallel'
			},

			Base: function() {
				  this.position = null;
				  this.start = function(options) {
				    this.options      = TACode.Utils.Object.extend(TACode.Utils.Object.extend({}, TACode.Utils.Effect.DefaultOptions), options || {});
				    this.currentFrame = 0;
				    this.state        = 'idle';
				    this.startOn      = this.options.delay*1000;
				    this.finishOn     = this.startOn + (this.options.duration*1000);
				    this.event('beforeStart');
				    /*
				    if(!this.options.sync)
				      Effect.Queues.get(typeof this.options.queue == 'string' ?
				        'global' : this.options.queue.scope).add(this);
				    */

					var timestamp = new Date().getTime();
					this.startOn  += timestamp;
				    this.finishOn += timestamp;
				    TACode.addSafeMethod(this);
		          	this.interval = setInterval(this.safe(function() {this.loop(new Date().getTime())}), 40);
				  };

				  this.loop = function(timePos) {
				    if(timePos >= this.startOn) {
				      if(timePos >= this.finishOn) {
				        this.render(1.0);
				        this.cancel();
				        this.event('beforeFinish');
				        if(this.finish) this.finish();
				        this.event('afterFinish');
				        return;
				      }
				      var pos   = (timePos - this.startOn) / (this.finishOn - this.startOn);
				      var frame = Math.round(pos * this.options.fps * this.options.duration);
				      if(frame > this.currentFrame) {
				        this.render(pos);
				        this.currentFrame = frame;
				      }
				    }
				  };

				  this.render = function(pos) {
				    if(this.state == 'idle') {
				      this.state = 'running';
				      this.event('beforeSetup');
				      if(this.setup) this.setup();
				      this.event('afterSetup');
				    }
				    if(this.state == 'running') {
				      if(this.options.transition) pos = this.options.transition(pos);
				      pos *= (this.options.to-this.options.from);
				      pos += this.options.from;
				      this.position = pos;
				      this.event('beforeUpdate');
				      if(this.update) this.update(pos);
				      this.event('afterUpdate');
				    }
				  };

				  this.cancel = function() {
			  	      clearInterval(this.interval);
					  this.interval = null;

				  	/*
				    if(!this.options.sync)
				      Effect.Queues.get(typeof this.options.queue == 'string' ?
				        'global' : this.options.queue.scope).remove(this);
				    */
				    this.state = 'finished';
				  };

				  this.event = function(eventName) {
				    if(this.options[eventName + 'Internal']) this.options[eventName + 'Internal'](this);
				    if(this.options[eventName]) this.options[eventName](this);
				  };

				  /*
				  this.inspect = function() {
				    return '#<Effect:' + $H(this).inspect() + ',options:' + $H(this.options).inspect() + '>';
				  };
				  */
				},


			Opacity:  function(element) {

			    this.element = TACode.Utils.Element.$(element);
			    // make this work on IE on elements without 'layout'
			    if(/MSIE/.test(navigator.userAgent) && (!this.element.hasLayout))
			      TACode.Utils.Element.setStyle(this.element, {zoom: 1});
			    var options = TACode.Utils.Object.extend({
			      from: TACode.Utils.Element.getOpacity(this.element) || 0.0,
			      to:   1.0
			    }, arguments[1] || {});
			    this.start(options);

			  this.update = function(position) {
			    TACode.Utils.Element.setOpacity(this.element, position);
			  }
			},

			Fade: function(element) {
			  element = TACode.Utils.Element.$(element);
			  var oldOpacity = TACode.Utils.Element.getInlineOpacity(element);
			  var options = TACode.Utils.Object.extend({
			  from: TACode.Utils.Element.getOpacity(element) || 1.0,
			  to:   0.0,
			  afterFinishInternal: function(effect) {
			    if(effect.options.to!=0) return;
			    TACode.Utils.Element.hide(effect.element);
			    TACode.Utils.Element.setStyle(element, {opacity: oldOpacity});
			  }}, arguments[1] || {});
			  return new TACode.Utils.Effect.Opacity(element,options);
			}

		}
	},

	ResourceManager: function(locale) {
		this.images = null;
		this.texts = null;
		this.defaultLocale = 'en';
		this.locale = locale ? locale : this.defaultLocale;
		this.dllPath = GM_getValue("dllPath", "");

		this.loadImages = function() {
			this.images = {};
			//EMBEDDED_PICS_BEGIN
			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhEAAQAPYAAP///9LS0nh4eOrq6i0tLYCAgFdXV6GhoUFBQWxsbKKiok5OTsbGxoeHhwwM';
			B[i++]='DEZGRtra2t3d3WpqatDQ0GZmZvPz82FhYYyMjKampu3t7be3t7q6uiYmJp6ennx8fHp6egMDA5SU';
			B[i++]='lDw8PHR0dH9/f+zs7L+/v0pKSjY2Ntzc3HFxcdfX166ursLCwqurq+7u7sXFxa2trfDw8NbW1uXl';
			B[i++]='5dnZ2fn5+aqqqry8vODg4PHx8czMzLi4uMPDw/v7+6ysrMTExL6+vrGxsbKysvLy8t7e3paWlo6O';
			B[i++]='jqenp8rKysDAwMHBwcfHx/T09OLi4ru7u9jY2OTk5MjIyPj4+Hd3d1JSUoWFhd/f3/f396ioqPb2';
			B[i++]='9rCwsKWlpbW1tdHR0ZCQkPX19VlZWYKCgv7+/svLy6SkpM3NzSMjI4mJib29vSkpKbS0tOnp6efn';
			B[i++]='56Ojo8/PzxwcHJOTk11dXTIyMujo6Nvb2/r6+hQUFJqamtTU1G5ububm5vz8/FRUVLm5uf39/SH/';
			B[i++]='C05FVFNDQVBFMi4wAwEAAAAh+QQFBQAAACwAAAAAEAAQAAYHyIAAggB/gn59fIOKgnt6eQB4d3x2';
			B[i++]='fnWLAHRzcgBxcABvbm2Ke2wAa2ppaGdmZWQAdmxjf2JhaWByX15dXFtae2tZWABXVlVUWlOCk1Ju';
			B[i++]='XVGKZFyFinlQg09lXFlOl01MS0pJSEdfRkWXRH5DQkGDUV5ji1fPg1hAeD8+i2luPUQAY3i42eEt';
			B[i++]='TxMdOXCAgYLkhg0AdfYAYOOmBpkuNNzMACCjxjQANoasGQMkBgAYXF5cYuOCBoAWLsaAYbHi0hhh';
			B[i++]='AMw8iWcjkaBAACH5BAUFAAAALAAAAAANAAsARgdfgACCgwB4Kn+DS0JCXXSEj0ZiO1geHQBKWRwl';
			B[i++]='hCVyJAAjIgAhIEyDMxtjj4M2bxhbq49RVCkAeih/UycmADQvAC19LR8nNSAqgntuHkZYdoJ2RDwA';
			B[i++]='O2kaXW2xhBlF2oEAIfkECQUAAAAsAAAAABAACABGB12AAIKDAC4XhIRrWRA2eBtXO0IPAHlhDi+D';
			B[i++]='MmIdAEcUnWo9iABRE6MNIaNtRjQAaBYAFXI4hGwVABMfeXgSK2pWADYRgi9Kbms2PoJ8TR4gc11j';
			B[i++]='ZkpLDBmjdHgmgoEAIfkECQUAAAAsAAAAABAACgBGB22AAIIAY4JBSH+DiouLKSJqHAt/a1kpdhs9';
			B[i++]='USsmCSUwCwhag2A3XQBZaABuC16MUROFinhIjCVdbAAYDX9gYhOKbBUAVworMRcpVR0ATVeFL0pu';
			B[i++]='Gj58gnxgJGpVaWNmSksMGa4NF0YsjItbOIOBACH5BAkFAAAALAAAAAAQAA4ARgeRgACCAGOCXiaF';
			B[i++]='g4JLQkJddIqCTmEnD3p/a1kpdhtvJSk7XzJvenJakZEKQQAzeHyDYDdPAEBIAH4fKahRE4mDMV2o';
			B[i++]='wmMnZwcZiiVdyC03Y1gdeYpsFQBXbk5MB1EewVpOhS9Kbho+rwB/U0ZVIztjZkpLDMiRdGJ6EheD';
			B[i++]='GUW+gkNMhEWCQiFKiDsGpghURA9AIAAh+QQJBQAAACwAAAAAEAAQAAYHuYAAggBjgl4mhYOKACVd';
			B[i++]='GQAtLmN2fnWLAGA3TwBAMQBvbnuKbBUAV25OTFslZWQAU1Fjf2tZKXYbbyU0EVsVEHFiWgAvSm4a';
			B[i++]='PnyCfzYxH0ZQilETiYMtUoNmSksMj4sZeGhWHUtCQl10l20feglogxlF1IJdMIo2bxhbPopjFhwj';
			B[i++]='OQjxcLOjCZM8TaLcMMDGzZw5wuqIaliDzAUeILoAgDDkzyAbQ9aM6QRAzhk2l1JGgaMx5SUiigIB';
			B[i++]='ACH5BAkFAAAALAAAAAAQABAARgfGgACCAGOCXiaFg4JLQkJddIqCJXhfXxh/a1kpdhtvJTQRW1p7';
			B[i++]='mVgAdXsAbG5begZMHB0ATkl/g2A3TwBAMQBvbm2RAFETiYNJZIo2bxhbPopjYlUXNIpsLtV9C2N7';
			B[i++]='D0HCJV0ZAC0uY3Z+ddYVAFduTkxbJWXJdmyFL0puGj58gn92pLC6AsCMkiUMxkV6EQNDmS6DMhQx';
			B[i++]='JqiFmUFjeLjZ0YRJmRwQNFhg0yWMhSmDbAxZ80fOAgAj1KwQNsYUgA9UbGHxJygQACH5BAkFAAAA';
			B[i++]='LAAAAAAQABAABgfKgACCAGOCXiaFg4oAJV0ZAC0uY3Z+dYsAYDdPAEAxAG9ubYpsFQBXbk5MWyVl';
			B[i++]='ZAB2bGN/a1kpdhtvJTQRW1p7tFgAL0puGj58gn92Um5dUYpRE4mDeVCDZkpLDI+LTUxLSklLQkJd';
			B[i++]='dJcvNy5bQYMZUNOCEyuKNm8YBmCKYwofWSWEeLjx44SCkShXgJB4kaSAlSkA6uwBsATOGjF9zFTZ';
			B[i++]='AoDNhD+DXqih8mcEAgBxqhS5xEBECgB6UPwpQYHBpT/6AHyRgGwKMkGBAAAh+QQFBQAAACwAAAAA';
			B[i++]='EAAQAEYHx4AAggBjgl4mhYOCS0JCSCuKgkR+Q0JBf2tZKXYbbyU0EVtae5lYADwRAEkcLngjNR8t';
			B[i++]='ADI1f4NgN08AQDEAb25tkQBRE4mDeVCKOh8OBFqRT249RIp5KEsAF3JjMmhkwiVdGQAtLmN2fnWK';
			B[i++]='bBUAV25OTFslZd92bIUvSm4aPnyCf+xIcdMlCgAzSpYwGBepCZMlSpIM+mVL0RWDgviIcICHzQc8';
			B[i++]='A9rkwUNkBZIbNgbpOLHgzxcLAGJ42CNsTBNBZcTYmvJPUCAAIfkEBQUAAAAsAAAGAA0ACgAGB1mA';
			B[i++]='AIIweH+Ch4IfVVUWO4iIBWEGYUCPllBDlogXBAhYj2kAfxRnaG1GPxklKzhgUABJlXlVPFtxdQcz';
			B[i++]='iE16KmNZaABpXC+HURRmABhWAEQsK4ifAE8dgjYAgQAh+QQFBQAAACwAAAMACwANAAYHX4AAUlcA';
			B[i++]='XgCHiABydx1EEkCJAHRHZygVPpGHTBeZXxJ6VHmZAg8nfUmZiHVKqU0YBhQ2mSQLHRlCSjo6OYdv';
			B[i++]='bwBXH0wmXTRuiFpfcQBpMQAwiDRWNQBLLgBgqWZPYwCBACH5BAEFAAAALAAAAAAIAA8ARgdbgACC';
			B[i++]='AHyDFxISIyuDAFUoLmB9HzUAARAAbW41ZIM6ZwiMAC1fjFhPJGh2oWwuNKKDKXNegytOAAwgW0IA';
			B[i++]='KxIcfWCqZWgXcXWhAFETY4N4H10yQHlNgzZDawBAgQA7';
			this.images['icons.spinner'] = { name: 'spinner', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhDAAMAPMAAP///2ZmZoCAgExMTJmZmTMzM7KyshoaGgAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAACH/C05FVFNDQVBFMi4wAwEAAAAh+QQJCgAAACwAAAAADAAMAAMEMBDIiaalqF5wQK6GdYwe';
			B[i++]='YJziFUpFsQEE0bZbDNA1MQ2XYA3AiWD4AwSOAN/mGLBEAAAh+QQJCgAAACwAAAAADAAMAAMEMBDI';
			B[i++]='eaalp15QQK6IVYwegJziFUrDsAGG0bZbDNC1MQUXYQXACWH4AwiOAN/mKLBEAAAh+QQJCgAAACwA';
			B[i++]='AAAADAAMAAMEMBDIWaalpV4wQK6HNYwecJziFUpBsAEI0rZbDNA1MgmXYQnAiWH4AxCOAN/mSLBE';
			B[i++]='AAAh+QQJCgAAACwAAAAADAAMAAMEMBDIOaalo14QQK6FFYweUJziFUqCsAHH0bZbDND1MREXYhHA';
			B[i++]='CWL4AxiOAN/maLBEAAAh+QQJCgAAACwAAAAADAAMAAMEMBDIGaaloV4gQK6DJYweMJziFUoEsQFF';
			B[i++]='0bZbDNB1MRnXYRnAyWH4AyCOAN/miLBEAAAh+QQJCgAAACwAAAAADAAMAAMEMBDIKaalol5AQK6B';
			B[i++]='RYweEJziFUqGsQHD0LZbDND1MCFXYSHASWH4AxyOAN/meLBEAAAh+QQJCgAAACwAAAAADAAMAAME';
			B[i++]='MBDISaalpF5gQK6CZYweIJziFUoIsgFB0LZbDNB1MB3XYB3AyWD4AxSOAN/mWLBEAAAh+QQBCgAA';
			B[i++]='ACwAAAAADAAMAAMEMBDIaaalpl6AQK6EhYweQJziFUrHsQGC0LZbDNC1MBVXYBXASWD4AwyOAN/m';
			B[i++]='OLBEAAA7';
			this.images['smallicons.spinner'] = { name: 'smallSpinner', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhEAAQAIABAGaLsf///yH5BAEAAAEALAAAAAAQABAAAAIOjI+py+0Po5y02ouzPgUAOw==';
			this.images['icons.empty'] = { name: 'empty', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhDgAOAIABAGaLsf///yH5BAEAAAEALAAAAAAOAA4AAAIMjI+py+0Po5y02qsKADs=';
			this.images['smallicons.empty'] = { name: 'smallEmpty', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhEAAQAPewAJO0ckeBD2WVOGOHUSVXDJO0c2WJVPz9/Pr7+U2YAFKgAIfKAG6PXWuNWuTq';
			B[i++]='4XawOG2kODl5AMbnAEyLDE6JDEKDAICoXCdZD3aVZkhyM4CecpW4cypbEnLGAFugD4KpXL3pAK7m';
			B[i++]='AG+pADd5ADBuADyBAI/UADx8AEaIAKLOczh7AKHMcnm5AGSmAB9TBqPNc5feADd3AJi+c8XkANTt';
			B[i++]='AOHo3pbUAJauiobXAInaAFSQAEN9DHnKAGOrAG+eAFKVAFWYAJvEckSGAHjBAHzGAKfgAILMALzn';
			B[i++]='AIXHAJW1c7HDqDp5AM7rAMrsALzjAJa6ci1qAJS1c67eAJvIAM/ayZjPANLuAKjdAHu5OFGeAEmN';
			B[i++]='AD9+AEaOAGKrD2SrADV0AESHAJLFXKfhAE2WAIaxXHW8DFGcADZtAHCRYKXgANzxAD9/AHOTY9nw';
			B[i++]='AESLAHC6DLHfAJ7OAFioAKXeAFaXAFWcAHexOGe1AGq8ALnlAIumfoaieIPJALniAEWHAEiQAH7D';
			B[i++]='AMHPusvsADh5ANPrAGWWOKnYAJraAFelAEp0NtbvAEOBAD6CAI/RAFOYDIXDAHTEAIvJAFyuAJDT';
			B[i++]='AIfQAMjqAF2XAPL18NTvAOzx6kqGDEGBAEePAFemAIXAAEyNDE+XAF2nD3K9AMfmAEySAIK7AKDW';
			B[i++]='AJTbADt7AD1qJ2m3AJ7HclWZD0eIADR0AP///////wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAALAALAAAAAAQABAA';
			B[i++]='AAj7AGEJFLgCS5dQD4IMXAgrRRkkC4bckWPGkQyGYSL1cQLHFB9VnRiNIDPwxQImVirludJIVIwB';
			B[i++]='DL5sEPhmVBtMTY4UMbFIwys0gzTBWkWEkBpFgkCI0WHg1Ss2JcAAsGNEAo0ZU0KISHXJ6Q0Vfwp5';
			B[i++]='gDTHB4FEls5QeXVgjwtJCQKwwtNigINMBBqwbSCER4csASAg4hDIqR4HrzCgyHGKkgIBTxJwgcIA';
			B[i++]='gVMlqHDASGMDFABYn8YocDUAQY0LXiYdkvJogsAom9xwImEgQytAVQzFAZJk4IcTaypE0NKDhadS';
			B[i++]='dCwwLLBjyRY/pOr8oFCA4UAAAgIEEPB5YUAAOw==';
			this.images['icons.safe'] = { name: 'safe', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhDAAMAOZuAGWlF9TyADh7GGeaTojDAGSlF02WAEuWAZK2gMjoAMnpAFGZAVKbAHy7AGSm';
			B[i++]='F4S4QsXYvOD1AFWgAWKkF22sAfH279HxAE6PASpxCEWPAWqoL3e+Fz1+HoK2RWCpAcbZvbXOqazg';
			B[i++]='AMnlAGq0AFaZAVaeAaXgAMHsAFqmAIHFF3a6AGKWSL7jAKDWRuX2AJ/OAGmbUJ/RAJHTAHCmAU+K';
			B[i++]='M0eRAYO3RtDgyDl7GYnEAInDRWaZTZK3gaXZRH66AYa5Q4C/AL/rAITJAPv8+kiFK6TYQ0SCJuTt';
			B[i++]='4Mzdw2+vAcvtAIaucmekL+zy6W+fV360QnfFAN30ALjiAMPnAGSvAK3YANHoAHilYi91DrnlAMbh';
			B[i++]='AKfEmeP1AGqcUpLLRvP38Y/OAJDEAVSiAYG2RZnQRYG2Q4C0RrHhAH7GF+rx50WEKMLmAIfMF///';
			B[i++]='/////wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5';
			B[i++]='BAEAAG4ALAAAAAAMAAwAAAeAgG6Cbi1saF6Dg0VnWSEyUGIagz1KURZBJiUDMEyCUi4RAScUK01L';
			B[i++]='GW5kClwia2FqR208NWMpLDMcAhcQbSBEIwUbSVdDCE5tWxg5Qg46Hl03aRUfWDEJYB1uVBI4SF9G';
			B[i++]='PlNWQIJlKAc0OyRVWi8/g08MBgsqBA0PiYJmEwA2iYEAOw==';
			this.images['smallicons.safe'] = { name: 'smallSafe', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhEAAQAPeaAGcAANnAwPbw8H8bD7OAgIAMALR5cnogILR6c+PQ0HYEAIwOAK4nHJVCOIoN';
			B[i++]='AJokIZZDOaYcD3YHApwNAJQOAJEPAKUSBnwPAHgMAL9IPqhkXJgZDIQwMIANAc56c4YnGsx5crR7';
			B[i++]='dKREOIUXDIURCJwRBJoNALV6c8d6c40QAoYpHYQlGcR6cqgcEJZEOnADAMRkXMt7dJczJ68XC8I2';
			B[i++]='LblCONyAeosKAKUPAZ8VCHgEAI8XE6RjX30LALp6cpAdEL9qZqdZULByar57c79FOuCZlMBAN3oY';
			B[i++]='Fdd0buCUj4QcD6tnYNNjW3IFAHIEAH8NAJZDOH8MAJA4N6EQCZMOAJA3NokOALkuJHQEAHwEAKFD';
			B[i++]='OrkpHpgWB6lkXMWDgLh6c9mLhIAQBI02Mn0YDLFkXKAOAIgNBI0kG34kI6BgYMleVrV2bnoPA3wW';
			B[i++]='CnsdG58tIKJUSrBCOKUaDctVTXEQEK5DOME+NIcKAHgOAIUkGIMGALR6cogqH6wmGqpwcI1AQMcx';
			B[i++]='KJccD3gDAJdQUNFwaYoZDLMbEL09MpcOAIQHA30jIY43LIQIANqMhpUlInIIBqsiFoYqHpQ3K4YY';
			B[i++]='DZocD6UOAIsOAO3g4NCwsP///////wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAJoALAAAAAAQABAA';
			B[i++]='AAjvADUJFAiiRosIcVgMXKjJwyEjdrbMqDRhwxCGMNQUSWKDCQ1DOExUIDMwBhEvVQAAcLRDpY47';
			B[i++]='Vr4I7ANGCqZMmf5wwIlJgaURmlAwaATkgIBMlxJkEnDgxoICe+pAIoTkQRqcOAfpQZSiABRKcjLM';
			B[i++]='AZToZqYAgspw+RFmQKASDK5MoXN06QEzb2QoGSCCQg4LWcziTPBIS5APEHwsoMLID04CBB6LWcMH';
			B[i++]='gaZCDrBgCtBTwuYAaAzkEXgiSg8nL5qQOHPEjSIei0IM7HLhSYEOkyTBWSLEhQaGCMbgwcCmzQoV';
			B[i++]='kSwzFGigwYABDQwwDAgAOw==';
			this.images['icons.unsafe'] = { name: 'unsafe', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhDAAMAOZdAKwxH4QHB8d0aNmxsfDg4LhwcPjw8OjR0alRUcYyH8cwHNmXjpwaFY0ZFqsX';
			B[i++]='CMl0aK41Jq4yIdE1I79EL+unn5EhIbhxcZo1NbtBLpkxMdifl7UiD9g1I8h1aeF5bpETDOWXjNqi';
			B[i++]='mqEcFbE+McIyH74gCbQrGqA3NOujmrokEcCAgNF2aMFsaeuYi8wyHs0yH8dyY8xwYNp0aJ8VCM8p';
			B[i++]='F7BIQOmmnZQjIaFEQud8b6FAQJEMB86Ge5gaFN1SPdp1aOJqXNeXj6ZQSsWIg9eZj5w2NciQkNuX';
			B[i++]='jqlQUL1OP6QYCZYmIo4MB8VOP7ktGsqIhspPP+iFdpMgHdU2JNOHe9iwsMBFMJYaFK0xH6FIR79N';
			B[i++]='P+DAwP///////wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5';
			B[i++]='BAEAAF0ALAAAAAAMAAwAAAd+gF2CXR4SLzKDgxQ2IEAcLlBHgyhPFwEnDAE3NQuCLUUHXEYqXAdS';
			B[i++]='Wl0/USwZBlxcBhUzTg8JPjkiBa8FHyUpWCQ0UztVr1sNGBMRKwoOCFwEBFwWQjAdXU09A1s6SFsD';
			B[i++]='WTyCQSNXTEoQSzhDGoNESSYbVjFUIYmCAgAAAomBADs=';
			this.images['smallicons.unsafe'] = { name: 'smallUnsafe', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhEAAQAPeSAHJHEIVgMKGFYLSic8e2oJR6OOHZAIBgD2k7ALOhcufXAPbz8JBnAMekAIps';
			B[i++]='DHxUIM2yc9PIAMWdANHCsJlyD35ZAJhuAJZrAKyBAJVpAHdWAG5AAOPb0HdMALuZALmRDJx4AKZ5';
			B[i++]='D4ZgAMOscr2SDLWjc8miALabAIVjDKJ4ANzLALiLANOzALupAL+wAOjZAIVfALqmcsWfAM+wANzK';
			B[i++]='AHRGAI1iAIBcANu/AKeSXNK+AJ91AKt/AIljAL6qc8qrAJl8ANe3ANvQAOPNAIldAJ2AAJVrAKp8';
			B[i++]='D5JpAOzgAH9TAJ90ALOIAJluAIFhD76qkIplAMKfANjNAKiIAMOdAMatcpl0AJV7OLuRAKOFOJx5';
			B[i++]='AKJ6AMegAKuRcOzdAMywcqZ8AN/LAMWmXMunAIZYAJV1ANS4AJ5+AMKbAN7NAIFiD7iROLWidLCO';
			B[i++]='OL2TAJRqAItnAHtZALShcptsAHxdDK+DAJhrALejc6uAAJBsAH5aAI9nAKiSXIBaALCXXLKIAL6T';
			B[i++]='AMyrAIxoDKB5D5dvDNrQANa8AJZsAMm8AOXRAKBxAJt7AK+MOKN0AH9SAI9sQLSdgP///////wAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAJIALAAAAAAQABAA';
			B[i++]='AAjtACUJFPhlzZEQjEYMXCgJAgkuVHgoajSHkA+GYsYMAQHg0SMANcgw8DMQwg8vUwREWhmJAwAb';
			B[i++]='Pe4I/KDgBYAFLFcScIQEhaQqDZKc6LJyQgCWD5qIkNMmkAIaDx5BCkBg5YINdRhcGSQjTCIzUYxU';
			B[i++]='XSkggxs7TihgmWEoCJqjZN+waLDkQBYMgCSY+PNo5ZMLOFTo2FIghgU8TFaAkQrpkYc0BqRYGSBJ';
			B[i++]='0KEdKYhAWgnpjBADiBwILHEDyh4lmyNBKlIowiI2A/nE0dOnAwIEcFq4AJKD4QA6GirAyKOljAPK';
			B[i++]='DAUmKHBATYEEDAMCADs=';
			this.images['icons.unknown'] = { name: 'unknown', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhDAAMAOZUAL2mYaR4QLSPYPj08IZLAItSAN7PAOjYAKl6AdrHsJNdAJlkAMihML6nYriT';
			B[i++]='ALuUAJNhAOHLAMOqYtC/ALuacMCqYene0I5ZAKh8ANO3AM2tAM63AK6JAMGrAMWkAMyoMIpRAOrZ';
			B[i++]='AMGrYtTBAJ9wANi8YJVwAceyAL6YAOPGAO/oAPHlAMeuYNrAYqmKMKNyALeUAKZ9ALybAOLOAMOl';
			B[i++]='gNOwAKeBAMCgANO4YJ55AMWmMJtlAJplAN/PALKRAJNmANvBYdS+AL6cAOHKAL+fMKJ7ANC0YPTw';
			B[i++]='AL+nAeXOAaN1AMeuYdjLAJBaAJ1tMKmMMLylYNK8oOHTwP///////wAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5';
			B[i++]='BAEAAFQALAAAAAAMAAwAAAd7gFSCVCUfDEaDg0BJHi8gOwhLgy0rMAIJA1FOJBKCET0BU6JTUgpF';
			B[i++]='VDghNzQWURSiBBxQOjMHDwsColFKMk9EGUMpBQNTURgbJ08sQho1PFJSTkEqE1BUNg4oBKIBI0cd';
			B[i++]='gg05MU3QTgZMIoMAJj8XED5IFYmCUC5PAImBADs=';
			this.images['smallicons.unknown'] = { name: 'smallUnknown', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/AL+/v3DWcPn6+QDUAAHCAfT09LjquOrq6gDdAE7PTriq0YzdjNnZ2eXm5zXK';
			B[i++]='NcDAwCjGKOj46Nbz1vDw8Ae9B5J9uBnEGQDJANzc3NLU1cbuxuzs7eHh4fX89QDgAJbglgrFCs7w';
			B[i++]='zujn6qTlpJmcn1AujaSlpbO1t8zNz9PL4qyvsd/g4XxiqqGfrb/Bw6aoq8bFzV4+lsnJyfTy+N7Y';
			B[i++]='6nFVot/f3wC9ANXV1QDEAADPAADYAADiAP///8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ/w';
			B[i++]='Jwk4CLmkcslsOp/QqHRKrVqv2Kx2e4MsOsPhAnQZDHbotHrNbrvf8Lh8Tq/b7/i8fqC7WEJDHQll';
			B[i++]='CB4ePIiJiouMjY6PkJGSk5SVlpeYmZmGCAg6BB9CARc7hpqnqKmqq6ytrB4IAwQaEjkDh665uru8';
			B[i++]='vb2wOg4BOoWOCDt8OsrLzM3Oz9DR0tPU1dbX2Nna281nCI4eshC3jbEXOQQ36uvs7e7v8PHy8/T1';
			B[i++]='9vf4+fr77gQXOuQY7UA3oNEOHTm8hOjBsKHDhxAjSpxIsaLFixgzatzIsSNEAwEoEBKIbgejARcI';
			B[i++]='BOjQAAOOlzBjypxJs6bNmzhz6tzJs6fP/59AZ9o4IMFBDmKLBuYwqcgTBQMHGDDAYKOq1atYs2rd';
			B[i++]='yrWr169gw4odS7as2awYGHDoEcBW0iRMEw3IseAADgwcGhzYy7ev37+AAwseTLiw4cOIEytezPhv';
			B[i++]='gxVSezggpUhpXB4ILkDocVfvhAKgQ4seTbq06dOoU6tezbq169ewY5OeIIKD2hAECiayrGjuggZq';
			B[i++]='NxQQ4LG48ePIkytXHlXGAxkMDvQQIIC2DRw9buioDLf3DQkYMIiYQNyighLoS6SAWCF9CYYoTJCY';
			B[i++]='T7/+fBkMBZywz3++igYuuLDBC8sVaOCB+WHwAAAMMvgABsQVcMB12W23W3eJ6HCDADgMVf/ARSm4';
			B[i++]='p95DLYjYwwr9pYhCDwCkaJ8JH6LQgAonIGjjjR5hAABDDfKIQQ8SUqgdd0spoiFnHlokQgwirtcQ';
			B[i++]='DCK+t58JDDRg5ZVXrvACCQT6h+WXVn7YQwMFrNAAjmimSdEBC/LIII8PHBAkdkNeWGSGNyApZ0Uz';
			B[i++]='1BClkz3QwKSJ+9VIEQrz9TDfimo26ihHDPTIopsAMDBnhURedmSHe1LU3p8MzTAooSQYOhGiJChK';
			B[i++]='AqOPtuqqRDJI2lCDMlxaJyK84amnmBJ9CmoPfkYpZakVoaoqq68m++qCsjYIwAO2WogrhohsmqRE';
			B[i++]='56U36oi+lrAtiySYgMK45JYbHwkqqAr/gLnsbqDsuwfG+qZDtEabqZF5csrrQzSImC16KfzrrcA9';
			B[i++]='MOAifxmoejB9LsDrcHKRzrsjiwxaOiGd0vKQa7X5XvuQqO5V0IN7JboHQ4jp9YDqwvM1zHJ9Zz4s';
			B[i++]='c0dsTizps3JejKmdmnbcqUN9hsyQsOgp0APK6PWw3wvlRaRfuOpStMGiM1e9kY6U+gikzrdqTC0P';
			B[i++]='1v7cULc1zDC0sCIf7Z7SxB6aKNUUwW313BUJoKCzz0K4tZAZbwy2z/v2UDJ6MYjQkLA1NIT0sKZK';
			B[i++]='ZKzcEkFO9+QRNfdcdA3ZyzO+uzoEpXuAjixiDE0vzna6FMnw9qoVSU756xZpPu2dHHfO/5Cg7hnt';
			B[i++]='0Og0OGS6C/6dIPzwxM9Xo3wmEK/8CSrMlzDs0MfOdd9fhy0myOmlvXvuD5legHwvn5nBy+g2Hf35';
			B[i++]='EMnuNe1/294CC/CzoL1D8bOgu0M01M9QAS4srzwAMTuR/5QnA/Oh74D7m969dKUvBDrwgXvD2AJr';
			B[i++]='18CCneAFL1CBuxiyH/o0ZAMdbAF+4MMqGTWkPidYQUNMtbJEMUQGDZuOClQIQfSpz2/W42DMIsPB';
			B[i++]='iKCAAQ1BncpKGEBTrYBVjRtTEnsQIAGoAIg1tKECN8fAa41veDRU2vBGqLIAOmRcDTEhQ15Arhc8';
			B[i++]='T2kPacASmUilKB7whtUD3AsbcsQV+v8wi1wE4wsD+B8ruQCJaVwjGt0oRb5NsH0VRAGNLphF5dHx';
			B[i++]='BcJTgRAnkDzhvYBXpmoAIB2iRokIkpB0gyP7cgjKUs5MlD2znSlX+S5Ucq5DJoilLGdJy1ra8pa4';
			B[i++]='zKUud8nLXvryl8AMpjB5aUgqUtAGPkimMpfJzGY685nQjKY0p0nNalrzmtjMpjapWczZpbJD2wyn';
			B[i++]='OMdJznKa85zW7Ob6volMdLrznfCMpzy1qU4cdmye+MynPvc5znrGEUn8DKhABypQf47yngRNqEIX';
			B[i++]='Sk6DspOhEI2oRKHp0Fe2c6IYzahCK1pFjXr0o/vk6DFBStKSulOkiLyoSVfKUmyi1Fr/LY2pTKX5';
			B[i++]='UoTO9KY4rSlAccpTmeoUnD0N6kp/qlKhGlWjRD2qUpEqQWOmdKlQlWhSo0rVjTbVmxatqlYHOtWt';
			B[i++]='ejWfXf2qWOEZ1rGa1ZxlPataw5nWtbr1mm19q1xpetV1ZnWueOVmXe2507z69Zlx/atfAytYvBK2';
			B[i++]='sHI9LGLdqtjFqrWxjjUrZCMr1slS1quWvaxWM6tZqnK2s1D9LGiVKtrRGpWoGTAtY/f6TxysgAOq';
			B[i++]='VasMiBqV2Jp1BT8VzwRSa1uvrmCKWJVLDj5gGw4IB7a9pSoKNmAd1rIvM5vpzAEmsAEGoCC5R0WB';
			B[i++]='DSZAG9usADe6Ca5wP2AXvOilMehN/69618ve9roXMI+ZimTKcEjMXOApUZnKWfbL3/76978ADvBW';
			B[i++]='0sIBAQQgN7gQr1xSspKWBOXBEI6whCdM4QrXZChFOco36osrhCiElSBuFUhEMoANZyrBTUEJOvjB';
			B[i++]='4ha7+MUwjrGM4eEPgDhiIDcISDmQwY0e+/jHQA6ykIcMDW88YiDDKIYvlszkJju5yeEgQC10/OQq';
			B[i++]='W/nKWH4EMBzwg1GUAsVZDrOYx+wKWMjCAD8QBCFMQeY2u/nNkYCFJ0ARhjGU4Qx6KASYUSFnPfj5';
			B[i++]='z4AOtKD9bAY/ACIMRDACErTwjxLv+RKcQMYFzrGFSlv60pjOtBa68AVEe/rToPb0CP8gsAMlYwIW';
			B[i++]='PM6BA0YggFC7+tWwjrWsZ03rWtv61ri2dQRAUApIo7oPFgiABHJN7GIb+9jITrayE2DqOKNaxQkw';
			B[i++]='QKuVTe1qW/va2I51BxzQa2fz+AIOWEAEsk3ucpv73Lfe9i0ejYhIA1vY6I63vOeN7gg4QAfdXkSk';
			B[i++]='+ZCDaPeA3gAPuMCNHQEIEAPM++6DF8Y98IY7/OGhloDBTc2JWLz70BDPuMYbLgELHLzdneCDHwKA';
			B[i++]='5o2b/OTzDoHH9RxylIAg2mBAucxnnm0NgOAMzz5HAkYQc5r7/OfINgAI8J1qB3yA4UBPutJvbYAa';
			B[i++]='e1jcS4+61GU9ghsgAQLwnrrWt+4T6Q/cINga4LrYx/6BaI/97CYPAgA7';
			this.images['bubbles.safe'] = { name: 'bubbleSafe', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/AL+/v/v6+vT09Ndxceq5uerq6riq0dnZ2d6NjeXm58DAwNFQUPno6MkpKeED';
			B[i++]='A/PW1vDw8Ms4OI96ttYDA9zc3NLU1ezs7cQEBM0DA/DNzeHh4e7ExMgXF/z19Z2Kv8QKCuGXl+jn';
			B[i++]='6ualpZmcn1AujaSlpbO1t8zNz9PL4qyvsd/g4Xxiqp+ipb/Bw6aoq8i9214+lsjIyfTy+N7Y6nFV';
			B[i++]='oqeWxt/f378DA9XV1dIDA9sDA8YDA+UDA////8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ/w';
			B[i++]='9xhELrukcslsOp/QqHRKrVqv2Kx2e2sgOsMh4pObTHTotHrNbrvf8Lh8Tq/b7/i83ozhZIYdCxgT';
			B[i++]='Dg48h4iJiouMjY6PkJGSk5SVlpeYmTyFOhMXIEIDGDqGmqanqKmqq6yYDp4bDxeErbW2t7i5rQ45';
			B[i++]='EQM5hY0OnTnFxsfIycrLzM3Oz9DR0tPU1dbJZ6WMEzsNtIuvGDsXN+Xm5+jp6uvs7e7v8PHy8/T1';
			B[i++]='9ulJOTqNOjsfpIt05OiGIEOAHggTKlzIsKHDhxAjSpxIsaLFixgzLiQw4MOOb4n67dinSAeGCwM6';
			B[i++]='JKCAo6XLlzBjypxJs6bNmzhz6tzJs6fP/5gqCjyIsANYySQkETnA8IFAgQMHKNiYSrWq1atYs2rd';
			B[i++]='yrWr169gw4odS/YqhQMaAgz4eHSkIm4ICuCgoCFBgbt48+rdy7ev37+AAwseTLiw4cOI9yZQETVA';
			B[i++]='hFEhkSZa2qDHXLsQBGjezLmz58+gQ4seTbq06dOoU6te7RlCCA1oM8yK7BaRjgsIEqC1IOCgxt/A';
			B[i++]='gwsfTnz40xgKYhwogFCAaxs4etzIQTspD08PKFAIAcF3RAMkwpNAwVCCeBIIT7AYwb69e/YxEAYw';
			B[i++]='8b4++xQJWrSw4KK4//8AykeBAgAUWKACFBwkQAHQSUedbZIhksMNAeBgQwECTITCeeMtVP8Dhz2o';
			B[i++]='YN+IJ/QAwIjvsZDhCQmkYEKAMMaoEQUAIGSgjRT0sGCD01WXyISWXZhhRCHAwCF5Cb3AIXr0sXBA';
			B[i++]='AlBGGaUKLozQ331SZgnlkAkIoEICMoYpJkQFEGhjgTYqgCGD0fUIYW2HAGkhhhHJQMOSSPYwg5Eg';
			B[i++]='0vciRCew1wN7JY5p6KEYHXCjiWcCcMCObT54iEjWySlkRObhiZAMfPY5wp8PBTrCoCMUiuipqDoU';
			B[i++]='w6IJGRgDpA76KOENQdL5kAdLhofknbn24GdEopJqaqrEpkogqwYCoACsbk4aYZy0zjlkQ+CJ12mH';
			B[i++]='mYZ3rYkjsHDCt+CGq94IKZAKgLjoWlD/7LoAroqmQq4yKykPlP4Y7aUNzcBhtbrySwIM/vZwAIr1';
			B[i++]='VUAqwe21wO7Cwyn6bo0mFvgom7G+Wem9ti7E6Xke9HDeh+e9sKF4PYiKMHsKn+wemAy3nFGZEC+q';
			B[i++]='7Jo8zlvvrLVOm5Cd50mAUK7hGdDDyOH5aqV3DM3XrbkQWUCoy1BfRGOjOOpIcbP0PsuDpRknlC0J';
			B[i++]='NMjwc64+D32e0aA6FOzTELEd9dsRBTBgssomaHXNskKbs4fnwRBCQrnSkBDRTH4KrKBuO5Q43Iw3';
			B[i++]='dFxyyyUkb95bY6yzkufl6TGHMExLuNHlQhQD4qVGtHjjqEs0ucX27o3QnucJrRDnMyj0/3kL95mg';
			B[i++]='++68s/fieizwLrwJKbBncOrIq361zVpzPeTG4pU9e+wLfS7AeiqDWYHK5CKd/PcLre4snJW7XsMK';
			B[i++]='6K8gvULpryC7QjO031wLwwsPAMsh1i98DN6D7//dkaKc8/5HwAICsGLju5jrDmACF7ggBepCCH3a';
			B[i++]='kxALTJAF8UmPqViUEPeYQAUJAZXJBIWQGCisBwFIAQgNCD7xZY18A/QVy6ASwoac4AAJCV3JNog/';
			B[i++]='UKnAVGnrQQKC2AP9pBCHLGzh8gRoOYRsb3cr9NXuMlgy/CnkWwnhIEJcAC4XHM9XCxliQ1rgpCT6';
			B[i++]='z4U305u0SpiQH9aQISeIIhWxWEL84f8HSi0AYhiJKEEznnGJrMPZGkvmogZGUXhtdIHuUqDD6+3O';
			B[i++]='BdMCVQL0qBAxNoSPfmQcGpvXxEx60mWbhGEnP0nKdYVSgRYqgSpXycpWuvKVsIylLGdJy1ra8pa4';
			B[i++]='zKUud1lLvAVSjTbwgTCHScxiGvOYyEymMpfJzGY685nQjKY0p9lMXyawdRaipja3yc1uevOb4Hym';
			B[i++]='NV+IymCG85zoTKc61znNcaaxfNlkpzznSc96btOdnAySPffJz37yE5+i1Kc/B0rQgnYToOU0qEIX';
			B[i++]='ytBkIhSb5myoRCdK0IcKMqIUzahG5WlRYG70oyA9Z0fhidGQmvSk0ByppVDK0pYuU6X/93KpTGcq';
			B[i++]='TJgKlKY4PalN45nTnn50pyX1qVAbCtShGlWiRT2qUg2a1KU6tZ9NfapU6RnVqVpVnVW9qlbBmdWt';
			B[i++]='evWeAfwlSb9K1oOG9ZoXLataqdnVtbrVoWclJ0TfSldmtrWueL0rXumq1726ta9+VStgA0vWwRLW';
			B[i++]='q4Y9rFYTq1irMraxUn0sZJ0q2ckqtbKWNSpmMyvUzXK2p0CtwGfLulMVaGC0Xz0BUJ+C2q2qYKfb';
			B[i++]='gYBoWzvVoIyUGyCAjQZ4c1raLvUEFnhOXN9JmQpRADMWOMAJfCtU1ULANbqVzQQod50dgEAudLFL';
			B[i++]='YrbL3e5697vgDS9fFtOYx1jnnZtg/4pToCKVsrj3vfCNr3znS1+snCUtA5gNdel1kpSs5CcADrCA';
			B[i++]='B0zgAht4JhcaSlG0gdaSDMQLGSilhE+1gY4MgsENVkQ4xnGPDnv4wyAOsYhHvA4klIEf4wAJOIhx';
			B[i++]='jRa7+MUwjrGMZ6yMbDiiH78Ihi52zOMe+xgX2JkFhn9M5CIb+chK6cUPRAEQJDv5yVDehScI8INA';
			B[i++]='XHjIUc6ylreslGF4AhRiIMMZ9EAKLJuCE2ROs5rXzOY204EPDdhAGIRQhCNsAQNl0PGZvYwBcWzh';
			B[i++]='z4AOtKAHrYUufGHOiE60ohEtggY02RXDEMgOIiCCACz60pjOtKY3zelOe/rToA71p/QZwAHrUIIT';
			B[i++]='E+jDAB4g6la7+tWwjrWsZ70AU0MC1SdZAAEsPete+/rXwA62pgPgaDNPxss5wEAEEMAAYTv72dCO';
			B[i++]='Nqg7EIFHC2MYyebAqqXN7W57W9oMiAAwzIzqHFxA17z+trrXze5WM6AB4wZHpB/M7Hbb+974vvS7';
			B[i++]='493lV2R7AH/It8AHfu8HcIDfhXhFqrVNZYI7/OHezgAHCGEIXH9A12CAuMY3LuwNTDzhnci1CDLO';
			B[i++]='8ZKbPNYEIAMaJjCQCICg2SePucxDTYAL4LnPXoD5zHfO80yL4AZIaMC2e070oiMaBDfQtpyNznSm';
			B[i++]='i0DXTY+6w4MAADs=';
			this.images['bubbles.unsafe'] = { name: 'bubbleUnsafe', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/AOzRsbOkznleqO2DBvTl1L+/v9WKMfr6+d6rcPT09OV+BvGFBurq6tmXStJz';
			B[i++]='BdnZ2eXm58DAwPDw8N16BZJ9uOS9j9zb39LU1ezs7eHh4efDmtN9Gfrz6s50Dfz59dR2CdeCH+jn';
			B[i++]='6uGye/Hbw/nv5NR5D5mcn1AujaSlpbO1t8zNz9PL4qyvsd/g4Z+ipb/Bw6aoq8i9214+lsjIyfTy';
			B[i++]='+Nl3Bd/f381xBdXV1eF8BtR1BeqBBvWHBv///8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ/w';
			B[i++]='RxAZPg6dcslsOp/QqHRKrVqv2Kx2q3VsGhXPcFgpTRS7tHrNbrvf8Lh8Tq/b7/i8fr9TKGoGI0MH';
			B[i++]='DTUKAws8iouMjY6PkJGSk5SVlpeYmZqbnDwLC30fGkIINTuJnamqq6ytrq+bCwofIwQfp7C5uru8';
			B[i++]='vbALOQYIEwORA305ycrLzM3Oz9DR0tPU1dbX2NnazWiojzsOBgrejLITOg436uvs7e7v8PHy8/T1';
			B[i++]='9vf4+fr77g41OTsggbtFTtGOHDoMVCBwoIfDhxAjSpxIsaLFixgzatzIsaPHjxFHIOhgqCCPHegC';
			B[i++]='NtpRwwECDxAe4JhJs6bNmzhz6tzJs6fP/59AgwodSvSmDQwEDOjIURClDpWLBkzoAADDgwcWbGjd';
			B[i++]='yrWr169gw4odS7as2bNo06pd69UCDgg9EOhQsFIJVEUKdFRggMNCBggMAgseTLiw4cOIEytezLix';
			B[i++]='48eQI0suDKHFVQ+F7jq9K9VAj76AJSQYTbq06dOoU6tezbq169ewY8ueTfu0hBAZHmQgcIPuos2M';
			B[i++]='UFaImQFDgoYgkytfzry58+YMHsyIMOMBgx4HDty2gaPHjRzB7TJS4ICABQsMJCC/GOCE+xMrJFJ4';
			B[i++]='f8KhChcm8uvfn3+GwwMp8CdgfixA8MILGMDw3IIMNvifBREUIKGEEVjQUAIMcOcdeL+Jt/9IDjcc';
			B[i++]='gIMNDCSQ0Qr0wRdRe/T10MKAMKrQQwEw8ueCiSpAwEIKDvboI0gWFODQhENa0AOGGn4X3lOMgPgZ';
			B[i++]='iSZeFIIMKcb3UAwp1hegCw9A4OWXX7YAgwkKEgjmmV5GCUECLcD145twVsRAhENKOGQEJWbYnZId';
			B[i++]='MvnhDU+WeBENAmRpZQ8WUJliDwHyaJEK+fWQn4xxVmqpRw8QOWOdBTyA5J4cGuShIk6OKKhF8xnq';
			B[i++]='EA2KLtroRZCaIKkJlF5q660UzaDpQxPO8OmGS95VKpSoZumelYUay6gJjlYU66y14iotrhHuOmEB';
			B[i++]='EfzKp6h+kgqoqVFOxKJ7raqYKrkt0uj/ggrstuvufSawMGsB79aLwbT4NqirnRD1qm2oJ43Kw7Cn';
			B[i++]='StRCiuPCl7AMCffwQI0CXjArxPq9kO/FzWXKr5AzSuipnsD2Key3xErEKn0U9EBfwifEgOJ7PcRK';
			B[i++]='cX4Wz7yfmxjn7NGcHGuKbZ5JAgzcn4GG+5AEybqXssrGnhBADy+7tywM60kEoAkuzGsRBpPq7HVH';
			B[i++]='QXJa5JEgbxtwtwOTXPBD554gwL1MZ7l01FoyC2ukXVuU99d8X3QAhNdiayHZQQfbpNpG97BwCA8Z';
			B[i++]='K8BDdC/bLEXP7k2R5X1nPlF001V3nUP/Gk40uBBhSd+hcb8nAw2QtxigvBbNgDetF2Gu//ntGIUu';
			B[i++]='8uFFP5ToyhGlKIORrcP8AoEpJK/88vnxiJ8Ly0efAgv5SYz79bmXLbTABEd58ntLQ4RwRJEngJ/N';
			B[i++]='cF1gc7xVY+9+RLpzO3LvigtgvwDhQ3S/AE9HZMH+oHuB9KJXAJy1YIDRm0H73sdAwoFKdN6iXwMn';
			B[i++]='OMH4nW1+pHNYCmAAAxbAbVn6eQgGAoQ1/9inVjl6yH5S0IKHNEtmkXLIDCyGHRa0kIIMtODQIpjB';
			B[i++]='FLjpKi6ciAoe8BDYxQyFOGtWC2o1uR5AoIk9ONABWEBEHOZQexBMG/3Up7wbMkp5JowZziDCroek';
			B[i++]='0CEwaBcMrMeoiDxxIi/gkhUbqEPuIf9Ohg9ZYhAlogIvhrGMMsRZgbz0Aia6EYoOQeQcb1dHtHXP';
			B[i++]='PjvaoBejl0cYJI8FRjSf8mAQrmZBwJAQeeNEFLnIzDUSgyUrpSq9dkreZXCVsLxYK0dnAxTY8pa4';
			B[i++]='zKUud8nLXvryl8AMpjCHScxiGvOYwSzc7mjpg2Y685nQjKY0p0nNalrzmtjMpja3yc1uehObypSf';
			B[i++]='K23wzXKa85zoTKc616nNcF5wnOyMpzznSc96ftOdO9TiiOzJz37685/nxKcdnwTQghr0oAYVqCNJ';
			B[i++]='htCGOvSh6VQoKiFK0YpalJoShedFN8rRh2aUmR0NqUj9+VEeknOkKE0pO0uqz5Oq9KX/MO0mS4cV';
			B[i++]='05ra1JozZehNd8pTH+SUoD0Nakx/uk+hGjWlRHXpUZfK0aQy9akbdSpUpwpRqVL1qge1Kla3StIH';
			B[i++]='LtOkXA1rP7Uq1rKu1KviBKlZ16pOsrL1rd50K1znmk250vWuGEXrO9WK177mNWRpBatfBytNuxK2';
			B[i++]='r4Y97F0Tq9i5Mraxb30sZNcq2cmWtbKWDStmM7vVzXL2qp797FRDK9qnkra0Sz0tao2a1AusNrJ6';
			B[i++]='zWepWpCB165VBUmNjm3L2gKiWiAEEnDtbrfaAix+lQd5GY5ujFPb4U5VBRjYTmwF1pnPWCA0VlGB';
			B[i++]='c4+KWwncJjc2IIADfBNYvOiFL34B/8xk1sve9rr3vfCNr2Eqg5UeZCaLPJAKVaKDFbb4978ADrCA';
			B[i++]='B0xgsFhANweQC3nLa5CWIOAAMSmKhCdM4Qpb+MIYzgmJkvIBptQFbb/JQTgWEssS42oEIiDJOByB';
			B[i++]='EgfcpRwKOEc6+EHjGtv4xjjOsY7fkQSAmCRg4/2xIo6hgG0Y+chITrKSl8zkZ3QjEuAYhpB9QeUq';
			B[i++]='W/nKrgCHLXCB5S57+ctgfgQwDPCDUnA5zGhOs5pfIQta/IAQhkDEmudM5zpH4hOhGAUZzICGPQxA';
			B[i++]='zmz+Mx8GTehCG/rQdsgBIAQxBiIgwAAOSIIWajCBU0wZE0SeQA24wOlOe/rToN6CA/9KAAYxNPrU';
			B[i++]='qE51owEAgh0UgxOfGECRddAADfRA1bjOta53zete+/rXwA62sIHNgVbHgsg1AAECCDDsZjv72dCO';
			B[i++]='trSn3QBAVwLPMf5AAwBw62l7+9vgDre4d90DA5z5zguQtaIVwoFxu/vd8I53sDGzYkmkuw812IAI';
			B[i++]='mC3vfvv73/HmgAGIIeR7Z3vb3Qa4whfO8GaTwAA5sPYiYl1kQFSg3Q3PuMY3rmoSgCDi5MDzQfK9';
			B[i++]='b46b/OQbJ8DHAR3rkW8AAYxGucxn7m+VHyIRBp8AqQFgapr7/OfiHsEGbp5pbQMA40BPutKjPYIS';
			B[i++]='AKQP56g10pdO9aoDGwAdmEAONK0oEBJY/etg5zUAbpAEEJQ87GhP+6k1cIMOwFztcI+7BhAAgLjb';
			B[i++]='XeZBAAA7';
			this.images['bubbles.unknown'] = { name: 'bubbleUnknown', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/AL+/v/n6+QHBAQDUAEnOSfT09LjquOrq6gDdALiq0WPSY9nZ2eXm58DAwI3d';
			B[i++]='jTbJNuj46Nbz1inIKfDw8Ae/B5J9uBbDFgDJANzc3NLU1cbuxuzs7XHWceHh4fX89QDgAJbglg3G';
			B[i++]='Dc7wzujn6qTlpJmcn1AujaSlpbO1t8zNz9PL4qyvsd/g4XleqKeWxqaoq7/Bw8i9214+lsjIyfTy';
			B[i++]='+N7Y6t/f3wC9ANXV1QDEAADPAADYAADiAP///8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ/w';
			B[i++]='F1FYBLmkcslsOp/QqHRKrVqv2Kx2ayE4PMOhg6IbDHbotHrNbrvf8Lh8Tq/b7/i8fqC7SDRDAQQX';
			B[i++]='AwgfHzyJiouMjY6PkJGSk5SVlpeYmZqahwgIOjkgQhwXO4ebqKmqq6ytrq0fCAMCGhE5A4ivuru8';
			B[i++]='vb6+sToPHDqGjwg7fDrLzM3Oz9DR0tPU1dbX2Nna29zOZwiPH7MSuI6yFzkCN+vs7e7v8PHy8/T1';
			B[i++]='9vf4+fr7/O8CF2VyMdqRg4KpRjtASXAgoofDhxAjSpxIsaLFixgzatzIsaPHjxINcKBACNzAJDsa';
			B[i++]='DbgggIMHBhhwyJxJs6bNmzhz6tzJs6fP/59AgwodatPGgQgPchQ7mSPlok8UDBxYsACDjatYs2rd';
			B[i++]='yrWr169gw4odS7as2bNot2JY0KGHgltMnSoakMPBARwYOjA4wLev37+AAwseTLiw4cOIEytezLgx';
			B[i++]='YAYsqvZ4UGoRwaZP/fTAu3dCgc+gQ4seTbq06dOoU6tezbq169ewR08Y0YGtCAEDLKNcRNcBA7Yb';
			B[i++]='CgQASby48ePIkyefOqPBjAUHegQIMNsGjh43dOjGPPdGBAwYRkwYjjGBifMmVEisgN6EwxQnSsif';
			B[i++]='T1/+DIcBUNTfL38FAxgwbPCCcgQWaCB+GDQAwIILNoDBcAUcYB122ilymVw86HBDADgYVf9ARiq0';
			B[i++]='l15ELojYAwv8pZhCDwCkWN8JH6bAwAooHGjjjSBhAIBDDPKIQQ8RTpjddhhquJmHGI0gg4jqPRSD';
			B[i++]='iO7pd8ICDFRppZUsvFDCgP1d6WWVH/bAQAEsMIDjmWhadICCPC7IYwMHBHndkBbupoiRHcZ50QQt';
			B[i++]='QNlkDzUsaaJ+NVqUgnw9yLdimow26tECPbLYJgALyEkhkYvgiaRF7PnpEA2CDlpCoRUdWkKiJSzq';
			B[i++]='6KqsUjRDpA8xOIOldCZyYaY3HKlnRZ162kOfUEY56kWmoqpqq8i2qiCsDALQAK0V2mpnIpruOpF5';
			B[i++]='6IU6Yq8maMtiCSekIO645MJXwgqoAlD/7robJOuuga+6CZGs0GJ6Z655hilRDSJie54K/nYbcA8L';
			B[i++]='uLhfBqgaPB8M7zaMHKTy7sjigpVKOGe0PNx6r676QgRqey70ALKIMYSIXg+mKiwfwyrTZ6bDMH+0';
			B[i++]='psSROhunxZfWyR21+G4KEZ/tteBQsOcl0IPJ5/Wg3wvkTZQfuOlatIGiMVfdkY6T+ggkzrVmPG2G';
			B[i++]='PVv7ELct0DB0sBU4hLSwpFJULNUWwW313BcFkGCzzj64tZAYa8wzxyS2J8MIDwUrtNrtKT2soYjK';
			B[i++]='TZHjdEc+EXPOQfdQvToXGXbHT7b3p8giymA24ifrh65FMzSe6kWQS+46RphLuzPYgDsU/2h7RkMU';
			B[i++]='eg0Qrd0DDP2hIPzwxMtXY3wnEK88CivIh/Dr0MPOdd9fVxvmx+ilHVG/EfleQHwtm5lBy+c2Hf35';
			B[i++]='EsXu9ezWO+RCC/C3oH1E8beQO0Q11O9QATAsrzwALzuR/5Q3A/Oh74D7m569/pYvBDrwgXu72AJp';
			B[i++]='10CCoeAFL1hBuxyin/k8ZAMdPMF93qMqGT2EPihgwUNIlTJEOWQGDJPOClQIQfSpz28U3BQKXkaV';
			B[i++]='FU4kBQt4yOlQVsIAkooFqmqbmJT4OxgEYAVBrKENFZg5XNVufMOjodKGN0KUBRAi4nqICR3ygnG9';
			B[i++]='4HlKiwgDmPi7KUnxgDes3uZe+BAk+v9QIinQYhfD+MIA+qdKMEiiGtmYxjdOkW8TbB/KaHRBLSqv';
			B[i++]='ji8Q3gqGOIHkCe8F+iIVAwQJkTVShJCGpFsc2TfHUJoyZqPUXO1OyUp3pdKKHTqBLGdJy1ra8pa4';
			B[i++]='zKUud8nLXvryl8AMpjCH2UtEVnFjHfKBMpfJzGY685nQjKY0p0nNalrzmtjMpja3WU1jyk6VyeSm';
			B[i++]='OMdJznKa85zotKY31wdOG6TznfCMpzznyc114lBT9MynPvfJT3LaU45H6qdAB0rQgf6TlAEtqEIX';
			B[i++]='ytByHrSdDY2oRCcazYfC0p0UzahGGWpRZGJ0oyANqT47ysCPivSkKD0nSXOY0pa6tJ7/EjxmSV9K';
			B[i++]='05pOc6X4tKlOd+oDnPaMp0ClqU8TGtSinnSo4TSqUjeKVJMu9akRbSpUpxrVmH7zolTNqkKlqtWu';
			B[i++]='9pOrXg0rPcEq1rK+k6xmTatDrcpOrKr1rf5k6z1/Cte6bhOtds3rTeUK0KTq9a8V5StC/QrYwjIT';
			B[i++]='r4YtLGIT+9fFMjavjn1sXSMr2bdStrJpvSxmy6rZzYa1s57tKmhDm9XRknaqpj3tU1OrWqU2NQOt';
			B[i++]='taxgwcmCDsQ2rSlo6lRuW1YWIDU8E4Atb7vKAipedS6hqE0HgmPb4U41BRuozmwzI4HNYKAzG1hA';
			B[i++]='Cpxr1NxOYDa1YcFtciPTRNAFBHfJ/8teHMPe9rr3vfCNr3wDAxnJEGB2OOQBAi4QlalUJS0ADrCA';
			B[i++]='B0zgAhu4K2tpy1vKUV7zssQlMCGKhCdM4Qpb+MIYxolRkKIUkzTYVgphSCtHvCqRkKQQCEGJQJ6y';
			B[i++]='knT048UwjrGMZ0zjGsfjH2V4BEFuwGBzJKMbQA6ykIdM5CIbORrfgARBiGGMXzj5yVCOMpTFIQBb';
			B[i++]='9FjKWM6ylrcMiWA84AekMMWKuUzmMpv5FbGYhQF+4IFBFOIUZ46znOcsiVh8IhRhGEMZzqCHPvv5';
			B[i++]='z4AO9J89MeZUdAIPZvADIMJABCMgYQuQjrSkJ03pSF+AEDswRKExkeY+XEEAIfACGP8YTepSm/rU';
			B[i++]='qE61qlfN6la7+tWwLnUASECZMoh5E2mWAAlizete+/rXwA62sFkdAQ5YACCZhrMlYhECCAz72dCO';
			B[i++]='trSnzeseGOC+mNZ0JcRBAGp7+9vgDrewIeCAWn9j04z4wA4eMGpxu/vd8I73D0RgbGRrOxyyYLe8';
			B[i++]='983vfgs7ANdWyrk37WVn+/vgCE94qsktAYCc4RBjDoYEDK7wils84fQ+tg7QcG8eBMMCFL+4yEce';
			B[i++]='bwMYweGETsTHI0Dylrs83B64dgjsrWlZWEAEL8+5zqPtARJg+xue0EEIFr3zohu91xAAQVJsnZAQ';
			B[i++]='rPnoUI96qxl+6T5ERepYz/qpi20ZARfvWutgD/sPNGDsG4hC7GjPusnPruogAAA7';
			this.images['bubbles.hrev.safe'] = { name: 'safeHRevBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/AL+/v9Zubvv6+vT09Oq5uerq6tBLS7iq0dnZ2eXm58DAwN+Ojvno6OEDA/PW';
			B[i++]='1vDw8I96tss4ONYDA8orK9zc3NLU1ezs7c0DA/DNzcMDA8cWFuHh4e7ExPz19Z2Kv8MKCuGXl+jn';
			B[i++]='6ualpZmcn1AujaSlpbO1t8zNz9PL4qyvsd/g4Xxiqp+ipb/Bw6aoq8i9214+lsjIyfTy+N7Y6nFV';
			B[i++]='oqeWxt/f378DA9XV1dIDA9sDA8YDA+UDA////8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ/w';
			B[i++]='5whoMrukcslsOp/QqHRKrVqv2Kx2qzEsOsPh4nORSHTotHrNbrvf8Lh8Tq/b7/i83nyZcIYCBmUN';
			B[i++]='DTyGh4iJiouMjY6PkJGSk5SVlpeYPIQ6Eh8gQgEXOoWZpaanqKmqq5cNEjscDjsSpKy2t7i5uqoN';
			B[i++]='ORGhtYoNnDnFxsfIycrLzM3Oz9DR0tPU1dbJZ8GJEhkTtIuuFzsZN+Xm5+jp6uvs7e7v8PHy8/T1';
			B[i++]='9ulJOTqMOjsf+4p05NgxYQEGAT0SKlzIsKHDhxAjSpxIsaLFixgzamRIIMCHWdp49NsBEJEOcQE6';
			B[i++]='JKCAo6XLlzBjypxJs6bNmzhz6tzJs6fP/5gqLDiIsCOHtpElDTW48IFAAQQIKNiYSrWq1atYs2rd';
			B[i++]='yrWr169gw4odS/YqBQQbBASYlQjpth0LCuCgsCFBgbt48+rdy7ev37+AAwseTLiw4cOI9yZQAbVH';
			B[i++]='BFEmkyRdOqHHXLsPBmjezLmz58+gQ4seTbq06dOoU6te7flBiA1oMWSQEJlk5AUJ0FoYgHCj79/A';
			B[i++]='gwsfLvxpDAUxEBRIOMC1DRw9buSonZSbAwoUQjzoLfEAie8kUDSEAJ5EwhMsRqhfz159jIQCTLSf';
			B[i++]='rz5FghYtLLggzr+/f/gUKADAgAMqQAFCAxTwXHTTHeLWITncIAAONhQwAEUolBceQzVo2P+DCvSF';
			B[i++]='eEIPAITYHgsXnpBACib85+KLG1EAQEIE0khBDwkuKB11iERoWYUXShQCDBqKp9ALGponHwsIJODk';
			B[i++]='k0+q4MII+9UH5ZVOBpnAACokAOOXYEZUgIA0DkijAhYqCN2ODkrW4w0/WiiRDDQkaWQPMxDpoXwt';
			B[i++]='RnSCej2oN2KYhBaaEQI1klgmAAjkuGaDhjxoiI8UyhkReXYmJIOee47QJ0R/jhDoCIMaauqpD8WQ';
			B[i++]='qEIExuAogzxCCGelQT7kQZLfGVknrj3wKVGoo5aK6rCoCrgqgQAo8Cqbkbopa5y1NuQdeJxuiOl3';
			B[i++]='1ZI4AgsndOvtt+iNkMKoAIBrrgXEpuv/n6pmLtTqspCK5OykswL50AwaTpurviTAwG8PCJg4XwWj';
			B[i++]='CrxeC+omLByi7c5I4oCNqglrm7Y9S6tDm5bnQQ/ldVjeCxmC10OoBquHcMnseanwyhqN6XCiyaap';
			B[i++]='Y7yS8kCpvQzRWR4ECeH63QE9hPxdr1Ry11B825IbkQWCsuw0RjIuaiOOEjMrb8X0QjteeTTI0DOu';
			B[i++]='PAddHtGfPgRs0xGh/fTaEgkQILLJHkj1zLFmffFCHmMbgkK40qCQ0Ep6+iugaj9UONuIO2Qccsop';
			B[i++]='BG/dNtdraUJIlncnxxrCUCvgRI8bUQyEkyrR4YmXPtHjFCd18+R5lgf0QpnPsBDnLdRn/8LtuOeu';
			B[i++]='XovpsZD77yakoB7Bphd/etU0zxu51j1kDF7YsLvOEOcDpIeylxWgLK7RxnfPEOrNYr383TWsYP4K';
			B[i++]='0C90/gqvLzTD+sy1APzvAKj84fy/x8C99/zP/Sjkq4tW/wbIP/BdTXWSCxICTOACF6QAXQmRz3oU';
			B[i++]='YgEJsuA95ymVihTCHhOoQCGfIhmgEhIDhPVAACn4IAELiDwAJjCCKmtMBB1yAgQoxHMj06D9PqWC';
			B[i++]='UpWtBwn4YQ/wg0IbrpCFdEvdm5iXPdypsFe4w+DI7LeQbilkgwlxgbdcQLxeMSSIDmkBk47YPwPW';
			B[i++]='LIAkVEgPQUjDJ0rRiiS0n32c1AIffv9RiDMkIxL/p0SL4ewELGLgE3+nRhfcLgU4rB7uXFCrTyXA';
			B[i++]='jgsBo0PwqMfEmVF5aKykJp92SfFlcpOgTFgnEfijEpjylKhMpSpXycpWuvKVsIylLGdJy1ra8pax';
			B[i++]='TGL4SEkhH/jyl8AMpjCHScxiGvOYyEymMpfJzGY685nJ1OUBl9hLaFrzmtjMpja3yU1lSvOM9eqm';
			B[i++]='OMdJznKaE5rfxGQ4z8nOdrrznddMpyfXCc962vOe8JQnL22Az37685/a1Cc1+QnQghr0oMQUqB8R';
			B[i++]='ytCGAlShdiOoQydK0XZCdHwSrahGN7rNi96MoyANKTr5uMuBivSkKD2mR+mZ0pa2dKX/P3KpTFMK';
			B[i++]='02rO9KYcrWlGccpTh+q0p0D1KUmnudCgGtWgPz2qUv2Z1KU6tZ5NfapU2RnVqVp1nFW9qlYDOlRw';
			B[i++]='xnSrYOXqxEpa1LCadaRjJWpEz8pWZ2a1rXBNaFfV+dW42lWuafWqTe/K11++ta9x/Stg2yrYwZ61';
			B[i++]='sIYNK2ITu9XFMvaqjn3sVCMr2adStrJLvSxmj6rZzQa1s57tqU4rEFrFznWellHBBkq71RPo9Cms';
			B[i++]='vaoKapqdB5A2tk9VQQv7aIhXgAA2G9jNanF71BNYwDmnnUwfJkQBzFgAAScgLk9d+wDXAFc2tOFt';
			B[i++]='b3cAArnQxS6JCa94x0ve8pr3vHxZ/0xUAiG+mmmCKU6BilTKQt/62ve++M2vfrFylrSsJbvajdQF';
			B[i++]='MhAAAeTmJwhOsIIXzOAGO3gmFRpKUY6iPAcNpCAYCKWGUcUBjwwiIBU+RDjGcY8Sm/jEKE6xile8';
			B[i++]='DiTkAMBtGcc3GDEMCVzjxjjOsY53zOMeLyMbjehHAIwSkl0Y+chITvIqrAMSJTv5yVCOMo198YNQ';
			B[i++]='jELKWM6ylm3hih0Q4AeBGESRt0zmMptZKTXmbhjG8GI9uPnNcI6znOVMCFZs4g58mAAGwiCEIhxh';
			B[i++]='C4AOtKAHTWhBX6AMoxgzJTZxaCx8wAtg4LOkJ03pSlv60pjOtKY3zelOe3rSAhABUfteTAhFQ8IV';
			B[i++]='ExDBp1fN6la7+tWwjrWm/XwBfdS5EoTQAANkzete+/rXwF51DwhggB0g+taRGIYBgs3sZjv72bFm';
			B[i++]='wAJGjQZkN2IYE4g0tLfN7W57+wcYMMKxTa0JCURA299Ot7rX7WoBENvY2RgzIXyxa3bb+974vrS0';
			B[i++]='J1AUWlgbzTmYQL3zTfCC4zvcGqh1tYMxb4Eb/OEQV3eHE66PRANcAw6IuMY3/uwOEJsMti4EISSg';
			B[i++]='gT1z/OQo73UHRFDscbtCA39Iucxn3moGgIAoiOZEU2jO855zet9FyQF8fU70ole6CPwmh6qNzvSm';
			B[i++]='/2DiN/iE06de9I5IHdNBAAA7';
			this.images['bubbles.hrev.unsafe'] = { name: 'unsafeHRevBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/AOzRsbOkznleqO2DBvTl1L+/v/r6+dynaNWJMfT09OV+BvGFBurq6tmXSdJz';
			B[i++]='BdnZ2eXm58DAwPDw8Nt5BZJ9uOS9j9zb39LU1ezs7eHh4efDmtR/Gvrz6s50Dfz59dN2CdeCH+jn';
			B[i++]='6t+udc15GPHbw/nv5NR6EZmcn1AujaSlpbO1t8zNz9PL4qyvsd/g4Z+ipb/Bw6aoq8i9214+lsjI';
			B[i++]='yfTy+N/f381xBdXV1eF8BtR1BeqBBvWHBv///8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ/w';
			B[i++]='RziMHA6dcslsOp/QqHRKrVqv2Kx2q3VsGhXPcFj5TBS7tHrNbrvf8Lh8Tq/b7/i8fr9TKCYIJEMG';
			B[i++]='DWcDCzyJiouMjY6PkJGSk5SVlpeYmZqbPAsLfR8aQiITO4icqKmqq6ytrpoLCh8kBB+mr7i5uru8';
			B[i++]='rws5CKQDkAN9OcfIycrLzM3Oz9DR0tPU1dbX2Mtop447DggK3IuxEzoON+jp6uvs7e7v8PHy8/T1';
			B[i++]='9vf4+exJOTuP3ra65dCBoAIBAz0SKlzIsKHDhxAjSpxIsaLFixgzamRIQkSHM+IS7TDnj9GOCQ5E';
			B[i++]='eIDwAIfLlzBjypxJs6bNmzhz6tzJs6fP/58ybWAggEBHjpAjdZRUNGBCBwAYHjywYKOq1atYs2rd';
			B[i++]='yrWr169gw4odS7as2awWcEDocUCHApNKliZSoKMCAxwWMkBgwLev37+AAwseTLiw4cOIEytezLgx';
			B[i++]='YAguphIqtSip3KYIeuDdKyGB58+gQ4seTbq06dOoU6tezbq169eiJYTI8CADgRtvFVmuXJdlBgwJ';
			B[i++]='EG4cTry48ePIjzN4QCMCjQcMehgwINsGjh43cvBWukiBAwIWLDCQIFxiABToUbBoSCE9ioQrXpyY';
			B[i++]='T7/+fBoJDaiwz39+CwgwwIBBDMkVaOCB+VkQQQEMMhiBBQglwIB12GmnW1yL5HCDATjYwP9AAhSx';
			B[i++]='4J56DJ3nXg8u9KfiCj0UoKJ9L4C4AgQtqIDgjThuZEEBCTXYowU9SEhhdtvJpaFmHoIoUQgzjLie';
			B[i++]='QjKM+N5+LzwAwZVYYulCDCcQ6F+WYF6pJAQJuLBWjmimCREDC/bIYI8RfDjhdUReyJ0iR3b4oUQ1';
			B[i++]='CCDlkz1Y0OSIPexnY0QrzNfDfCyq6eijGT3gY4tuFvCAkHRaKBKGeN6A5J4RtfdnQjUMSqihEiV6';
			B[i++]='wqInNArpq7A+RMOkCjVIA6YVFpmhp3oq+ZCoo/bgp5RTnnAoRKqy6mqszMa6IK0NFhABrnVuemci';
			B[i++]='eSYJkYnomUoisCh42+IJL6xg7rnoxnf/QgusFpDuuxg0K++Bs765kK3UasrDbp1+6itDLozIrXoD';
			B[i++]='zzBwDw+8yN8FrCpMHwzzRnycpPby2CKDl86Zq51G8qptQ6W6R0EP7g2MggwipteDqg7PB3HL9Z0p';
			B[i++]='8cwZsWnxpNLKOaS+/GLrMagLSTAseiOTTCwKAfSQMnqFdlleQ/qR225EGDBK89UY7Vjpj0FqXO2+';
			B[i++]='nPrsL3vuCRCv0VIWvXSxxz6UrNURwY313BIZoGC00kLY9c669ttrie7NEIJCxAqg0NpNt+3Q261K';
			B[i++]='JDfdkD+0XHPPRZdQvn2L/TeUTi4k5Qw1HH7ifuxGRIOijzuUeuSsT4Q5x7uOnZCgJTM0/+IMQIqu';
			B[i++]='Mgz+qeD778DPZ6N8LwBvvAotzMdw68y77jXPYfOQLaghp1e057UvhHgC8sG81gUwr/t08+Qz9Lq1';
			B[i++]='HcsegADsC3D9Qu0LkDRDFsR/OQzHG1+AzC7kbzwN4yufAPeWqcxJ72f/GqACBXg+sF3rgLJ7gApi';
			B[i++]='EIMWnK1p9FEIBvZDLvzAx1UzUkh9VOAChRyLZYpKCA0gJp0WlHCBDHyeAaenJBWcSSomXNwDFFK6';
			B[i++]='lYFQZsdygasUBwHF9SBABmjBDmEYQ77Bzm8fA9/vXlio33lwZTJbiLkUEsKExOBcMVheoRhSRIfA';
			B[i++]='oEpMHGADewbBzV1RiDlsyAqoeMUtqv9QZv+5EgyGSEYjJsSPaWTdGqNHQ/jUaIJUNJ5CtuS7FvSQ';
			B[i++]='e7+Lga+OBQE+LqSMDgFkICE3yAcWcpOgxFon07e5UJoyYqOMXYdSwMpWuvKVsIylLGdJy1ra8pa4';
			B[i++]='zKUud8nLXt7SiehTpQ18QMxiGvOYyEymMpfJzGY685nQjKY0p0nNaj4TmA4k5TCtyc1uevOb4Ayn';
			B[i++]='OKGJTTZma5zoTKc618lOa5aTkB5rpzznSc96evOdnoynPffJz37uE5/a9KdAB0pQcAJUmAVNqEIX';
			B[i++]='usyDQpGhEI1oQR2quW1K9KIYnSdF22jRjHr0o+Hc6DlBStKSVlOk+jSpSlfa0AI+saL/LI2pTImJ';
			B[i++]='UiTN9KYqrWmHcMpTkOq0oz0NKkR/KtSiDtWlwXyoUZc6UaRmE6FMjao/iSrVqtqTqlbNajuxqtWu';
			B[i++]='opOrXg3rN8Eq1rKe1KnmTKlZ1zpNsrL1rcx0K1zneky50vWudr3rXPOq17fyta9r/StgyyrYwYa1';
			B[i++]='sIbtKmITm9XFMraqjn1sVCMr2aVStrJFvSxmg6rZzfL0pxfwrFl16oIMiFasK/jpck7rVRfo1AIh';
			B[i++]='kEBoWWtVF8jwpTygSwV8AxzT0papK8BAddAaPcxoxgKcicoKfhvU1EpANrSxAQEckJukKkK3d8nL';
			B[i++]='XhzD3e5697vgDa94AwOZqfSgAQ9k/yMPmvKU5UzlLPCNr3znS9/62ncrFqgNW9wClweKBCUqYQlQ';
			B[i++]='BkzgAhv4wAhOME08RJQPHKW/ctFNDr5hkFNaOFYd+Ug4GjESB0RYEeQwhz5GTOISm/jEKE6xO/jx';
			B[i++]='4U1RNySLKIYCskHjGtv4xjjOsY6bsQ1IeEMEORhGL4ZM5CIbmRfeqMUtjszkJjv5yY74BQJ+cIBS';
			B[i++]='wBjKWM6yllURi1n8YDIKOMSWx0zmMkPCE6AQBRnMgAY+uPnNcI6znPEwADG7YgF1zkMOACGIMRDB';
			B[i++]='CEjggqAHTehCG3rQEziDKa6ciWIkGgsOMAEYxODnSlv60pjOtKY3zelOe/rToA61pf8NAAAETCAH';
			B[i++]='YfYEJ2KBAACI+tWwjrWsZ03rWneaACLYgKLtjIkd5IADtg62sIdN7GK/ugcAaICD0SDkSnziAMaO';
			B[i++]='trSnTe1ac6ACRUH1IRg9jh0ggNLVDre4x01uIuR619xOxAAU0ABwl/vd8I63rJGtbEPwmsPAALa8';
			B[i++]='983vfmv62qbWtqoZgWdglMDfCE84wnGt635sexwDyAEIDq7wilv83SQ4gAlOvYOHdyLiICDAxUdO';
			B[i++]='cmrTe+PMRgSeFbABkZf85TAXtgc00ACUd3zlJuhzzHfOc1hzgOY6ULSvc97zohv90yXAdqL3/AFX';
			B[i++]='H/3pUL80rkGgA3Q4PepYz3pHRnAOAzVn/etRB8ABvK7pIAAAOw==';
			this.images['bubbles.hrev.unknown'] = { name: 'unknownHRevBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/APn6+QDUAAHBAfT09LjquE/PTwDdALiq0erq6uXm54zdjNnZ2cDAwCjGKDbK';
			B[i++]='Nuj46Nbz1ge/B9LU1ZJ9uGXTZRjEGHLWcgDJAMbuxvHy8tzb3+Hh4fX89QDgAOzs7QrFCpbgls7w';
			B[i++]='zszNz6TlpOjn6crLzJmcn1AujaSlpbO1t7+/v9PL4qyvsXleqN/g4aeWxr/Bw6aoq8i9214+lsfI';
			B[i++]='yfTy+N/f3wC9ANXV1QDEAADPAADYAADiAP///8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ9w';
			B[i++]='SCwOQRSCcclsOp/QqHRKrVqv2OwSdKtYMNqweEwum8/V0U2Qa1gg6Lh8Tq+LCbeLTtdWPOyAgYKD';
			B[i++]='ZQQfOjs7AXwOIH+EkJGSkhgfATsdBooXOQUjHJOhoqNlIRU6Bh2ZBgEBFx8FBACktLW2TRCnqTw8';
			B[i++]='q4uvSbfCw6MPDagdvL2+OhdeIcTR0oDGyMq8qpq/DX7T3t9kDw6IydfYmYp8sT3g7e5THA4Bu+bK';
			B[i++]='2TvNz+/7/EXxmPXqqUp3wUG3fgjbFQAYUCA6V51kJZxI7MEHhg0D3sv3hqJHUiMaYCqXMeNDRiNm';
			B[i++]='fVxJBYIFB2xyyJxJsyZNPTvoldx5zxUn/5tAgwodSrSo0aNIkypFKkAAN1BFFHy40CqR1atYs+ZU';
			B[i++]='tbPrtVVaw4odS7as2bNo06pdm/VXBWhCOBSgmoqk17t48+rdy7ev37+AG/YUAEKIhQsjAytezLix';
			B[i++]='48eKMwUQgAFCjgB2IWvezLmz566ZdLix9rm06dOo/3aY3ABzRm17YsueTbu27du4c+vezbu379/A';
			B[i++]='g9e+ZCDjjhwRMF5jxUnAjefQo0ufTr269evYs2vfzr279+/gpwvQE6Dh8Rw7AuLrE6KH+/fw48uf';
			B[i++]='T7++/fv48+vfz7+////yEWBBBHTVc1565rgigAUcJLAADhBGKOGEFFZo4YUYZqjhhhx26P/hhyCG';
			B[i++]='SKENCEDgQA6omHOgOQZcEAEBCCywgAY21GjjjTjmqOOOPPbo449ABinkkEQWaWSOGiywQQ8UXKai';
			B[i++]='TAgqE0AOCiCAgwYbkIDAllx26eWXYIYp5phklmnmmWimqeaabH6ZgA0z9uAAVdesqEyLDfRwZQII';
			B[i++]='ZDDAn4AGKuighBZq6KGIJqrooow26uijkA7qAQkbKBmCAOUpYycvUyrg4AYeDAAAgKSWauqpqKaa';
			B[i++]='aow0MEDDAgi4N0AGJNiAQw836FAnlNfoIAAEM5KQwaj4HXDCsSesIN8EyJ7gnggomCDttNRKS4N7';
			B[i++]='AKRQ7bbSspAADDB4EIOq5JZrLrYaMKD/wrrrMqDBqAMgYCuuumrKqzI63AAADiQOkN8KzSYb3wsB';
			B[i++]='9+ACtwiL0IMKCFeLQgY9iJAACymca/HFAGqggnvscqxBD/HOm+uu6PV6g5794kfCDAEr+54MATur';
			B[i++]='LQoLJGDzzTe7EIMJ43aL8882Q9xDAhm4kADGSCdtHwLqcrwuxwwgEPKtI9tbMr4n8yv1fTW0ELPL';
			B[i++]='PWjAcsHaVmyfCNL2IK3CSrfttn8LdLyw0yosMDW9JEfJQ74ob22f11+7V8PYZJtgdn1om6C2CWy/';
			B[i++]='7fjj9NEg93vs0nB31bxsunfWKddHcMwC9wB4zD2UfV/iizcO+eqQqzs5uyowcHm9md/L/wvfWvtL';
			B[i++]='n7HIEi4ws703u7AJKIhg/PHIQ2sCC4urkPzzHrAuvbmSPw1f5bPnbXLfusunQcC8H7tC+CfMQH4P';
			B[i++]='CzS8rQSLqz8tDNPHj2rc1m+88Lp2y0s17Txojnvn8Rlcs17QgwEGTAYAQ1bE3Ect+DGQWkeTnwT/';
			B[i++]='wzT7yS12UtMf3qymt//5DT5da9YE3AO6Yx2gBwk8Vul4Rqz5ZIt4zbOPB9Y2wRr2R2N08xjINIi5';
			B[i++]='/tluc9xbVrNaUAMSgm6EKBSe6c6WNhrax4k2jOJ9AJAu2MXuXTsUGf/8x7kPuudzx5pBBAsYsxa8';
			B[i++]='J4UyM9zpmsi4+0BRinCkD6tcBav3ZP+Pg9vLHXxg1iywkbFZMyiie9C4QubZhwZsVN183hjHRubn';
			B[i++]='jrW72u262D2xNeuE8AnYDD52RuHBoFspCKUoRymtikULBaNMZQpYIC32OfKV+IGkDyUJRD32QIDI';
			B[i++]='QmImLxkfQmYgWg88mgQeuLwWwvKY8pElF4PYgxe04Jkt0CV8oNkCTMJHA9R0TwZgoMpUqmCMLuhm';
			B[i++]='KmlgTGSaU1Y83OIPPdi9c7rznMpcJyXds4AUxCAGLIiee7Q1rfd4gJ8ouNazGiex91ArBS54z+ES';
			B[i++]='10/30AB+PQAACxL6TnPGk5bs3GcEZaTQ+YhgAe8xZMQIOsbDuaBxh3NPAlLqHnBJFKT/FbVoOrWH';
			B[i++]='NWYOU5QULZ0oBRqxMcLHeO8pqHticLwYuLJ08VnpfGBAs5jCc6Z4rKkteXrSjspHBDnlKVAdOkZv';
			B[i++]='2QwGKE0qS63qVGRetIPzjBjF7JnTVL5HZ6FkgUh/KcoYCA2pKg0rfJQ6n7GWtZFnzSMA/0rYCQZW';
			B[i++]='qoMtrGKld9hJogwFkI2sZCdL2cpa9rKYzaxmN8vZznr2s6AN7Wa1SFPH8ssHqE2talfL2ta69rWw';
			B[i++]='ja1sZ0vb2tr2trjN7WxJG1XT2kC3wA2ucIdL3OIat7a8jSRaUXbc5jr3udCNrm6TO8vlnla62M2u';
			B[i++]='drcbXOou87rcDa94x8td78qTueRN/6961ztc82KUc+yNr3zn+1r3Wve39M2vftdrX8Hu978A3m5/';
			B[i++]='ERvgAhu4uQP27YEXzODp7q+0tcRvgydM4fo+uLcRrrCGN4zaBGeYwyBusIf/F+ISH3jE8DWxiv+L';
			B[i++]='YvSu+MXzbTF4YUxj9cpYwjXOsXhvrOMeh5fHPg4ydoEs5CI7l8hGTjJxkazkJjt4g8r1r5OnnFsm';
			B[i++]='U/nKsbUylrfMWi1z+cs+8DKYtyzmMV+5zGaeMprT3OQ1sznJbn5zkeMs5yDTuc49vjOec6znPdO4';
			B[i++]='z35+MaADreJBE7rEN5bAodt84SgjdgOLTnIJZKwBK0W6yC7w8JRAUKkNIEDRl85zrf8aXd3lXCBP';
			B[i++]='e/KABkJd4xIkgFILcMGlMuVoKeUABFbCEgkSgIMSsNrEH921C+I0J1qX2tQvitGMjsTsZjv72dCO';
			B[i++]='trR3lKQlNck1GObUBRbUoAeJ6NvgDre4x03ucleIRCZCUXEgrCk+cKM9i4334wREoHkYCEqZ4QVz';
			B[i++]='cuCc8Pj73wAPuMAHTnDqjEcHxt7VDbAdENgI5+EQj7jEJ07xig8nJ8bpBGlSw/GOezw1q/nVZfL9';
			B[i++]='8ZKb/OR+CU0DfkABxHAF5TCPucw1wgoBKEEudHn5zHfOc46vgg+FGYJUqHIJthj96EhPutKXzvSm';
			B[i++]='J6IVzoALEVwCk6VY/epYz7rWt84K9a4X5QZPGUIQAAA7';
			this.images['bubbles.vrev.safe'] = { name: 'safeVRevBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/APT09Ndxceq5ubiq0erq6uXm59FRUdnZ2d6NjcDAwMkpKfno6OEDA/PW1tLU';
			B[i++]='1Y96tss5OdYDA/Hy8tzc3M0DA/DNzcQEBOHh4e7ExMgXF/z19ezs7aSSxMQKCuGXl8zNz+jn6eal';
			B[i++]='pf78/MrLzJmcn1AujaSlpbO1t7+/v9PL4qyvsfr5+XleqN/g4Z+ipaaoq7/Bw8i9214+lsfIyfTy';
			B[i++]='+N7Y6t/f378DA9XV1dIDA9sDA8YDA+UDA////8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ9w';
			B[i++]='SCwOQwaBcclsOp/QqHRKrVqv2OzSc8sEMNqweEwum8/V0M2yUwQa6Lh8Tq+LBRZKjkJRIBZ2gYKD';
			B[i++]='hGUYHTk6OhE5OxAegIWSk5SUGBkRDJqLFBYGIRqVoqOkZRWYmjybERQdSSulsbKzTQ0ZOamqm3te';
			B[i++]='SrS/wKMLCrgMPMe6DIwUXhXBz9CBw8XIyJs6jX6R0dzdYgsQ1NXWDJw7SSLe6utTGhCZ4/G7zG/s';
			B[i++]='9vdFKwo6xvHjmsoaQfiDr+A6A7n8ySvHypMAWAYjAluQQYfCi8dW0YMjsSOpEPswitSlSGCIdB5T';
			B[i++]='UmkQAAKbHTBjypw5M1HCkRc1UqDJs6fP/59AgwodSrSoUaM3/IQqggBRhAiKokqdSpVfP5xYr1Xd';
			B[i++]='yrWr169gw4odS7as1KfMnAnRYIBCpqtY48qdS7eu3bt48+ZdZcGDkAAU+OkdTLiw4cOICyuzgKGB';
			B[i++]='BXiJI0ueTLmyXAY5IAQQZ7mz58+g8UZoA9kfwxyoU6tezbq169ewY8ueTbu27du4c7OGCnecjh0d';
			B[i++]='BMtjtcPCjePIkytfzry58+fQo0ufTr269evYl7Ox6e/3DovxsLVBUEFEj/Po06tfz769+/fw48uf';
			B[i++]='T7++/fv41WMI0MFtbx7egVeNDp0EoEEBE+Cg4IIMNujggxBGKOGEFFZo4YUYZqjhhg5OQP9AAxDs';
			B[i++]='gItvMAmYUSsCEHDAARPY4OKLMMYo44w01mjjjTjmqOOOPPbo448yTnDABSsEsEMEJH43zmgIEIDD';
			B[i++]='BBeAQMCUVFZp5ZVYZqnlllx26eWXYIYp5phkXllACyyKAEFgA5ZYDQN9iPBkAQRIAMCdeOap5558';
			B[i++]='9unnn4AGKuighBZq6KGI6imBBCBcMGQFj7WpJDK/IVDAkBsAsEJ+nHbq6aeghhqqijMkMMMBBPQg';
			B[i++]='wgqM2oBDDzfkIKmJEVjQAIsgSLBpfAOU4GsJKaz3wK8lnPeBCyQkq+yyyc5w3gonMCttsioUAAMM';
			B[i++]='G7wg6rbcdvvsBAmgIK64CUywKQAEuAr/q6yUuolMDjesgIMNBAAgXwrEAqseB/n20MK0AH/QAwoA';
			B[i++]='M+uCBD18UIAKJ3jr8MP5TYDCeeNSPEEP6Kob66zVwNvDvPXGB4IM+QaLXgz5FhutCwcU4PLLL7fw';
			B[i++]='AgnaUgvzzS4j3EMBErRQAMRAB/0eAeFSLC7FCdSb7qsbtzvpMR6DbO97NLCQssk91EByv9E2/N4H';
			B[i++]='yfaQrMBCl232fQdUPLDRKByQMdPsHhNgxzd8TO/U7g179bNbc02C1+6BTYLYJJB99uGItzeD2uiN';
			B[i++]='O8Pb63L8bt1Sv8dvyvr2YDXmPXQNn+CEG5746ImHy/i4KCQAedNyuws15Xe71+uvfeur/7evtQ9M';
			B[i++]='ggsf9O7778eSoALhKABv/AakJ9/t4ken5/jqcQPoOg9Rx85eDfnO7msK2pcgQ/c9HFCwtA4QPr6y';
			B[i++]='MCivPqhpNz/xwOK6vXTkTptYfcjr0VA7Bz0Qe/mvMcDXrxJ2vmWlr4DL+tn6Fogfor1PbalTmsai';
			B[i++]='N7fJ2Q1/6akasR5wHsz5agA9EKCvOkezXbEHWrsj3ns2MDYGutA+EmObxTA2P9ZJ72nUgx0G0XO7';
			B[i++]='ErCABh3EHAdDSCwSAq49oGvhe5T4wibCZwXgQl3qzEXDCUrudRfE23n+5z0F9i9lLECPCFX2t8+F';
			B[i++]='jYntQaMT18geUpkKVeiB3hVzmMX0oP+MWFj7IrFkAMTzjJGEw3vPDM5YOPiokY2IjI8c60e3Op5H';
			B[i++]='a8QCYXryJYMapOePMKDWCTbJyU4mq2HIckEnR3kCFSSrfIlMpSJrSMHp3W9qK6jdECcZSfX8UQLI';
			B[i++]='QuDPHIBA4ZlQlcBczyJbh8NXbpEFyGTBLNOTTBZIMj01aOZ5JAADUo4SBV5sgTVHOYNfBvOb5xnm';
			B[i++]='De2nQy2C85zfFGcFsVi58J3gBS9QAfLOEy1loWcD9XSBs4xlOIWhZ1knaAF6ACc4e55nBunrwQpU';
			B[i++]='IFB0ppOVczRm5xS4ooGy5wMHQE8gE9ZPLwKuBYY74s5E2oNrLTSjDn2oFRlpwXbykpP/De0cJ/eZ';
			B[i++]='MC+mp3fo8ed5XuC7F6Cyc+opAElLyrKUglOdriznQdEDUouu5wMxpSlOD+rFarkMBiEN6lCBalSV';
			B[i++]='wi2iSk0Yw94Z01Ey9QWbVMFGccnJF+iMqzvLanqE2p6tdnWNSC1mWO/KVxfmlZyO7Ktg1ffXRs7L';
			B[i++]='BIhNrGIXy9jGOvaxkI2sZCdL2cpa9rKYzexkV0pMwM7LB6ANrWhHS9rSmva0qE2talfL2ta69rWw';
			B[i++]='je1qOTtOw9pAtrjNrW53y9ve+ra1tF0nHT/72+Ia97jITW5sg5tUuyn3udCNrnR1y1y9One62M2u';
			B[i++]='drFbXc/edrvgDa94d9td2473vOhN/+9py9vS76r3vfANL3vZ6d742ve+z53vcOuL3/76l7f6rd5/';
			B[i++]='B0xg2QYYdgVOsIJTe+DrLvjBEG4wcSFM4QRLmL8VzjB+L6zhDveXwx4OMXxBLOISn5fEJk7xdlGs';
			B[i++]='4hZLl8UujnFyYSzjGv+WxjbOMXW/ylL66vjHvcUxkIfMWiET+cjr5XFnzYvkJs9WybVtr5OnjFoj';
			B[i++]='U7nJVr7ykbOs5SFzucs//jKYcyzmMde4zGaOMZrT3OI1sznFbn5zieMs5xDTuc4dvjOeM3xhB+zZ';
			B[i++]='y1AWbvUu8Ocfj0DCHsJBoXPcAv2OxgOOugAB/LxoNYNAv3BSgJwmUIANTKDSKh5BAewa9ahI9fgY';
			B[i++]='j3YSlEBQAByMANQexiir0TQBNbHp1KpAkYpYBKRe+/rXwA62sIdNIyFdQAQBMDWuAVSgAyWIQ9CO';
			B[i++]='trSnTe1qW/tB9AKRiHojXEplgzyDDXfi9tOf0iw7I8QxTnbWze52u/vd8I43c7aDpO4Ux9z/WIRu';
			B[i++]='9s3vfvv73wAPuGt4c5HfGIAzoUm4whcemlo14Ej/YbjEJ05xvGBGAT8AjHAqzvGOe1whi1ECW/wT';
			B[i++]='8Y+b/OSe4YtfhtCUHEDFLDCPucxnTvOa2/zmUUGLAsBQBJa45ChAD7rQh070ohv96EFJCgKW8oMg';
			B[i++]='AAA7';
			this.images['bubbles.vrev.unsafe'] = { name: 'unsafeVRevBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/ALOkztWJMHleqO2DBvTl1L+/v/r6+d2rcPT09OV+BvGFBtFzBdmYTOrq6uXm';
			B[i++]='59nZ2cDAwO3Ut9t5BdLU1eS8jpJ9uPHy8tzb3+bCmOHh4frz6tN2CM50Dfz59ezs7dJ+HNaCH8zN';
			B[i++]='z+jn6erLpuGye/HcxPnv5NR4DsrLzJmcn1AujaSlpbO1t9PL4qyvsd/g4Z+ipaaoq7/Bw8i9214+';
			B[i++]='lsfIyd/f381xBdXV1eF8BtR1BeqBBvWHBv///8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ9w';
			B[i++]='SCwOMYeIcclsOp/QqHRKrVqv2OwSc+McStqweEwum8/V0W2hA5EI6Lh8Tq+LIxxJTiIJUEx2gYKD';
			B[i++]='hGUlJzk7OwkSOgwYGoWSk5SVJR8JAwoDixIbDCMdlaOkpWUEmJo8m50nDBEGprKztE0EIDmqq6x7';
			B[i++]='H0m1wMGlJri6PLsDCb1vws3OgRoBuQrH1cjKOn6Rz9zdYx3SxtarnMqfET3e6utTHQw51OPjrIy+';
			B[i++]='cOz4+UTgO/Hy1go2KetDYZu+g94YBPzHMGCnc+kQSgymAcQOhhiP0ZPwgdnEj6VGBOiXsWTAZDkc';
			B[i++]='hQLJsgoBEgE26JhJs6ZNm3r6+StZshyf/5tAgwodSrSo0aNIkypVygEEBVFFKJyQkECR1atYs+4Y';
			B[i++]='II6n102ctIodS7as2bNo06pdy/ZqAkYgwAgxwIBqV6948+rdy7ev37+AAzdUsGgDBiEHJJAUzLix';
			B[i++]='48eQIzdWkGBDCQIbFkvezLmz5895FeQIkHgA6NOoU6tmvGNBgAQ7x5XLQbu27du4c+vezbu379/A';
			B[i++]='gwsfTrw47qqxre3QkbkhIx0LbkifTr269evYs2vfzr279+/gw4sfb51NIobLdVyUtyOlnxI94suf';
			B[i++]='T7++/fv48+vfz7+///8ABihgfSUckAds7M20nnISLHCAAQ5cgMOEFFZo4YUYZqjhhhx26P/hhyCG';
			B[i++]='KOKIJF5oQwMExATPOOkteMwAEnAQgQcPPHCBDTjmqOOOPPbo449ABinkkEQWaeSRSCbJ4wUPZNDD';
			B[i++]='ATokwKKC4ySgAwUN4HBBBiI04OWXYIYp5phklmnmmWimqeaabLbp5ptiOvCCjQYEcIKLLVoDYwA9';
			B[i++]='aOlAAxYgIOighBZq6KGIJqrooow26uijkEYq6aSFWmCBCBk0ScACUlaTp6dXOtCkBwgYMOCpqKaq';
			B[i++]='6qqsstrAAzVAUMMDDcSHwKU24NDDDTkoR2U1CSxAgI0iWGDqfgCooKwKLdRXwbIqxBcCDClUa+21';
			B[i++]='1dYQnwEsYOtttS44IIMMHsTQ6rnoprv/7QUQFOCuuxBcYCoCDeS6a6+gqmdNDjcYgMOJCPDXArTM';
			B[i++]='0pcstD288O3CIfRQwMLYwmBBDyE44AIL6mas8YAXFBDfux9f0AO99vLqq77V8NsnwPuJQAPBzco3';
			B[i++]='A8HRdgvDAw7krLPOL8SQgrng7ix0zhP34IAFLziw8dJM59dAux+7+zEEDZCsq8n5uqjyv1Xrh4AA';
			B[i++]='NMfcwwUvE9xDtxjnF0K1PVTbcNNwxx3gAyA7HHUBD1h978la37By1/mBHbatZZuNtn5rp9B2Cm/L';
			B[i++]='7fjj99VQt3zv1qA31sd8eszWLOP3LM0F9yA4zWenkDZ+iS/eOOSsQ97u5O8WAMHl+Gb+/+vmfnMd';
			B[i++]='8H0zL1t4wZ8r+7vDKcAQwvHIJz9tCi4sXoDy0HvQ+vTpSi71fJXTzve+uXde3wUEH6xsC+KrQEP5';
			B[i++]='PTwAsbcTLL6+tTJQL/+qdF/vscPu5l3v1bXzoDkPnAMcfRDwuwr0AFrlU8EMBrYsir3vWvF74LWU';
			B[i++]='Nr8KCuhp96ub7Kq2v71ljXt/2918vgYtAx4QdCoAQA8YqKzSxeBY9uFW8ZyXHw+4zYI4BFDH7hay';
			B[i++]='kXUQc/67HQC7J0D5BE8FAigaCk3IwpqZDnFsu2F+pJjDKurHAOyKnezk5cOS9e9/ARRhfMpHAwqe';
			B[i++]='kGACkE8TS3e6+6SOiveBoxXnaJ9Xxf9qVrWy1Q+/KMQwzqd3yxLbGX1XtBUirFvNy08Nosg4/ciR';
			B[i++]='jpDcj/Y+mDIiipBsCKQPwWggMjUiTAbgYoEoR0nKamGMWjAgpSpZ4IJqtS+SsJTkHrdXyRASroT1';
			B[i++]='CR991mgBaklQaROQIPNgGMti1meStkMZ7mzZAwAI4JkCMCF9oCkAFdLnAtSMjwVksEpVFsCML+im';
			B[i++]='KmtATGOaU49epOUydXfOdrqzi/xT5xCZ+QAWxCAGLpBefLplLfl4gJ8w0Ja0Glcx+VyLBS+Qz+kS';
			B[i++]='18/41CB+PTCACxL6TmMiM4jKnCc7z0bBGinUPiF4gHwSSTGCmvF0L2hcG4220h6MS6L/Iq2oRWdJ';
			B[i++]='yXV2LpijpOjZRilQiplxPseTT0HjEwPkxeCVZ6OPA1rq0pvJ1JwXBaMlHSqflH60PiHQaU+D6lAz';
			B[i++]='hitnMlCpUpma1KfONJ011WjnQnAxe+pUlVWNgShdQNJejjIGhTydA8Q6n6Xeh6xmtWJU+zjVwBrW';
			B[i++]='goPNqB8Py1jqJbZvK1uBZCdL2cpa9rKYzaxmN8vZznr2s6ANrWhH21m0JhOy//KBalfL2ta69rWw';
			B[i++]='ja1sZ0vb2tr2trjNrW53W1vTYhS1NuCtcIdL3OIa97jIva1vpbqy5Dr3udCNrnR5u1zCNne62M2u';
			B[i++]='drc73OoqtnvcDa94xyte7wKXvOhN/696jWteEKZ2vfCNr3xh295avne++M1veutrU/3697/b5a9a';
			B[i++]='AUzgAjtXwJwzsIIXTN14pjXBDI6whGWLYPBO+MIY9kGFr5vhDi94w/f1sIgBDOLgjvjE/i0xileM';
			B[i++]='XxWz+MXrdTGMZzxeGdP4xtq1MY53HF0d8/jHyPUxkIfcXQef1r0mJrKSi+zBI9s3yUuOcm6FLOUq';
			B[i++]='x5bKVs4ya7GsZS1zuctW/jKYpSzmMS+5zGYmMprTDOQ1s5nHbn4zjuMsZxrTuc4wvjOeWaznPaO4';
			B[i++]='z34eMaAD7eEST4DQZzbyb5GcAUQTGQUgvkCWHA3kFwjYShjIVAYacGhKz1kEAt5Tn/4u4AAPXMDT';
			B[i++]='M0aBAzD1ABtsqlNOBpYOMJClLYnAAThAAapPHNJbz0lkDMjo/3gAIxm9ykZKSrayl83sZjv72T5i';
			B[i++]='UgYMACVYx9pTDXqQqErE7W57+9vgDre4M3SiFG1gRfL0lHsoQIDGuttxESDBgZIz7GpQphHRIY++';
			B[i++]='983vfvv73wC/jnk04ytOJacaszGOwhfO8IY7/OEQzw1yMLKcd5hmNRjPuMY13hrMEHzjIA+5yAUj';
			B[i++]='GhD8IDEfH7nKV87yeVgJDHSxy8FbTvOao8YhlTnMEKRClbb4/OdAD7rQh070omNlD3ExwksCwIal';
			B[i++]='OP3pUI+61KdO9aoTpSlPGUIQAAA7';
			this.images['bubbles.vrev.unknown'] = { name: 'unknownVRevBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/AEnOSfn6+QHBAQDUAPT09LjquADdALiq0erq6uXm59nZ2cDAwI3djWLSYuj4';
			B[i++]='6CnHKdbz1jnKOQe/B9LU1ZJ9uBbDFgDJAMbuxvHy8nLWctzb3+Hh4fX89QDgAOzs7Zbglg3GDc7w';
			B[i++]='zszNz6TlpOjn6crLzJmcn1AujaSlpbO1t7+/v9PL4qyvsd/g4aeWxnleqL/Bw6aoq8i9214+lsfI';
			B[i++]='yfTy+N/f3wC9ANXV1QDEAADPAADYAADiAP///8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ9w';
			B[i++]='SCwaj8ikcslsOp/QqHQqLTQ+1Kx2y+16v+DhJVO5YcPotHrN/kLIudtt1K7b73i0g/Gw6HQWEgV5';
			B[i++]='hIWGh0MOHxE5OgM7Ozogg4iVlpdcHCMAORaPBqCSF5ikpaZHHAUAIH47oB0dBgMVIae2t5VWFX4D';
			B[i++]='rzw8sToVELjFxm0hZK2usL/ABjoPDsfU1Vx7fY47sB3Ov8HS1uLjTQGqjZ/N3t+REdPk8PFCb7s6';
			B[i++]='zN3rzsERHPL+1th4McuXr8OOB/3+KbzV45ynewQJdhgAYKHFUvRaGVAXUWIHEO8uiiQUYEQEgRw7';
			B[i++]='Row14AEdMBAaVBCQo6bNmzhz6tzJs6fP/59AgwodShSnhYcb8alcCiuSBaIVADBISISBBEePIGnd';
			B[i++]='yrWr169gw4odS7as2bNou/paylZfrLQDAD0YJSQAAE9J2+rdy7ev37+AAwsezDQWtBxnMljYppSw';
			B[i++]='48eQI0ueLJilgAsQcgxoTLmz58+gQ7MFl0HHRtGoU6tePXiigAebOxrYEfeP7du4c+vezbu379/A';
			B[i++]='gwsfTry48dyfOu7IIWEbQVlPBciZTr269evYs2vfzr279+/gw4sff10Ar4jLc+wgGCnHAwYhesif';
			B[i++]='T7++/fv48+vfz7+///8ABijggPYVkIEEeOWT3nrrDGCBABlwkIACOFRo4YUYZqjhhhx26P/hhyCG';
			B[i++]='KOKIJJZoYoY2IAABI6ats+A6BgRSAAIKKKCBDTjmqOOOPPbo449ABinkkEQWaeSRSCbJowYKbNBD';
			B[i++]='A5q5WBODzgyQAwMI4KDBBiQg4OWXYIYp5phklmnmmWimqeaabLbp5ptiJmCDjT2cNIA3LzoT4wM9';
			B[i++]='aJkAAhgQIOighBZq6KGIJqrooow26uijkEYq6aSFYoABCRs0GYIAdzqT5y9WMjDhBh4QEACBqKaq';
			B[i++]='6qqsttoqjTQsQIMCCMhHwKU24NDDDTrgOaU3OggAgY0kYHAqfwecoOwJK9hHwbInyCcCCiZUa+21';
			B[i++]='1dIgXwApYOtttSwkAAMMHsTg6rnoprv/rQYLqOCuuwtocCoBCOS6a6+e/uqMDjcEgEOKBPS3ArTM';
			B[i++]='1ucCwT208O3CIvSgwsLYooBBDyIkwEIK6masMYEaqCDfux9r0AO99vLqq3rA3tAnwPyRMAPBzc4n';
			B[i++]='A8HRdouCAgnkrLPOLcRggrng7ix0zhP3kAAGLSSw8dJM64dAux+7+/ECCJCsq8n5oryvyv9WvV8N';
			B[i++]='L9Accw8avIxwtxjrJ0K1PVTbcNNwxy2gAiA7HLUKClh978lU8sDvyl7rF7bY8tVg9tkmpJ3f2ia0';
			B[i++]='bcLbckcuOX401D3fuzTojfUvn/rNNcv5HUxzwT0MTnMPaO/HuOOQT+765O1a/q4KC2iO/y/n+v7y';
			B[i++]='d9cB45fssocX/Czw0DpsAgoiJK/88tOawILjKjAvvQevV59u5VLTh7ntfKcMeO/2aUDw78quQP4J';
			B[i++]='M5zfgwIQezuB4+1bC4P19LNKd/YeO+xu3vVefTsPndsd6OpjOGi5oAcGJJgMBrYsisXvWvN74LWU';
			B[i++]='Vr8KDuhp+asb7arWv71lrW8CDBx9wAYtCshndMo6QA8YqCzU/exY9+HW8aCnHw+4zYI4DFDH7hay';
			B[i++]='kXVwcwDMnee+5yxovaAGJxydCVdYvNSpjW031E8Uc0jF/QSAXbOjnbx8WLL/BfBzIpSP6JQ1Awoi';
			B[i++]='kGYvmA8La5Y41UHxcfuZYhXniB9Yyf+KVvPh3ge9xzv6zAxaYzsjtGaARPms0YXP0w8N3ti6+8iR';
			B[i++]='jpDsjx5xpzXdgRF8ZYOWCulDsBmITI3FgwG4UkDKUpqyWhijFgpMycoUsKBa74ukLPkzySBWcoh9';
			B[i++]='7EEBl7VETmqyPofEALUkqLQJSNB5MJylMu1Tyy8SsQcueIE0X9BL+kzzBZukjwauKR8MwKCVrFSB';
			B[i++]='GVsATlbSIJnLTKetfuhFIYYQfOqMpzqb6c5LykcBKYhBDFhAPfl0y1rz8cA/UaAtaUGuYvO5Vgpa';
			B[i++]='MB/FMQ6g8qHB/HoQABYwVJ7ppOct3+lPCtaoofcRgQLmk0iKHdSMimsB5BQnnwSwVD7/46roSDGa';
			B[i++]='UXZ2b2vPNGYpL4q6UhaUYmakT/Lmg1D5xEB5MYgl6urj0vvA4GY0nadN94jTXP5UpSC1jwh4+tOh';
			B[i++]='RtSM4coZDFbK1JdmNarL1CgI7Umxi+WTp6ycT89IyYKSCrOUMSjaUltKVvo09T5mRSsk1crHAQr2';
			B[i++]='sBYkbFUNi9jGVk+xllwZCiZL2cpa9rKYzaxmN8vZznr2s6ANrWhHS1rPdvGmkf2XD1bL2ta69rWw';
			B[i++]='ja1sZ0vb2tr2trjNrW53y1vbnpaqqbVBb4dL3OIa97jITS5uf0vJta5MudCNrnSnS93eMteWzlVt';
			B[i++]='dbfL3e56l7jXdaZ2v0ve8pr3u+Gt/+dzz8ve9rrXuOnd6OfeS9/62le28c2ucO/L3/66N7+F9a+A';
			B[i++]='B+xdAC+WwAhOMHQNHFwFO/jB1vUfanG5Xwhb+ML4lTBwKYzhDnt4tQzm8IdHDOEQC5DEKFawieeb';
			B[i++]='4hYLeMXrdbGM7Qvj8c74xu2tcYVxzOPy6rjHQCbvj4NM5O0OuchIju6Rk8zk4y65yVCOsAebG+Ao';
			B[i++]='W5m3T76ylmmb5S17+bVd/rKYfRDmMXu5zGbWMprTbOU1sxnKbn4zk+MsZyTTuc5EvjOegaznPfO4';
			B[i++]='z36+MaADLeNBE7rFhj40inU8AUXDWcNUXuwGHM3kEtRYA1miNJJbEGIrfSBTG0BAo/81zWcShHhP';
			B[i++]='fdJAAjygAVLjuAQJwJQCWrCpTkW6SojJ0pZIkAAclMDVKRYpr1tAJztNmAcxEgSNbKSkZjv72dCO';
			B[i++]='trSn7SMmbSAAGdBMYzoHqgdFaEInCre4x03ucpv73BtK0YoaYQAp3dJTOnAPfBxL78kZCEG9UNCU';
			B[i++]='OPML6ORAOuQJuMAHTvCCG/zg5TkPe+IQm4jMpjbHibjEJ07xilv84shxhXJy0ADT8Js1IA+5yFHj';
			B[i++]='mhBoe+QoT7nKIQOOHzRgMSlZucxnTnNvWGYQdsFLzGvO856rxjDxPoMQGGCeAWQlLUhPutKXzvSm';
			B[i++]='O/3pYTG6BeZihJjMpChYz7rWt84O9a57/es+EQAIpEKVIAAAOw==';
			this.images['bubbles.hvrev.safe'] = { name: 'safeHVRevBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/APv6+vT09NBLS+q5ubiq0erq6uXm59nZ2cDAwN+Ojvno6OEDA/PW1tLU1Y96';
			B[i++]='ttdzc8krK9YDA8w6OvHy8tzb380DA/DNzcMDA8cWFuHh4e7ExNNjY/z19ezs7aSSxOGXl8MKCszN';
			B[i++]='z+jn6ealpcrLzJmcn1AujaSlpbO1t7+/v9PL4qyvsd/g4Xxiqp+ipaaoq7/Bw8i9214+lsfIyfTy';
			B[i++]='+HFVot/f378DA9XV1dIDA9sDA8YDA+UDA////8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ9w';
			B[i++]='SCwaj8ikcslsOp/QqHQqHWw+1Kx2y+16v+Ch5oG5YcPotHrN/jIekN3tNmrb7/g8WpGI53IVIAN6';
			B[i++]='hIWGh0MKHxI7FRE6OhGCiJSVll0cIwKNjwueERgal6OkpUcAAwIgFTk6njyfGBamtLWUVhiOrq+w';
			B[i++]='CzkYDLbCw20WZKyQvDy9ORAKxNDRXHwQrJ0Ly9nMztLd3k2om7rK2ss6ORLP3+vsQm+51+XyCxES';
			B[i++]='HO340tQ7rbvy8zog3MtHsFY4EDni/fu3QIeAghBJvRuHbaFFWBjURdxICMAIRgk9VbxosSGEOmAY';
			B[i++]='bMBwYYfLlzBjypxJs6bNmzhz6tzJs2fM/woUSQrtpQNoTwwCEgwkkgBhhEeQokqdSrWq1atYs2rd';
			B[i++]='yrWr169UyQ0V6gns0woQRAkBIMCR2LFw48qdS7eu3bt47ZaVdOZBBVd5AwseTLiw4cH0dmhgsCPC';
			B[i++]='yMOQI0ueTHmsLwh+H1fezLmzZ7sRLkBwXDLSn9OoU6tezbq169ewY8ueTbu27durO1nUsQOEjoX0';
			B[i++]='Kuy4MKe48ePIkytfzry58+fQo0ufTr168patNJtz+VveuR0QElgA0KO8+fPo06tfz769+/fw48uf';
			B[i++]='T7++ffQDHqwiXY73ju7aFHXBAxwYcAAOCCao4IIMNujggxBGKOGEFFZo4YUYZsigDQUwAP+SZv4B';
			B[i++]='uMwCgQxQwAEHUGDDiiy26OKLMMYo44w01mjjjTjmqOOOPL5IwQEZ9LBBY/1xV04EOyRQAA4UZCBC';
			B[i++]='AVBGKeWUVFZp5ZVYZqnlllx26eWXYIZJpQE2pNiDBH8FaGQ2JEIAAJMGFDBBAHTWaeedeOap5558';
			B[i++]='9unnn4AGKuighBZ65wQTiJABkBZcEIGa/6mZgIEZdBAAefdlqummnHbqaacnzoDADAcUUF4AidqA';
			B[i++]='Qw835ACpiKExkKIIE2DqHgEm5GqCCuk5oKsJ5YXgQgnEFmsssTOUBwAKxzZL7AoGwABDBy98au21';
			B[i++]='2CpLAQIpdNstAhSQF0ABqrLqajYhapP/ww1vchgAfCr8uit6HsjbAwvO5htCDynke6wLE/QQggEr';
			B[i++]='oJDtwQjfR0EK5XnbMAU9jFtuq6+qe0MPOLj7nggyyMureTHICyyzLhxgwMkoo8zCCyVU+2zKMJ8c';
			B[i++]='cA8GTMCCAQnnrHN7BXDbcLcNI1CAxKtSjO6ay6yLscbt0VCDyB/3QEHH9jJrcHshENsDsfvu7PXX';
			B[i++]='9R3gML8/p3AA0eZWnI3SGQ/tnq9Ql0cD1VWXcDV7WZewdQldg+333+vNMLZ53s6AttHbRbr2xW2/';
			B[i++]='y169Is/bw9OR92C1e3nv3TfgnAPO7eDepoDA4ecmLiLbTKuHq650zwt3rq3zW4ILIdRu//vtwpaw';
			B[i++]='wt4p4O57B50Hj63gQJ9XOOlqJ8146uhRIO/quaoAvQkyTN/DAf4228De2RcLg/Dgdyp28Qzz2+3Z';
			B[i++]='5BZdOg/pLr602+nN/asHPcwvbwzx6ipw98Z+z7+xOAufAO3Ts/KNTXRDS1/ajqY45b3Pcedx2q8c';
			B[i++]='UJ7I5YoAPchfrizXMlulZ1mz4117OsC1AZqQPgsr28MipkDEsQ9pPEAd/M7zOhPUgAYVjBwFM/gr';
			B[i++]='Dt5tPZkrYXuEeMIiugcA2wqd6MLFwomtr30ObBy9fiUDEZgncjUwjwZHZjfMaY2I6wGjEceonlCN';
			B[i++]='qlTmQR4DT7e8GfYgZL+KWv3kJQMclv9nixzcXXtm8EW+uUeMZAzke9RoOos90DxT+xUGz0NHiGmx';
			B[i++]='hzB4FgomSclKEstgw3JBJTeJghUQa3uCDOUgW/hEGMrQcfLT1Q4ZqUj04HECw/ofzhrwP915UJS4';
			B[i++]='RA8hX9jAGLbRcR5ogTBbsMrzDLMFizwPBY5ZngnAgJObTEEA7wXNTc7glrnMZhPVlzxfHlKb4Azn';
			B[i++]='LqHoTSleDwUveMEKgFceZhXLPB1wpwuSFay+Dcw8xkIBC8xzt7y9szwz+F4PALCCfYYzm+M05S/b';
			B[i++]='GUAU8VM9ITiAefQoMHtO824s6NsPabbRHkiLoBI9KEJJ2c1TloeWlDSo5ShJT4FN8zz/tTPPPcvz';
			B[i++]='Atu9AJSWQ48BOurRkolUmwntpUl70NKMPjQ9IVBpS2MK0GlC62Qw0KhOeZrTn47UiSVdqMAKhk6V';
			B[i++]='btI8K5vkCigKS0q+YGZVpZlUz7PT9VDVqmMMKhu/Cde6mlCuhjSnXfcKPry6L2MnCKxgB0vYwhr2';
			B[i++]='sIhNrGIXy9jGOvaxkI2sZBmL1TXm1QY+yKxmN8vZznr2s6ANrWhHS9rSmva0qE2taklb2UL+FbOr';
			B[i++]='ja1sZ0vb2tr2tqVtLS/nmjHc+va3wA2ucFerW3KibrjITa5ylzvb4ip0acyNrnSnK13nCnV51M2u';
			B[i++]='drdbW+vyFrbcDa94x+tZ716WvOhN/y93zfta9br3vctlbxTBC9/62te28i0nfe/L3/6eNr/H9a+A';
			B[i++]='BxxaAGOXwAhOsA8MDF0FO9i/DO7tgyds3wjvl8IYJq+FM8xh9G64wyBeLzct294Qm5i6Hz6xiuM7';
			B[i++]='YtfOd8UwTm6KY0xj3864xjim7Y1zzGPV7rjHQM5ti3d73iAb2bQ/PrKSOZvkJTu5yU5WMpSjbOQp';
			B[i++]='UxnIVr4yj7OsZRxzucs0/jKYYSzmMau4zGY2MZrTDOI1s5nDbn4zhuMs5wlbuAF17rGFM5DnHJMg';
			B[i++]='whRYUp9rzIL8IukDi8pAAfA86BXbQAT5bdObKGCADlCg0SYmgQEUxShHdZMHh15Sk+FEYAAckADT';
			B[i++]='GI4oqVmQIrb0kpywKNGJUtSjWtv61rjOta53HaMfBWlIj/o0+yowoAIdSEPITrayl83sZju7QRzy';
			B[i++]='ED9ABEN05QA84uGrtgGXn/1oB9ZsioBwiGOdcpv73OhOt7rXfR1+BNs7w+EPQ0yDm3rb+974zre+';
			B[i++]='950bwCyENxuogHY+Q/CCG7wzobFAYwZ+8IY7/OF6acYPAu5viFv84hgHDpIGwRa3MDzjIA/5Zvay';
			B[i++]='gzMIoSkJAYvKV87ylrv85TCP+VbOkhYjqIQlPsm5znfO8577/OdAxwkIkrKUIAAAOw==';
			this.images['bubbles.hvrev.unsafe'] = { name: 'unsafeHVRevBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlh4QBxANU/ALOkznleqO2DBvTl1L+/v/r6+dulZdWJMPT09OV+BvGFBtmXSdJzBerq6uXm';
			B[i++]='59nZ2cDAwO3Ut9t5BdLU1ZJ9uOS9j/Hy8tzb3+bCmOHh4dR/Gvrz6s50Dfz59ezs7dN2CdWCIMzN';
			B[i++]='z+jn6erLpt+udc15GPHcxPnv5MnKytR6EZmcn1AujaSlpbO1t9PL4qyvsZ+ipd/g4b/Bw6aoq8fF';
			B[i++]='zl4+lt/f381xBdXV1eF8BtR1BeqBBvWHBv///8LExf///yH5BAEAAD8ALAAAAADhAHEAAAb/wJ9w';
			B[i++]='SCwaj8ikcslsOp/QqHQqjRgw1Kx2y+16v+ChiVS6YcPotHrN/g5IIMbtNmrb7/g8+lQ5SHI5Eh8R';
			B[i++]='eoWGh4hDGxgLOhIJOzs5HyaJlpeYXR0jCx85kAIKAgkplZmnqKlGBRELKY87oTyiCRoDqri5liYG';
			B[i++]='Gn+xCgo8swI5ILe6ycpsb745wMLDxMYny9bXXBt9fwmh0dLTB9XY5OVMPa0fjwKy4NKiOQcb5vT1';
			B[i++]='Q82w7e7gowsd9gCxaTug45O3ffsU7DjwL6DDXOg6GdSH0J1CAw8zosIH6lvFisXmaRxpqACngt2C';
			B[i++]='fVw5TEGCA3XADDBQggEDHThz6tzJs6fP/59AgwodSrSo0aM6JcBSybJpsB1KjTLQsKBCQyIV1EGK';
			B[i++]='xLWr169gw4odS7as2bNo06r9ys5j07eiBKhNkEDCAVM/CixY5/at37+AAwseTLiw4cMVnyb4cIaE';
			B[i++]='hB19EUueTLmy5cuFXVIa8AEy5s+gQ4seHVhBPMcCSKtezbq15B0MDiSIPExuAkC4c+vezbu379/A';
			B[i++]='gwsfTry48ePIeUOizQN258R1dciZQ7269evYs2vfzr279+/gw4sfT/76zWcVd0jfgVCSjgMVTPSY';
			B[i++]='T7++/fv48+vfz7+///8ABijggATeNwYHj/Slng7suQMVAyR04MAFOFRo4YUYZqjhhhx26P/hhyCG';
			B[i++]='KOKIJJZoYoY2iDDAAZ64tWCD0gggAQcRNPDAAxfYoOOOPPbo449ABinkkEQWaeSRSCap5JI+XvBA';
			B[i++]='Bj0YoEMCDuIE4zAJ6FBBAzhckIEIDYQp5phklmnmmWimqeaabLbp5ptwxilnmQ7EgGMPjVz5Ij92';
			B[i++]='9dClAw1YgMCghBZq6KGIJqrooow26uijkEYq6aSUGmqBBSJk8IANAzBApTR7gqqlA096gEABBaaq';
			B[i++]='6qqstuqqqzaiAAEKDzQwHwKY2oBDDzfkAE6oWDIwAI4iWIBqfwCsoOwKLtxHwbIrzBcCDCpUa+21';
			B[i++]='1dIwXwEtYOtttS84IIMMHszw6rnoprv/7QUQEOCuuxBcgCoCDejKq6+iMghODjcUgIMNDSDgnwvQ';
			B[i++]='MmtfstD2EMO3DIfQAwEMYwuDBT2E4MALLair8cYFXkDAfO+CfEEP9Nrb669W7nuDnwALzJ8INRTc';
			B[i++]='LH00FBxttzA84MDOPPMcwwwqmAtuz0TvTHEPDlgQgwMcN+30fg20C7K7IEMQcL27npzvlfyyHDB/';
			B[i++]='CARg88w9XBBzwT10m/F+IVTbQ7UOPy333AM+EPLDUxPwQMlZ4zsMsDx0/e/X+4k99q1no602f22r';
			B[i++]='8LYKcdMt+eT5oXA3fe+iwPe9KOsrjeAt7/eszQb3YLjNaauwtn6NPx455bBT3u7l7xIA/8HmWv+d';
			B[i++]='8ucrD+4yfjJAm7jBoys7/MMqwBDC8sw3P60KLzxOgPPUexD79elaTnV9mePud3O7DwM64fddUDDC';
			B[i++]='yrqA/go1rN/DAxF7O8Hj8VsrA/b4t2r39h8/7O7eWOPc1lTmtd/VBwHDo0APoLW+FdCAYMuqWP2u';
			B[i++]='db8JXotp+csggaLWv7vZ7mom+x7gxmfAW51uBQpcIOlWAIAeQFBZqZvBsfDDreRJbz8egJsGdygg';
			B[i++]='j+VNZCQLYO7A5znx9S509ineCgJwtBWm8IU3Ux3j3KbD/VSRh1jkTwHYVTvbySuIIewc145Ivvms';
			B[i++]='rwYYVGHBAkAfKKZudflp3RXzM8cs2v8RP7GaVa3o4z0xEtB39akZtMimxmXV4GguTFi3orcfGlAR';
			B[i++]='cvyp4x0n2Z8+DpB3BaSP2Rhon4LVYGRtTJgMwNWCUprylNXKGLVgcMpWtuAF1ZofJWdZSSGKMHyB';
			B[i++]='I6PLEAitFNbnfPZxowWoZUGmTcCC0JshLZd5H0vqroi5zGQPABCAagbAl/WxZgBaaJ8LaHM+FpCB';
			B[i++]='K1tJgDTGYJytpIEymcnOW9nSj5gEZDvnSU9nEnGM0nxAC2YwgxdYbz7dshZ9PBBQGGhLWpGzGH2u';
			B[i++]='1YIY0Gd1jRPofGhwvx4U4AUOpSc77TlCXQIUgzd6KH5C8AD6MLJiCU3j6mIQOTgizaX/PRjXRUuq';
			B[i++]='0Y2+85JGlOYxTZnRtJnyoBVLY32WRx+FzmcGzJuBLNNmHwfANKY5q2k7OYpLEk6UPiwV6X1C0FOg';
			B[i++]='EnWiaQzXzmTQ0qY+lalStWkYcRpNeYYAY/vsaSuxOoNSvuCkwzTlDBC5OgeUtT5Ozc9Z02pHqkLT';
			B[i++]='qoRNLA8Ni095Kvax+GPsH23Agspa9rKYzaxmN8vZznr2s6ANrWhHS9rSmha0a31mY23gg9a69rWw';
			B[i++]='ja1sZ0vb2tr2trjNrW53y9ve+ha3qb3nZH9L3OIa97jITa5ydRvcjrJsudCNrnSnS93fNreqR6yu';
			B[i++]='drfL3e4a97qHza53x0ve8o4XvKs1/69618te5KJ3uO2Nr3znO9v3xpO19M2vftlr35z+a78ADnB3';
			B[i++]='+9tW/Ar4wAhWLoFBl+AGO9i6fYOnfw384ApbuL4RZiuDL8zhDvtgweL1sIgdDOLnjvjECC7xf1HM';
			B[i++]='YgCrmMItjnF8XyzjGs84w6qFr413XF4a8/jH3vUxkIdcXSET+cjQNTKSl3xcJTP5yb51MpSnnFsp';
			B[i++]='U/nKtLUylrf8Wi1zmcte/jKWwyxmKpO5zFA+M5qZrOY1I7nNbiYynOMM5DnTmcd2vrON86xnGfO5';
			B[i++]='zy3+M6BRLOhBj/jFEzB0mnEs3PtmQNFMRoGKL8AlSCM5BgTOEgY0lYEGJNrSdRYBgf9ldAA/XcAB';
			B[i++]='HrgAqHeMAgdkalOd+lSOwaFpLnlJBA7AAQpWzWKS4tpOI8uThGszoxrdKEdMSrayl83sZjv72T9y';
			B[i++]='EpSkJOtZ/0oCEJLQA07E7W57+9vgDre4NwSwFbWoStAEVQ5iU4EBQPbdlDtQgvahHgZcCRwukYB0';
			B[i++]='ysPvfvv73wAPuMCzc557i8pTzLFNchbO8IY7/OEQj3hvlvMR2BhAAsxxjcY3znHSwGYADMp4x0dO';
			B[i++]='8pILxjQg+MHFPWPylrv85R/RTCX0wheY2/zmHVeMDs4ghKzAYi1AD7rQh070ohv96GYJxF2MMJOa';
			B[i++]='3AQpUI+61KdO9apb/eo/YUAKqnIDlSAAADs=';
			this.images['bubbles.hvrev.unknown'] = { name: 'unknownHVRevBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhfgA3APcAAMbIyf///9EjJM8iJdQlJ7IfILAfItEmKK0gIc4pK8YoKagmKNIwM8JBQ+Gz';
			B[i++]='tM0iKNIkKdAjK9EkKdEkK8wlLM4mKsklKea9vuvJyuXExbceKMkhK80jLbgfJ8sjLsklLrwwOeK4';
			B[i++]='uuW9v+bIyuXIysQeLMAfLr4fLcojMMYkMcAlM8EmNMIrOMMxPs01QsU2Q8Y6R8c+StJcZ9uRl92Z';
			B[i++]='n+KrsOCpruO1ueHU1cEXKMQgMMciMsQiMsMiMr8iMcUlNcYqOscvPskyQck0Q8w2Rso3Rco5SMs7';
			B[i++]='Ssw/Tcs/TcxBT81EUc5GU89JVs9LWNBMWc9MWc9NWs9PW85QXNBSXtZmcdR5gtiHj+GaoeO8wOni';
			B[i++]='4+fn6IWFhoODhPj6/fb5/Qo4cfP3/CFMfkBmj2aFpXuWslhrf4ylv52yx6y9z7rJ2MzX4khJSsDC';
			B[i++]='xPz9/s/Q0czNzsrLzMzOz8vNzsXHyMPFxuPq69nj4+Tr6+Tq6uLo6ODm5uLn5/v+/uDj4/3//+7w';
			B[i++]='8OPl5cbIyL/Bwf7///b39+jp6dbX14WGhtPa2efu7ebt7OTq6drg3+Tm5Tc3Nv7+/dTU08rKyRYV';
			B[i++]='FNzKxOPSzeDLxqVWRqJYSaFYSZ9aTKVfUaxvYq1yZq50Z7R4bMadlJE8LqFYSqRaTKJZS6BXSqNZ';
			B[i++]='TLZ9csCPhuHMyODa2e/h3/fw75o6MJo9M5xCN+fPzPXt7JUnHZYxJ5YzKpg1K5g2LdywrKYTC90n';
			B[i++]='HL9YUr9oYuq+u+O+vNUbFtkmHtcnItgqI9ouJ50kIOSFgeiwrui9u+nHxtUjIKAbGdMjI9UmJbMg';
			B[i++]='H7ghIbMgINImJtIsLKMjI9I6OsRZWOa9ve/KyuDPz+fh4SopKfz5+fn29v/+/s7Nza+urvz8/Pn5';
			B[i++]='+fT09PPz8/Ly8vHx8evr6+Hh4d7e3tra2s/Pz83NzcbGxr29vbe3t7Ozs7GxsaWlpZmZmZGRkYyM';
			B[i++]='jIiIiISEhIGBgXV1dWVlZVVVVUJCQh0dHQwMDAYGBgICAgAAAP///yH5BAEAAP8ALAAAAAB+ADcA';
			B[i++]='AAj/AP8J/AfqFKdOCBMqXMiwocOHECNKnEixIadToAYOtKRJVrQFIEOKHEmypMmTKFOqXMnyZDRZ';
			B[i++]='miwNXBUqWhAhRHLq3Mmzp8+fQIMKHUq0KFAhQaKFWvUvwKlhLo4kUbJkiZKrWLNq3cq1q9evYMOK';
			B[i++]='HYu1qpIkR1wMOxWA1SsQSJY0iQJlCZMpUvLq3cu3r9+/gAMLHky4sBO5UZosQQLiFatKrl4wgUJF';
			B[i++]='igwaIkiM2My5s+fPoEOLHk26tGnTJDLUqEIZCpMXrSpBjtFEyhQZGe408sNHj2/ffIILHw58uPHj';
			B[i++]='yJMrX868OXPfexo1MmcFipQmMVxZqvQqhhMqUWzc/zlH7tu3QuC8uSkkTly49+HAvS/k5tu4+/jz';
			B[i++]='69/Pv7///wAGmJ84hYyzSiyAlJMIDjJI8UR2qXDnnWU4kDcOPPewEU889gwCyBbllGOOOeeM2I49';
			B[i++]='8KjnzYosttgiJAG4KOOMNNZo440zuuFGBqCgggoG4JjjxxVNPOhKhN01AUUV15gTgDz+9LPPPfr4';
			B[i++]='U88W5qiTjiSSrOPlPf48AmMAZJZpppmRdDPmmWy26eabcMZ55jGfUIIKKLD0Qs4eNDDhBIQSNuGE';
			B[i++]='DNcEUg4bYQ4SyTvwvFFOHI5Ewg47bRAIQDztBPDNG+p0KsgWAThyyDhZqlMOIo8UIueqrLbqpjeo';
			B[i++]='XP9yyyqrnHIKOXpgwQR2RwbaBKGG5ONPlP08gs0+GIY5ybD93POkP/hs8ciw1GJTDj7F7sPsPeSs';
			B[i++]='6eq34LqpzS64TEMIDbjoogUeNMgF6CswMMGEDFpsAY8+/WBjTzzLxoOPP/zcU8+//aAzjz9stIHN';
			B[i++]='I/G8E4+27/w7ZT3C7vNGuBhnbOYuzVRTTSwN6DJOHjQowcS78c6rRSABYOvsN1X6Cy0kkJiz7CBQ';
			B[i++]='5nMIG/ros88j2srsbADn8OMPOxonHW4GMmBhQhUxOAAIIzQgwQQMvboCQ1WEBrLNvy/36zKZ5Szb';
			B[i++]='BpRs1OPPJPLAY0/Q/9pDpjnaIq303axCMoIVt13/0Ac5VMeFdYSRcV2oN2BrWqU8YwdQtj84I2yP';
			B[i++]='P/vY0wW2/shsD4zn1I3356vW1wch3wBOwxFLwNAK4S9c1bU39uhTDyTf5IPNO7HPQ+YWbDzCzeT5';
			B[i++]='RJKPPpNMgg8b2HRTDzbybBNqPo9IAvr0q5ZO9RFKwMY6EklQUSg45nlDpnmQePON+AF4UQYZXSyb';
			B[i++]='zxnrk2FGGfSTYb/969O/BvX8v2k9DUZAgvYg8wLueS8Q4PCWm74gBjCwYRLYMAMYxrAGNZwBDWNQ';
			B[i++]='QxrOQIYLogENZBBDGvpHwjOVTg80KIIAV1eJVrTgCEc4YHrk9I0xiIEM62vgGOhXhjPs8Azwgx8Z';
			B[i++]='/9IwBjCooYRI1BTgZlCEIwzQhTA84DdY9QcvhMGKfyCEF74QhjB84Ytd3KIX3dCHPiQRiW5Y4hCc';
			B[i++]='aIsI2aIFRjACFbTgCHZ0wx14zKMe89iObrTjj4AMpCAH2Y49GvKQiEykIvHYjTsuMpHtKMciZiAE';
			B[i++]='I7SgFm6EYxGksLJ41IMLoAylKEWJiFGa8pSoTKUqV8nKVqqyC/I4hyJmEARLYrISbyzCEKBwDUfM';
			B[i++]='IQ5yCKYwh0nMYhrzmMhMpjKXycxmyuGX5phkEIpwSTeyYAhDcEIvBVEHAHjzm+AMpzjHSc5ymvOc';
			B[i++]='6EynOgFAhzpEk5bUvKUtrikEbTqCm+vMpz73yf/PfrLTnZMEghAuuZ15IqUJ2+ymPxfK0Iaus53m';
			B[i++]='wMMMBMoCWmynFixAChMS6tCOevSjEJ3kD4RQ0e3QggVBCMISOPrRlrp0nyG9wg+CwIJZmJQFQADC';
			B[i++]='Su+p0Jf69KfkjOlMa2rSFeRUCSwFqlKVClFFXIEHQFiBTStBC6P+AAlJXapWXRpTHUR1qrNQARCu';
			B[i++]='mtWtmtWhXf3qdsI61iSU9axw7WdMS6DWf9TCB0DgwRF62c64+nWhbZDlFejqA1oIxBY++IFe+drT';
			B[i++]='vzp2nYM4hFNL8AMf2OKwidWBERj72M6qM7JOzUFla4HZH2iWs55NbTlBewXR+oC0ds3sZh3RV9X/';
			B[i++]='2jacigrtaAVSixP0wARFQO1th8vaHPDgBLDtLQ+AK9zh2jawoV1uck/AgxIEl6fOHS50Wytd3lKX';
			B[i++]='ubRtbHY9G9lFcNcE061ucANR2/F6lg7bNS56vVtdF/QyDuJ171/bWQ48tFYH87XrCXZQAvuGqA10';
			B[i++]='0G9n2/CGLejhvOktgRRGBIg3DKINGM6whjfM4Q57+MMgDrGIRyziQdThEFtoBA6WUIIUIJe3Jqiu';
			B[i++]='EKyRiHIYwhyReIOOd8zjHu8YHXLwsZCHTOQiG/nIPobDkQ9RDnKc4w4z8MUGUPBiu8a4BzmQgSru';
			B[i++]='cAdG4OHLYA6zmMOchzyM+cxlTrOa18xmNtvh/81wjjOcy/zlNrOZEXLOsx3woAcuh4ABA7CAB6qs';
			B[i++]='3B5geQk0uEEWFs3oRjs6CyKItKQnTelJX+ACkr50pS/NaWp4+tOgBrUxRk1qY3A6050ONTUuEGpe';
			B[i++]='uPrVrjZGMYghDGRA4AGDhq0rNMADQ/egBDmgrKF5QGxip+DYyE42sj/A7GY7u9kUoEAFpk3tak/7';
			B[i++]='ANjOtrafwe1leNvbwAj3L8b9C2FI49wMSEAFDrBubWOb293+di7mTW96+8IXvwDGM3DdAVcIxBPJ';
			B[i++]='2EGvDa2DghtcByVI+AYWzvCGO5zhD4i4xCM+gIpbvOICyLjGN75xZXjc48gIucjJTe5gQAMaCfR4';
			B[i++]='wAA4rvGPf1zkyKg3vX+RC3Iv4xkU4EAyOjGQUCSDBzsIutCFjoKiG73oHki60pfOgaY7/ekciIDU';
			B[i++]='p071qUsgAhKAgNa3znWuE+DrYA87yhWgAAtYoAJZ77raIQB2bxNgGW+PO9ghMAEPJCMUGvlHKWSR';
			B[i++]='DAQY4O+A9zvgB094AxTg8IhPvOIXz/jEO+PxkI885JlB+cpT/vHMcAYCNr/5Akj+85i/vOVHX/lk';
			B[i++]='yKIUeRcIJz6xiVK4/vWiEMXrS5GJ2tv+9rXHhO53z/ve7x73wA8+8GNPfOKTYhTIHwUpjj8KTAgf';
			B[i++]='98S/BCaKT/1SbAIjGgkIADs=';
			this.images['bubbles.small.unsafe'] = { name: 'unsafeSmallBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhfgA3APcAAMjHyP///+bk5erl6Onk5+Pc4enl6N/c3+bk5uXj5cPBw/Tz9PLx8uno6eXk';
			B[i++]='5ePi4+Hg4dva29DP0M7NzsvKy8nIycTDxMPCw8HAwbSztAQybBJCdzZdikdVZIWOmFl6nW+Mqn6Y';
			B[i++]='s19seZ6yx6+/0L/O3fP3+/j7/o2mvvb6/fz+//v9/vHz8r/iziu8ZC68Zi+9ab/hzSa4Xi2/Yyy5';
			B[i++]='YSy4YC+7ZS+3YjO4ZjW2ZTe5aDazZMHgzTG3YTG0XzOzYDOyXzayYjmwYkC0abPcwsHfzMHey8Le';
			B[i++]='zNbh2jSwXTa0YDazXzauXjisXjutYbG0sjGlVDixXTeuWzirXDmnWj2qX1CPZC2aSzqwXTquXDqt';
			B[i++]='XDysW0yuaLPMus7c0t/m4dzj3jCXSjOYTjqpVjqjVT2pWT92TViKZVqIZmeRcnCXenKZfJTDoMTb';
			B[i++]='yjKVSjKUSTKSSTORSTKORz+mWFiMZVeIY1iJZFeHYl+Pa2SPbmqTdG+WeHGYeoGjiY6ultHd1NDc';
			B[i++]='0+Pr5SqAPC+FQTOKRTyfUkKkVjhzRGGMaiVrMzykTzuXTDx8STp2Rj55SdTe1kCcT0N8TUd/UcDT';
			B[i++]='wy93OkWgUlSmX12SZdTh1uvy7Nng2jeFQczdzjeDPkOcS0KXSjmDPjmBPkefTjuDQECHRavIrc3a';
			B[i++]='zkiLTMfYyEGUQ2WfZnqre0WZRkeaSEmdSkWURkmZSkyeTE6aTlSdVFqkWl+mX2ytbMTaxMzdzNjg';
			B[i++]='2PD08OHl4eLl4qGjoUFCQfv9+/P18/f490ybS06eTVCfTlKfUVGdUFWgVFmkWFiiV1qjWV6mXFyl';
			B[i++]='W2GnYGKnYWSoY2aqZWmraHCvb3Sxc1dXVrOzsiopKExLStrY1hQSERoZGbu6uv7+/v39/fz8/Pr6';
			B[i++]='+vb29vLy8vHx8evr6+fn59nZ2dbW1tPT09DQ0M/Pz83NzcXFxb29vbi4uLKysrGxsaurq5WVlYuL';
			B[i++]='i4eHh4WFhYODg39/f3R0dGVlZTExMSIiIgoKCgQEBAICAgAAAP///yH5BAEAAP8ALAAAAAB+ADcA';
			B[i++]='AAj/AP8J/Memj5o1CBMqXMiwocOHECNKnEjRoRo/XQYK9IKGEiVQoUKKHEmypMmTKFOqXMmyJUpK';
			B[i++]='h/wAExjIDCVYsYYNI8azp8+fQIMKHUq0qNGjSH/qHCbsVSI/AQL0oTSs2LFkyrIm28q1q9evYMOK';
			B[i++]='HUu2rNmzW7MqS3bM2KtDXnRJenWMWbNnzpgxe8a3r9+/gAMLHky4sOHDiPkyW4a3VrJilPw8iiSL';
			B[i++]='mbNo0GyVusUJl+fPoEOLHk26tOnTqFOrxnWLDWZotZCRWvMn0qxa0KBNu3XgwIMECIILH068uPHj';
			B[i++]='yJMrX85cOQEEEA4UmGQL2jNmp/T8kTSrWbRnpQ6U/xv37VswcN+6BQsXboH79wvAdfsWTpz9+/jz';
			B[i++]='69/Pv7///wDiF98447DQQDkHlNIMNMtkt1130EjzSAQFxoPPNfLEc0874pADwQMQQBBBb+zcEw84';
			B[i++]='3Hij4oossthNAN20KOOMNNZo440zBpBJKb34UYou40Rgii3PNKgdd7U8M010AcTjjz/74GONP/eQ';
			B[i++]='EwEA6lBAATsYtGOPP/owENWYZJYZQAO9gGPmmmy26eabcH7jBxuT9AHIJIE8kMs0zRj5YC3OTLOL';
			B[i++]='Aw5c408+7JwDTzwVPKCOA+a00w4G4gRzTjzwqHjOOpyuc0AA45gzjgPrAGDOE9pEAOeqrLba5h9p';
			B[i++]='9P8RyBeTpIFKA5pMUwt2R95WizS7jGOoP/wcms8+8txzqDZP9nNNAPD44ws51PTz5JPZUPBlPtlc';
			B[i++]='64uqroYrLpxdSBIJJ8FcwkgX42gizTK8PrjMr7yQ04s+/lhjIr714PMkNfVQ82Q70VpDQT7W0NPL';
			B[i++]='vf7U82U/AAvsDzzjVmwxmZgwwsYXgUwyCirtvkuLg5IYQ8sywCbAjb/URMWvv9e8OA6+vRQcATX6';
			B[i++]='6JNNPszeo6wvL4IzpTwXFz1uNz5Oo8oqq4xDgCa2MIOMg5EYowwztgwagMAtB4AvPSxHJc6U8ESb';
			B[i++]='j5P72NMLPd36LG1U4OTjD9FG180qN7qUwjQbLID/4wDUa1FtMi1ZO7C1P13ja0/YoNJstjz+9IPP';
			B[i++]='PNTsQ+XPcA9t9+asevMNN9yE87ctW1Ety1bRDNqNPfrcE9U1+sQjjzauBxAONfls46Q12MCeTTbX';
			B[i++]='UGNNPPfew00A3whPMefMw9mN6JpEw5bpqA8KzDcqRqViNzF6M2YIIHjAbD4ogGD+B+aD/8H6IIAf';
			B[i++]='QggkNC9/m8//Hc0xx5ySRm2y4A+N9ceDEwc00IFsWEMEGuBACUoAPw6MgAQjCAEKRvBAEGwgBPPL';
			B[i++]='YJnqpwlo4I8U2qGM/6z3ojep4AMb+AD4OLABDqQPBRyQYPlQEIIPjOADGhiBBncII+hBoxjFAGFt';
			B[i++]='/2ABxGcM6husUsEJUvCNBZyAG0tMgRRVkAITpGAFUrxiVALIwwz6TRPPAKIQHUFEYxgRAe14hxrX';
			B[i++]='yEY2VuMd7qiGHOdIRzrGsRruiKM72sjHPvrxj4BcIzwCCUh4UIAcYDQGMUixPzISwxjN2AUC4mEP';
			B[i++]='eljykpjMpCY3yclOevKToNzkPELJSXtkAFfN0Akj/0DGYciiFpKcAAUmQMta2vKWuMylLnfJy176';
			B[i++]='8pfApCUFzoEATdRCGMNYpSNewZRlSLICFgCANKdJzWpa85rYzKY2t8nNbnpTmhaQQDGPGQtlviIW';
			B[i++]='wnAmAqD5zXa6853wjCc4xamJZeBkFI18hTBgQf+LZ0ZTngANqEDbGc5iMqMVrsAnIJbpCn76c6AQ';
			B[i++]='jahEAVDQXNACoQptxCtc0Yp+rvOfEw2pSL9pgXQgwKKsaEVGU9EKViTjoSONqUyvaYEJnDQZKQVF';
			B[i++]='HgDRiFSwwqUwnalQZVrTm/pUpzz1aSqOEdShOnWiRc3FMXzaiZ32lBVLbepTtyrQqE6VFZ1IAyAO';
			B[i++]='oVSmfpSraO2qOk761ar+46qfeOlZ00pXeK6jAgLIRTKomod/OIKlrLDEFxBA0boatp2cSkAuLJHS';
			B[i++]='TiDCr58QhSeQMdjCHvay20xsLpDhCVFs4rGOiKwnLCFJy2L2tNbUrCU6+1nIVgISpCUsSFFLWwD/';
			B[i++]='qBYSlWitI3AL29LOtranvW1uQfsJQxSCC5X9LXAvuw4KnNQShTCEbiFhCDIgd67LDS4FBIAELkSX';
			B[i++]='EI9lBHWtO1h2Zje4AFAsF8hgCPD+gxGFmAN5BWDa8x62udxd7xzcC985UIELYECAOpRr37ReAB0I';
			B[i++]='6C4Z9hveQpThv2AI0QUKfN8LJOABj9Avfx08hSEAgkIRuAAGFEDiEpv4xChOsYpXzOIWu/jEF3gx';
			B[i++]='ijFggQOM4wBtqAIVyrDhMjTBB2woAIUgYA50GPnISDayOtKR5CY7+clQjrKUp5wOdUzZyeZ4wIEK';
			B[i++]='wAYozIHB73WwFpIwBCMUoAACGICa18zmNROg/81wjrOc50znOttZzW++85oNYIABJEA6XcgCFb7s';
			B[i++]='XkcUYgxaWIIPcMAGIrQgBi2ItKQnTelKUzoGkMa0pmPAg057+tOeLoKoR03qUpfaCKhOtapRfYRW';
			B[i++]='u/rVsI51q9tgBCKwYQqDnoMhBpGGf5hhEWWQQhJ8QAMZyMAGNniBspHNbGS/AAbKjra0nw2Dale7';
			B[i++]='2cjGgba3nYNuezsHOwi3uMMdhHKXW9xBILe5zS2EdrvbCfCOt7znTW8nVOHeVIACFHJtCENE5h9r';
			B[i++]='EMQcxiAFRfegBzVIuMIXnnAa0MAFEI+4xCfu8BrQ4AYYP/jBb9ADH3jcBz8IuchDDoSSm/zkJvhP';
			B[i++]='gspXzoSWu1wKU3A5E5pA85rbfAo4p4LOd77zMpRhwf1exCEGYgZBKKIMWZBCFJa+BCU4/elQ74EN';
			B[i++]='ZkD1qlu96i5A9sGVsISuL4HpX1+62JeOhbJn4exoT7vas4CFtat9C1vQghbgTve6093neM+7z3Xd';
			B[i++]='b0MsIhF90MgdEjEIOcABDm9wg+IXv3gxuCEMYbiC5CdP+cpfAfJhYLzm34D4N3j+854/vOhHT/rS';
			B[i++]='m/70po+D6lUvh9a73vWEIMQgKNGINWhEIF3oAx7qcIbe+973dDiDFazw++Ib//jIT77yl798O5zB';
			B[i++]='DtCPvvTrQP3qW//6d9DDHjQSEAA7';
			this.images['bubbles.small.safe'] = { name: 'safeSmallBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhfgA3APcAAMjHyP///+bk5erl6Onk5+Pc4enl6N/c3+bk5uXj5cPBw/Tz9PLx8uXk5ePi';
			B[i++]='4+Hg4dva29DP0M7NzsvKy8nIycTDxMPCw8HAwbSztOTn7QQybBJCdzZdikdVZIWOmFl6nW+Mqn6Y';
			B[i++]='s19seZ6yx6+/0L/O3fP3+93h5fj7/vP09enq6+Xm59zd3o2mvvb6/fz+//v9/uXo6eTn6PHz8rG0';
			B[i++]='sqGjoUFCQfP18/f491dXVrOzsvWwCvSxDvW1D/HhuPGqBvCrDO+pDfKuEOypEeyrFumoGOalG+7e';
			B[i++]='u+3du/isC+ymFOiiFeehGOWfGOWkG+KhIu3XrOKaGuafG9+bHNyZINycJOvau+vbverm3+fj3NaO';
			B[i++]='GuKYHt+VHuCWIdqVIN6VItaRI9eWKK+ISOjh1smBGNyTItqQJteYONjIseTbzcR9HteLJMR/IdCI';
			B[i++]='JtWLKKiDU6WDVKKAVKqIXKKDWKiJYaiMZdi3jOTc0ePb0PDq4sB6Ib94Irx4Irh0I7p3JNKHLMmA';
			B[i++]='KpVuPaOAU6SBVKqNaa6SbrCUcLeegMCpjeTd1KlnHa5rIrJvJpBaH86DL51sNpRmNKWEYNB7KMB1';
			B[i++]='K5doNppqOuTUxCopKExLSvb08trY1vn498V3L5xeJ5tsP51vQ+jd08h5NcuCRq1/WN7NvvXv6qtk';
			B[i++]='LObWysRvM6lhLb5uNOXe2ejl48ZvOKhfMKVeMKdeMqpkN9e6p+HUzKxoQLpiNL9kN8BmOsJoPLhj';
			B[i++]='Obt9XcOOcxQSEcNoPr5nPcJnP8BoP75pQr5uScZ0Tsd4VMqDYuTMwebTysNmQMJqRMBpRMJtSMdw';
			B[i++]='S8RvSsVxTcdzT8hyUMh4Vcd4Vsh5V8l7Wsl9XcyDZM6Haebc2Pbx76ypqBoZGf7+/vz8/Pr6+vb2';
			B[i++]='9vLy8vHx8e3t7evr6+np6efn59zc3NbW1tPT09DQ0M/Pz83NzcXFxb29vbi4uLGxsaurq5WVlYuL';
			B[i++]='i4eHh4WFhYODg39/f3R0dGVlZTExMSIiIgoKCgQEBAICAgAAAP///yH5BAEAAP8ALAAAAAB+ADcA';
			B[i++]='AAj/AP8J/GfnUCFDCBMqXMiwocOHECNKnEjRYSFEaAYKTDOnUydXr0KKHEmypMmTKFOqXMmyJcpO';
			B[i++]='kBBlEpgnUCdfvX79Qsazp8+fQIMKHUq0qNGjSH/q/AUsVyNEAQIc6vQr2bJmzrI228q1q9evYMOK';
			B[i++]='HUu2rNmzW7M6a7ZMWS5IabJ9yrXsWbRp0p49m8a3r9+/gAMLHky4sOHDiPk+g4aXWLNknRAl8hTs';
			B[i++]='mbRq1IrJMnbqmOfPoEOLHk26tOnTqFOrPmbMDmZqxJjFMnTHkzBi1KhdM3bggIMECIILH068uPHj';
			B[i++]='yJMrX868eIznzzPEcHDiBKli1KY9o0Xozidh0apN/5N1AsI4b95wfPPGDQc4cAviy1/wjZs3cOHy';
			B[i++]='69/Pv7///wAGKOCA4bwHTgo4iCNOJiqYc4Is0VADDXfegUeNNYmwMM4M8eCDiTzx3ONOOOU84MAD';
			B[i++]='D0DQWzv3xFNfNzDGKKOM3ATAzYw45qjjjjz2mGMApciiDSKyZCMOC7MUM82E3X1HzDTXUBdAPP74';
			B[i++]='sw8+l/hzTzkQALDOBBO0c4E79vijDwNRpanmmgGQU8M3bMYp55x01mmnN4jYQcoheJCSRwPYXBMN';
			B[i++]='kxUSI801rKzQACb+5NMOOvDEQ4ED6zRwjjvuXBAODujEAw+M6LAjKjsHBDDOOeM0wA4A59CwDQR2';
			B[i++]='xv8q66xz3lHHIXlgQUodlqiwyjXEbNfkbcRYw4o4jPrDT6P57CPPPY1uU2U/mAQAjz82lJNDP1VW';
			B[i++]='ycsEZebDS7c2wErruejaicYnnpyyySiPoCHOKtZAI2yF0BTLipv6+HNJi/3Wg0+VOdSTQ5XuXHvJ';
			B[i++]='BPlcQk8NNQRcZj8FH+wPPOlmrLGaoDxiBxZ5kAKLJfPWOwyFnygzDDTGysDNwDlEFfDAmNQ4Tr81';
			B[i++]='KAxBDvrow0s+0t4DrQ01fpOlPBsnnS43RF6jyy67iJPBKsU8wwyFnijjzDPFJBrAwTEH0C89MEcV';
			B[i++]='TpbwXJsPlfvYUwM94wqNbVTf5OMP0krnLSs32cj/ArUdmXyzAtVrYa3yMF2v8LU/YfdrT9mm4qy2';
			B[i++]='PP70g888Oeyj5dB0H63357J2wx43CwxezFZYB7NVNYlyY48+90SFiT7xyLNN7AGAk0M+7lB5iSaz';
			B[i++]='88ILJjlcEk/E99ToTfEYg+68naQPXg1bqa+e6A3ewBgVjNzc2E2aIYDggbT5tADC+R+cH/4H7IMQ';
			B[i++]='fgghkPD8/HNyA470yyxDSx21BZM/NderkZ04oIEO8OISItAAB0pQgvhxYAQkGEEIWjACCIJgAyGg';
			B[i++]='nwbXZL/BUSN/segOZf4XQDu94AMb+ED4OLABDqivBRyYoPlaEIIPjOADGhjBBndoo/utghrJSEYI';
			B[i++]='/2vjiyBOI1HekNULUOACbywABdxgogum+AIXmMAFMJgiFqMiQB5qUHCrmEYQh1iJIirjiDFwxzvW';
			B[i++]='yMY2tlEH79CBHOdIxzraEY5uzKMe98jHPq4RHn7sIzwmQI4wKgMZseBfGZGhjGiwIgbxsAc9JknJ';
			B[i++]='SlrykpjMpCY3yclOYnIensykPTDgq2joJJF3KOMvgkGMR0pgAhKIpSxnScta2vKWuMylLnfJy17G';
			B[i++]='cgLoiMEqiAGMX6CyErlgCjQeSYEKAOCZ0IymNKdJzWpa85rYzKY2t/nMCkRAmMTsxTFz0QtgLDMG';
			B[i++]='zeSmOtfJzna6s5vfXAU0cAILReYCGL4YBjOd+f/Ofvrzn+r0pjCfcQtc1BMPyMRFPvcJ0IY69KEA';
			B[i++]='ECg2hlHQg1IiF7i4hT7RyU+IevSj3KyAOmIwUVvcwqK1uIUtmsFQkLr0pdSsgARI2gyTuoIOeKBE';
			B[i++]='LWyx0pbC9KcvlSlNd3rTnO60FsvwKVCXClGhYmMZO00FTnVqC6QqlalY/adToWqLVNQBD5A4alI5';
			B[i++]='mtWyanUdJOWqVP9BVVWwlKxmjWs72UEBGWCjGVGlwz8qkVJbiAILMYioXAerTlHZVRQmTUUk9qqK';
			B[i++]='VqCCGYAVLGEni03DYoMZqGiFKRZbicaiQhSPlCxlRztNy4ois5tlbCg4AdrAdpS0sAWAaTkRitT/';
			B[i++]='VoK2rA3ta2M72tnWlrOqcAQgzhDZ3fJ2suyYAElFAQhH2JYTjmgDceF63N5OQAZjOENzGbHYR0BX';
			B[i++]='uoBNZ3V7CwC7nqENjuDuPx4BiD+AVwaiHS9hk4vd8/5Bvez9AxjOkIUYrMO48jWrBdIRg+y24b7d';
			B[i++]='BYQb9puFE1kgwPO1QAIakAj74lfBXngCHjQEAQtcQAEgDrGIR0ziEpv4xChOsYpHbIEVk/gCFTjA';
			B[i++]='OE5giTCAwQ0XdgMVlmCH8ozjAedIh5CHTGQhr0MdRU6ykpfM5CY7+cnqWMeTlXwOB5ADAiewgxb+';
			B[i++]='gOD1KrgMUXjCFQpQAAEM4MxoTjOaCaDmNrv5/81wjrOc53xmNtMZzQYwwAAScIACoOELYOCyeisB';
			B[i++]='iDWUQQpLIIIdoOCDRjv60ZCOtKQn7YMjWPrSmL40EjbN6U572tNWCLWoRx3qK5j61KhOtapNbQkr';
			B[i++]='QMEOXgj0HxyxiDr8IxCTcAMXorAEIPzgB0IQAg+GHexiB5sHPRi2speN7B4429nGDjYRpk3tIlj7';
			B[i++]='2kUwgra3rW0neNvb23ZCt7/97SeY+9xVSLe6183udlchDPAGgxa0IGtHOCIy/zCEIv6wBi4gWglK';
			B[i++]='CILAB05wgQMBCDtIuMIXzvCDBwEIQ4g4wAE+BCUs4eJLYILGN67xJnj84yD/eBRGTvIpmPzkXOvw';
			B[i++]='wsmnQIWWu/zlXog5GGZOc5q7wQ0HtvckIDGQQChCEm74Ahe2QHQpGP3oSJeCEoSQhKY7/elO30Gw';
			B[i++]='AZ70okuB6FjHehe2/oWue/3rYP9CF8IOdjOYoQxlMLva1672m7v97Teftb0dMYlGHEIjcWjEIvrA';
			B[i++]='Bz7sQQ+AD3zg2aAHNaiBDIhPvOIXTwbDq0HwkN+D3/dA+cpTvu+Yz7zmN8/5znPeD6AHfR9GT3rS';
			B[i++]='M4IRi+gEJQyhEYGg4RByEAQcZk972r8BDmIQQ+13z/ve+/73wA9+8AcBh0EY//jIF4Tyl8/85seB';
			B[i++]='EIXQSEAAADs=';
			this.images['bubbles.small.unknown'] = { name: 'unknownSmallBubble', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhTQFzAOZ/AP///+MeJufm5tjW1r0rJtUkJswdJFAti7g2K+U1K7UtJ8snJvfItblJN71p';
			B[i++]='VsZcRq+pqeZHNMpINdmbh8g3LOMkJ5Zyo+yFaPvd0ep1Wct6ZNUeJfa7pehkSce5ttijlPzg0+W5';
			B[i++]='qPrazOnIutg4LfnTw+QrKdSCar6WicpqVPW0nfOkiteMdudRO8VxXpibntq1qNrCuPCdg9sfJfOt';
			B[i++]='le3Sx7gcIa6Jq+6Sd+hbQuGrl9pJNm9Llb+povbBrcdhS8spJvXo4ezNwcSlvMSzrr2IeMUpJr+i';
			B[i++]='mPDZz9dtU97Hwup9Ydza2tB1XdljStm9s8FRPO2McPGYfdSRgMsuKLgmI7c9L/XTxNYuKdCtuvXX';
			B[i++]='yuuji+ltUdAlJtpUPdQoJ70wKOeyn+EeJsN9avr6+eLg4OXj482Hcd4fJqCfoOzr6+U+L+e+sMId';
			B[i++]='IvrWx9ciJf36+vLy8ubl5fv49/nOvdrY1/fv67UxKNiunvPNvs+gqcA/MN0mJtE/MPjz8f///yH5';
			B[i++]='BAEAAH8ALAAAAABNAXMAAAf/gH+CZlMPYIeIhwgUi3uODRISUA8PPz8pTU0aJyydExMfOjphYSEh';
			B[i++]='bKgjqkKsrTVCNbGys7S1r6+sqqpspqQ6H1ufU1OdJ8YamUmWD1BQEo57i4lgBNXW19jZ2tvc3d7f';
			B[i++]='4OHi4+Tl5tsKDkGC7H8fFEZf8lhfWPZYJPl9fTv9Xl45nDjhkmHJkgtRouCQsWULDRphOLDx4WME';
			B[i++]='gzx5rmS8wvGKlo8gQ4ocObKjxjwMGEzkECLMw4YypOBIeOGEwQxJknRw8q/fPhL35AktQLSo0aNI';
			B[i++]='kypdyrSp06dQo0qdSrUq0i5dFhD4wA6OCyML+PCpUMGEWRMJ0q5ZGyFCixY5/zp0IFgQIQ4pDB+q';
			B[i++]='4MCBYko6dEoILuGmsIjDiBMrXsz4cOHBgFP64KvioYxOMmleWJIhAxcuHXLkaNF2TdoEZsmqrhCg';
			B[i++]='tevXsGPLnk27tu3buHPr3s27t2/YYtDM6GLEAZw/U+KNJXv2dAK2buHG/Vw3oRQpK1bQUKGCx4Hv';
			B[i++]='FgAP9v59iAgiL9KrX/+CCAYM6Nmzh6CESBoiMQRHZjCZg4oJGoxxxhkONBHFBZtx5plco5G21mmp';
			B[i++]='rRYAa7kZYMOFGBoww2+4iWGhAWhwKOKIJJboW3AFGDHFH3cAsdxZaKkF3VtxzVXQQQrhpd12HFjw';
			B[i++]='HXiDXeHjj0PEIN+RSnhw5P96aQghggdpQBCDG4DRIRlfNLgwAgBc2jFFCggq+Floo5VmGmpnSUgW';
			B[i++]='bhs8wOWbQbBggBgmxmbACQCcYECdfPbpp4hoFAAGEmB8wVxzMrZF43QE4RiFjtrtxcENP4JXmBuU';
			B[i++]='VjoEBC9AAMJ7IIQqKqcecOree++JkKoIbojQSh4l7EeRfyx8AEAQH8DApQs3OfGDEwy2sMNPZ9Jj';
			B[i++]='Fh8FjIXGG8yCGMAbBjj7Wpu3mhICl3oGsMEGaBiwQQDdRhuitt+Cu21r2woXrbMGNBFCEyBuMMMM';
			B[i++]='0b4WLbfy/qnvviSKMYOKYGBRllnOzSidjdXdJUOkkupRqaUiZPHwAZu+4IH/qBh/SiqpqHac2KWQ';
			B[i++]='WckfZWMIAcAZJzSh6wRJpPAAC1NoAAVAkpzBwg9UkACEJWHt8cMefIDBDBQn7GEA0UXv6Rq1IbTh';
			B[i++]='dAoANG0AJWCc8IABYDSRMhjeUqKh0FBscLQhexjTBBjy7vHAHjOo7UjKGoqt9dprb8jv3XjnhkYX';
			B[i++]='UAQ88GlrraEoXHLRhWCOC2+3lw8hkFepBa5OTDGpGWO8scWheoxYq26ELNlk3J3wxK0AOvBAChk4';
			B[i++]='gMSbbETSxJsATEFFFVxW0QWeJ3ThJhJBAJACnlwGkYLS2roZAoZ4ymkDl1uyAEXvwUPRxpZQGMAC';
			B[i++]='ACO0YUPvUL85Ate43xn1/5sTRDuB93kSn/f67LsmRhd7BEww4G1FV2Ojh9+VHY99+eD44yL4H5HQ';
			B[i++]='AwEPGPCAB0yDxUylhAY68IEgi5WsKEMDGbhgdVxCwhlS8AMdxK4B1ypa79bWuwbQDgBVWADudMcl';
			B[i++]='HUxgDwAgFBRiaIOluQl2MQTDDJaHPeet7gQ2eB0SbIC76XEJDDPUwbVSYAMdIGF44cMToW5ogxki';
			B[i++]='wRG9y1b7tsi+AhBAfoATnKJEU7gb2WVhDOvfkL6TqQNYYI0HaOMQlLAk9tSAU3VMDwRCJrL+cIcG';
			B[i++]='W2CBCyYAAwxw6QxQsAMAmuCC64XhBwBgAwUo8AArzK52QFjh7pyGpwlQgv96dqNWEKzVu/Lx0Ggz';
			B[i++]='zB4ajAgFGH4Pe767XhPOFwQdJE18egofD4kIADlZL31cDCbevAjG57DlLYSzkUHsgp00UqSNcRyC';
			B[i++]='5KJJJCV1CgLYzCY20xMDPGrzm+l5ggT7CLrKbMEFw2DkD84XghnikA1Qk+QaKJCzE1Yhk3lagPHa';
			B[i++]='cDscAuAB5WKa09owKABAYZcCDUAbrvUAI+LpdRPY0iHO9yYopk+XXOKlnnCpPmF6lE/ExEIYB3e/';
			B[i++]='zuBIYc70gcN+ZIErSHNiFgDBSyeHucqF6nLuUcxjetCeKn2OghUcAxkA4IImJOEM2HueQRswiT1Y';
			B[i++]='cREsmMAPFIDJKeRzn/3/ZMGFHmAD4gnUaTZYHVcz2jYabiCsADCaFAEAhhCsTgdteAAQH+DBpkUx';
			B[i++]='fbt8HVwXCsyP+rVOxCRBouw3HZMyE43c6R8MKsWDwsz0RzwI1UwrdjGbguByHvjYpdyAHiKIjD/l';
			B[i++]='fEh2tBRDUCiSBQ84nw4u6Ls9sOFkTSghAbKYgiwuAJJsaAMf9hAE4eGpaWsSpbVWN8RdKvRaOkgt';
			B[i++]='AOBa1qi1AXhN4Ksnz1e+u+ppl2Ao5QSy2NG/etc3XkSAYI1J2Lng7wI4WEh2EtsXNjiOByM4zGMP';
			B[i++]='wAMkSJZIlKvcezDrmMcMhqdE+Kl/KkOD7MjgBA7Y0ps+8IAOQOG1XJoA/z8eYDIuaYAKVNAAl4Rw';
			B[i++]='vhPsLGpGqMACYptBKLyBNRV4ww1hp4PqGXcDbX2TDtCm0C3pyZ1oA4OCsQe+i+I1o0db3QSup8Xv';
			B[i++]='Grk34RWswUq6zBztj70+gGMWWiWC+SpBVJNV0n0QeED7WAw9WyaCmMfs5QDPasAPKXBMcHCCFDhA';
			B[i++]='AxqgxGe8AIVLOGMHEeiD0H5GhbVQoQE/IAAQ7mmCBdjACAUwgYj3bIMuqGZCFsLQhdrwhgBEWmkb';
			B[i++]='aIPb2lAuA2gPRJfWVhsmIT1ufahbXUV1V8t20F8W+ciwxo0XKaBkMR6MOieFVGIpAscbJOaxQ8DY';
			B[i++]='ZGugwDxCIBbFrmMaKP/iRwIbWCY4CFNOPgOa0PSENKTJMwkCdxqRpgVGMKoAsgqwmkevKTczyJdu';
			B[i++]='tmW325Q1TizoHRTaHet60ybJSy5jBvLXTP4x7o1v1MOlDqMEgAcbYwV/45VrwOWGX6EwQiCzxMV8';
			B[i++]='Zv+k+dkzOdBmOjMmMjUo2xEQXODO5JxvoylNii63yidEIb8aIAUKfmJ37U1z18x627YmI64RhFK9';
			B[i++]='9O8vgeFIYjAgKlBZ9lOa25x/9ePTz/UHzaKVwZppchPPVFs0H69fyEdOcgidHNwpV3m5WU52j3o6';
			B[i++]='QzVPe2y8uAecE1YgOcmEMToxhU984AN4yDsM9v4EGDzh70+IgeAHT/j/wg8+8IBPvN/3zvi95/3x';
			B[i++]='v7j7Bz4hDGKwAGVmS0YKnPAAzj+Azl6IhOj7IIF9mJ4CfZik6ldPBXpi+PUYBkLsqQAE2dee9rXP';
			B[i++]='PRAWwPve+/73wA++8IdP/OIb//jIT77ylz/8LhSA3q+ZdR9sDZdK/MAB2HeAC8YwhiJ4vwgoCD8K';
			B[i++]='jjD+I5i/B0fogfrXz/72ux/96De//Oc/fvHbHwXg//73uc99F7gg+9hHCUMDBQ1QgAbYAFaQgAq4';
			B[i++]='gAhgBQjwgBAYgQ94BwhwBxZ4gRiIgQpwBxuoAB74gSAYgiI4giRYgiZ4giiYgiq4gizYgipIAAvQ';
			B[i++]='BdAXANJna//wAA7w/wE1MFT+1IM++INAGIRCKIQ8+CZDRQZIWIRDuIRM2IRO+IRQGIVSOIVU6IRz';
			B[i++]='MAJTAAUwWACwwXbb1hZ0BgUo4AdlwAQDcIZomIZquIZs2IZu+IZwGIduWAdyWId2eId4mId6uId8';
			B[i++]='2Id+2IdMUAZ2cAYEYARdEH0EgHqCswPNwAYCUAdMYAYCMIlyMImWeImYmImauImc2ImemImVKACh';
			B[i++]='CIpyMIqfeIqomIqquIqs2Iqu+IqwGIuYaAZmKAAwcAdG0G5euIgN8AFmUAdloAZKWIXEWIzGeIzI';
			B[i++]='mIzKuIzMCIVkoAZlKAdTQACH2Bo3JzgS0ABkUAdmEAfN+I3gGI7iOP+O5FiOQhgHv0gGUJCL1kgA';
			B[i++]='CDB9EZCNADAAAmCO9niP+JiP+qiP0FgG08iFNJiIOCeP9LiPBnmQCJmQChmEZECLNaAA1Yhva0CQ';
			B[i++]='9biQFnmRGJmRzEgGAjAAQaAAC9COtPYcFKmRJnmSKJmSQNiRAACS7Shez9EHDTCPFamSNnmTOImQ';
			B[i++]='LOmSAQkGStYHVkCTOTmURFmU4biTIdmT4wWUQmmUTvmUUBmFSNmOYEQBQVmQUZmVWrmVsDOVPYkF';
			B[i++]='aGGVTcmVZFmWROmVIRWWV1mTZtmWbmmSaPlFYGkCjDCWb3mXeKmTA9CSSUlMX2AWdYmVeTmYhGmP';
			B[i++]='cVkoZUEFCGCXhdn/mI65jIdpKBWgmIz5mJZ5mVMYlwQgmVRwB5WJmaAZmkKomZzpmYIpmqiZmv5E';
			B[i++]='mqzRmZ+pmrAJmppJbhUABKbJlrGZm5g5mxMCBFR1mroZnI7JmwGwAL+Jm8KZnHlJnL75msr5nGVJ';
			B[i++]='nMbpnNBZnVkpncdpndoZnXvJk15EAAA5ncC5neTplNhJneWZnip5nuOpnu65nt3Zl9UQntn5nvZp';
			B[i++]='k+yJnPe5nxeZn/z5nxnpnwA6oAopoAR6oPtooAi6oIYZn+0Inq0hnvrJoBT6jQpaoRjajBeaoRyK';
			B[i++]='jBvaoSBahR8aoiQKhSNaoii6hCeaoiz6gyvaojDalQ4akBBanPUZ/6M46oMvmqMtuqM8mqI++qMl';
			B[i++]='GqRCGqJEWqQdeqRImqFKuqQV2qROyqBQGqUIOqVUSqBWeqUAmqVayp9c2qX3+aVg+p5iOqbqWaZm';
			B[i++]='Wp5omqbbuaZsap1u+qbQGadyqpx0WqfCead4qpt6uqex2ad+qpqAGqioOaiEGpqGeqi7OaPfSZ/o';
			B[i++]='qajumaiQ+piSOqmNWamWSpiYmqnLyajzGaE3yqlh6qk1KqGi6qWk6qjteapqmqqg+qis+pwcWQd2';
			B[i++]='4J2faqOwGqvJ2ZB18JAReaumqqvq2Y//+KD0OQcD0I3CWp7oOADqyI40CpBdQAATQIvBOIzLKpzP';
			B[i++]='GI0TQI02d6v/cv8HMPCIkSiL5nqu6Jqu6rqu7Nqu7hqLtFgHtoiLunirNGgEBDAGZGiGf9iv/vqv';
			B[i++]='ABuwAjuwBFuwgFgGfjAGhViNAfA+9kqDWgEFU1Bh2aqcQpCFW/gae2MF4EknrjEc+OqCIjuyJFuy';
			B[i++]='JnuyKJuyKruyLLuyC0tv/mIEGgCR4/IaM1AAWcF8OruzPNuzPvuzQBu0Qju0RAu0zjeDe0MAGoRo';
			B[i++]='aOCxave0UBu1UlsbKGIEZ/AHXmGIM9C0U9u1Xvu1RxYcIGsc7PABYGCIVpG2aru2bNu2bvu2cBu3';
			B[i++]='cju3UEEcYMAV7fAHhPAA59C3fvu3gBu4gju4hFu4hnu44ZAO68ADDoEAADs=';
			this.images['dialogs.alert'] = { name: 'alertDialog', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhnwA3AOZ/AP///8Q4LOnGuPbJt9UkJua5qMRINuXk5MseJMd5Y75DMr4rJ1Aui+MeJsda';
			B[i++]='ROzq6swnJtmllPTz8+Q1LMEqJ9aMdfDu7t/e3dibiLsrJ9WDa2lHlNZGNNjW1nhWmb9bR7UtJ+d1';
			B[i++]='WcdrVPO6peqGarsyKcdjTOKrl/Hh29VaQ/Dd1bSpp8x0XdthSLocIfOli9VzWuqUebUyKfOtlfj2';
			B[i++]='9t62qedkSvTSw+vNwNiSfO6ehMVRPNVQO5ibnsOwqtR7Y/WznOmNcvnTw+eynro9L82Dbea/sdYx';
			B[i++]='KbklI9hpULksJtrY2Pbp4vfCruzVyup9YaYnI7o5M93CubchIuLg4MepoMIeI8koJr1OO+ejjM0x';
			B[i++]='KdusoM6JdfDZz70tJ9y+tOlsUNza2sk+L/HOv8NdWPGZfufm5tCsuPv7+vnOvb8xKeXj48Sit8Ew';
			B[i++]='Ka2JrfrWx97c249sobg3K/Li36qgn/z59+7Br95cRNrY1/39/cVoY8t1cfLx8M21rt68wf///yH5';
			B[i++]='BAEAAH8ALAAAAACfADcAAAf/gACCKDkmIiw/GhU5GCcnQwVGdgKUOJaXmJiUm3ZGBZ9DQ44RXEVF';
			B[i++]='ORUVGho/LDAiSSYmDrQ7trYGuboKCkRyMsAlAWrEbV7HyF4Ly8zMGSBIzdLT1NXWCxlRGDR/3YI1';
			B[i++]='DilJISEkQTFZL0MjI3YDA2NjN/P09fRjafHvAk12duwFhsw4UQFDBAwiigQhQeIHuSRJbLS4c4eH';
			B[i++]='RR4cMmrc2MaUKQUUjhwhMJKAyZMoU15h8QFCypcwY8pMCeGKAhXeauxIYQPME3NldMwAMqJJkwFp';
			B[i++]='0gjxwMDDmzdxGDCQ0qOqVR8+rFqtsqLKCCAzKkRAEcEJDhEwPnxoAaPFDgU7/3gY4MBDDAcxcgKI';
			B[i++]='EaNG2IS/GQBg0HCiCxIIVqYgQOBCMePFLiAzjoyAhYPJkRtMtoKggefPoEOLHv2ZAIQoc/6g2HkH';
			B[i++]='TIgn53S8mFH0aFIhUZsKcSOVwYoVb4QI97GiR593A4yOKACk+YwEOKQkSICBRQIVeSI4EKECgBEs';
			B[i++]='RiqICV8CAI4cDroAyKHlrxIAJpCIYIJEgwAAJXIwYVJBAQAiLACwAwtdENhFBVYUoEF66mlgxXxM';
			B[i++]='nKBBZ6RVaGFpEJDxBxc83NHTT0GUMRtRtinFm1QenNGbbytI4aIUvxn3RQ00/vMVWDPkUAQKeTjB';
			B[i++]='hQkqYPCBChWokAMWNZgQ3v94FZR3ZBcaEAFle0gIIogGV2jAxHZM8MKECAKwgAETGmBQQQEnOKBA';
			B[i++]='ggsKKAIALkBZApkUXmjnaARQYMROPb1GQlAvkGibECoysIGhKErVh1ZVFcdoD1t8NcMML2iQgAjU';
			B[i++]='AZBAjwIYwQUALPDAS3gBeFqeCeXh8MkPVMI3xQ4CalCAFbJawSZhTDSy5Q73CVCCgg7AGawLADjA';
			B[i++]='Zp13JuuZaXvw6RpQIxaFVBoCHOoBG71tgINUvwknRBrEGWdUEyPQUQWOL7wgAgo1fLApC0FikcOQ';
			B[i++]='R6rAghE5KHDgqV5AqYQGBhwxQZUmvBrrrPN56cAO+ymwnwssvFUssMLCKSf/ncpmvCxcHvpZhoi0';
			B[i++]='GTXtodpiK5Ufb3DbootfxOgDjTVsYe6k6epgKQoAWICBITgLgAWQ3inAwn44aAACfFqkB0ABcvw1';
			B[i++]='hZXrIVArAhXsN2GcGGAtNRMAIEixC8NCCCWyGttJQAY7tOZnDLKF/E4auZ3xhslsPCXVF49i9egW';
			B[i++]='lKb78Q8mqDVOElhgsZOoRNiFVwlqaKGFEu1psQASITUwwRVTZK4YAQhYQYBmLmT22GOch74YZ6NH';
			B[i++]='loNlBZJdtoUELLBDC67BxnYWofgjADzy2HODPPHgEM8YA1Dijx2giJLF8hjEkEMQFZDACgzUQ9RC';
			B[i++]='CthbNNdde3WvhRiOhx/+/xVakA9B+VekD8H67LfvPvvpx08BFhHUUMRh7+ev//smgRa7A7TzEyMi';
			B[i++]='cIIaRGITmTBLJiyxCSM48BM1EMUJInAQDOQAFRooQgJawQIRiEAWavlA4QyABV70ggi+8IUcVigD';
			B[i++]='FgIDGCCQAQhmSMMa2vCGOJwhFHYIhRz68IcgUAIFrvC5ZS0AgLXjAgacUAeoOfGJUIyiFKcIxTxQ';
			B[i++]='8YpYzKIWt2glFXBBDguAgBGRCIMfbKEOFniABdbIhzbyQQJwjKMc4ehGN67xjg/I4wPMsEczmOEA';
			B[i++]='B1jDGqhASCpc4JBwSGQYFrmERjrykXjogCQnSclKWvKSmMykJjeJySUcwP8JWFjA5/53hyTAIAJp';
			B[i++]='tMAbJUCDVrrylbCM5SvnWMc72lKPuNQjH3fZRz/68pfADKYwh0nMYhrzmL6kQhioMAcsUMAkshNH';
			B[i++]='EerARwlYkYvYzKY2t8nNbkrADHCgggCU4JJomkAA1eymOtfJznZmkwZmWMIDREABCMjOASagwRos';
			B[i++]='gAZ3+vOfAG2nBA4ZgQxc4Z4fAAAVLBDQhjr0oVdEwwGWoAIhLsAADkjoQiHK0Y5C1AwdAAAIKHDR';
			B[i++]='HQRrox5NqUrZCVKRktQAJlXoA1ZK05pis6UjvShM83CBmdr0p0CNIk5JqgASAqCnQU2qUoe6AAWI';
			B[i++]='AQsAgINPlUrVmjJVDLz/iOpUq8pVjzI1AFkNw1a7SlaHfjUARACAWMvKVrOGNKcBQCsAlmCGttr1';
			B[i++]='n0zNS1rDUNe7+nWdZ00rXf9KWG4yVQ0BkINa+1rYxm7xsGpQ7GAdS1ksHrYNkmVsZTcLxcvKYK6a';
			B[i++]='5axoBcHUNrThs3gI7Wg5y1QvnBYAHVDtaivb2gV8NrazXW1tj4bb3Iq2tV7grWx9W1imLoACwiUu';
			B[i++]='a99K0uMmV7m0Ze4ykAvb4UL3rsalbm+v29jsPpe7xZWuc6sL3u6KV7vWLW9XJUpRizq3DrHtp3rv';
			B[i++]='OtALFPSgC7hCBrZgyLHOt6vwFCs99WvQK2CBBkugwgPk+1+uwjOc41xf+gmUYJoFJCCeF+CjLTfM';
			B[i++]='4Q57+MMgDrGIR0ziEpd4jhK45RqW2cx6ZkgPRDxoAmhAhQ4sAQ6HzLGOd8zjHvv4x0AOspCHTOQi';
			B[i++]='/7iQhcxxGBp5ABU40yV6QoEXTAIBL2BhCzSwADK3zOUue/nLYPYyj335AD6g4IthJMAVPtCNLWSA';
			B[i++]='yvqVwwesM5062/nOeM6znvfM5z77+c+ADrSeKUjoCNTZBFhQQn6pjJpu/GELXiBiTUiagUpb+tKY';
			B[i++]='zrSmN83pTnv606AOtag5fUNLL0N9NbmJo7uhgj2UgAJDhPU1Zk3rWtv61rjOta6XoYRe+5oZsL4C';
			B[i++]='SaOQA250IxAAOw==';
			this.images['dialogs.small.alert'] = { name: 'smallAlertDialog', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhzAH6AOZHAOvr6+Pj4+no6OXl5d3d3eHh4dbW1svLzNfX1ubm5dnZ2dXV1d/f39fY2NXU';
			B[i++]='09zb2+Dg39/e3tTS0tHR0dPT1Ojn58zOz9ra2dDP0N3c28PDxOfn5s/NzdTU1NnY17KyssfJytLQ';
			B[i++]='z9va2tXW18rJyNrb3M7Q0c7PzsTGx9LS0vHw8O7t7b/AwsfGxsbHx9na21MyjcHBwby+v9HS1MjH';
			B[i++]='yJyeo97e4NnX2Ono57++voVwrN/g4LSuwvDv7/T09MO7083NzcnJyczMzMvKysfHx8XFxcbGxv//';
			B[i++]='/wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5';
			B[i++]='BAEAAEcALAAAAADMAfoAAAf/gEeCg4SFhoeIiYqLjI2Oj5CRkpOUlZaXmJmam5ydnp+goaKjpKWm';
			B[i++]='p6ipqpgfra6vsLGys7S1tre4ubq7vL2+v8DBwsPExcbHyLaMrUA+Pj0q0dLT1NXW19jZ1Sva3d7f';
			B[i++]='4OHi4+Tl4tDl6Obr3Ovg6u7x8uvOQB+KH80qPSv9/v8AAwocSLCgwYMIAwJIeHAhw4cQIxZ0KLGi';
			B[i++]='xYsYM2rcyLEjt3qtDuXzsa8fgJMoU6pMuWIhS5QtXcqE+bJmzJU0cea0qfLmSZ8/d/acGbQo0Jo6';
			B[i++]='XR4VWlRmzKVPh0qleXMpT6REdVptqpWr169ZkyYFenSrWK5RmU5VK7WlW6dw/1/6U+HsnqEPJM2e';
			B[i++]='RSlgr9+/gAMLPtl3sOHDiBPzVcy4sePHkCNL9rsC5N1neidr3sy582LPoEMPLiy6tGnKde+qyHy6';
			B[i++]='9WPSrhPDjk27tu3blOnaJfRhNe7fwIMLH068uOEVPXzsHtSbtfHn0KNLn04dJ3Llqp3vtQGje3cd';
			B[i++]='PXDq8N69bwIZNdKrX8+eBwKUJ9jLl6+BggweJEiwqE58Nv//kPlH3HXLCdJcYwOMR54OONVAXncL';
			B[i++]='zSfhegMAkMKEEvIAgAA55HBfDgCGKOKIJC6UXIFHHMiYggv6oBIPD3Y3AAbt1cCDjTjeeGN6MQBQ';
			B[i++]='o4455pjASSzkQEIOPZao5P+STAKHnAooqpiYgzEymNIMMUKIXg0ymJTUBjmkp6F6EOy1Ag5Npqnm';
			B[i++]='mrSdeBc/iv2QJQw6uHgSd3MOwEJ6MqC5kp8nOaDeCuoBihKahrKpqIgCLurZk1H6hhieWVoJQAUs';
			B[i++]='xgjAliDuFZ+NG6jXWaOORkcqTqeWSuJHkWq3EqWV2plpjAHsyaVfEgyq3pCR8cCiDhbUNisMGnY2';
			B[i++]='Xgmqzpaqqk5C0ypiwy54EpVzwrCpmH7FIGao6UUmgIM6DCDADONhEFsA4wmgbg/jFfsaS64yK29w';
			B[i++]='BGZ3GLVz1inng7MGQCOGEuaAw3o86GiwjQY78Nd4dq5wAJ0qxDZeStzpEO//vLctizFi9RYiJWAV';
			B[i++]='wPigrwvC+p3J0wI8XwLcqsyeuWdVQGdP4QFwA4vFrkAynT+sMB4BJ8mpgwrjKQjACzivhK61KJVA';
			B[i++]='Z8h0dsfDt94x2MPMJ2FtAQw13Fz1QuPZoAKdCuoAcwJlT7zxaRo32TFvq8W774JLVz0ssg8G8ILL';
			B[i++]='8rXgI9/s+aVgDTjxDEC7ADitQwk3dEc0DMgCIGcNPtM5AA8rGI64SmqfpIMOAvRApQ8ajke50AnC';
			B[i++]='ELHI08Jgw3jBwlgDppAfbvnDMLg4Hgi0M73276G9zZykgAVQpQCPVwvDDAAYT9616UGwQQLUV88y';
			B[i++]='9RnsyEMC6pGwwQY4fA++//jfe6AeBWf1k6kONrSuITcQI54cN2EHHT8MPftI7EJjW3pS3SUTgOxO';
			B[i++]='4gOsiY5YDpoB1WCAgQJabGbqWlf9ygOAx9lggH2BEPBU0rY1dfAvwjMQ8RYWoxFUwHbV+gFKHqSn';
			B[i++]='9OTgKLO5UHoIlR5eqQRRKNmRCQDzOe8wr3O2s91sjmW/Oo3nAZ7znQCAmMQKqIt2OgiA7ASgM66h';
			B[i++]='pGJCI5rpMleD1H0uYknEW7haZ4FyoWQ8F9vLBzeoqH5AyV5/WWKMVFjBaHWHcP97EPQ6dZZP8aBl';
			B[i++]='fznfX6KRnKKlLkYF0CBKLGi/GrDLdQA45IPah5ICdG4FqaOahqDmrszpAP9ToFve52DUM3aVbQaP';
			B[i++]='bJ/aKveD+oHNd2yMZWRYBUe/4EtqKbEj+1LCQlvJAFe6qqFfdnSCwVwtdxOL4BlhABQiAgBG4DlW';
			B[i++]='S5Kprg2thInHmuIz6VSzij0SRo9zUM1WUIG6PfIFAKDTBlr3Awf9oGZolKU8H9OOZ/llbl9bpB1P';
			B[i++]='6JIB6HFLv9wLCcTEvfRUKH2CFMsAczm08dCRdpVzFwB8MJ4d6q+hkHzcQ5m4tJTIsQIOchfWoFYD';
			B[i++]='FYjMYvvy30IoqrqwbUBtj/zB5E5yzHmq0abpc5PHwLidKlWgZkmM0QCAmjXyDIAGBDtYwRC2HhkU';
			B[i++]='VExMXapU13OBvRQNjBX/BcAj0deuFVSsQg6FZg/2RTlnbtV2mOMcLB1EOW3ajnD78kE/7tiDrRGr';
			B[i++]='JXS6AQAwMDMi0olXDpXjD7tDVJwaFjCQqmVS/BkjfupzjkBhrHeaB7j1RAAAGqgstmIWAHzpgHmR';
			B[i++]='HAGLatCXHrygaqR9JNnuR8kBiNY7eEyJJWNEWpDur4Is/Q4YU8e8sTFNBa38mgVfyjRC4U8Avevh';
			B[i++]='YR21xuPoFG6ZGZYNCotC2C6FhQC4gGZJgJItAS5RYjFJX5TFDZ/sYwABqEBL+FGBAZSXH1AZIUra';
			B[i++]='W80K1uyJfWkJXd4yX34+6SnohU1L+lIhoJIGuQNIpEqr09zlTiaxO81M/8EmzAOYqaQHM6Bw/lYi';
			B[i++]='QAqn5AQgDrGIR7wSCIz4xCGGwJlok8aV8Lc20IyQFR1sTRpr5bnDy4yX4piq+jKpwah6lGMq5x0e';
			B[i++]='HNSjNk6yibATYb1410ZHNkF7ZLIj9RQTJbGdVkqeDGWUTMBdE5APHzlQg6qeBD1H/g2QW/wYuqxm';
			B[i++]='NUBWMhtpGeG91CAAEawB+k5Cge3h4M+A2sGd/ywADQR0WoXpSw0AxYI+AboGCuPzH1OCgy8nIFEg';
			B[i++]='qIELnlmDB7DZMXFeVKjlLCIIQ/cs35JBo1nAXZRcqEMyyIEE4CPRAaCTMFkeWEpiwAMK+NoANZj1';
			B[i++]='SU4g0WEXGyUowNGtSf89aiU1WyfPZlYIUyRfnEQA1mGigatrYIAUpIACDEAJBbI8AAWkZNHnBlSY';
			B[i++]='gEQ5V29PJReyigtqsIDnRBvUgbk3ifSdb0eZenhqjC2hNOBunTDgzighgbsGtuwRZLlDJiHUshGQ';
			B[i++]='5UAdGyWhInWQ1axxJU37Y585SZVtBLSTyHA9QOXyAQDFZRmQJtZo6gsPrgyALwPAUCmYtE7QfVM2';
			B[i++]='8XtUHTfOs531Jr2E73uApvT40HQTARyd0Bx0OvjO8mdKg7fqOgHv736uGB9zPegcFMvXtcJk3sCJ';
			B[i++]='w3xJ9M1rPN62EybkoAFUorg+drBvpu6kJrrHzv52uCNZQAeezHgJM/j/Gkumg6fit380pna2HWbU';
			B[i++]='eO93iSLv4rIzh+/1jWDmCd92rxN+MZov/OeRTPq+b9zwaGe73x8velSjKvBJgQ3gGaOs2Icc8LIH';
			B[i++]='THMbj/qNz972YU99kHPP+Ma3/vScr71gRs2NKMFJ89YMfeg3hFzqS//6yox+8mU/eOhLn/q9n33i';
			B[i++]='U6/MxZMf9OYf/d9Nb/rcx7780U984JV/+/YLv/OfOTCp1N4o96P++OxXesH3dj6GfAXYRpZnIM+H';
			B[i++]='fQzYgA74gBD4gKDHeaoHf4UneonGf7zndtXEfdZneN43gBjYdwcIgh7Fe+2XfSKYdmnXed0XgsaX';
			B[i++]='fSrYfS3YgS1ofZt3/0LQB34fKIP8J4Cvp38UqHrgB4DQxn6wh3yuJ3y213x3ERMRGIVSOIUS6IJW';
			B[i++]='iIPc54Pb54JYiH8d6H1f+H3YN4Tb14ViOH0YCIZhaIU2WH5quIZumHkvWIbT14Xxd310qHZOpHlO';
			B[i++]='9HlcqIV1+IfG14N+SIP/V4E8SID2t4IlqH0BmF8JmCIb0l4JEAADgF6WGACauIkFUAAQAAEMwAAE';
			B[i++]='EAEZ8AAPIAIXoAAecAMIgAAGgAAOEIuxKAG0GAISYIshkIu6uIu82Iu9yAG8CIwhIIy5SIy6CIzG';
			B[i++]='eIy+uIzMmIzOyIzLmIzDGIzQWIzUuIvSWI3TmI3YGI3aaI3K6I3d+P+N1WiMz9iN3GiO5MiN5EiO';
			B[i++]='t0iL8BiPEiCLsdiKCHADHpCPIrCPGdCPGRABEfCJENCJBBkABbCJlniJl2g9DElFzmcVa+QWAtED';
			B[i++]='FEmR+2CR6GCR+zAPHNmRHvmRIBmSIjmS3wAP0mCSJHmSF4mSF9kPFZkQShEXf4FjBjJgVBiBTlQB';
			B[i++]='OvlEOpmT+HWTQBmUQjmURFmURnmUSJmUbKiUddh7QHgoZGgYkTd4Trh3ONiTWIl9OMCUXNmVXvmV';
			B[i++]='YHmTG6AuYxmWQNmDZnmGX2iHKsiWYUiCcXSEi+gX+FeVvGGTQ7mT0reHD8iXW6l5fwmYaTmYhFmY';
			B[i++]='hnmYSVmWXPmU6xf/gD33iMC3GHbJHHipl/i1hz6JXBHElz+5l9dnmdIHPoEplg+omJ+Jk5uJmAJg';
			B[i++]='mqrZmqHpml9phmM4f7L5gWhZhFuYgYZom4FIGpNZk9S3Ae1VAcIZPsT5PViZnMSpk8gpnD2JXHzJ';
			B[i++]='nNAZepmZmpc5ndA5ljuJmU7nmRWAA9tZndjpdJy5h6PZnacpnuP5l9yJfZzZgOd5mu75gIHJmhAY';
			B[i++]='nwy4lfipLvspffGpn1OYm154g9r3hk9Ug3GolmvJgQxqhkXYhgY6mw9Kg8r0mykyYMfJMj2ZAMwJ';
			B[i++]='nvopPt8pnJqJAxqanMjJnCi6nMe5os7JocLpnNu5mj6plznZky+q/5nhiZnvyZ+aqXlj2Z/WSZ1P';
			B[i++]='xJ6eeX2j6UTgKZ9IWZ4RuJXfKZjTGZhIGnqAFkH76aTmCZtwyIZoGZQT6qViuIVYWJsRyD9RMmAn';
			B[i++]='CqMvOj3N6ZwgeqLLCaNxmpw8maI0upzQqaOrKad7ipVrmp04uqI1iqSYCZ7HuZqbCZrk2ZlWypPd';
			B[i++]='GZ1a6XRJ6qiLilyBmaREup2GyoeWmqeJmqQ7eaSYyod3updMWqObGqOpqZx+yaOEBqWNKoWymZR9';
			B[i++]='GJsBSpRmyHRv5DFoKj4DQD5vSj0VwDLf86vAypxyaqN+uqbHeXTaOahPmqfKKaJySj7HuabOiZ7f';
			B[i++]='qagz+pw6iaVZGf+eP+mtz+me39qjOemkgWqo0Mqq+HWk3Rqj5Tmlllmd9cqu9iqtPTqeRhqki0qj';
			B[i++]='HgqgrxqpRQmk8/maWsqUFvoBvfo91LOmLvqwD+uwEns9wgqsGHutGEuswMqxGPuxyKqxGuumcYqt';
			B[i++]='cJqjJuqumFmWHIqVmyqtyjmdxPmX1CqifjqygyqzNTqkdBquNlqnMPucxempOYquMIuo0SmuO0qe';
			B[i++]='8iqdflml/YqeCTu1EWifeLiwK4BcFMuQHks+XUusHDuxG/CrGSqcxLqiXeuwH7u2Y9umaSs+Hmus';
			B[i++]='YzuyzcmqyWmcISuoynqtdmuj0zOtz1qyKpqsflutyQqiWuu3e4v/oi8qqCT7nez6Z9/jqHubpz8K';
			B[i++]='uQPrr06qrvv6l4TmoZi7qR8qlISWrlQLm2b6hOrCtb+6kNNDPQNQPcNaPWz6umx6PWr7ujj7ohab';
			B[i++]='u2v7tl5rsRyKux3rtbX7osbKt256snorssubpoxbt9WquIzLosuKrCCastirptFrrSnauIebvcwq';
			B[i++]='siqqky0LaIsLucnJoaArPhBotWYJPgdrlPBbpFRqv/npgCZypk4nvAkQu7g7sWBrPQAswBULtrVr';
			B[i++]='wAMMt8cbwMGbwGo7rBDssCH7pgxcwRrKpijKsdtLuG27oRHLtsZLnANQuCX7poVbtxmMwBccwX8r';
			B[i++]='ssPLu9ZLvv/r/73Pu5x4a7LZC7r42pN/5rKS2p1Y6qHrSqcve6SRurRTGK3jGoWT6p/+yqOcSoV7';
			B[i++]='mLoeM4lc+7+xq5DVU8D/27tb/MVbvJCwW8Za/LrCS7FojMYSS7Zcy8bT86vWY7sPa77E27uzK8AN';
			B[i++]='vMYcTJy4W7IcfMPAaqwZvJwlKrHB6scWDLLG26YQu8jYSqyG6sjg68PKOcnHyqJ7qqbH2rhK65eW';
			B[i++]='7LP7ip07S6XlCqlBGq30Sp1lWZ5PDJrnCamcy6TzS52FKrNWzBsbArsKycVhzMtjrMUA3Mu+7MVa';
			B[i++]='nMVZ3LrG3MXI/MUH3MwTzMxs3MxZbLvT/Ma9S8fXQ8h87LEHvP+xBrzGIgyjPcyq48y8fYuj1Pqt';
			B[i++]='w/qtP7ytPwxo5QytlEvKO0unOMqoKJrP9TzF7iqzgeqp11mudVrKq/qeqqqverq07cm0Tre/qju2';
			B[i++]='wyzMxLyQwWzRFH3MFB3MBFzGYWzRvSzGZvzLHW3GIu3MJT22c7zSJU3NHg3NbwzHtDvAM83SDGnN';
			B[i++]='FQu3NP3NiDyscny26BynlsuiySqzzjqnTkSycpqpBW2ncArQMNqjLXunKiqtNiuiWpudw1mtjHq4';
			B[i++]='P7uHIgqeUX2Z4Xq0Se2T7Cqj2Umt9uykEH3Fq1nRIL3Rcl3XxTzGch0AKM3M1MzRdB3SvPzSe23S';
			B[i++]='Gn3Thj3TXlz/wNfs0iwdx4vN2JDtv5Ct2Dp903MLydgaPomLrEHrsuDDt9VrwYqrtUiXvs27nZ9d';
			B[i++]='2t97vteqmZNLnGr9vUF91tuakyXKydwr1nqJnFY9s+0as+QKyie0sBsi0QqZkHad3BWdkCIt12IM';
			B[i++]='2GRszID93MLs0RdNwHiN0lys3ZHdxSRN3cvc3Ywd3uJd3jrtvXQ71dicyZ1srSbbp9Kr1DYM2spb';
			B[i++]='txUsnCWMrYJrzjkJp9JL1P7NrNl7t3aLrxkb1PNcnOFjnC+Kvq2NHFFyKf+7iZh4iQhp4cqd4RpO';
			B[i++]='zH+N0XeN1yCe3dW93SBtzJYY2HNt3dwd2R8N4tXdzGTc13tt/8yOTdjLPM23i83W48d1HLFia7Jy';
			B[i++]='C7KFLMdmS8HurbvmK9oA/sICzsAtfMGCO9t9y87zDLjOi9l8O9XqG8+YW+BZqVURTqK9rImYiNwb';
			B[i++]='fuZonuZqnubAzLrOLN2BXdIfveLRvczkTd6QXdnfzdEArLtwzMdsS7s47uSNfLxqS7yHnrssw81t';
			B[i++]='K8IsTMcZe8K/+7EDnKFpfN4u7MlTvumc3ums6qKTC+Z3cSnoVeEXnomZeNzIjepmnpBkvuob7uoY';
			B[i++]='XtdmTsy1vuZqHufPPdG+vNFtHt0jXeJvTuLD/uLHHeMvDezSvNix++eCXtmHvtOFLs4IHLZ9/uzF';
			B[i++]='+uwO7OwNHP/AvSvoYvvtOS3Tt5u7zKnlnj6t5Vuii0u4fovu554ApRXh7YWQ9p7qqY7hrV7m997v';
			B[i++]='rH7qF47rts7hJc7mzh3iJC7sG+7ryT3iIu7iDZ/wDH/jNv3YMq7R3XzYCWzTjV3TeS7NiC3ZFk+x';
			B[i++]='eZuh7SzDfiqsgju8e9uyHT/TI8/G62vIKq/B1DPvo67FFG7h/c7z/87v/r7q+W7Xro7q+r7zPL/c';
			B[i++]='uZ7XTE/wTh/xdv3XHV7Mx6zhxi3wIR31g13eLm3x1Oz1HK/jH2/YeK7xbiy7wqryyMzGZa+h3rzB';
			B[i++]='w4v2s5vHXDv3DSnqV1zvQU/h+O7z/g70fH/qph74FV7qZK7/70c/64Wf9KxO60Ov+Fgf+ZI/+Vlv';
			B[i++]='4dTD3NvN9Zq/+Zzf+R5P9+Vtx3Ev2T3u8Ts++jvOuz197nuMwoF+83ivy3q/97Rf+3t/7K9O+6UO';
			B[i++]='9IC/80Hv934v9LdO+ZP/+LFO/JBf63yd+dg93i793TAe/eH9y4nNunod/bru+YW9+Y9tzYNOu4y+';
			B[i++]='sR1bPTh/xRNu++hv+weZ/r1fAIP/77ev+8c+67ce+H2f18O/+7t/+JAPCAOCAwGEhIWIhoOLjIqN';
			B[i++]='j5CLhZGUAwmCl5eWm4IBmpigCZqZoqWboqGji5mnpqmYqK6WpaS0s7G3rLSxu7amvbq/vMAJG8ai';
			B[i++]='G8jKxMnJ/8Wlxxu3FQA9H0fY2R8AGwHe3+Dh4uPk5ebn6Onq3obfnZLtku6Th+yd9If59OH3g+z/';
			B[i++]='/d4BDIjPUaVHBRt9WqgQlMNVkT7NgqjqoUVYFAfJUvWrlTBOuWoRG1krV0hUIU+S7AVrpSgB1a5l';
			B[i++]='w7ZN2jpwBW7q3MnTXb19PXsqmjfvJz99k+z5M5fPoNODjBJChCQR5FRKVaEyymoVq1aqLTuyUvkR';
			B[i++]='5C6yK51BG6mWZDNns6hZm0lTQIKgePPqzWtUXN9xf/8hIqrTaGDAiYoOVvruq+PHVyFLdsgVakGU';
			B[i++]='Gr1qHDVyYkNOLkOL9jVArsyZH+x+y8naW850r8XFlt2z9f/e27hzo6tn+LHURvYS/kY4mdLvimMv';
			B[i++]='QmWYWRO9T54iZvZ4tuUtT8I6uyKFdvStQabpHtl2N0CB2ebRsz7v+nxs9KvZm3fdfv76cfB169+f';
			B[i++]='Lp6/pYk0pZRPkAToyG/4JHVgY4wJ9B+DBC54UGVPNSeRLSadsplyKOmylUjcWVhVh1spdFeIGI0F';
			B[i++]='ooYbxCReauXd596MNLq2w2vs3ZfeejTWON+O7u3w45DywfeeOTjyd05+uPVlmE+E+cUPOLwNNVBB';
			B[i++]='gk3ZWDuCIbWUPAc1CI9WyT2C4SoXbmjmVJhJZ5UvIHJ0oUhoxikINSucpo1qPfbp55+ABureaj/a';
			B[i++]='lt5tTBL/quSi/N2j5Tm8qSMQYkkBROU6vS02XGSVbFoRWGFGNuKnlHkkYkclKrcKAHm+KAAhgsYq';
			B[i++]='66w9AikofYnSp1euud7UK6PABpvgl2MWWNyxyDq2abJdfUYVq3rWBSutBUAAwazWUqvtoYfyuCSu';
			B[i++]='hg4prLjjlmuulA7+V06VFT4InIJWbumuvMYW2y6zX4nZyAYCtEpXatPSeGOP11ZbbcEzDizojQj3';
			B[i++]='yADBgQIpcXznITwxuIXih+PGFL+X5Lkg7/TrbVZmGVyAif0U74KXggkcu/jGLLNG/UY7Hg7mXfsw';
			B[i++]='BAzskG3B1wYtcMLaFm30rTna2ieh8nVb32xQN23kjLqe/ztyyDcd1h+lAzrpqL0z47ts2DT7i9qr';
			B[i++]='AVgrtMHWMuC22z8fvIPP2b79tsI0NlzxeXOr7be2eB8dsZ/26Zokj7axJmR7R5ZztU6Pk4v15HhZ';
			B[i++]='Fuaw8qBMdrKFJFCzqwMcnG0BcNftt98MnM5zz3efnvrqbs/ds+yy93x63wXcOPfcgstqX42D2qq0';
			B[i++]='0jJK3biRki8aOeXMT0nOX+lGtRuBJ3OpL+YsN5jg2M99/u+r59lt9+o2lC/++DzDnvrbq6PedvtA';
			B[i++]='3/hw7rzrDoHPFQOtutoGF6334ADclrh09LtbBWBxyWseThTYqGFBSWuUKpmBJvioKkGpUvrwkqXg';
			B[i++]='0Tnvof+mAmlrmw0YMEK3lfBtJ7TbDuC2Axuoj33qe+HuYqe23dUPfwnjWbV4x7oV0m10frof4HpX';
			B[i++]='wKXl6Ig7Eh6tcKWocISLiUxqmuOWx0ApOW9SkNKayk42mJI9aV1UipemTLY9AAVkAB7UBghJ57YI';
			B[i++]='uNGN4kshAUg4QvOd744knN0K72ZDH+7Rj7trGx4HGYH7qa1nufvZ/VZIOoitjUavA1r/Gsa/3vnI';
			B[i++]='cE70mBQJaLwFekxkUkyguZAiyq3xJSikHCNg+vE8VTIlQl9Do9m0gTOejTACBHhjBOiIwjj2cnyE';
			B[i++]='XN8gYbfHEtrRBgRAZvmWaYMW/hFu/WOYtXAoOkWK7mD/2GSb3oKmv5/ND1CBq1WOBqaw/xEuiU8L';
			B[i++]='njiN+Lsk/iljTKyiPPfysndVaHthXOWWqISDWdIEhCvU5S4jUMddMmCgdiSoDXC5y2QygAC5TGHp';
			B[i++]='6mhM853QoHl0Yfp+yT5hwvB9r6Phw77Jv/3xz4Z5Sx3vDvk3IfIuUEA0pyQBVUSaGnGdSSNSE9VT';
			B[i++]='uKo1j4rK2wkEC6OgK8VyINbjpz/HcxdbunGhCyVoLgdKR4oe1G4YPahBn1pVOFpUor4cpFgF2bcY';
			B[i++]='gtSjJk2rWk1Kt3JWsk+G3CY232rJ/tXVaD4lx8ccN89vKSmVQ91aKguDAwDY7AN3eWj5IKpMiCbT';
			B[i++]='sZCF/ywJGetYhnKVoHSE40AJukuolg+XjKXoV5dpQo7e8YXn22hqx4ra042QpWmdpHtkei28rU1/';
			B[i++]='2ALUTGdkziUGj1yG6lUoB+hXb0BggU5Mbl77CimhoLKfh01AAZAZ2RJE9rrXbegcz/fGOQqUsstM';
			B[i++]='JmlL28tjMvOpnzWtVvMoVl62V4eoY19b3bdC901zddWCW+oaadfaylZ1sXpk7+ImSZ/9kKbpPFwT';
			B[i++]='e7pTHfG0asFdzeJeg0DaAHVXISOlFqH7om48FLLWdWyIIcvQqYLWsY1NMUTdSOISm5izmCUhQkur';
			B[i++]='zMwyU5kF1ah+IWDRq/a4vfXl2R7hBlaPro+lr9uhSv8j+TAfLjlu+QPibLmJsNvu7X9U5uZ07Trb';
			B[i++]='vfkuPsT7LeMKSOadwjNcv01cwcCcZuQul7lNUmp0SUcA6444u5U9MXb3vGc9r7iyKz4vCsnHS88e';
			B[i++]='06EHxbFEjwlM1Z61vcA8Lezup8KOGlKFP1zrWq+5P7ZVk8ugtmkQdXvEUncLpxJDYqkVjLyRRY3V';
			B[i++]='e4WzfqgxZ+rWOcS4huiI78xn0LLY1nwONnYZClVI47HI3DW2+BTqXmXjEXa7TJ+mp03taquVt1H2';
			B[i++]='9N/+1Nu71hVyem2wjPAjMtxcGB0VWCpiCxCBDDjW3dXF9QOwy2vsurvewiZxLiO6Yhbr8qARFajA';
			B[i++]='pbr/1a1GFcYvZmain8rZO0Zb2sKEuCAnvWOWFlK1q3Xde49cukYf+XRzTaS0uxy+amlUaAKWFSXz';
			B[i++]='1u1sFmpQRUpcKfm6YHgqlyfnvnlQZpPuww4AAvGu87yDXoIHFL3Ojh36re1MAKMrHd5BF3G+px5Z';
			B[i++]='6gJ7qn8W+L4tG+itd3eqb0PvVTWr1azCsaNHtqOlI07x9y2y44+eOMShyTeUw3XKXr4yXbVNam/H';
			B[i++]='ilvIVQ/ModbE43IszPXhVmt+hWZh2WYAhn3RAHZwa6OXoOgPcLrTi375zV/+8gSAOtVHD1mo+znY';
			B[i++]='enb3Gx8q8AwwfI6P/bBmO2u3qIZ97GOf/erLjtU3/9q+jZvFPdAljfZIqnVnaGUyH6NM4E5fG8D9';
			B[i++]='JZgkv0nyvlPr1DFHXNIIZ+qaDl7xT1TU4ns6/sJRMVEbUPfPm555zD8gAxnIvAgyn3n4Q9T+9/+8';
			B[i++]='/ulP//i/H/4A6G72B28ASHoGWHr91mIDt4D+toDLpFAM91no9YBZ9Wyo5WwY+ELSdkiZZi0XB3IA';
			B[i++]='lmUgOIIgWE37c2D9dVsGVlYiiFv5806C83LlN24fU35lxnjkVnPoQDUt4nOUt38vcAGX9wJEuHki';
			B[i++]='wHmYZ13+VwLw935MmAGgB2IHGG+gl2sG6G9eN4Xi9VhTFVWLZWsEB1ULOHZqd2NmGF4C10Z4hFFI';
			B[i++]='Jv8+Eqc6pXNIIRVScJhW6CNIxpeHbreHcmdI1XdXLRdmNDh+2yeIh6cOURRrObiD3EJrkscAn0eE';
			B[i++]='JUCEkvgC+td5l5iJVfh5Uqd/Tad/8MaETHddI0Z/e7ZQW7dvkXViLhZsvOZrXudrLgaLLNaFqVhs';
			B[i++]='s9dVAgV8A7Vd7VVH5EWHEXeHRiZxq1VxcWhfPtNkPrRIQaZ3zHdSVeY3LGiC2aRldjdqfqcw7ZQ8';
			B[i++]='G8NTofRJOpeDUsNm8uGI/3JARmeJRziJlvgCD/ACCkCJ70iP9WiJCnB5FyAC7SgC8Nh5/HeERkcA';
			B[i++]='88d+74Z0uoZ0nOiJuWaFU3h6Whh6vWZZ/7Z6VCX/UDcmYxmlkaSVkVWFgcSXgcSYWhj3cfATX8a3';
			B[i++]='P/r1Q8Z3TSF3ZV4mZdwnODL1TqkGYdsXc+i0V4dTg07TTuiIGgFgAyVwARdAhAowj/TojpQ4ifzI';
			B[i++]='j/THefz3eVEZhC8wf5rXfkdXf6EHhfeGeVDofvGHlVjJf5tnkKInkQf5bmmZb0xIda4Hkf2mS1jH';
			B[i++]='dd11lwyoUOdlaJ21VViVaOMFkoIpVi6khyV1jZ/WgopUUlmWbYzZUtB3d35nQFSDU20WjuTnLdUi';
			B[i++]='MQkQeelIlExpj6FJj0dplEE4hJqof6JpiZF4efPnfpaHhKnpiU54a6zZlaAXll+phE34lu+nllKH';
			B[i++]='/3pYKJctlmd4SZxUh4qMpUtfyF5qd3t3tFAmhGxjhXYxhD4pCXcmRTqwRYcbOFMiGI0pGZ4mKJOh';
			B[i++]='Npno6Vsy2GWe4JlCCYlGmZT2SI/zeJRJeZ8XcJ8KwI/6qQD5aZXzOYmoeYT/aJax2X7tqH8JiolD';
			B[i++]='6HQLmomhV4oImJbzBnUhFoDxl6Hx15UEqJbftVm693WsWItax1kvNqIMBXB1CWO6p4YVOHEed1Vu';
			B[i++]='iFbfCYdGpl8dp18XN56GyWQgGEmkQzdJFp4p94f8dXc1mXfY1mUN006sMTotZ5ks156HFQARcHlJ';
			B[i++]='2QAK0ABcmqX9qZRBGJ/+KZ9HKY9N+Y/v2KCXKP9/81eWDyCEsamaqamV+weFdBqF75d5EDVv9VeA';
			B[i++]='fCaA95d0ShdZekqhCHiAuBSXJDpsf9aFealVhgZ8t3dwuwiiZwedbuOLMyqpu3eByeg67WM7fRhS';
			B[i++]='LOl2HFeHihmC75NNvNVNobZbkulpSLotMJcz7NGZVRoB+qmlXNqrWyqPv6ql8gmmYDqEAVqJnyeE';
			B[i++]='QiiJJeCfb3qV+/iU0lp//QeA/Xetebp/F9B+aVqnEaqn7Nd+QweFgmpv+HZdgIquQlehAjioqJd1';
			B[i++]='L3ai/hZV8jqXeZmGZMd7HMk6z0aSxfc6QTY7/MpxMBRp+TWNEyc6PrqBU/aYzWdND2tfUapNRQr/';
			B[i++]='ahMLg9hEpS8ylPLYq7zqsV3qq8OKn+44mkRYpmaqrG/6APxIhP74lPAokMYajwwqoA+gAHS6jpMY';
			B[i++]='kAaaldSaswVpZ6MYoaRoZ+2Hf7sZln8KWe4akX2GYijGdVUXtbJ4r773b1VVhjQ2Ppp6gZ56Wh9F';
			B[i++]='Q3LYgXronenDnXxobS2VmNv5VmtbZVx2sSz3govJM54gAFVqAx0Lsh67pV/qt1Y5rA0ArHtrpr5K';
			B[i++]='hMK6pVpqpox7n5RolP7osm+6ra0JkDzbs2ZpuQUalZwHqBj6uZ/7efB3eaMrocDptEsbqKcrlyh6';
			B[i++]='rzYQlwDHcGVnkShkUNQ5mG7YnZ6atnXIZK81/54kWJ6LyW7WZI2T5HwCZp5G6ow02TYFgKsbGwEv';
			B[i++]='wLd9W70NcANcir1cOr2+2qX3ubjzCKb2+Kb6KALbKoRMaZryabL7B5ssy7JPybKoib75mImWZ5Zk';
			B[i++]='2aZuKpVMW67g+oQM6nRNuKEDGKgbapzHWYuRBW8f+l3y2nqr11gtBnCUuot/GXYWWYEOF20HpT4G';
			B[i++]='NWmgCrxvuFF26FpyKMKwdV+y+l9xuzMoB2Vwy21RSlvZAnm5yr29OgIjYAA8jAANgABA7KseEJ/5';
			B[i++]='mZ/76QF+u6s4nKUei8ODy72UmI9GWZTRanRHqY/zOcXLmqzxicXKGq1t+qZHaL5sSpb8B49xev+/';
			B[i++]='90uV7RtiRod/XXl/BIi0bfl0aml/55pdIqqKfIx11zVvrbvHC1iXyHS1EOhdMlbIVxVRebRVHoij';
			B[i++]='OFqSnzqMIGWMGqh8oOq2aouqIchpskpgKahlQ/M3NryxBHAB1Auy+hmfH9u9fKu4rZzESVnEJGvE';
			B[i++]='5nuyppmfVBymRvnF5jvLpQmghDvFkUu+maeyvRy/LQu5V0mfL3uVm7fGmYuJZpmh/9eE18x/BcyW';
			B[i++]='p7tnvAbHTSeRAtiWU0eXGdyArFjBkMpZB2dixuZGX1tx1GiSJmmHN1qjM1rCJKiYKFxbLzW3KimH';
			B[i++]='8VXCBVDK6ZgBXKrDCj0CqdzQDe0BHuDKTLz/yv3ZyxYdn48buF/ay/KYyxd90fyYzFaZzEbJsh8N';
			B[i++]='v9NsoGxcs2uKvv2XhFEZlk/IoePsp+Jcep7rud2803hGdVh4oge3WI0qx6qYUF51dr6HQr+3bMuG';
			B[i++]='1JB6S7craQKtcfFcsGa7ySxHN8cbV6pzcgDmwglr0KhRAATQAAt91mat0HzL0A5d0fGJAEZslA1g';
			B[i++]='xLDMq4r7t4mLn8Dqn0Vs0cQ6n0o5xcCsy1Tc133tn0j5n3FNxUXppvS3n1Apldlqrf9X2eIqm5bX';
			B[i++]='ppxop79prU9YgBo6wKMbgOq6lqlrnFendQ/cwPJKqSuKircko5IKnRr8ULJd2xnYhyYVyXGo/514';
			B[i++]='GKNn9Z0Lm6oa9zBirQ1kndBnvdw+TL1BnNANYADNHcTNTb0MzdbQ3cQTjdcUPctfOtd8jZ/fS8ul';
			B[i++]='6Z/N6tH4edF8zdiQe8wlAJXvy4+uKcY6C7+dR8YI6rPwfcbiaor+raeYiJAznacyfaDWdb9JF3qd';
			B[i++]='3bT6NnqCjJwKWHD4WpFt9EIVyKIXGXZqE3w3aoFN7aLrhVFOjTof6D71bKqOFrxU5jY54Z7IjdA/';
			B[i++]='jAA6DN0jIOPLfdYGsMM5bgBp3eMO7dARjcp3XdepPKZ5fdh0TdfmbZq4LIkcjbJhKgJ8Td5C6I/b';
			B[i++]='ar7Ruo9ajL7IKqDM6nn7p8YH+rOYm6ciMP/AnT2hpA26rnfTpn2ocMzAbb66wyZ6JUaL9vqoVtuA';
			B[i++]='D2h1oeVQ0tlsYjVws0txm2rbI9mvwxR393xpG7WM+HXcNFEAJZDWNb7cO44AO37pmt7pPszQMg7E';
			B[i++]='z13jzm3XwmrXQ564IdvKIPvcP16mRGzLvdysiM3RQbjeV2yJVlyUk0t/5ovlFm3lp3mykwi5Rbes';
			B[i++]='8Qin1LzGnUuVXWmg/heV4TquSRftBkmQTYfNfDpi4GxvsKh6JHp6X5d1AkepaojBILpw08lLnRXV';
			B[i++]='73zuwxSHheTbkKlDLF7cI8Xbk7yB2jmHOLpLO4BGh0XpZp3jOs7DPHzwQIzwni7qrd6rmp7/wzO+';
			B[i++]='0Gbd1tv7sX77ynmd12TK5B0dpree8SLvuFqa5Ordy1NevmbMs5EtrlvM6+5blq8plVjpvtGcuVr5';
			B[i++]='xuRKrqrLzfaGdOQ8kYBmnKoIixE+1Oh852R3qYg+qbPtNoqawLr3wLJd9ZFm1aF6wnkIyXPIu9aW';
			B[i++]='nScUAC4+6SVw42a/wzp+6Qgv3Wvf6T288KJ+A6IO0a7+w3T/w1xa99UNshFN5BrvveKd3t9buOG7';
			B[i++]='uCjf18Ae60iJ2FD8q+Grn6xZifuIeWicxvstf1HpefEXwO6I83lK4KAYhbeGuoEq+r1mYkOdigjM';
			B[i++]='e3e52rObr7itexmwXk3P9cRH1Vof6F9F/2PAWOEEO9Anrs+rE20C/yI78AIKvQAdsAAL4AAj0PwO';
			B[i++]='4ADLv/wOwPwIvwAGwPzYj/3Xv/1rj/DXrdZsHf7RPQIdkPYKzeMynuNuD/HZvfd+T71bSsvhrb77';
			B[i++]='OaaBr+T4v+S7DggXDyUXhCWEIheFD4qEiiKQhS+HJZOTiIWCD5simw8ZnqGeJaAZBJugh4MZGaSD';
			B[i++]='JaumraQEBKy2rLK2n7qntbWmv8K/ERHCxcPGw8nJygTOz9HSxdQRDNURNsjY1NfGxQwM2tbV4eYQ';
			B[i++]='5gwQ6+Ho6Ofv6fLX6eXy7urs+evu+PP11goMEPDhiMGDHwqUaDCi4YgFCx5CjPjQocMOFv8zVnRo';
			B[i++]='oCECjQ1CihSJoAGCkyhPGlhpAAHLES9BhhwxsqZNkQpy5rywU6dPnj4VvFA0VMGFF0iPFn3x06hQ';
			B[i++]='noqiOiIadVJUpZdUDRK1FdarUJTCat3kdatZUbdk5VqbC5jbYLZslVg2DC7du3jzDhsHjptfcn3N';
			B[i++]='WfPmrdg4bYdtPFPM14a2Z4UB/5tMubLldpjn2VA3sOBBgx8glEBAEaLEER0iUsDYkAJECrAnLnC9';
			B[i++]='IPUCA6Y1evzIO+XJESVp3mTIkKZwmh0NFIcp0vhwkR5C+pQuPaiCBkeBQlVUXdFOnkOvei/6tKr4';
			B[i++]='q0aBptfZyFElpEjfxxcrQtUhSA86Per/xB9//rKe5MILAWJRUsshBxIYVlu/PLAMXMHYdcszzkD4';
			B[i++]='DTcAFRYNNsQQ4FgtjA02mDgfGkYNhX9hWFhkHka2WTgvjsgAAeHIKKNl4NAzomEQdPYZaBA8gFtt';
			B[i++]='E3WwWpGpJZkkRB3YZpsDty3ggAMrQdRSSy6tVNFKWZrG0komJceRlhk1wGVKIZUUXUnPNdScm8/h';
			B[i++]='9Jx1PUWVHlGG7CSCU0wNlUgiTxnF1E5LWUIIfIRwckF9jjBSCH2JvPJCV/zlB0l9n3hSCyoAllLK';
			B[i++]='L3YN4yABIgiji4Rx6WXqMbU441er0hDzDV7dACaiPfbUKI+Iut5KjmDzQHDjYPqEs0Nm//Ho2Gs4';
			B[i++]='FPZI0I9HhFbCkENOVOWVVUY5ZAfUftmRRw6Z1NC3GU2Emm66vWnSur6RNJIHHijAZgPRRVcdvPDm';
			B[i++]='FK912F1QnUj+NpCTwNcNbJ127G2XcJ1OVQVfUUqVYNQhUXFy6aWWYpyfJxdvzDFXvJAFi31lAYgK';
			B[i++]='Kqxk6qmAo6rqFjAORgjMhBNOoxc0HcK64TYp9kzNY8Y4poxiP9+IY1/d0KMrs5kdXQsEATz74wcM';
			B[i++]='vCBRk03ChjXWDjRZ29ZGKumaRFYy+ZCTFKBWWm7oOlTbQ1+OK7dFH1U0E0NslqSccGbC5JJJeZ8U';
			B[i++]='500F54TU4DWpR+dPUUXyZybi3floUv9EWXXUVItmnsiikTJyMSQvdGLfK3NR8hWBZIXSS1wRHmLK';
			B[i++]='XHQxyKBbzoyqzIW3WxPrMUBjM2OruFeDYjnBExbZZea8+OI/yzuWzvLq1BOZr9xALfVnoSnAZNhZ';
			B[i++]='r2akbBilpnZFTXY92/bWjottlut7S6ZFFNWtW3LzpomABynhHzhNbPrd7pX3U1NJrnOD6BRuJAYz';
			B[i++]='oJyuQzB8xYte9RKYBA1WMIKFhyoYJM9QKAef7AgqO5AwiuM0d7FMPI5RJQOZyFjhiq6UoBNcEUHK';
			B[i++]='dhGzlukMVDhbxujmMiAb1kUvdsldND7EmGWcaFY0qlHwfBcjoyFvV0pMGmWoYT3PYA//AtrbmveM';
			B[i++]='BJspdQBKs+Ei2LTmPe99bWuymR+WsIQSwPkGJdcC421YMkcunUklYGoJvNqoJsTxLSTxUqCZ6HWd';
			B[i++]='+z1wcAyU4AQTaZOgbAdidlKYU+hkp81hjiqJeoRYzqIJ1aXsZPwRBCyoArJTfAIXs8NLpjZxClDs';
			B[i++]='IlU+ZBVdfLY7ZhjRZtPoWYakpywbJQ1DO/qVLr0BmQDgwIoIYUADwigB2KTAmc3UGti89qU5QsSL';
			B[i++]='U3LAarSZtrQtKSIYkY0B0HhNKFXLJSrxDUXalhFyjas3b0RnSeaJJnrVZF5rqt9wEqmc6lCwYIMS';
			B[i++]='Sp2OItA+CeVhB+1Tn7Qjno6JAoaJ/whdCTmHn7Wski0pY9lFA+RJB53FlKzcRahC5bKSpsqWtEKi';
			B[i++]='qzLwKp9JplaAGV6KFuOYJopjRb7jUC0YEDVkgoYBHnCABKKZghlMYAIYoEBRYUOBGTBVml80nza7';
			B[i++]='t5oZZO17WyzS2XSjGiZJSUpgxA1M6hiTj3zLrBlhDd3eeIMbIKCAb22rGwW3LnbRtTjDcUm9CihI';
			B[i++]='e9nrOvuqyT+tE9DC6mSDhmNPQimXCUMo4gHxgdwLEeGfVX7sE4LIKFfYUsqYofJUERppzUxaS9Li';
			B[i++]='haR5gcZfCANMl8LURM/jS4o2sw2eXg8hEdDeU5s6A6c2dbdPLR/XlGSbr3Wta8X9ov+Tjisb8I3T';
			B[i++]='a+IcEkzouLY0gjMi2KUItdQnzzvSk64BPIn+0qQ/NIk3vHMiWFMmqYDA2ku9AyPcAv3Fk4je6Tt7';
			B[i++]='WpxPAOUdUlJsURIDmei4MrqvaHYUqPDFIOoCIZnZ5ZQP+iHraHbSlx0jh6Y4IvBc+5cNs9RVe1kM';
			B[i++]='hYRWq8YgRlnELIYxfRotZc7GqL2NsYxnHGPeria4XwPub31rY6g6iUjNHZIczWWbjWC3IeFcgFnh';
			B[i++]='maXufqnJ6GTjG9c4E3ZqZI3qNIkDCecT+OhXATf4cua+AxUPKCw7Somc5IBS2UE8ihKNnU9YRGax';
			B[i++]='y45MUyGdMEbb4sqVteIVrfCzrFj/GiueVZjBh74LNh7AjQ4NDRmIeZVMOewXvsDod6yIwIqh9YHc';
			B[i++]='NimaN5bmjZ9JaqUWdQYSSMEEmpqCVrdaqTXugFWxVkYuThVr6IMukYoLPq/x2rqo+THZGnLkslmk';
			B[i++]='JcD5FgANucfznpckKZFyuwBX1/QqUk7/Ygp3GqYvMIcZXzdoQAHb6tbAEsw7CqiEdxyrAP1MKlJa';
			B[i++]='WZmlHuqV+kwqK6SDISvPcrKUuW4tqaKFgEoRMoKzUmY3A14zGq0q1M5y0sbAKYtQxAxk1AWmreop';
			B[i++]='pxlwA1k7s7dHPSrIe+vqkjM11ayeMW97aySnbi2bYiQjU7lI4xo71bdiLO5Ymxtk/7LSUdnTPbZD';
			B[i++]='PrJOmLQTJHuzH5eMwxLB6fM5Tm+jA6duQH0lDjuO5IkjGItByJ0HPGkOj3uOIoJ3OwpjbcYYKGSo';
			B[i++]='sgClBbQKKpDcw/IL2BEowqlE6awiEKrc+X1DRswhqFo1IaTlFFc4tRXPqJELTR+T0wQwAAWOagIT';
			B[i++]='YAADJjiqqoc61JBz/vOcP3nom/lMWM/g1LuVQDZXz/pp0pq41iJXtXhOe56778lNpx+Z3Hl7OzY9';
			B[i++]='nvmbMrY6gjjqTBB/gYTX1zc3QkAp9svMp8oFKZaTUDou+l8figq3z32Lun0trpjdnj/LlpTecvCn';
			B[i++]='epDsUHshiFdo4cX4sGtj1fhNT//NBsuc8allbILe9r/Vp1dUqSYBEzBUIRACEhACvmVUKddboeY9';
			B[i++]='zyQBXJRV3BNVRQI+tHFG26NFWgNkvwZku4ZdGIEbt2cROlcREaEl7jNld5VOcAR88fRexrdP8fUv';
			B[i++]='hxQnFfQdlJN16eZYV0FZGbMnl7VZnsILn4RRqtNRFsUKbLdnalF+M/N2rEN+AAdwpzR+uABEm7Ip';
			B[i++]='oABhU5gzpWVhwbBEFCcLjsdiH0AACNABIYd5lSdypJZqr/ZMX7QaEQhrv/VqplZ6MkdysOFUJZcC';
			B[i++]='nJdN27RF5gN7IShW34Nr1jR7dVR7IPhVQyYRHREllniJUUIl1+I+EcE+ZIJXdXX/HMzBRsrhdH6V';
			B[i++]='NzgIUALzAgTTit1BJ3sCKHtiQrH4X6qwOY2Vi6aTH6LEfJ/jMZniCnM2MsTIFQrSMh5lcH6mUQPH';
			B[i++]='Z1mIftIwWupnC8pAaIN3fjrjd78kPBqWM11oCvaHPZEnARiAVJeXeZRXeeqYeUY1Af53eZqnaiGn';
			B[i++]='eRPQavOoavIoiK0GehQwVLAhgUYigcFVga7ngVEFRlAyJTwHJcn1RV8DPtYiTqbBc2RjJSRIkUP3';
			B[i++]='PqexG1EmdczWXvgyEoEzHPgDSL4RbhCUL4GUMCTUEyyZXzohMekxduJBMYTAXmsmFZ3Eb0a4jBkF';
			B[i++]='Q/L2jU/YDAzGWQTnfU84IDUD/1prAWInQiE/FFoOhyLyd3gncgsMUACPNzUE0AAUsI5gaXnlWI7m';
			B[i++]='eHlmeY5geQJiiVTzWICcV4/+2I/cxE0UQIjKtUWfplRQpU2rt5B9iSQYaXS994iQaJG5t2xw5HvS';
			B[i++]='Zl5gMlf5E5JUB5nvBXxhUm36si+FkzAX027tVkKfg4tXATqiCToSVR+XUpplh3YwdGcWIzpeIVFo';
			B[i++]='kVFCmRYmw4U9yVkqBEPPmIVOiFHk9zKoklqL11IKt0TVeAsFUAFoSAAjkAJmqZYWYAJq6YZh6Ybl';
			B[i++]='aHnqiHnpGI/0GI9FpWroGHLqCGN8GDZbREYtt1tNQmpGAnOEiFx1yZfJRXtiZf9kKbgRJEiCyFGJ';
			B[i++]='TUdW/pmfPhcTRmdlGvER1IZAInEDf7RIDJSZ3VaD/qQTMKlY58FfUCFQ1cc4nckIQfgnqPkACoBg';
			B[i++]='xfgVmxCi26dvH/ONUSiVbzdwV9hnuLkyu8l3eQE7dpdapoVhXUgAybmcCDB5ZHlUGBACQ3oCIYBU';
			B[i++]='B3ikRLqkTMqWTgqPSPWGlIePIueODFh65ilqMhdqYoRcyMU1soFcZ8SlwFWQqeGIzUVsFflkTWZH';
			B[i++]='UFYtP1dNtwEu9INHf+OmwRES/dRPT4dt8BVY7bVP1BFfEOpt+7Uee9InJ3RfkXShgiJCj0JnYPFQ';
			B[i++]='3XcpokQJPTh3EwJhHNWFmbL/or7Zm5uKhCgjIL4gfrfjaHsneHzHCQ7So5yWAQYwAZWnlmqpjre6';
			B[i++]='jpRXlkIapObIq/MYpGHZfzWnciunhzPXAQJ5XKzXrEdyVbZ2Y643TcZVmNZae733LeaSXdvqczDY';
			B[i++]='Lh7pVnzkVyUpkjfBJjrxL4o0HV/GXgFzbTihHl+nOBTlUBqjMaJwqcCIqYdSMmXhqZZlhCdzm1AI';
			B[i++]='IaDqZ58qjVN5F9IYeIxHFhkAAcoJec0JlhaAARZgASeQsRbAAWeJlsM6njOgjrSqq5knpcRaYysH';
			B[i++]='ciIHTToWev1YlzrWj83EenLprF86rR8IidnyVdfaTni0dOBiETXxR9/KRnNl/08QBEiAVHUuOUmY';
			B[i++]='qV9odhUX9EFKIUH9dR7hEQnnYYskxDkktFkDm4RdQToIxlGpk6KhNSCttIQEK5vOKH4MC1puZyoN';
			B[i++]='W4VJCQkRO7FT8wALQI4YcAIYq7EcywEZW3nRaQKJO6zaWbIhG7KL+4aSW4+UW4/7GHJ8SLPHCojc';
			B[i++]='o2MxV0bBlawXqIFBdpG0N2RDRp9xakexV5i/hxLlxYIkwafVZk/w0lZRG27hBmbU8VYy6KB8Mh47';
			B[i++]='MUo3yTmciYuC0i9DAWevsAgbY6kOBTJ8plmfBIwoeoTL+H1OiAso87Z7RrCJtr0WFb7OmCrVK7HL';
			B[i++]='uQBlmbGDe7gYS52PC7nr+P+xE0CkldeOrka5k+ePnBeInxez/8i/qtdFUzLAdais3UNV02qQkihH';
			B[i++]='zTq6uKY2TAVkasOtVtKRj+gtdvolsRu7MOimbOob+gMvblRXa9JI03EDF+AB37YvKulIT5u1EhNC';
			B[i++]='g1LDB3VYSNEU9UGLEJWi0st2HJOoR2G2c0cK31hwAqspcLuETPwpRllDPklh1ygr3PsJsNq3C7Ck';
			B[i++]='Z8mkB2gC9muW2Fl5mBed71ur2Am/8GiWbVmPpxeAmZePKfDG+lesT9VMAmmH/xjAd9hMtpaQh7h6';
			B[i++]='5ZOsY7RNfzl7XtStDkm6Zxp7yeEtFklWdgonjjlP9oMm5CKSb1Re5fUuIIn/ACBpQHylu/IlWNJB';
			B[i++]='r9+xE5hpytXXCA01CFZhKJSQFJUAC2ynmh6Dol5xATOklLi5fUeMtqXqvdUbsABbcFDYQvOGvpxW';
			B[i++]='AguguInLsdAczRlruOd4AoKbq2ZsqyFLq9xMq+04jybruLpapd4pAQFozq7WeamGgJ8HlyWHAQCo';
			B[i++]='crE2gXjppcr1pbd2XFFVPhJpuj23ul/CLXFqmCEMgx7cRgdqrmkCQYGaL0FRfBCNQDnYL0TBOBOq';
			B[i++]='J23GfQQ2FnU2zLMpqgYHzGPrvcHsdmghzKUqUiDdhfgBARuAhhfQARvLsTN9Ahxg0xyQ0zp9ABzA';
			B[i++]='0zdtzRlrzYFrzbgav48b/52JK9SRm3lkSaWWe48ml7/4WHKrBsCZu1vfOYfoyTUwd5AT4cCpC4le';
			B[i++]='U5dl462+h1buZHSlwZ8dCUDGoT6/txLkNtcG4G3kFpl4ndef7NCHyplrxrU5vFAXVJMAghSuKtID';
			B[i++]='W4Tcl9H61sMaDbCkSnCk4Ao0lBYq05u/ub3AoAy8KAhXjD3MjLhBPdPSXNqJ+75kTJZiGcbDurjR';
			B[i++]='/Li/usZQSrlwPId2XHIJGMADKIijB8Ba6rlKNYAx64+dC7pXNaa6hj6jWzYGQCUErZ+dCGUi7Gwv';
			B[i++]='SJmUrNA2IZkrKTD48smtyK4CozgVhG7tmpPZsUFTcQgdBGDX94vQ+5mR0P+Z/gE6bwYW6s3RGi22';
			B[i++]='GnVKI42UpTp+qwMzegsBCYCGL0ABM30ACq7gpt3gDk641hzhG4sBN32xZ9mG98jGVup/6Eiy89iO';
			B[i++]='SpqAnVe/BXhUcTmB2gRq/9jGKfdMvhVNAuy5t8aI5YMbUyKz3HTPDYyJ1NWIczRduccbwIESQz5I';
			B[i++]='B/obQx5t8gS7zfZs6gVI6ZquvztBOLFe4n3KCKMvkICZZpZfm/MT8f3XZYco6h0fhu06ZFHf963e';
			B[i++]='grAIO8malEqpKNqTn0XnqHTnNCOqiiIIDFDgnKYAFGABByDoC77gPV3oC07opV3T0hnhSM2xYnzh';
			B[i++]='beirbdm4a3zpgnjpwUr/6ZbLcoA4s0+lj6HOaqYG3DOHlwNZlweZGgypkNgaXddULc7NyEEetItZ';
			B[i++]='0FHmP+KSETHIbCcxbm2lT+j60IQy2JV0KY6EWHTiZdoxi6ucOYiQm3u+H4lAWYtyy5cFsDKkFvht';
			B[i++]='th2FZyF1xFFIhePbtl34WASOhiJAAe07zYJuuB3LsYMu6PQ+79LszJCO7+r46B/b7xYQAmBJeSM7';
			B[i++]='8P2HjmSpxiR+VEQqpGXZ7+UIgIH4agGpuamXx7pd1VRl3GdEThu/s7BXh2GdgtqiptLtgt/KJoj5';
			B[i++]='rQ4EXn260Nyd193N1/ol84MiWMBrHS68wpqJH/2R5m72Zo1lOo6d390n/2B1pkL+zTLhd8ThNwo8';
			B[i++]='0QAiwAADYOAUkNOGjtM7zdNa39M9Te/QLOGXp7HneHlHWr9kP6REygEHSOGCa5apjXlQWvYKz+lo';
			B[i++]='WbIn0M0h9831eHlwjLmWG/GuZocTOHqgp+IzJ5Axl3OuUYe14T0Iydxzarp4iibI1+R39LorqORS';
			B[i++]='5l1gsqD/kpmO6sLtuh12EsProRPk5hNhFmbE/vQFox3hvbw2qQll4bwCNt91hqL4CozSi4TK2PSH';
			B[i++]='HfyepIyMkBMP0OdoqAAzQOiDjujR3PwZC/2IPv0KzvXV3/Xw/uCKK7iRDsYH/7FiOfYOr6ROyvBN';
			B[i++]='3Y5GRZYEiI8OaI96uP+PdizAn7d6hNzVrO5FynXPNy6twAUIMzMUgjMdgx0LFB2JiQuPCwYGkI8j';
			B[i++]='C5aRkgibBpsIDZ8NoqOkpaanCgqkCJKtmpyvNx43srSyHh4Nqrqin6qpwMEKF8MXFy/GuhciIgrN';
			B[i++]='ziIPD8zU1NPMxcYlqcTAFyUv29DR0g8ZGebo6ujl0uTR0d8v8/PgJcQPJdPm0uf+/+nKqesnEN2z';
			B[i++]='BwwSfDjCsOEHDxROcLAw0YLFixgtHMC4caPGjxcPVJSIgcMJDChTYjDB0sSJEy5NZJwpcybGlxZc';
			B[i++]='wkw5oafPnz1VAu0poajRoxJSpDDaMwUFpxSiSi06VYKDq1ixMmIkYav/1K9SGWVd4ACBAwNnHZBt';
			B[i++]='lcnSiLduL40w8HYS3Ud22WZi66qTJ7+eGnhAMNjTJlyCR4UKtSuUqFyjdj2eZYtWKg/CgDXDFqwa';
			B[i++]='tWzfjIkerc0zM2KjM4uGtoxZO3fXrFV73e9c7XS2CeamzZt2CX3SfoN7oatEwoUNGX54EfFjR5HP';
			B[i++]='D0ifLt15xpfYT6pUmf2Ehew3LWrHkD2lhZUt0wNlOcHEzxkT4E9IMX8+BqVD2/eUD5S+Uvz/UWDU';
			B[i++]='UkZR4ABSX3WwiIIMKkjBAgw+kohaaj1ylVVZNUJWBw5MCGGFlOA1F16StNUXYJ4UxgkrktTVol+t';
			B[i++]='GGYYKplxowAul2GG/xguOOJYYzc1XiaMLpKJUiNnrKEmQmvMALSPNBfAds2TvZXzTpXABcSPbez8';
			B[i++]='s46XYAJ0gS4IKZScci9MoBF1bFLHwUZvUlQRRzbVeZKcHOSJEnbk5SRTTTldlN6ghPrZUkrs+bQU';
			B[i++]='gUnNl9RT67EkFEomoDSBBIo66lRRWXXqqSONbMVhh15NEuJddL2FFyQGMAKhhYmYympdkczVSV8y';
			B[i++]='AvaKjCsiQAtllNVCGWGQJYYKKbv80g1qQAbTbJANGGPjMMiIhswwClwr22anKRDbkqfBI65n7oT7';
			B[i++]='TT75CJeluLy55u404ZCTbnDA+cMPQcFBOaYzx515xAcXSPCmm9PR2f+RRR4dnOdG3t00kZ7kcXde';
			B[i++]='nTTJ5F2le2o3HkohdIxBCBN0LPLIJHfsUwgmjNyUokpFNQOAhHz1n1ISRBWgVE7NPMgii2RloAM/';
			B[i++]='XyUW0K6GaLTRQmvlyKmRVELrqrZq8soCvN76iq29mLIYskECecrXqRAJjI/BYNb1C84aI0IDaNvY';
			B[i++]='jDHVqg23NujyFmU5ookwD9zI6K22vNX+JtprXdY2UG/nEFAOAfYalEqZyCX3QQMCT0cCdZe3qbnl';
			B[i++]='ba4JJ0UNUxyS6N+9hNJ55HGQ0ngan9Q6opKuhMEEs89Ou+0//TffzPTVF/Lvv4O8MqNFEU9VzVHV';
			B[i++]='TBXQyIflVQcpZOj/adIccrigq1u9SkmJJLpSddW2YiJXB7K6+AooX2t9wyfA3lIZLbrAn75gZP9Y';
			B[i++]='jDDM/m03MeGCuwxpgsvSb16Qr+B8QzjhUNeyqoWMvr0mS/diR5UIhw5ieEAE/TrT5CYgnctl7oNt';
			B[i++]='etPDpDOwzYFAOic8mHcOZpPQiUciqDMJdia2Ehr2SVCl25N62KMSHmIMdbGTnVAyRZTdFcWISZlZ';
			B[i++]='TyAVlabMDEECigrQDIS86o2KQmRZi9AmpBa0VMhUIFqVXZjWvUmoCoxWYwWv1ogiA4DiMClizI2I';
			B[i++]='ZTazGQkBN8qMkXgECh7lIhS5yMUvVIGZG6TCkMLglmZGcxppZet+/6GRlpKWtIxyWck0mJySa9oh';
			B[i++]='QcPZix/6QAcE2/GMCzJgAJFziAI4iLnNTSdObEohm0BCuhd2B2XbsR2ltuM6PuUyYr+klKQwpkv6';
			B[i++]='CBE9g0LPx0KAKQmA7JkFQh5Vctayp9QsBTsbxFLCEhVRcdNBP0seVqSYtC8ibXtkoVCHLNEBuGDC';
			B[i++]='nXTZ1YrkCYsGvEITlojRrni0Pn76CkWfYN8m6FcsyYSNFwhtgCEReUgd6QgbmFnNsiDJDdQ80hvM';
			B[i++]='whaT3DWuJ5mmXJucIG06ia7fPCBKsyGlZjIouQaUhHMHyJxIBubBhYnEgyZ8JUXwJMNefqeWO+0O';
			B[i++]='dlTXnYsI1XQu4f8Yx4TnRP/4biiVYk/KiNi7AC3KeAJKCqaqukSYgeV6ooJQ0U51orJmIouQWNoj';
			B[i++]='FsGqvKQVnbiaBItaURa0lOgsaqSjH1mBGF/xqjDrA1uRjLQLYvBikIjFlmLL5oxUvG1Zz8LfafLR';
			B[i++]='yNRwi6Pbwhvc7gGORj7gBSE9nEitJBB47MIACogAKv21wViisIOurM4rZxsd13buOz1VXUnyxNve';
			B[i++]='zvCnFPNIeCxCzF+aQCk7bI973KOfFCyXPS9Trnzgg80kAsgnzhzKppzyMmzyrpoJItUVp4cVC6Xz';
			B[i++]='LG1tGhgtpBbr+axT5+3iXsrqvVAE9muOMUwnBmOLxQR0fqfgI2H/CSstRTLjBpuBRmM/gw1wVeNa';
			B[i++]='oO1Mgx3bGkaSRqJ/+9s7MEkNuXXYM3ebF5R2IQLVplI5DTDBAUDA4haTAAQkeDGLZUzjmG6OtxqZ';
			B[i++]='E261A9w5iZCEedrpRfY0ExeWrnRH3VMwlwq8leXnZEwFXpRBZqlJncwn/NlPIbbMZUIYDykXChrQ';
			B[i++]='xsnWsULInWh2Sz71mRf66hMWc40EAprWVhbZM4549qORLLoagx40ofnz34VBo5lhPKNZeWvsRCt7';
			B[i++]='rdRU+H8cBte3TKM2dZUgGqMUqbwefIyTQia1q9VgimM64xNmTpaeK1hsXcviVVcHYaom4ecgphKT';
			B[i++]='9Na3eZJIT7uj/xPsEEo9Pfm1sCUFFNw5uT/XFcR7eFe8/yjveFMR86cm5IhJZC97RxMfieQiK26X';
			B[i++]='yBXZTlWaXzSCBswlv//txWEm40dc6FXAAI632AzA0CDdT6OQvXBq7p2ZDt/Ifw6uxv/2dgxMgpbD';
			B[i++]='DcyGoZvxRlCf+F+r9JwsbUxxnMb0cqVuMYtRAAKOo+DjHQ95i2PNQoSZ3CM47q1IsvOwGbpcIkmG';
			B[i++]='SUxQFhOZX0wmGDiP6XJJ5ds9l3bMBcp0s/xkoiDl6FAcsxWxB6EEMeJBRvt2J6hG125jrxHa1l4i';
			B[i++]='VAWXNf4FoGAHzGBEESOtFQsxWXPMjowkNiIhixSVySPZOPwtS/9SEpMXvAcA9d2acUR6o5kcrTsu';
			B[i++]='g0cTsxYBFgCBCzqOAhc4fvEab/XII7/iE4Ig8dT5iK0HNrCJT1y2tMSJReak61vvNgRLHjIvd9vL';
			B[i++]='HkpVuc81wQyGXanbrWx2vdsdfV7WPOtBewJZPfqFyHuVdFaPVNpbFZ1FBBdxs0q9a7aLmvlyibPM';
			B[i++]='JXxoZlFecwWYG5jC+3vMTLuLlTX0KUYyZhtbHRWAWmCwPVnKUoXaihE3uGXLsh+OtGkMfY1IXrh/';
			B[i++]='AMdRUrIP73AjHnABhqdBBoABB0ADiwd5lGd5KzYdKWR5kXeBreZqsuZqQCU6PiZDpIdDw4YetBMy';
			B[i++]='zzQyk/IxQAH/MjTTVJnSErMnezIoCDO4Zd6lTU7RAdfUTQ3yIPClTmm1FSMCCeXkReU1X3dRK9P3';
			B[i++]='TmjWCgpFdvbkZmUFZwP1RqxAP4B0I7KgcAcoDH5kSGhHI1zzNx6gSXezYdUwNsGQUMRBJPRwDJXk';
			B[i++]='UbMBD7HxAH4Xac5AgO9SYYHkcIfHgC8WY4IIYxRYeS3GcRiogRPoSqnWiJz3YxQxU3gCJyonZA9D';
			B[i++]='EShBVCcQAq6jJxrDEqgnTDmHc6ZDJ7o2MpuoMsBTOz8RdMrVXPSBdEZxIAfSKJeyVdiVRI9iFTwj';
			B[i++]='RYogFq8iK912VnTRXkhzRURoV3vBRgH1bXRmKiPAInImTzDC/33/JCOmUCR9SFhhQwzg4EjUsiR2';
			B[i++]='CFHOsg3HgDbD4UBLsg2VNQ2MtCQXJDf8I44e9i60QYek1Q5xuA+psAkKQAChJjkLYAEk8IA04ICN';
			B[i++]='p3gzZogROIEwZoGVpzkVKJEWyGKplnkmB0Ok1zA45oE59BITETov4RIvJB7kgRM7YSm3o5KOgik/';
			B[i++]='wRTFVmyWIjyyKIu6RzOGsDNhJSE8SQlPNypbgXwdUnym4oxFaZRolXzLN1eHsV9e5wG6MldxNSN/';
			B[i++]='NX7tZgy4EGh39z8XkJW30CPphz8VtpWU1JWQ9j8RlX48An+T9BkLR4DXcGmVdA/jUknbIgLbcGce';
			B[i++]='AJAPNzkq9v+QghhjpPZiNBCIFZmIi/h5rqaYMwVk0PGIjvljIsERs5QRImSJFONLS+UxS9aZPLFL';
			B[i++]='uRQUtWNsPqFLpJk723QgWVWL0uMVX+U8PBgiaqV83ZMJsFAi0Xd9XJdmXReN9bV92BhIelaF6qYY';
			B[i++]='6kYsevVucQQsDrWWeWRvblgt0DlYqrEkC/cN+ueO2SA4J5VA9cBZgpMag7EA/xiQDtEBGEACB7me';
			B[i++]='hSmYgthBIBRjLpaYGCd5hSiBHAgdOTZLN0ZbHTh6IRknRmUSSsZzoTia20EyK5h7RVeCP2E7/qEU';
			B[i++]='3QVmzCMq3vSaFupe4VQVVNFNHfIhYkUJWXc0tskXUvNXhEH/NXPmdWu0PrlChZ7QdQHFPtk4CuRX';
			B[i++]='Cs9ZIwRHGmn4DIwVNhh1DBgVpHnDox0GeAWmfyhFaXO0l+apHCPAgBsXco/XeBBYaoGZpe9pmBrn';
			B[i++]='kBHJmIq4agIKiRMxmQHqHQ2Dpjo3VLfGWyWxiak4VcjmE4QymjWBc6izHSUoPC3DZYXgXUuhKcZD';
			B[i++]='i5wyi4Xaoc3DM+AkFqPCMx/6KlYURrTpNGYkbm+BADJKGCy6V4XRCVFjotqnqZo6C1ZZqu1GP4JR';
			B[i++]='JPijGqRxGaORleOnV/Z2JNZASpjkf6BBd9MwR4TBl6w1AhpxpRpnmPI5iIK5YhZnY5OHgSJHiDJF';
			B[i++]='cWEKetZx/1QvB5KYaZlAhR7ERR7J5Jl6Sin64YrHlh/eNa5YNhRPsSmcQgETMJTkRHxM55MkWnwb';
			B[i++]='shdk9Zsn2iL6dW5VgyK2MH4E5UcGwCOG8YXtZoClShgK8K85kn5lqXdwaFFqWD9qKH6qARr0IBwH';
			B[i++]='1FkfxRvgeY7zoAqY2gAZ8KT/MpAs5ngt4IAu0Hgt8HHr+bIv+3Ewy54ucJAg4IA0kLM726UTGaYD';
			B[i++]='cwIHILRyQkKRSDCvJlwekaZMK1RIBjq3pmuug0s8FEzF9UvA5SfIpExDZHu5QxQCwprkxSnuqiDM';
			B[i++]='Q06/+BVTZCBB8zPsRT2Puhar0gGscAlypl+G8amWupuScP9nUTmyA9Vf8LOwpGqAz5kYgsR2kTEk';
			B[i++]='s6oLXThIGcW4l5GqTamFsvAJhqRQQJo2/MZ38FiPGhWWY2IqCFCyfYkAQttiPdtiD5iQM1asMKal';
			B[i++]='sYulgnmBwsqsP7uImvOIbdq7kvljuSZkanpURgVELDGSpTNsKYl6HaM6zOsxJlOaw8agTZV7WkUg';
			B[i++]='yeNew1eLGFIgZ6sgiOAVYgUqyacX5kUhUukJZ0WM3iNurNB8dXGb+rQ+N0Bvl+uitJCccKSpvdC3';
			B[i++]='bvR93Jg2Eit3CEuqahkkdeS4OmpoXDktpAFh+8ZI2dBAw2FYkuABpnt4BHmQN/t4j5eyLvaQiyhj';
			B[i++]='qxt5jqf/ccJarNGaebdVpvspWxuonzolOsJ1cqKjpr8WbIeSH595H7BHe4ciRPxBXVvVKC55RCGT';
			B[i++]='XUcRfFDRIVKhk86jCAuyIUD5KeUbI08JC7y5xVMpI7PgCS6aV/h0qZ+AZgmVNX1bnMQZlrM6LQI8';
			B[i++]='Uc8iAoZUlquBI/mWN5gFYnrXNt6QLW80sBmsQcCqeB5spYX8cS1Ls4oMch4HeYvXs+qpeK5LiIXY';
			B[i++]='Js/qHKC3MJo8U77LW9DxmJ/8yZDZyZ58ayunW505bD9cXCJjp7+kw8IGe37aXeD1ZUdBfL4oXtgW';
			B[i++]='jC2im28xmyEKCU2YffHEIuVWF7wpF9GIqZgKUOgGCm9h/09vdJyYocBhQ7EIq34F/JXOKZbgKH7d';
			B[i++]='0nd4yWANBmI+uoanUG6iYCt48QAB0JcjsGKLPM+K3HGJ3LIPmMgrW5g0sM+IjIgKyWIljJgTeGqb';
			B[i++]='40Fgusmv9jAgeBMsR8q9CyefnGO8dRIEisqhWIKWgjHtATsdrVzf1Tsr4Ts3KB/YtDMxswjblKG6';
			B[i++]='vDSN6qHFNxZ3O09rMYwkSiJo5jQ53XVypUZ+oSIx6sVP6W595YW1oABbI82K23YGNWCzymcN283n';
			B[i++]='nEhx7KNpqEfAwMepUC1WSLLvzFodkHj0TLOJTNY0y3gnjLsPqdZYupAZR2OBCa2ZU0KhDLyzpXKc';
			B[i++]='N5kZ8f9qWYswQYZrImRr4jHYiMIToqmCKdjDCNprh8Ie9OGK3ZVlEUozt7y9xOfEBsKosGKhUwSU';
			B[i++]='UgEirbJ8ldA0agZPZOd1rbDFx7zaWjNQjmGjj7EYp7q4SA1gr+3UyrAMjrXbzvK4emQKBWUshCQ2';
			B[i++]='beyFauTVfekAHNDPLdDczK2e/dzPKNAChIzIJwyBWhqI7Kl4DhjQkfdisGVxHiRTzxqtFC3RQCag';
			B[i++]='z+HXNrW7sjamwluKRMbRy2TYKfBMpem1QwEf1OVdCtKn0YVN8XHSc2pNUVQz5SXTl31Ooh2EHOLg';
			B[i++]='9IrTIyKMRmO2mR3TkoqEu2Kj/9va0/zhueBGm8DNa9n/nEJiuGrodgAWls7J4jlKpGVZI3h0sJox';
			B[i++]='GBecAV+tgBwgyWP9cRrQ42Tdsuy5uq1rz4ScwrFbrNmNcYMJWzZW3q6myZtXeoAttSC5eQTqHXQt';
			B[i++]='raNzwzvXQ1wbO/VhvcVTPE8WZUhxi9d7FFBhM+m6IHAelGMhNGARIWbWVnoBRmnRRWlhV1jsF1GI';
			B[i++]='t5cajcgcv1EjI79iqrOdPnBhbqOgzpHBxsXtLFn92+h8UG4MLap6zXNGNe7clwZwAgX5waybsi1r';
			B[i++]='zx6Mz0Y+ycMKYzu75FsqwjKWiBIIpq1k14HNyaXsyaBsEiV5EYFdpsGu64INcyPxUzCREWIeZYKK';
			B[i++]='xAx6/5rv0V32UYK6cykuyObKwzxId9nTVj14ztNwUaJtNo2l4EaqPejOTNSDURhW2OEzIm+nIOKP';
			B[i++]='Me8GTLHw9n5LPRlRvSN+ZEF9Nn8a5cDiFyOfzloLwAE+vsg/TrMLb92Ox88cfJAksM8H6dzU3Z6w';
			B[i++]='nvGwi4Hk/Z4hpIjx6SZvIhE1bFQiyXLU+h3HW0PjoScQ7aYlsZm32JLzwUz3TTMhICD4Edk2CHu4';
			B[i++]='x01VVMVEo6hP1zPjxCA/aFayOb52HnUtYiIxYivY+F/FIsZ4CyPOnCJgSTZnXAqO4X0Km7/9VAtf';
			B[i++]='+Sucmyxdj9tAAo5ms/YVK7KPgAAFr0EUcAAuoAF4///jC4/3Y93wi6yy1x1jLTDqjqee2e1iOHvk';
			B[i++]='N9vqHw+8I7/C9+lKnazreu1jw3UdvmY62MGJNFe1wsQSOeGtVfagu5OLskiL8aUVCX4hVURODgJt';
			B[i++]='VDROzPOD3N4hUmc1fyu/ukKMRyM1f95G6XbbguRHCDvi9Ju/h6HuxrK4+G5+aH8KxHlngEQkPsIj';
			B[i++]='XSl3bsciZ1ECOS45yt3c4A/+0w3+KosC0b2ygliYHqyQhZzWO5v4OUuIrZ7kTw6t9j/ex3qs9o9C';
			B[i++]='mAcIB4KDhBwcB4eGh4SMjYMWBxYcFpSSlCeYmRiUJp2enhghGBOjExMhJhihoyEpp6qwpq6mtLQz';
			B[i++]='Eyn/JrWytSm+vhISvykzxBQTxRTDyxQUEs3NDg4L1NXSBg7Y0tMd3QsdI9/VBgYI5eUI6erpHuoN';
			B[i++]='CA3x8PH09fXw6+3r+Tf4++YAyZHbhy6dvHoePNBT2CChAnsQGyh4eJCeAoUXG0qceGGiR4ofM2IE';
			B[i++]='+S4ctRIBPhxZyfKDhAMtNMicqSEGTRQ4c2rQicJFzp49XQgVCsJFURBIjSKlAYKEUxI0aECNOjWq';
			B[i++]='1aJKkTZFeoAEV6eCSHQd5HQRWUdh0SJCpKhtWwsnDpyoRHcu3btzEd29hAFTX1ghToUIvIuWrgkn';
			B[i++]='OmEiDKsUrVm9om2bTJkyNAqWK3eozHny5c/dQneo/7ZghEB0p9MVNHd6YAMD9tTdSIhAH4J+6s79';
			B[i++]='M6guYcWEDCUinHhjYkPa8YJv3PhReOwGN5IDD8lxogfrxIHP7meg2oOULFtSOOATJ02ZMVuob2G+';
			B[i++]='BQ31RoXi9CmUxtDyQ1EUtX8fBH8XVnnllFYgHFBgI2IRkuBTBR44loEOqvXghIxQEkklmMAFFwcY';
			B[i++]='uBUCBx+CCOJgI64SC2OqmIBKJ4h1kqIqJ2zSGAappPjJjTfOoOOOzcwATQcUANlMaJNJ0MEzEjgA';
			B[i++]='ZDfOPDMkk5thttlk2pgzDTVUdrfANth0eQ5pI5hmgGlitsaabgClswA7vcl2m2rnrIaObevEM5tG';
			B[i++]='x//5dtx2s013XUYbOeTRdfZgJ52f2iV0w6IeFKdAbdf9+ZCa2XynUnhHfDAeCuft9NN897kX31Dr';
			B[i++]='udBCeZ+mWh9TAErVlIBdDTgggVoRdRQI+t1KIFO0CojUU7EGK1aChEDCwVxtZRLjCcciC1cmGmqo';
			B[i++]='7LKpDLaKtaEMpu22pNCoSyc7IiNLCiEEE8y22vaiIzEpULDuBE2a64yPn/045GfSYObAZdZs2S9p';
			B[i++]='Wo4zAgILkPPNmOMkTE13py0MJ8MC3danPv3Q1iikq6m5W239RKdcRPe00+g7EivQcZ/WMdoocA6x';
			B[i++]='fBHLML883cvYAdeQldmIAB6mH6RAQkydopeeehr/xJRqTvfZF9WqAFbFlNK00vq0VVRH5Z5XWg2r';
			B[i++]='9YJbP6V1sGkNEqGEk6xVodmGoL2WW3oZ4ldfGU6biQkWeIvjjTHCyOyyHM4I2Cmn/CIBLsqwS0wx';
			B[i++]='zxy+rrufzRAMNEk6EHln0wTMjTcFT6P5llcuTE42rcUZOujZsFZw6BGnqdqZqK+zKG65ofP67LTT';
			B[i++]='vvLrg8LM8gUzS2qd7q83Grzu1hHsAAKWYprpDAdwep55nB4tU/Q/zRR9pyiwYP1O06u6KlNYRz0r';
			B[i++]='gV49SGyBwxooNq1kK3JJs3Jj4lYibeuFobKDnYDuhyHCv/ezifFLjFIBuALWwjGmoFFhhIEkczkp';
			B[i++]='/19Bipe8ggQkHTXJGdCA4L6UtA3SeHBz4gih5wq2sC11KRtYkkZu8gGplWlHYrdBDcNY8w+WHUcj';
			B[i++]='CmGIyR5CHYogpCJ5suHtBNUniRVxYvoQnhIvpruLYexPBuuAAXR2qfBoijzcQwF71qMeLbYnJ13U';
			B[i++]='IhfdE6AW+MdUp+qJ1dhTHva8RwM0QEEcraIeqVDFjlIREIN8JYgGoa9BYtmKrIDlCLOorRCCONb8';
			B[i++]='mOW+SlwoEpGYBCQsMb++qQJ+jXHR22YkilFIgDCuAIaOHlOMdimjlCkQRjPaJQxgtNIXq0yGuyoY';
			B[i++]='QSEB6V9YAlguwyGQcIwAStDYkTAd57hmGMkbV/+CWGtKWELdDCRjrFsmNGuzG0XFUDWvk1gTE8K7';
			B[i++]='jsjMN8HJ5uskwjFG9cmcdFpHd4A0guTxjAI/o0kMYqA9erLgnvjEZ9C6xx4xjvGf8CnVqfKDtDpa';
			B[i++]='7T1Ue0oevcbQhsaqfBB1BLEmNNFBmAUSjxCEsSZRyY6urW2SMERILek3VYhCFIIhjCkC08la6OIW';
			B[i++]='wxwmMg73mFAuw1wp2NeUOqikbuSUSRQEagQp2LnSNGxMAtGl5ww2EBq27mHlGBjr0iGm1aGpYGKi';
			B[i++]='RpjCFLqqxk43tukYOxQVxG0KKne8q5kHLsA74PGpYyz7XKUKUEXxgEAD2surTFhAPed5SnpexEn/';
			B[i++]='F/vJk+txr3tg1OJ9VmXQqjF0oXl81ViyBiHKAvJBTQkbWjDKLElAMpH20wtIRXqIjbplb4uJX2JK';
			B[i++]='mskadXJGNYJFje5WitjWNJUOnOBloCQaoC6Jc1tyBgejJFR7XaanJARYUzcWu9KFznM1FBnLqMlc';
			B[i++]='5mbzZvpATQ3HatZIOeoiPIRddZHoQuHNzmDZUMDOrPgSoBUNsT8BaBeL5kX2dO+vYFzP0kJ1H6Ac';
			B[i++]='pWpZU+hikxIV/3jFKGMBC/rC4qAGc0V9fowahYr1yJGyhaMYJu1eOHGXTliAbp6YwN1uJOIQH6YX';
			B[i++]='uxjMSknRLdn2IhQTGFxuZ2xKeDljSUvSlwep/zGaolYjStJIbi6budUi8zJMquMum6rLJhfOJjdG';
			B[i++]='poCRp4yAgcXDNL2BWXQDhSjyesAAevpTo34XqXLYEE/HsZIzFqAAuirvAzPAlU3uiVeZ2MQmN+kr';
			B[i++]='qgprWOv59Xn0JdoWR0WUobhqoQ1lEK/8w+hdvWqQaUmfZr/GiLE1SEKQ3PD9TBA/xXCa05xM1/5Y';
			B[i++]='msDGmOIwoQwGK80lOVZzJnIOBAZun0G5JLH6cbjerWh66o3RaNVhXI3YmlKjjtOlLppmQrZVucrs';
			B[i++]='iDG1mi2U7p7OSV5zPtk2JHMHckQWbZtl+x2x4Q3OJDBFN/OsvYed3k4Ia559utvPWfQeGkVFgv9W';
			B[i++]='GWW/Svke1Y6SlaiRb5BeA5+BgLU1BYFF0sSqqFomaSxIxGWSHJ3khepyCbh8OoB9QYWyGpO3ACYm';
			B[i++]='FS6iUYoW6CRaU65IDpTGkSQorw3uSzId3DHnQGiNzvk4YD/Wki95aWxjL/OX43D2VK8Jp0XV5mXU';
			B[i++]='VpminHxdcRsAN/3Ak8s+4rvk3CMiLFO68ShgAPXWdSU9O8Be96rPOu8TaOl5r7ppcqrBAtQn8hUj';
			B[i++]='evz5k1vpkUBDYXSB/Q2szC4owWFTOCPoZ9G1YNRCiHekw+WX8VFrK28yqpsi2IKWRCiCWdnSlgNj';
			B[i++]='HIxaDM5dy4Bl6BsIyx/tNBrt2kaQlHTcJW3/EEj5UvnpZp+wpoquINyhTZK7SqZelqT3qgm2VAsi';
			B[i++]='w6FDyiA8FA5IHuIQ2KQj6dE0dpnKceWIjABk8ejOvrq+3pbE2ex5zec98cz2tBetn++mCZ3zLG+4';
			B[i++]='/4RpZIRKoqniFFcl+v4KHfivstZgiYJNfYRwaQD4CBjyLHCjP/oTAox0ecriNoZwLdZyUqNWGOMi';
			B[i++]='LixWCypFCyr1SSnXgfGSQfvSgcGQLylAJJvhazKHDT6XVELnVKmxVMpkJcfWAFJFEGLSe89lGkYl';
			B[i++]='VVL1GqaRbfPgdPJAEMUHJxuTRL7xTeD0JxYRG1hSbl+XKRJAAjVxZ+nnbuR3HvZ1NEEhFG9n/yqL';
			B[i++]='JR+5gndJUYZawSv8wWi+wn+XBUgKpkfCIiGCh2mOcHiadoeMVDfLMhdyA0BxMzc24gkwZQrFQC9D';
			B[i++]='FS+vRgszxkCtpGsRlCQgSDnApSQiFHTSl1VYVmXrUCbqxHvHdntrok4rZA5Qtxu083S4kU77cGYk';
			B[i++]='YzNMVifgNg8QwRAf8xoL0AxsZm5WNAF3NXYsME949YtXOBNZaGeAlkXrJkapMlBhxEZqFBT/ARRq';
			B[i++]='hFDUqDRPAyBHgRO44m/scyBzSIdqYRbiOHGfVXGJcQl+UWKu5Xj7oz+IgQqzcBifllKDwYGLiFM7';
			B[i++]='QlOjVFOcN2NSkmNChTm5dDzXQCXJxTBCJv9zmSMQx+MlBKMlynRUAgEOz3SDVoZ1NjMSMqOE3dZE';
			B[i++]='C5FDelIoDnFDz+c6b4Ib4bYPkuMLDuAButgSU/hn8aZu0FNnfFV2xngexaiMn9JPe8aFXNiMzLhY';
			B[i++]='BYY1cNgIAhghsxJp6RNIlUaHDLcXFVY/pWUJmqYJMeKHy6KACjgtXamAmQQYgQFjsDUKtzBiM/BS';
			B[i++]='OmICwvQkJyglkCOCMpZbnOEvQXYlohEk37ANm6FcB2kOp+NcA3EDpYNsCLlcJVkxMRMPxnEPqqN7';
			B[i++]='S3YbSUQP4rY61HV81JREHMMmYGaEgKkME9CSEBCFH2AC2SMDqJmaqikD4qdX6zdnQSNY+NX/HukG';
			B[i++]='aEB5m/klWKICIK1CFYzWb1HTH/sHaWDhCGOTUQyXSIYEWvSznA1XNgX4PpOAP37BIRGoLdliUpnn';
			B[i++]='eOZSLoOTQJ7AYeF5N8KEDDPgeionGt9wgpuhJZWzkKRRl8qlc2MyZRDjY/1CGegQimoiTeQQJqXB';
			B[i++]='S2cSEF9le0mGhAiKhHXiDld2kRWBD9BhTpLZDrghDSPokqSJATQQjOJnk/lUjMO4Hv50fvLVWI2F';
			B[i++]='UPNmKu+hjG3XonXUAj9Tb1TTKkvBK8U5gEipf4jmUDcqaRllnAN4nBlVWoY3eQ/YUR1SUnMjnqBQ';
			B[i++]='YiXmC7xQCqOgCoADGKjWjzSWOO3yC7gw/wu3cI8pZ6H3iBnHNRkkJIN2OQ2sATqnszr2WSa8FDrV';
			B[i++]='pHs4hH3QMaG28SUAOmUjIGV8GqBYhiYE4Q5mVhvUIFwi8JJgx4vZM0/k94t3hmfkJ6nEmJOAJmgi';
			B[i++]='epuegh5qdz3QYzQD1UanojT9Zivc2CuXNjYYVSFCCo4SYocaZSH2c4eaBp11YReXMFssplLe+Z3l';
			B[i++]='MmqNwT8mkoG35UqlJ1z/2GPB9XINJC8TZEu8lp7sGRqk4WsmUYk9h3MkpA1ckjlMxm0nuUJPxVzY';
			B[i++]='NqhEiDrEZpniBoTBhRltRpoW4AL21JrAWK+tSWfhp0/agz1s916CRl/cQ1ieQh8ARTUv6v8eL0qN';
			B[i++]='t7JoXOFvlZVZhaRRP1p5dDiOa/MWknQ/fIiHqnVJHYJacgM33oIKYLkscANyKAsKCoRAuMB5rsRA';
			B[i++]='spZKs/BKw4A4wuAjhogk+kI5ERmfZrolyESJCulBorMxMoRU6FqfmDh9yYaYnThNhvmJTVU728E5';
			B[i++]='SfKub6ahHIpPxUh+nBJQ6VevQTNYMrk9MsluQSFHZGRH+Ne2gZRw5ZOqACi3DYuUp+ofi1YhDVcJ';
			B[i++]='GYsse3Ms0pIJIqUsz/IsFxdb+yOW7EiPpHZAIXZbm2cKYNqrlTGCOqVyLxeJJ8dTnYENCECQoBMw';
			B[i++]='KHgwBlMwo6swMPiKUOVMcdKmaZIx0oX/mZGZrksGTdn1VbcILx4wmm82AVT4i+KXmizAmsMrfsB4';
			B[i++]='E7XZbvJUqe5mfs4bsH0FFGEYhvDRX9t4t24YcDc6eBRGYSCwqnW7cOiYIX4oSQsYcehojrj6YR8G';
			B[i++]='Yh5GW97CsgrUpTLbamBKgbNApZ6XSsowgjvLU1giJBRUS+dZwB1wnkNyi8F0L7ZEJJ2jgrokukRb';
			B[i++]='MJ5pfDRUZce2ilqWKMABQ8HHVSMUoKnDHabDggx5nw7AkrtLmiFAA/SUA8DLtY5ahfXatWM3jFe4';
			B[i++]='rzPZfkijH6aKFfYmf/KHsG5LLMtpNph2eAfyveUYUmvjSKBVCB3lFpS0FwKUCSKmgVn8/y278Ans';
			B[i++]='eyNf/L6pkAIjR4hcegvkIrlgalynxCOGeIj6QqYYBBqst3pSEnPAFTA412vUupejsVNBxkxEm7TP';
			B[i++]='dK5moq0rGMGhq0yLfIT5oHufc6GKmim+W4U5GamQSk/2xKHBaIWRSqkgOoyG9bUlWh/TS5TT2xOj';
			B[i++]='ir19N3D/t5xGSnjheJXrexdFOkkLyBYmCwsgEgvmwrgTuD+n8J23xY+FUUqFKC77eECkoAuhFGNY';
			B[i++]='qlv2Ig4c1EHdoCRGUkt0nJ4/20txijowaMJHplW+FJEm7J9O+w+Aeq1bJWRRe8hQazqtJpoZuqHz';
			B[i++]='hK8fWk+ZPMOWfM/CCLw1sU/Ok0ZbOP+b7yeN81GiXGTK2IsUXRhHoCKcwBk+guQ1/4fEh5dpiEeA';
			B[i++]='jgS4C/g/AoRJHDJANqJSfiMK+sNJVKq/YzmIMNWWMRXTBkwvObalpndMFGQN17y5BZkNBcmQ17Ct';
			B[i++]='vYRVUVWfp5NV5/yfBtPOOohV0mfOUcXOOxengWp7onPU3uxBoxEm4PBLviTC+6mJrdt59PxmJuAC';
			B[i++]='w7uaar3W40ev/mypZ0eTZhtvfCaGuNl+wom9SqEfY9grF60gAuiq4Th5nkXFlwdAh9sJqHBqLK15';
			B[i++]='KkYYLOXYvbqIUKqOHnaH4DJMW0prVruI2zA5ZlpUP7tj5lxk1WASJvHHaRrI46BsWLX/VRHDbFWV';
			B[i++]='VDqIkMxkwk6V2wFxGkjGVaP4tEzWaiFwPJP8ARZwmmxNvKxJvON3vGPrrzgc3c7zKchom+9Fdw+d';
			B[i++]='UDuaaJLlo2PRnG1BeUgcjo2QCFFpLFgpuKpFuNDicX9YNyYVGK6AUzM7DN0pDNFsa3OMY5fhxjwC';
			B[i++]='GtHQYxuUQbn1gZa7L7cYe3jsQdc6tENbuiahtEZmVEbVgk3WQm1SoEpLzlN9reasOkmdkNDVHQRz';
			B[i++]='DBggASv8Zi1sT8rN3Gl9rzEM46Hcbl5kKoolsP600Io1b1yE0EAZR/JBR2tbRmtrUNioKwGGFujj';
			B[i++]='lGHBUNxLeHc4uHwoNwoIah0HrNeJ/y4T0MssJlsussWfIEyzUNnQTN8YBCQ2XS/4coKi/Wq8JiTi';
			B[i++]='ICSUeM2AHGStQ2zoulQohK6q8ZH/gBqXSCZURburKycarseIDKdKHaitFmMGEAGkyQEtEAM5kAOo';
			B[i++]='WemYnumUngOO6tyT2umh/NxBE+pEM9exWaJsu0e/eRVWYW+jWkf19yu+soawokcVJXhpY3kXBkmL';
			B[i++]='sAh0gWHIcnmXd53ZQsyWPWKZzSNbakqdl1IiIgqG0D/amS0FpEoLbEGQ8yP6STnTeroKmZ4/Zg2l';
			B[i++]='u1TR9Lk0tMgGWphf9c7hHGx/Xl10YmatS+7aZTHtgA3CUNY8c9zFi5r3xNz3DOozPP+pY2s0NHnX';
			B[i++]='fSbX1i3KBt2TgdVY9edYa8iNsNKqEavr7uM2nUWrtyo/5Dud89NJWE4YoDalTgqlgtN5xv6dEoAB';
			B[i++]='MvvyK4drToJBDwSJ3JCeRNUBpqSefcmtnOsZAqPU465Ut3cm4Synf75cw5ZsGr7bADqfTLVMQ9vO';
			B[i++]='HP7gBuAMokBuDJChk87pMAzDly68qUnpZC/wZt/p93rDx4jD0auiRLyb63HddI0qA/YfvPloidYV';
			B[i++]='h0ACR4wW3wjLgj14lgfsmPQXYgnNhdG428mB6ML4n8SO5NIK9Uhqj+9AEqQvsAam0szGptdbKHjC';
			B[i++]='SP9cR0Xa2XrUTEvIDTNzk6HaW83/+g6egky7gkmL5i6PAFv/Zicw6f6+3PcEw73/78EL4x+K9vQU';
			B[i++]='0MW4r3+mRWvvbtN9X4lFd9C9RYRFHwxr94J0qsDpd4Gt0RM3vg2oSOBvFxGXsd1vjpIQCjHCP/mz';
			B[i++]='LQMUQJ/mYeRbCTbSvsj+uI9BzOcyzCrm+JMNCBISDoQOg4WGEhSEHY0dC5CRCwYGDpQGk5WImgYI';
			B[i++]='hJCFkoiSnZmdlZeUkJiSppisiA6rrqm1rKqRtqkjI7ULvMC9uKsdKRMYKQgMH0fNzh8YLTExLNPT';
			B[i++]='1dbZ2tosGizfGuHd3uTZGtnV4errGi3s794o6yj0LTQuLi359zQk/i407uHDB8IF/wgSBQ0SBMEQ';
			B[i++]='RMCCDxkGbEGRRouD/g4e8Odv44GPH0kc4MAB5MeSJU2qtMCyJcsTLi2cIEkTAwebNkPQ5HDiBAaY';
			B[i++]='MGOaOGECw08ME5IqTRpCadNjSo1GVWrM2ARjFLKm2Lr16tIQIVKADSGo7KFDhhhJ6EChmCO2jxpl';
			B[i++]='LcSWgqKsgkYtkKWJ0ClPryzp+ksJQSUEnhA7MKyrwwhIwQw8Zlw4lWJdiDNrPjW4MGXNiCtJmCAB';
			B[i++]='AQRmzpp9sICChYzXsGO7dp1jdrVv39DFOHeuGzVvuIODEz6unDht4aa9k0evuXMX9ZprcN7cYMGG';
			B[i++]='De8dZAhwoMKIDUGKtHCA/EeW5//LqzSZcj0HmUBfzqTZcyfJEDZv3vyJf0JTo2P5599TRmEwVEwI';
			B[i++]='umTCggsaaIJXKZRFVlVcmWVhLIVkpWFbi+i11yStNAIiKwvUZRddjcSyV199YUgIWqG0ssolgpV4';
			B[i++]='SWighVYZaJ0g4ApkkQgjTC05GlZIi4gMIgF+J0xgwDKpqXZCa7G99k1tuOUQg5ZcfiNDbsQFd41y';
			B[i++]='v+32jpnU5Dammtas48507bRQD0V00kmdPnLKWZ132DnEEEcB9UOCoIJyNKhGhiKkqEjruUdSeRag';
			B[i++]='NBJNI1VK3qWRRtrTfCfo5CkHOtknqqgYBHgMkyY89VRXVI32X1JIxbqUCTNMMEP/rUlRaJUiKVCQ';
			B[i++]='AluDdJBXLDA6kBVcjmz41l5HWiKLJ7L4FQsmsozIGWWvlJLJiCAKli1iK1oyC5C/PFIiL+ZC4ogo';
			B[i++]='Lra74ruXLEABVMqglho05+RQpQw59Ovvv/9uuY1uYQpHDjwIw4OCO/RMxxx1EO+JAnTN1TkQnQ8Z';
			B[i++]='uh0JBvkTEKDdfYxQnwdg9xEIJaPcKEjtSZpSS0FF+t578c1cHn3ziRrCpj75ZMF/ACI1QYMGItUU';
			B[i++]='hUtxNTSDC1rAdNFLXcVVhHbZJUGEFmZNddWIdPiWiMlSMEOJX78V7YfNKmZJZov52Eq0cLdbSLGA';
			B[i++]='fXJ2JNGmm20qI9od9yio+FiL/7HHJANllB9wQOW+sNX2pZVfFnzlbPxOTrltYZqT8JnMwRkxPRQP';
			B[i++]='NLFCfHpnuj4X4/MxDRh11NFGhnqkksuTSqpxnw2hgDvKKje60+yVojTqWGPx/FOn+IEV9VIF+lfq';
			B[i++]='Mc0X+BRUpbpqofJNHU2V1FxtuOFdWYcv9/jGlq/sXNUyS62z117mLGCgzG2s93XBtQgotmBLZGD7';
			B[i++]='x+tABy+SkE4OEAIHZMBez+BACwAGsNf0q3Fi2sZtqFGbaeiLBRXcxjl2Yw01pcNzINyTnhrmMIjB';
			B[i++]='aR4lDAfEKLKwOtUpUCJLFEf+lLLdMeRkvGPUyjCFnpbc7AAnsNRIWPKym70HUv8uQaJ8fEK0ohTN';
			B[i++]='AgZiCQagWKACIQ9AYxFaWLbiq6zcagZc9JUgpsa9CE3Nahq6i/yKNSwMxQ9JcxuNUr5oKwZ98VYa';
			B[i++]='sp+xAAiKRzACgJ+QEbPIVyw32g0UkqAA3t44SEKCBQMSMEAEECilFmCwNv76UiYfuC8vaXI2Hbyk';
			B[i++]='4yQ3HOOoQx7s6NwJw+GON7VShaOb2Oec47BTvrBQiYJdojIGEPDYMDwaSYkwhbmy9ZCnPQlK5kx6';
			B[i++]='AqmZ0SRTMDMKUYrmk58MZUHL20oxJuAArCkpEeErSyLACb40Vs1XKZiBF+XiPQmocxFZ69AiALiI';
			B[i++]='+ykyfoyQGz4DSZf5tcVEdqH/JzvrCVBCzEVEfAER2v7WrkLm83+H5GeJUiCVJ1HyCKtBAeM2ulFQ';
			B[i++]='CgyTGBTYliyHjZLmhhy4MU5x4OHKOLkQT6jDR0VYuLA56QNi3oFhoFqHu4xgh3Uy1KUMg6pD2Jnk';
			B[i++]='hioDgXnMA5JIVSo9R5yZTKQHtFhVlXpOjJ5RmObEaz7IQQ9yCtTAylWmMYU0EloSWJYkCLJkr1VW';
			B[i++]='IQ1aNwTONOaljQ5FUVz+F4lkka1sKWLoPqs1iq5577DHShE7DWG1QcxFQ3Kb185CYNEoYfQE3qiS';
			B[i++]='vjrpydgw0F9pwhJxHic5c3DQTG5ipUsp4jlUnlIdrcQYQGTIOod4R5Zykqnp/1xA2374kmQ3dFTw';
			B[i++]='TlLUYrLsJEas3ahGtcxluswlQIluFaFXReKNha3XU6t1s0g8ASVFjopQUjjHG77vyvEqYATfix7q';
			B[i++]='orBpaJDo04tc/ghYhP7PEfc9loZSgKEOEZJ8yYJo1/ADKkle9AMH0ABHFwybyTmOtLjh4DrQtMHg';
			B[i++]='gDRLEdYcamGr2jct55QPa20tZ/mc0fUSIPawx0R2GrvicoAjuhRPi4GnXPt0Cnmgss+l1AMz+WBx';
			B[i++]='QJPlWU9MAEUiQ7EnRykVfpQsPSVPD61yZFVVBPGV8oaFLGbZyhe3IoguqvNWFSKvWch5IfL1d794';
			B[i++]='KUtj/RvAuqRFzefEr1wAaf/mOnsIkd3sT2URxwEFM3jBlJMNhDPHpjZtUDkoVFhrXavCzaXScxYb';
			B[i++]='YQtLp5BfWhqpNizucI+LkkxxusbCszFJmhuzmAg5yTnZbvKMchMkH88nJKmikgUUq1I5KGkQktpV';
			B[i++]='qJzNMZoxfGg8BIf+GV9ilTlr92UnGNvCKzD26ldvSSybG+qivAbyQ3hOEkDTbNC5YDl8YukJZSdp';
			B[i++]='WQRrYLP7+uwlR3rhCJJpTOjAoLwbTJw0GRphG3Y0S9vB71e+lIW3FYjqtMOPgTfEOvhAAQwdEqih';
			B[i++]='zth3xj2qxFfSaZIgaIpEhJmQeSLknsgkmU4j8lAAlKqSL1nWzcseW799V63/lYVVSYsnZNESvu5a';
			B[i++]='xSpgnIDY6Be2aM9gzj9HbCzMJkhBCtiwh9WnnR3ApHEfOMEMlncFtcSNyUEw6qMktMG6sVLeSJjC';
			B[i++]='7Ph3nVa72+tgByEdc7jafcrbtqe9dRuhlIxltyiNmATkCHLZcnEGlJqgvECgmuzxOH5jsNRnyEQp';
			B[i++]='CvKGAmQgm8opUA6nQe+CF7vG1byRFy/le16Xdya9axiy9tJjRHpmdcgs9TzRXZ49te7tPLH45ate';
			B[i++]='KToTp5e7z5xscOXQnW5+8etfosQSNa4h7+H7BmGdexgJ9Y3SCScH0SB2GMVsWpF+pLgFJMC+wxsy';
			B[i++]='so1ZenWD0v5Q/zQyhKRM/3ZPDR7N3oN3H3K6JJriVHN5chT49Cd7IUjVBMRteLAQHuNUNEVIURRT';
			B[i++]='YQxH01ZMcV2jYRa5AivUBRXQA1dj1GVx5mbiVGbjlCTfA2xd5D0/Vz9pUT7jJHoo0hYl8k9yEUbj';
			B[i++]='9VgvAkAdcAy1twDkhjgHEAN/9jiyUWgDI0FpclKmRXzw9m47yHXrwAKqNEs3pVuqc2IMVxEylX0C';
			B[i++]='oWK+NSiAYii+tToz5UKFUihGhX4Uxx7FpGnt0VTQtSk8cXKo9neytl2gEjSx4kRINmQOMl25Ak5b';
			B[i++]='8R/4lypbJRVQ0RXGcDXcIzaKkHPnRGz6VReoF15zBiyDeEdacTV59AjyJP9gHCJYc2M1XMQVYPZF';
			B[i++]='r6csirBe4pUhyTJQLigsgwCDNyGDB2YBfsZg/+J7gmYbpNU4nIRJ/VIw3mBvwoEc+dZo0pF8LSR2';
			B[i++]='wkgR0CFpzzEQBTEouFSFVGgRC+cnNuSFO7RjPcRjJ9Ey6vcoLUF4UuVchCdkaGhrrIYTOqGGW+VV';
			B[i++]='YNUSIlcUBAgrUvEUrjI9yuN4q3IVZAFlLBdOYiRGYdQr5oRmVYNsxpIX+mVY/vSJ8SROjMUhfuQi';
			B[i++]='+nVYxaBOFbJmZBZOYnEfDjCD94J7DvRZHsl7N8g4HmVah3Zo71aSqKUcxwcPZuJhm0NLNRWTc1JT';
			B[i++]='eUKMAyEQMURUJiESazf/VLqEKJrGHo8SVdc4KcO1dzshf+DYKaM2ap7GQxkXFEhGRYmXVSlHPNql';
			B[i++]='atfzXQLiTRDSenjkhwyIa8tDFVyGRugTX3b1crt2NWGEWFoRkGKWJI4Fl1rRK1v2lmdpF8eAkRr5';
			B[i++]='DKx4gyDZkR4pb7kROV6CObiBmDs4MOkgYcwXme/gSnTSb7+ofKBTMSfmW7zlU0bFk9LIOyDRO021';
			B[i++]='MpIyO3t3jUSEXJNyZNVkPD5RjjixEwTGE/+HZFm1aqqWPDchmzm2H7aGlVipNd7Ej13WFWOkFCsX';
			B[i++]='NVJ2FXcpZmNmbG7plkoDc1iWjxcic+O1XhkYgjJ3WEySin+pGhwZkroX/yaDZpgRZmHGdxuSwxsT';
			B[i++]='dlrwiRwnhEr26Yv6MB01yUIvFR0rlA85JYUNR4UIsYzjhzLdtzsGAVSiGVymCX+2yTO2SSly1yjQ';
			B[i++]='BJsn139MmTOzGTTJs13viEUg6mRn9VbZIxaq8kjnaJVQ8SpM4SqZGF4U2FjQeRajV236CGczZzUZ';
			B[i++]='chaKYD80SnkWaUbGwEwmYEBPd26E6UCa9ZFOqi+516RXInwiBZ/sQGHtOXwa5nzq0IvHCB3ISDr5';
			B[i++]='8G+zpWLZFzK+1To92ZMhMXfm5xEJWjLUCEQ9JBM0IxQIsiCIF11muBPmWBRQxBJEFodDIW6pkngo';
			B[i++]='B4FOtpvdNT2uh07+eP81Lxek4EZGznapSTEDNRpH42WiS4ErciVG2QlnbcSpePFsdtEr05mInZof';
			B[i++]='IaCK5VaDMsAD5kmLUAo5VjJKUPqKt1p8v1FK8MB12KClHRSEO+iLJdQakrmflSl2E7FbrJOTHkOg';
			B[i++]='GmMdElF3MFY7LMN3pyZNHgdyRmQfn+J/xfNMUqlxASiOTYYfZWVWBMhlC/hyFkIajLpkH7qbbbVW';
			B[i++]='WkNPg4hOW/ZOuiKpK3iB49NynbpWpOFN4TWJFVlzFnletFcSJ2AAp1Fu0QCSvHdBDOQaN1gwiPme';
			B[i++]='jUmSvHF8POiY0Bd2/eZCMklix/hLNVRDG2N+oAljPDmzRQlxEXd36QH/hkP5KHd3TAgCKTDhTEj5';
			B[i++]='mzQhm2rYjksBXhaiNFGjoqriXf5xectzXfnIcrmCXRMoqV9hXlmjnE3rllVmXrryluY0LJpHqZi4';
			B[i++]='axHSgJf3E5FiAghQAAeWAiTAq55Fq7M6qzlAq33LOA1Uq1FnJbs4rJLpfF9HhKl0JyEEQvpZQmA6';
			B[i++]='ab20cNZ6cNihO31SfkI1VNLqYvAxakG0jZliM8M0lO5xHikxRUTjNGuYlY+UoYyar2j4re0nqAJY';
			B[i++]='NELjeJBnlhSYowOpeQFEZoaQK7vSgE+2RWWLNadXTgxIpGX0bNTJZawSvV1GTi9SKqAiAQ+QAAfW';
			B[i++]='AAkWpQ+Ue+h2q1Ui/3Wb9JHxFkqkdFJWerIsuW8J0zB5MqYWkWLO+ErBSKYdYxFr2pMNt2KrExC8';
			B[i++]='1Q92F5QQF2pFGzwkMIZ6F2u/mTxM+UhtGJz1+Icrh6L0yKiw4lbIKzXJORVWVE3mqK/Pg1ZulVYo';
			B[i++]='HE7YOQhYFoKhp5DQGaTgRJdxBK/ySl7QUxoQAAAHVgQLcAAoYEEM9Hv9IlIfNUG6QawSRoTCWkrg';
			B[i++]='kIu9uGEbFltaqFOBYg/OYSf6aw/0K8DRikvcYXa11VMO16Y1C5pmbJrbWpSjAmr5QR8cZ0VlFXLt';
			B[i++]='yo5DU5Zvtbu85oBK8SDY1MdlWZY1aiFcc2Y0qiHc45wvp6qOdV+Fdf9tRfc2+WWXedQrZuM3sSBP';
			B[i++]='a5mIONoWdDEvpGEABFABQnBRGBUBDmABIOCSitZCK8uyMklTXwoCoEPLZndpMQsoPDVDu/OmJSNc';
			B[i++]='FpeUEqpjovtMCbKnTIOoAQI1XtHMd5xNEEiWxHtzCqu8WbYr4kN5jjxY1dIj7MMYPGIYOkIJwQAM';
			B[i++]='oNEADYAA6LzOHtDOHlAkPHIjOeLOiIHOm1Ek74wY7WwA7tzO8KzOoHEDAi3QDdDP7XwDHjDQN4AY';
			B[i++]='CK3QDv3QN2AACp3OldELDXAJwaDO67zR9tzOFI0YIhABFRAEpoxRH2AED2AAxmBNm2JkhRofUiRk';
			B[i++]='8HFkx8O6uOuioPr/qXU0FUdzc2SpZfuFR/+UPqJQGN6yP5QxAvpDCnozCULiGRGt0eu80FKNAPmM';
			B[i++]='AAjdzgqAAAqgABdwAV0d1l7t1S/Q1RcgAiKgAC+w1mCtABydzlTtAW4t1nTd1WitAGgtAg+w1w+Q';
			B[i++]='AQ8QAQQQ2IFtA4Jd2IadAQSQAYqd2IzN2IqN2A8Q2I892Ytd2HtdAnyd2X1N2RkA2BHw2aAd2gwQ';
			B[i++]='2oYt2KQd2qid2qoN2qPNAK492qsd2ohd2qoNAaENARDQ2q4dAbDN27vN26pd2gQA2sNN3I8dAQEg';
			B[i++]='ACRtWYjzAUSA25I92YlN2Y1d2MTt2Z9d3NeN2r8d2zaw3azt2jbw/9qujdvmndu4XQDqDQEFwN7q';
			B[i++]='/d7pnd7vPd/vHQAFYN/2Td/0nd/qHQD4fd8Ajt/+7d/9PeAGfuAInuAD4N8DMAAJ0OAP3uASPuEU';
			B[i++]='vuABUOEQngAavuEavgEc/uEJ4OEhvgEkXgEJYOIbbuIVIOIaXgEmTuIdPuIkDuMl7uIrbuMrDuIc';
			B[i++]='/uI0fuIb4OIzfuM2jgNEXuRFXgFH7uI4gORG3uRO/uRQ/uRMHuVUvuRWTuRTfuVTLuQ47uE4juRD';
			B[i++]='buUovgE4IABD8AElXW5ovuZs3uZu/uZwHudyPud0Xud2fud4nud6vud83ud+/ueAHuiCPuiEXuhx';
			B[i++]='ztyInuiKvuiM3j/ojv7okB7pkj7plF7pln7pmJ7pmr7pnN7pnv7poB7qoj7qpF7qpn7qqJ7qqr7q';
			B[i++]='rN7qrv7qsB7rsj7rtP7pgQAAOw==';
			this.images['dialogs.message'] = { name: 'messageDialog', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhOAAUAPcAALbDybXCyLPAxrG+xK67wau6v6i3vKW0uaOyt6Cvtp2utZuss5irspCjqoqe';
			B[i++]='pYaaowAAAhYXGxUWGBYXGSEiJCAhIyMkJissLi8wMi0uMF9gYggLEH2EjoGIkKiyu1phZwkNECcs';
			B[i++]='MCElKDI3OzU6PpSirZmnsm12fT1CRnqDijU5PIGJj0NHSltgZFxhZVldYI2fq6Cut6SxuaKvt6i1';
			B[i++]='va67w6y5wYaYooiapJGjrZCirI+hqyovMpmqtJeoslNcYTM4Oz5DRq+9xXB5fpSfpX+IjZ+qsImS';
			B[i++]='l1BVWLS/xbnEyrjDybfCyLbBx42SlYGXoiElJ6ezuYebpIufqIygqZGlrpmqsqGyup+utZ2ss6W0';
			B[i++]='u6OyuaKxuKGwt6u6wai3vqa1vLPCyaCts6Sxt6Kvta+8wrfEyrTBx7K/xbC9w4+gp5eor5anrpSl';
			B[i++]='rJyttJmqsZipsKGyuaCxuJ+wt56vtqa3vqS1vJWfox4mKYKZoYqhqoWcpISbo4ebooaaoYufppGl';
			B[i++]='rIyfppWpsI+iqY6hqI2gp5qutZWor5SnrpKlrJGkq5+yuUdPUkhQU1VdYHaBhVtjZoSPk4KNka27';
			B[i++]='wKu5voeSlnJ6fa24vLnEyLjDx3yPlZCjqZeqsJ2wtpqts5mssqO2vI+eo6KxtqGwtaCvtJ+us56t';
			B[i++]='sp2ssae2u6a1urC/xK++w669wq28wbbFyrXEybTDyLPCx7LBxrHAxYGZoIyXmq23uggMDRQYGRYa';
			B[i++]='G4WcooifpY6lq42ip0dPUVNbXSouL1hgYldfYVBUVXiSmJWorJKlqZuuspSmqp+xtZ2vs5yusput';
			B[i++]='saO1uaGzt6Cytn+NkKi6vqa4vIORlLDCxmVvcWJsbpqprKa1uKGws5mnqqu6vXB6fLXEx6q4u7TC';
			B[i++]='xbPBxLG/wrfFyLbEx7XDxoONj36HibrExnWNkWp1d4qVl6m0tpClqKS2uKO1t2lzdJuvsLLExZGf';
			B[i++]='oLbExQACArTExJGdnVpgYFZfXAsNDBYYFy8xMAsNCgwMDAAAAP///yH5BAEAAP8ALAAAAAA4ABQA';
			B[i++]='AAj/AP/9K/agz4ODCA9KebCHjy4/Bv34eYDjhhSJCTNq3HiwjcB/xGhNmeKgpMmSIx382bXrz5Q/';
			B[i++]='Lke+PEmzpk2TedD9M/XEDbNTC4IKDeomy6ksbtwEZZZlgSdPRZkNnUq1qlBabtRQOcXMkFVPzJYF';
			B[i++]='fbpM7IJjx5Satcp2qikYgfSUcINFjIK7eO+6UaDMDR0s0MqVolvrERYFpvIqXsz4LrwcfwgZSxZj';
			B[i++]='ToLLmDF36TKHDhlIt0jNIYViBKlOozKrXs36cpcqvQYd4rJlC4LbuG/HEdWM9jNtG6rJYeMvhZwr';
			B[i++]='z67EGbPImRwuY3JLn45gi6BBDQ6l0gLmgPfv3rWk/2oGRkszbIwu2PlVAZS9ICF+uWt3zRePIKHA';
			B[i++]='699/INWhBoUoYsCABBZoAA0GRGPAF9JIcwcEROjzTh24NNIBBT/UAYwKRQABjIEgGoiKIoEQkkgB';
			B[i++]='KKaoYgE2FEBJitEUQAIU/XixggQ2UPKIPtFAsMI2tfCw4pBDAkKIbK0kScCSTC45CSutlLEkK5NI';
			B[i++]='4s8LQgijjyP4sOAPKxlMkE42NTRp5pkEtILIIIOsoQoaqgwg55xyzrLKLGkMgEYas4TjTyRC3DOB';
			B[i++]='C8IEE0wSY1gTQT8n0Onoo3Ku0YAib8iChiwCZKqpprOcIYAs4HwqAATTgNMCBmiEsU4lUVRCzTfD';
			B[i++]='8Pez6ay0ZsqJIoqs8coZAfTq668ANBFAPOQAAEsA/hwRSyX+eEPOEBbI448R8XCQz6/YZtvrrZtw';
			B[i++]='0o0ZAIQr7rjkhqsEPU68AgAU+0ACgjnn4CGCBrkgUe694irBADu8mIAJE+Pgiy8T6tgCri2WfHBH';
			B[i++]='JuNcwg0k5mQi8L31wKEIMod0o4QrE5crziuvgPuKOJmIA64ZJI8DbsfjnuODIh5MsQQ587A8rskA';
			B[i++]='jCNOuOKYvLLKNpOrxwwgUdGDFQwkrfTSTCf9xhuffAIH0lI/HTUnSDetddJ/aPJRFIpg18DYZJdt';
			B[i++]='tg7Y6aCDIg3oQHbYO5gt99wyCBQQADs=';
			this.images['buttons.yes'] = { name: 'yesButton', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhOAAUAPcAALfEyrbDybXCyLPAxrG+xK67wau6v6i3vKW0uaOyt6Cvtp2utZuss5Cjqoqe';
			B[i++]='pYaaowAAAgoLDy0uMjs8QAABBAoLDQsMDhUWGC4vMVRXXLO9x6exum51ewcLDgkNECgtMT5DR09U';
			B[i++]='WFhdYbnCyVJWWY6grKGut5OhqpmnsJ+ttqGvuKCtta67w626wrjFzbbDy4OXoo2fqYudp4iZo5Gj';
			B[i++]='rZCirI+hqyovMpiqtCwxNJytt4GMkrPByUBFSD9ER3N8gXeAhX+IjYSNkoKLkImSl5SdornEyrjD';
			B[i++]='ybfCyFdcX1xhZLnCx7bFzbO/xYebpIWZooqep5Kmr5Glrpaqs5GiqpChqZipsZanr5SlraCxuZOi';
			B[i++]='qZ+utZ6ttJ2ss5uqsaW0u6OyuaKxuKGwt6u6wai3vpKfpa28w7XEy6azuaSxt6OwtqKvtay5v6m2';
			B[i++]='vLjFy46fppyttKGyuZ6vtqS1vKO0u4WcpISbo4mgqIeepouiqnyQl4SYn4mdpIebooaaoY2hqIuf';
			B[i++]='poiboo+jqoyfpouepY+iqY6hqI2gp5Wor5KlrJGkq5irspeqsZapsJqttKCzukhQU1NbXllhZFtj';
			B[i++]='ZoGMkICLj2Rsb2lxdHF5fKCrr665va24vLnEyLjDx4uTlpWdoLrCxZirsZuutJmssqCzuZ+xt4qZ';
			B[i++]='nqCvtJ+us56tsp2ssae2u6a1urC/xK++w669wq28waq5vrfGy7bFyrXEybTDyLPCx7LBxrHAxam2';
			B[i++]='uoCZoIegp36ZoIeiqRQYGXeOlISboYWconiNkoqhp4mgpoifpXuQlZessT5GSCAkJSgsLSwwMSsv';
			B[i++]='MFlhYy0xMmBoalFVVp6xtZyusputsaW3u6O1uaGzt6m7v6a4vLHDx4yanZ+usaKws6Gvsm95e7TD';
			B[i++]='xrHAw3R+gK27vrfFyLbEx4WPkYmTlYeRk664uq23uZCorLjDxZarrqCytLDCxE5YWVxmZ2ZwcSow';
			B[i++]='MLTGxiYvLgoMCw0NCwAAAP///wAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAPgALAAAAAA4ABQA';
			B[i++]='AAj/APHhuxIo2IwHCBMidPLATh1gfgI9CPaAYbA+Txgq3MixopOMCKMIxPcLxi4ZuxyoXKkSCpRi';
			B[i++]='MA/xKQYIELFiDgjZZMmz50o+gFQC2iMMnypdDESpYsC0qVOmcLh0geOI6TQ4TaU93coVa1M4vFK9';
			B[i++]='yTNtFBcdXKEymOrI0VWvVEWlncsAbtMSg4ZR2bZF2pYFgAMD3gJnwYotolABoXTK8A5TcFIInkwZ';
			B[i++]='cOEFhQtXiWFISjQTYqIpGE16tIlS0cTIidaNQodupdZ8eMdNRenbuEeLUSBmdxgF6jgjUmMtTJwE';
			B[i++]='yJMjLwXGWpxSx+9BiAemFL0McUglaJegVBZr1tIo/x9PPgGiQg0Qsar2ZQ6C9/Dhf6mGgD2Cez6U';
			B[i++]='zWH1IdIXBPB8AII5dKCBgHvxJahgI4IcosgBB5BBBoQUVkgGNhBOeM8PFpSxyg2RHHCJL+FAcgE2';
			B[i++]='bbRBTYUstniAIocYoogBBrBB4404GjDGjTveUwkyPcRyQxIGSPAMjRYEYcA1muToZI6J/FHIFbDA';
			B[i++]='UsArBWSpZZZWdlnlPUNoU48a8ogACwRCsNBCDmaaIc6WcMZZwHmFNNIKLgS0QsCefO6pp5554nJP';
			B[i++]='Oa4sw4EzzTQBQTl4MiNJKxqA0+eklBLAYANWDHCLpgN06mmnm26q6S33nMODN8lg0MwAFZwDjjvL';
			B[i++]='WP9yCw9NfGrrrQMwoogiVghQiwDABivsN74CW0st95hjiwD2RKDENxOEIEAu9xTxDRLZCKvttgIs';
			B[i++]='okgDi5BzRifzBGDuueYeQc44nRhxBCgUmDMOJyRAMAknRHgAjTM5LAFAAOSgK7C5AAAggLnsKJII';
			B[i++]='CuPMQs4IAxN8MAC00DKOOZtUnI4nmQRwxCeTYILOEQEUHDHB/xIcAAqJYDGFxwAwcfIRbgRgRACy';
			B[i++]='BMAJJy8wwa65RvzLScCzBDDOyecW/C8AOGiRCx4ujLNOwBEb8U0nnBghAADkdPLvOOSYbG7Y5nbS';
			B[i++]='CdIlp1xwL2rgo8cdi3gxyiJ01013KKGMYoUVeONDPXfeVoxyzCh5M0L43nYnPorhdR9izEgbnKCI';
			B[i++]='IA1UbrnlNWReRQOZU04FDTbU0ADlhQjybSE12HD56oUogh56jwgUEAA7';
			this.images['buttons.no'] = { name: 'noButton', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhHwALAPcAAFBWWz5DR0FGSZijqWRrb0pPUomcppWmr6GxuZ6ttaW0vI6boqe1vKWzuq26';
			B[i++]='wYSOk7K/xsDN1BweH7bCyDxAQrnFy0FFR0VJS4icpYmcpY6hqpSmrp+utYeUmqe2va26wHiBhbfE';
			B[i++]='yrbDybXCyLPAxrK/xbnGzLjFy4mdpYicpI2hqZqrspmqsaGyuaCxuJqqsZior4eVm6e4v6W2vaCw';
			B[i++]='t56utZ2ttKKyuaGxuEZNUICMkaKxt05VWKW0uqu6wKi2vI6an6+9w7bEyrTCyLPBx7LAxrG/xYGL';
			B[i++]='j2NqbbXBxm51eIico5GlrJCkq5Cjqo+iqZWor5SnrpOmrZGjqpSmrZyutZuttJmrsp2utJ+vtZ2t';
			B[i++]='s4iWm6KyuKGxt6m5v1lhZFdfYltjZqi2u4SPk668wa27wGJqbbXDyLLAxbG/xLrIzbnHzLjFyrfE';
			B[i++]='yZynq4edpIWbopKlq5SnrZqsspmrsY2doqm5vqe3vLHBxqe2u5Cdoa69wq28way7wKu6v2pzdkVL';
			B[i++]='TbfGy7HAxZejp6i1ua25vYyVmIqTloWaoJOnrZywtjY9P56wtZ6vtJqrsIuZnWt1eHaAg4WQk7XD';
			B[i++]='x7TCxrnHy4ado5SlqXB8f3eDhn6KjbTDx7PCxrC/w7nIzLG9wIWUl32Kja2/w1liZG94ekBFRmNr';
			B[i++]='bP///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAKcALAAAAAAfAAsA';
			B[i++]='AAj/AE/BQZGiYIoMbywVXGIgwxKDECNiQHTqSaI5LDKyuOKIhRVGLBRVoaOxpMkVVFRosMGhhssa';
			B[i++]='WBq52BJKho4YM2gkQNDipU+XWlTE6XJjh9EdXbgoeLQoS4AOdV54ubSgwdGrO3AwcdLjR56veRjc';
			B[i++]='8SAKDI9RmAB8AcUjR6Y9YMGK6eHEiR0+ZfKWcdDnAx49Eh4IiFQqjIAxQATp1UvGjhMqQYqUmFxC';
			B[i++]='kJE0aToBGkQBBKkBmiwcMUGZMok9UqigETGidesJm84MuUCogCQCpiCZQRLBtWsREORIScImhPHj';
			B[i++]='xtsYquAGyaFPf5QUMoHc+AlKcmD4mUSku3fvazgFIfJUSYSaNUK+ex/i48CpJlHiy59PZYP9DfWl';
			B[i++]='zJ8PZcqpgAA7';
			this.images['buttons.small.yes'] = { name: 'smallYesButton', data: B.join('') };

			B=[]; i=0;
			B[i++]='data:image/gif;base64,';
			B[i++]='R0lGODlhHwALAPcAAAoKC0pPU1RZXSUoKlVcYElOUbO/xrK+xQ4QEY6gqZOlrpSlrqKxuYKOlHiC';
			B[i++]='h6q3vjU5O7PAx73Jz7jEyp+pro+Xm4icpZSmrpOlrZGiqpWmrpSlrZmqsp+utWBpbWt0eK67wXR9';
			B[i++]='gbfEyrbDybXCyLTBx7K/xbjFy6axtoaaooicpI6hqZOmrpanrpWmrZqrspmqsZipsKKzuqGyuZ+w';
			B[i++]='t52utZ6utZ2ttHuHjKq6wae3vqOyuKKxt6m4vqe2vKW0uoqWm3B6frHAxq69w6y7wVRbXq+9w7fF';
			B[i++]='y7TCyLPBx7LAxrG/xcDO1LnHzUVKTLjEyZymqm10d01SVJ6orIidpZGlrJCjqo+iqY6hqJSnrpOm';
			B[i++]='rZWnrpSmrZ2vtpmrsp+xuKq8w6KzuaGyuJ+vtZ6utJ2ts6i5v6KyuKGxt6u7wae3vXN+glVdYLHB';
			B[i++]='x4+coVdfYrbGzFtjZqi2u668wa27wGJqbbfFyrTCx7vJzrnHzLbDyLTBxrrHzLnGy7jFyoKKjYWa';
			B[i++]='oYidpJSnrZyvtZuutJmrsZiqsIuan6S0uYWSlqe2u6W0uaOyt6q5vrvL0LfGy7bFymt0d294e7TB';
			B[i++]='xam1ubbDx4CJjIadpIedo5CkqpOnrVNdYKCxtoqYnGp0d627v3yGibXDx2Bnaaa3u7C/w73M0LrJ';
			B[i++]='zZ+rrnJ8fmpzdXN8fmt2eERMTf///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA';
			B[i++]='AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAK0ALAAAAAAfAAsA';
			B[i++]='AAj/AFuloKKiYEELlwKpCGQBkwqCBiNGBNQKi6YXHGBohMHhxYtBXwx1IVRoo0mNXry8uHBlRZkx';
			B[i++]='NmLGrNFhRqI1OhocYiCDhsyfMm8kqMLpDBoeSHmIEQOmDgJEm4KY6UQmTNKrjHagqWJlkRofisIq';
			B[i++]='8jGqzYcBODw4WMWKTQ+wYsXK+WHFSg8QdPLmJUIHTiQCb+JIKvAggBtSn/TqndPICoYhSkxINrGk';
			B[i++]='sqNUqjxBCCGlVBEgRyZPPmDAiBZBQvSQWL06lB4mUVCdAmBJACgnlCCx3j1iSRYNd/qIGC5iwhM/';
			B[i++]='eKBM4fMHBQVRFSQQJz5hwokSLmLkqJSke5ISe/ZEMrDzBImpEY+a5Ekyybt3JEjSbGmVoUWW+/dZ';
			B[i++]='LNjAQlMWFhdowkIWXCiA331a3JdJKwEBADs=';
			this.images['buttons.small.no'] = { name: 'smallNoButton', data: B.join('') };

			//EMBEDDED_PICS_END
		};

		this.loadTexts = function() {
			this.texts = {
					'linkInfo.moreInfo': { en: 'More info'},
					'linkInfo.category': { en: 'Category: '},
					'linkInfo.states.safe': { en: 'The page is safe for browsing'},
					'linkInfo.states.unsafe': { en: ' '},
					'linkInfo.states.unknown': { en: 'Page not available for scanning'},
					'linkConfirm.question': { en: 'Are you sure you want to visit this page?'},
					'linkConfirm.inprogress.reason': { en: 'Page scanning in progress'},
					'linkConfirm.inprogress.question': { en: 'Are you sure you want to continue before scanning is complete?'}
			};

		};

		this.getIcon = function(name) {
			return this.getImage('icons.' + name);
		};

		this.getSmallIcon = function(name) {
			return this.getImage('smallicons.' + name);
		};

        this.getImage = function(name) {
                if (this.images == null) {
                        this.loadImages();
                }
                if (this.isIE()) {
                        return this.dllPath + this.images[name].name + ".gif";
                } else {
                        return this.images[name].data;
                }
        };

        this.isIE = function() {
                return (navigator.userAgent.toLowerCase().indexOf('msie') != -1);
        };

		this.t = function(path) {
			if (this.texts == null) {
				this.loadTexts();
			}
			var text = this.texts[path];
			if (!text) {
				TACode.d("Missing text: " + path);
				return "";
			}
			var localeText = text[this.locale];
			if (!localeText) {
				TACode.d("Locale " + this.locale + " missing text: " + path);
				localeText = text[this.defaultLocale];
			}
			return localeText;
		}

	}

};

TACode;
// VALIDITY_FOOTER
